package com.qmth.distributed.print.business.bean.result.statistics;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.distributed.print.business.enums.StatisticsStatusEnum;
import com.qmth.teachcloud.common.annotation.ExcelProperty;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 印刷任务统计 result
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/7/10
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class PrintCountResult implements Serializable {

    @ApiModelProperty(value = "主键")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    @ApiModelProperty(value = "课程名称（代码）")
    @ExcelProperty(name = "课程（代码）", width = 20, index = 1)
    private String courseNameCode;

    @ApiModelProperty(value = "课程名称")
    private String courseName;

    @ApiModelProperty(value = "课程代码")
    private String courseCode;

    @ApiModelProperty(value = "试卷编号")
    @ExcelProperty(name = "试卷编号", width = 20, index = 2)
    private String paperNumber;

    @ApiModelProperty(value = "题卡id")
    @JsonSerialize(using = ToStringSerializer.class)
    private String cardId;

    @ApiModelProperty(value = "开课学院")
    @ExcelProperty(name = "开课学院", width = 20, index = 4)
    private String college;

    @ApiModelProperty(value = "总考场数")
    private Integer totalExamination;

    @ApiModelProperty(value = "已生成")
    private int generate;

    @ApiModelProperty(value = "未生成")
    private int unGenerated;

    @ApiModelProperty(value = "已印刷")
    private int printed;

    @ApiModelProperty(value = "未印刷")
    private int unPrinted;

    @ApiModelProperty(value = "已作废")
    private int invalid;

    @ApiModelProperty(value = "已打回")
    private int reject;

    @ApiModelProperty(value = "题卡类型")
    private String cardType;

    @ApiModelProperty(value = "题卡类型str")
    @ExcelProperty(name = "题卡类型", width = 20, index = 3)
    private String cardTypeStr;

    @ApiModelProperty(value = "状态，FINISH：已完成，UN_FINISH：未完成")
    private StatisticsStatusEnum finishStatus;

    @ApiModelProperty(value = "状态str")
    @ExcelProperty(name = "状态", width = 20, index = 8)
    private String finishStatusStr;

    @ApiModelProperty(value = "已生成/未生成")
    @ExcelProperty(name = "已生成/未生成", width = 40, index = 5)
    private String generateStr;

    @ApiModelProperty(value = "已印刷/未印刷")
    @ExcelProperty(name = "已印刷/未印刷", width = 40, index = 6)
    private String printedStr;

    @ApiModelProperty(value = "已打回/已作废")
    @ExcelProperty(name = "已印刷/未印刷", width = 40, index = 7)
    private String rejectStr;

    public String getGenerateStr() {
        return generate + "/" + unGenerated;
    }

    public void setGenerateStr(String generateStr) {
        this.generateStr = generateStr;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCourseNameCode() {
        return courseName + "(" + courseCode + ")";
    }

    public void setCourseNameCode(String courseNameCode) {
        this.courseNameCode = courseNameCode;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getCardId() {
        return cardId;
    }

    public void setCardId(String cardId) {
        this.cardId = cardId;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public Integer getTotalExamination() {
        return totalExamination;
    }

    public void setTotalExamination(Integer totalExamination) {
        this.totalExamination = totalExamination;
    }

    public int getGenerate() {
        return generate;
    }

    public void setGenerate(int generate) {
        this.generate = generate;
    }

    public int getUnGenerated() {
        return unGenerated;
    }

    public void setUnGenerated(int unGenerated) {
        this.unGenerated = unGenerated;
    }

    public int getPrinted() {
        return printed;
    }

    public void setPrinted(int printed) {
        this.printed = printed;
    }

    public int getUnPrinted() {
        return unPrinted;
    }

    public void setUnPrinted(int unPrinted) {
        this.unPrinted = unPrinted;
    }

    public int getInvalid() {
        return invalid;
    }

    public void setInvalid(int invalid) {
        this.invalid = invalid;
    }

    public int getReject() {
        return reject;
    }

    public void setReject(int reject) {
        this.reject = reject;
    }

    public String getPrintedStr() {
        return printed + "/" + unPrinted;
    }

    public void setPrintedStr(String printedStr) {
        this.printedStr = printedStr;
    }

    public String getRejectStr() {
        return invalid + "/" + reject;
    }

    public void setRejectStr(String rejectStr) {
        this.rejectStr = rejectStr;
    }

    public String getCardType() {
        return cardType;
    }

    public void setCardType(String cardType) {
        this.cardType = cardType;
    }

    public String getCardTypeStr() {
        if (Objects.nonNull(cardType)) {
            String[] str = StringUtils.split(cardType, ",");
            StringJoiner stringJoiner = new StringJoiner(",");
            for (String s : str) {
                stringJoiner.add(CardTypeEnum.valueOf(s).getName());
            }
            return stringJoiner.toString();
        } else {
            return cardTypeStr;
        }
    }

    public void setCardTypeStr(String cardTypeStr) {
        this.cardTypeStr = cardTypeStr;
    }

    public StatisticsStatusEnum getFinishStatus() {
        return finishStatus;
    }

    public void setFinishStatus(StatisticsStatusEnum finishStatus) {
        this.finishStatus = finishStatus;
    }

    public String getFinishStatusStr() {
        return Objects.nonNull(finishStatus) ? finishStatus.getTitle() : finishStatusStr;
    }

    public void setFinishStatusStr(String finishStatusStr) {
        this.finishStatusStr = finishStatusStr;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }
}
