package com.qmth.distributed.print.business.bean.result.statistics;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.teachcloud.common.annotation.ExcelProperty;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 扫描任务统计 result
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/7/11
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ScanCountResult implements Serializable {

    @ApiModelProperty(value = "主键")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "课程id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "课程名称（代码）")
    @ExcelProperty(name = "课程（代码）", width = 20, index = 2)
    private String courseNameCode;

    @ApiModelProperty(value = "课程名称")
    private String courseName;

    @ApiModelProperty(value = "课程代码")
    private String courseCode;

    @ApiModelProperty(value = "试卷编号")
    @ExcelProperty(name = "试卷编号", width = 20, index = 3)
    private String paperNumber;

    @ApiModelProperty(value = "题卡id")
    @JsonSerialize(using = ToStringSerializer.class)
    private String cardId;

    @ApiModelProperty(value = "开课学院")
    @ExcelProperty(name = "开课学院", width = 20, index = 1)
    private String college;

    @ApiModelProperty(value = "考生数")
    @ExcelProperty(name = "考生数", width = 20, index = 5)
    private int studentCount;

    @ApiModelProperty(value = "已扫人数")
    @ExcelProperty(name = "已扫人数", width = 20, index = 6)
    private int uploadCount;

    @ApiModelProperty(value = "已扫张数")
    @ExcelProperty(name = "已扫张数", width = 20, index = 7)
    private int sheetCount;

    @ApiModelProperty(value = "缺考数")
    @ExcelProperty(name = "缺考数", width = 20, index = 8)
    private int absentCount;

    @ApiModelProperty(value = "扫描进度")
    private BigDecimal scanUploadRate;

    @ApiModelProperty(value = "题卡类型")
    private String cardType;

    @ApiModelProperty(value = "题卡类型str")
    @ExcelProperty(name = "题卡类型", width = 20, index = 4)
    private String cardTypeStr;

    @ApiModelProperty(value = "扫描进度str")
    @ExcelProperty(name = "扫描进度", width = 40, index = 9)
    private String scanUploadRateStr;

    public int getUploadCount() {
        return uploadCount;
    }

    public void setUploadCount(int uploadCount) {
        this.uploadCount = uploadCount;
    }

    public void updateRateAndSheetCount(ScanCalculateResult scanCalculateResult) {
        this.scanUploadRate = scanCalculateResult.getScanUploadRate();
        this.sheetCount = scanCalculateResult.getSheetCount();
        this.uploadCount = scanCalculateResult.getUploadCount();
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCourseNameCode() {
        return courseName + "(" + courseCode + ")";
    }

    public void setCourseNameCode(String courseNameCode) {
        this.courseNameCode = courseNameCode;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public int getStudentCount() {
        return studentCount;
    }

    public void setStudentCount(int studentCount) {
        this.studentCount = studentCount;
    }

    public int getSheetCount() {
        return sheetCount;
    }

    public void setSheetCount(int sheetCount) {
        this.sheetCount = sheetCount;
    }

    public int getAbsentCount() {
        return absentCount;
    }

    public void setAbsentCount(int absentCount) {
        this.absentCount = absentCount;
    }

    public BigDecimal getScanUploadRate() {
        return scanUploadRate;
    }

    public void setScanUploadRate(BigDecimal scanUploadRate) {
        this.scanUploadRate = scanUploadRate;
    }

    public String getCardId() {
        return cardId;
    }

    public void setCardId(String cardId) {
        this.cardId = cardId;
    }

    public String getCardType() {
        return cardType;
    }

    public void setCardType(String cardType) {
        this.cardType = cardType;
    }

    public String getCardTypeStr() {
        if (Objects.nonNull(cardType)) {
            String[] str = StringUtils.split(cardType, ",");
            StringJoiner stringJoiner = new StringJoiner(",");
            for (String s : str) {
                stringJoiner.add(CardTypeEnum.valueOf(s).getName());
            }
            return stringJoiner.toString();
        } else {
            return cardTypeStr;
        }
    }

    public void setCardTypeStr(String cardTypeStr) {
        this.cardTypeStr = cardTypeStr;
    }

    public String getScanUploadRateStr() {
        return scanUploadRate + "%";
    }

    public void setScanUploadRateStr(String scanUploadRateStr) {
        this.scanUploadRateStr = scanUploadRateStr;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }
}
