package com.qmth.distributed.print.business.entity;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务明细表（入库申请）
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@TableName("exam_task_detail")
public class ExamTaskDetail extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 命题任务ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("exam_task_id")
    private Long examTaskId;
    /**
     * 备用卷序号
     */
    @TableField("serial_number")
    private Integer serialNumber;
    /**
     * 试卷类型，多个以”/“分隔，如A/B/C/D
     */
    @TableField("paper_type")
    private String paperType;
    /**
     * 是否曝光
     */
    @TableField("exposed")
    private Boolean exposed;
    /**
     * 曝光的考试ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(value = "exposed_exam_id", updateStrategy = FieldStrategy.IGNORED)
    private Long exposedExamId;
    /**
     * [
     * {
     * "type": "A",
     * "attachmentId": 12,
     * "totalPages": 3
     * },
     * {
     * "type": "B",
     * "attachmentId": 13,
     * "totalPages": 2
     * }
     * ]
     */
    @ApiModelProperty(value = "转换后A3试卷附件")
    @TableField("paper_attachment_ids")
    private String paperAttachmentIds;
    /**
     * 曝光卷型：已打印卷型，多个以”/“分隔，如A/B/C/D
     */
    @TableField(value = "exposed_paper_type", updateStrategy = FieldStrategy.IGNORED)
    private String exposedPaperType;

    @TableField("relate_paper_type")
    private String relatePaperType;

    @TableField("unexposed_paper_type")
    private String unexposedPaperType;

//    private Boolean enable;

    @ApiModelProperty(value = "流程审批人列表")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(exist = false)
    private List<Long> approveUserIds;

    /**
     * 保存内容
     */
    @TableField(exist = false)
    private String examTaskContent;

    @ApiModelProperty(value = "卷型解析内容")
    @TableField(exist = false)
    List<PaperInfoVo> paperInfoVoList;

    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    public List<Long> getApproveUserIds() {
        return approveUserIds;
    }

    public void setApproveUserIds(List<Long> approveUserIds) {
        this.approveUserIds = approveUserIds;
    }

    public Long getExamTaskId() {
        return examTaskId;
    }

    public void setExamTaskId(Long examTaskId) {
        this.examTaskId = examTaskId;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public Integer getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(Integer serialNumber) {
        this.serialNumber = serialNumber;
    }

    public Boolean getExposed() {
        return exposed;
    }

    public void setExposed(Boolean exposed) {
        this.exposed = exposed;
    }

    public Long getExposedExamId() {
        return exposedExamId;
    }

    public void setExposedExamId(Long exposedExamId) {
        this.exposedExamId = exposedExamId;
    }

    public String getPaperAttachmentIds() {
        return paperAttachmentIds;
    }

    public void setPaperAttachmentIds(String paperAttachmentIds) {
        this.paperAttachmentIds = paperAttachmentIds;
    }

    public String getExposedPaperType() {
        return exposedPaperType;
    }

    public void setExposedPaperType(String exposedPaperType) {
        this.exposedPaperType = exposedPaperType;
    }

    public String getRelatePaperType() {
        return relatePaperType;
    }

    public void setRelatePaperType(String relatePaperType) {
        this.relatePaperType = relatePaperType;
    }

    public String getUnexposedPaperType() {
        return unexposedPaperType;
    }

    public void setUnexposedPaperType(String unexposedPaperType) {
        this.unexposedPaperType = unexposedPaperType;
    }

//    public Boolean getEnable() {
//        return enable;
//    }
//
//    public void setEnable(Boolean enable) {
//        this.enable = enable;
//    }

    public String getExamTaskContent() {
        return examTaskContent;
    }

    public void setExamTaskContent(String examTaskContent) {
        this.examTaskContent = examTaskContent;
    }

    public List<PaperInfoVo> getPaperInfoVoList() {
        List<PaperInfoVo> paperInfoVos = new ArrayList<>();
        if (StringUtils.isNotBlank(paperAttachmentIds)) {
            if (StringUtils.isNotBlank(paperAttachmentIds)) {
                return JSON.parseArray(paperAttachmentIds, PaperInfoVo.class);
            }
        }
        return paperInfoVos;
    }
}
