package com.qmth.distributed.print.business.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.dto.initMarkData.ExamDetailCourseInitMarkDto;
import com.qmth.distributed.print.business.bean.dto.initMarkData.InitExamDetailDto;
import com.qmth.teachcloud.common.bean.examRule.FieldsDto;
import com.qmth.distributed.print.business.bean.result.ExaminationDetailResult;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.bean.result.SummarizedDataResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDataSourceEnum;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.CreatePdfTypeEnum;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 考务数据 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface ExamDetailService extends IService<ExamDetail> {

    /**
     * 计算印刷计划下的总科次
     *
     * @param printPlanId 印刷计划id
     * @return 该印刷计划下总科次
     */
    double calculateTotalSubjects(Long printPlanId);

    /**
     * 计算印刷计划下的总卷袋数
     *
     * @param printPlanId 印刷计划id
     * @return 该印刷计划下总卷袋数
     */
    double calculateTotalPackages(Long printPlanId);

    IPage<PrintTaskDto> listPrintTask(Long semesterId, Long examId, Long printPlanId, String status, Long courseId, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Long printStartTime, Long printEndTime, Long printHouseId, Integer pageNumber, Integer pageSize);

    PrintTaskTotalDto taskTotalData(Long semesterId, Long examId, Long printPlanId, String status, Long courseId, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Long printStartTime, Long printEndTime, Long printHouseId);

    List<JSONObject> taskViewPDF(Long examDetailId);

    /**
     * 查询有效的考务字段
     *
     * @param schoolId 学校id
     * @return 考务字段
     */
    List<FieldsDto> findExaminationFields(Long schoolId);

    /**
     * 考务数据模板下载
     *
     * @param schoolId 学校id
     * @return 是否成功
     */
    void downLoadExaminationTemplate(Long schoolId, HttpServletResponse response) throws IOException;


    /**
     * 查询考务数据-简略
     */
    IPage<ExaminationResult> findExaminationBriefPage(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber,
                                                      String examPlace, String examRoom, String packageCode, Long startDate, Long endDate, int pageNumber, int pageSize, DataPermissionRule dpr);

    /**
     * 查询考务数据-汇总数据查询
     */
    SummarizedDataResult findSummarizedData(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber,
                                            String examPlace, String examRoom, String packageCode, Long startDate, Long endDate, DataPermissionRule dpr);


    /**
     * 查询考务数据-明细
     */
    IPage<ExaminationDetailResult> findExaminationDetail(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber, String examPlace,
                                                         String examRoom, String studentParams, Long startDate, Long endDate, int pageNumber, int pageSize);

    IPage<ExaminationDetailResult> findExaminationDetail(Long convertIdToLong, String param, int pageNumber, int pageSize);

    /**
     * 查询印刷计划下考点数据源
     *
     * @param param
     * @param examId
     * @param printPlanIdList
     * @return 考点集合
     */
    List<String> findExamPlaceDatasource(String param, Long examId, List<Long> printPlanIdList);

    /**
     * 查询印刷计划下考场数据源
     *
     * @param param
     * @param examId
     * @param printPlanId
     * @return 考场集合
     */
    List<String> findExamRoomDatasource(String param, Long examId, List<Long> printPlanId);

    boolean taskSubmit(ExamDetail examDetail) throws IOException;

    boolean taskFinish(ExamDetail examDetail) throws IOException;

    boolean taskCancel(Long id);

    boolean taskBatchCancel(Map data);

    /**
     * 删除考务数据
     *
     * @param printPlanId 印刷计划id
     * @param source
     */
    void deleteExaminationData(Long printPlanId, ExamDataSourceEnum source);

    List<ExamDetail> listByPrintPlanIdAndCourseCodeAndPaperNumber(Long schoolId, Long printPlanId, Long courseId, String paperNumber);

    /**
     * 查询考务数据-场次导出数据源
     *
     * @param map 考务数据查询条件键值对
     * @return 导出数据源
     */
    List<ExaminationExportDto> findExaminationExportDtoDatasource(Map<String, Object> map);

    List<Map> listStudentByExamDetailCourseId(Long schoolId, Long examDetailCourseId, String ticketNumber, String type);

    boolean taskNormal(ExamDetail examDetail);

    /**
     * 根据班级id组查找班级名称组
     *
     * @param clazzIdList 班级id组（集合）
     * @param separator   分隔符 默认","
     * @return 班级名称组
     */
    String findClazzNamesByClazzIds(List<Long> clazzIdList, String separator);

    /**
     * 根据id下载examDetail
     *
     * @param ids
     * @return
     */
    List<ExamDetailPdfDownloadDto> findPdfDownload(List<Long> ids);

    void dataDelete(Long id);

    List<ExamStudent> listStudentByExamDetailId(String examDetailId);

    List<ExamDetailCourseDto> listByExamIdAndCourseId(Long schoolId, Long examId, Long courseId);

    void deleteByExamIdAndPaperNumber(Long schoolId, Long examId, String paperNumber);

    TBTask importData(String printPlanId, MultipartFile file);

    void deleteExaminationDataByExamDetailIds(List<Long> examDetailIdList);

    void resetExamDetail(Long entityId, CreatePdfTypeEnum type, ExamDetailStatusEnum reject, boolean clearPaperType);

    List<ExamDetailCourseInitMarkDto> listPrintFinishExamDetailCourse(String printStatus, long startTime);

    int countMakeupUsedByExamIdAndPaperNumber(Long examId, String paperNumber);

    void updateStatusById(Long id, ExamDetailStatusEnum examDetailStatusEnum);

    List<ExamDetailCourseInitMarkDto> listPrintFinishPackageCode(Long examId, String paperNumber, Integer serialNumber);

    List<InitExamDetailDto> listPrintFinishExamDetailCourseIds(Long examId, String paperNumber, Integer serialNumber);

    boolean allExamDetailIsExist(Long examId, Long courseId, String paperNumber);
    boolean allExamDetailIsFinish(Long examId, Long courseId, String paperNumber);

    List<ExamDetail> listExamDetailByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber);

    List<Long> saveImportExamStudent(ExamPrintPlan examPrintPlan, List<BasicExamStudent> basicExamStudentList, Long createId);

    int countByPrintPlanId(String printPlanId);

    void updateTotalSubjects(Long id, int totalSubjects);
}
