package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.params.TeachCourseSelectParam;
import com.qmth.distributed.print.business.bean.result.CourseResult;
import com.qmth.distributed.print.business.bean.result.TeachCourseResult;
import com.qmth.distributed.print.business.entity.TeachCourse;
import com.qmth.teachcloud.common.bean.result.DictionaryResult;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;

import java.util.List;

/**
 * @Description: 教学课程服务类
 * @Author: CaoZixuan
 * @Date: 2022-02-07
 */
public interface TeachCourseService extends IService<TeachCourse> {

    /**
     * 分页查询教学课程列表
     *
     * @param examId     考试id
     * @param courseName 课程名称
     * @param pageNumber 分页参数
     * @param pageSize   分页容量
     * @return 教学课程分页结果
     */
    IPage<TeachCourseResult> teachCoursePage(Long examId, String courseName, int pageNumber, int pageSize);

    /**
     * 选择课程表并创建教学课程
     *
     * @param teachCourseSelectParam 教学课程选择参数
     */
    void selectCreate(TeachCourseSelectParam teachCourseSelectParam);

    /**
     * 根据用户id查询该用户所属机构下可以创建为教学课程的基础课程列表
     *
     * @param requestUser 请求的用户
     * @return 课程列表
     */
    List<DictionaryResult> findBasicCourseByUser(SysUser requestUser);

    /**
     * 根据教学课程id集合批量删除教学课程
     *
     * @param idList 教学课程idList
     */
    void deleteTeachCourseBatch(List<Long> idList);

    /**
     * 根据考试id，课程编号，教师id查询教学课程
     *
     * @param examId   考试id
     * @param courseId 课程编号
     * @param userId   教师id
     * @return 教学课程
     */
    TeachCourse findByExamIdCourseIdAndUserId(Long examId, Long courseId, Long userId);

    /**
     * 查询教师绑定课程关系
     *
     * @param userId 教师id
     * @return 课程集合
     */
    List<DictionaryResult> findTeacherCourseInfo(Long userId);

    CourseResult listUserCourse(String param, Long examId, Long teachingRoomId);

    List<BasicCourse> listByUserId(Long examId, Long userId);

    void batchCreateTeachCourse(Long schoolId, Long examId, List<BasicExamStudent> basicExamStudentList);
}