package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamStudentCourseDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.pdf.ExamStudentInfo;
import com.qmth.distributed.print.business.bean.params.ExamTaskSubmitContent;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.mapper.ExamStudentMapper;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.business.service.ExamStudentService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamStudentServiceImpl extends ServiceImpl<ExamStudentMapper, ExamStudent> implements ExamStudentService {

    @Resource
    private ExamDetailCourseMapper examDetailCourseMapper;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private ExamDetailCourseService examDetailCourseService;

    @Override
    public List<String> listPaperTypeByExamDetailCourseId(String examDetailCourseId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCourseId);
        List<ExamStudent> examStudents = this.list(queryWrapper);
        if (!examStudents.isEmpty()) {
            return examStudents.stream().filter(m -> StringUtils.isNotBlank(m.getPaperType())).map(m -> m.getPaperType()).distinct().collect(Collectors.toList());
        } else {
            ExamDetailCourse examDetailCourse = examDetailCourseMapper.selectById(examDetailCourseId);
            if (examDetailCourse != null && StringUtils.isNotBlank(examDetailCourse.getPaperType())) {
                return Arrays.stream(examDetailCourse.getPaperType().split(",")).collect(Collectors.toList());
            }
        }
        return null;
    }

    @Override
    public Map<String, Object> getStudentDetail(Long id) {
        Map<String, Object> map = this.baseMapper.getStudentDetail(id);
        Object extendFields = map.get("extendFields");
        if (Objects.nonNull(extendFields)) {
            List<Map> list = JSONObject.parseArray(extendFields.toString(), Map.class);
            map.put("extendFields", list);
        }
        return map;
    }

    @Override
    public void removeByExamDetailCourseId(Long id) {
        UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, id);
        this.remove(updateWrapper);
    }

    @Override
    public List<ExamStudent> listByExamDetailCourseIdNotIntMarkStudent(Long examDetailCourseId) {
        return this.baseMapper.listByExamDetailCourseIdNotIntMarkStudent(examDetailCourseId);
    }

    @Override
    public List<ExamStudent> listByExamDetailCourseId(Long examDetailCourseId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCourseId)
                .orderByAsc(ExamStudent::getId);
        return this.list(queryWrapper);
    }

    @Override
    public List<ExamStudent> saveStudentForExamApply(ExamTask examTask, String extendFields, List<Long> examTaskStudentObjectParamList, Long examDetailCourseId, SysUser sysUser) {
        List<ExamStudent> examStudentList = new ArrayList<>();
        for (Long studentId : examTaskStudentObjectParamList) {
            BasicExamStudent basicExamStudent = basicExamStudentService.getById(studentId);
            if (basicExamStudent == null) {
                continue;
            }
            ExamStudent examStudent = new ExamStudent();
            examStudent.setId(SystemConstant.getDbUuid());
            examStudent.setSchoolId(sysUser.getSchoolId());
            examStudent.setExamDetailCourseId(examDetailCourseId);
            examStudent.setBasicStudentId(studentId);
            examStudent.setStudentCode(basicExamStudent.getStudentCode());
            examStudent.setExamId(examTask.getExamId());
            examStudent.setPaperNumber(examTask.getPaperNumber());
            examStudent.setCoursePaperId(String.valueOf(examTask.getId()));
            examStudent.setExtendFields(extendFields);
            examStudentList.add(examStudent);
        }
        return examStudentList;
    }

    @Override
    public List<ExamStudentInfo> listStudentByExamDetailCourseId(Long examDetailCourseId) {
        return this.baseMapper.listByExamDetailCourseId(examDetailCourseId);
    }

    @Override
    public void updateAttachmentIdById(ExamStudentInfo t) {
        UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamStudent::getAttachmentId, t.getAttachmentId()).eq(ExamStudent::getId, t.getExamStudentId());
        this.update(updateWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndStudentCode(Long examId, String paperNumber, String studentCode) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamId, examId)
                .eq(ExamStudent::getPaperNumber, paperNumber)
                .eq(ExamStudent::getStudentCode, studentCode);
        return this.count(queryWrapper);
    }

    @Transactional
    @Override
    public void deleteByBasicStudentId(List<Long> basicExamStudentIds) {
        List<ExamStudent> examStudentList = this.listByBasicStudentId(basicExamStudentIds);
        if (CollectionUtils.isNotEmpty(examStudentList)) {
            List<Long> examStudentIds = examStudentList.stream().map(ExamStudent::getId).collect(Collectors.toList());
            this.removeByIds(examStudentIds);
            List<Long> examDetailCourseIds = examStudentList.stream().map(ExamStudent::getExamDetailCourseId).distinct().collect(Collectors.toList());
            List<ExamDetailStatusEnum> examDetailStatusEnumList = Arrays.asList(ExamDetailStatusEnum.READY, ExamDetailStatusEnum.WAITING, ExamDetailStatusEnum.PRINTING, ExamDetailStatusEnum.FINISH);
            // 更新考场科次
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByIds(examDetailCourseIds);
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                ExamDetail examDetail = examDetailService.getById(examDetailCourse.getExamDetailId());
                // 已生成了pdf，不更新科次
                if (examDetailStatusEnumList.contains(examDetail.getStatus())) {
                    continue;
                }

                // 更新考场科目科次
                examDetailCourseService.updateTotalSubjects(examDetailCourse.getId(), this.countByExamDetailCourseId(examDetailCourse.getId()));

                // 更新考场科次
                List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByExamDetailId(examDetailCourse.getExamDetailId());
                if (CollectionUtils.isNotEmpty(examDetailCourses)) {
                    examDetailService.updateTotalSubjects(examDetailCourse.getExamDetailId(), examDetailCourses.stream().filter(m -> m.getTotalSubjects() != null).mapToInt(m -> m.getTotalSubjects()).sum());
                }
            }
        }
    }

    @Override
    public List<ExamStudent> listByBasicStudentId(List<Long> basicExamStudentIds) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(ExamStudent::getBasicStudentId, basicExamStudentIds);
        return this.list(queryWrapper);
    }

    @Override
    public int countByExamDetailCourseId(Long examDetailCourseId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamDetailCourseId, examDetailCourseId);
        return this.count(queryWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamId, examId)
                .eq(ExamStudent::getPaperNumber, paperNumber);
        return this.count(queryWrapper);
    }
}
