package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.Update;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailPdfDownloadDto;
import com.qmth.distributed.print.business.bean.params.RelatePaperParam;
import com.qmth.distributed.print.business.cache.CreatePdfCacheUtil;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.ExamTaskUtil;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.CreatePdfTypeEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowApprovePassEnum;
import com.qmth.teachcloud.common.enums.TFCustomTypeEnum;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.service.MarkService;
import com.qmth.teachcloud.mark.service.ScanPaperService;
import org.activiti.engine.TaskService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.jce.exception.ExtCertPathBuilderException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务明细表（入库申请） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskDetailServiceImpl extends ServiceImpl<ExamTaskDetailMapper, ExamTaskDetail> implements ExamTaskDetailService {

    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamCardService examCardService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private ExamDetailCourseService examDetailCourseService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private TBTaskPdfService tbTaskPdfService;
    @Resource
    private TBSyncTaskService tbSyncTaskService;
    @Resource
    private ActivitiService activitiService;
    @Resource
    private TaskService taskService;
    @Resource
    private TFFlowApproveService tfFlowApproveService;
    @Resource
    private TFFlowJoinService tfFlowJoinService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private ExamTaskDetailMapper examTaskDetailMapper;
    @Resource
    private ScanPaperService scanPaperService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private MarkService markService;
    @Resource
    private ExamTaskAssignPaperTypeService examTaskAssignPaperTypeService;

    @Transactional
    @Override
    public boolean enable(ExamTask examTask) {
        ExamTask examTaskById = examTaskService.getById(examTask.getId());
        List<ExamDetail> examDetails = examDetailService.listExamDetailByExamIdAndCourseIdAndPaperNumber(examTaskById.getExamId(), examTaskById.getCourseId(), examTaskById.getPaperNumber());
        if (examDetails != null && examDetails.size() > 0) {
            // 有考场提交打印，不能禁用启用
            long count = examDetails.stream().filter(m -> !ExamDetailStatusEnum.NEW.name().equals(m.getStatus().name()) && !ExamDetailStatusEnum.READY.name().equals(m.getStatus().name())).count();
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("有考场已提交打印，不能启用/禁用");
            }

            Map<Long, List<Long>> map = examDetails.stream().collect(Collectors.groupingBy(ExamDetail::getPrintPlanId, Collectors.mapping(ExamDetail::getId, Collectors.toList())));
            for (Map.Entry<Long, List<Long>> listEntry : map.entrySet()) {
                int countRunning = tbTaskPdfService.countRunningByExamDetailIds(listEntry.getValue());
                if (countRunning > 0) {
                    throw ExceptionResultEnum.ERROR.exception("有关联该任务的考场正在生成打印pdf，不能启用/禁用卷库");
                }
            }
        }
        // 禁用，解决试卷关联关系，并且解除生成pdf绑定
        if (examTaskById.getExamTaskDetailEnable()) {
            // 正在生成pdf不能禁用
            if (examDetails != null && examDetails.size() > 0) {
                Map<Long, List<Long>> map = examDetails.stream().collect(Collectors.groupingBy(ExamDetail::getPrintPlanId, Collectors.mapping(ExamDetail::getId, Collectors.toList())));
                for (Map.Entry<Long, List<Long>> listEntry : map.entrySet()) {
                    // 解除所有考场该任务绑定关系
                    // 更新exam_detail
                    List<Long> examDetailIds = listEntry.getValue();
                    UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
                    examDetailUpdateWrapper.lambda()
                            .set(ExamDetail::getStatus, ExamDetailStatusEnum.NEW)
                            .set(ExamDetail::getAttachmentId, null)
                            .set(ExamDetail::getAttachmentPath, null)
                            .set(ExamDetail::getPaperPages, null)
                            .set(ExamDetail::getCardPages, null)
                            .set(ExamDetail::getPagesA4, null)
                            .in(ExamDetail::getId, examDetailIds);
                    examDetailService.update(examDetailUpdateWrapper);
                    // 更新绑定关系
                    List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByPrintPlanIdAndCourseIdAndPaperNumber(examTaskById.getSchoolId(), listEntry.getKey(), examTaskById.getCourseId(), examTaskById.getPaperNumber());
                    List<Long> examDetailCourseIds = examDetailCourses.stream().map(m -> m.getId()).collect(Collectors.toList());
                    UpdateWrapper<ExamDetailCourse> examDetailCourseUpdateWrapper = new UpdateWrapper<>();
                    examDetailCourseUpdateWrapper.lambda()
                            .set(ExamDetailCourse::getPaperType, null)
                            .set(ExamDetailCourse::getPaperPagesA3, null)
                            .set(ExamDetailCourse::getCardPagesA3, null)
                            .in(ExamDetailCourse::getId, examDetailCourseIds);
                    examDetailCourseService.update(examDetailCourseUpdateWrapper);

                    // 全部任务都取消，更新计划状态
                    //所有考场都撤回，印刷任务状态改为就绪
                    examPrintPlanService.updateStatusById(listEntry.getKey(), PrintPlanStatusEnum.READY);
                }
            }
        }
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getExamTaskDetailEnable, !examTaskById.getExamTaskDetailEnable()).eq(ExamTask::getId, examTaskById.getId());
        examTaskService.update(updateWrapper);

        return true;
    }

    @Transactional
    @Override
    public boolean updatePaper(RelatePaperParam paperParam) {
        ExamTask examTask = examTaskService.getByIdIncludeCourse(paperParam.getExamTaskId());
        // 提交印刷的考场，不允许修改关联试卷
        List<ExamDetail> examDetails = examDetailService.listByPrintPlanIdAndCourseCodeAndPaperNumber(examTask.getSchoolId(), paperParam.getPrintPlanId(), examTask.getCourseId(), examTask.getPaperNumber());
        List<ExamDetail> examDetailsPrint = examDetails.stream().filter(m -> !ExamDetailStatusEnum.NEW.equals(m.getStatus()) && !ExamDetailStatusEnum.READY.equals(m.getStatus())).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(examDetailsPrint)) {
            throw ExceptionResultEnum.ERROR.exception("部分考场pdf文件已提交印刷或已印刷，不能修改卷型");
        }

        List<Long> examDetailIds = examDetails.stream().map(ExamDetail::getId).collect(Collectors.toList());
        int countRunning = tbTaskPdfService.countRunningByExamDetailIds(examDetailIds);
        if (countRunning > 0) {
            throw ExceptionResultEnum.ERROR.exception("部分考场pdf文件正在生成中，不能修改卷型");
        }

        for (ExamDetail examDetail : examDetails) {
            List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByExamDetailIdAndStatus(examDetail.getId());
            // 4.没有未完成的命题任务
            if (CollectionUtils.isNotEmpty(examDetailCourses)) {
                throw ExceptionResultEnum.ERROR.exception("部分考场包含的命题任务未完成，不能修改卷型");
            }
        }

        // 更新命题任务中关联卷型
        ExamTask newExamTask = examTaskService.getByExamIdAndCourseIdAndPaperNumber(examTask.getExamId(), examTask.getCourseId(), paperParam.getPaperNumber());
        List<ExamTaskDetail> examDetailCourseList = this.listByExamIdAndPaperNumber(examTask.getExamId(), paperParam.getPaperNumber());

        ExamTaskDetail examTaskDetailOld = examDetailCourseList.stream().filter(m -> m.getExposedExamId() != null && m.getExposedExamId().equals(paperParam.getExamId())).findFirst().orElse(null);
        if(examTaskDetailOld == null ){
            ExamTaskDetail examTaskDetail = examDetailCourseList.stream().filter(m -> m.getSerialNumber().toString().equals(paperParam.getRelatePaperType())).findFirst().orElse(null);
            if(examTaskDetail == null){
                throw ExceptionResultEnum.ERROR.exception("备用卷不存在");
            } else if(examTaskDetail.getExposedExamId() != null){
                throw ExceptionResultEnum.ERROR.exception("备用卷已在其它考试中使用");
            } else {
                this.updateExposedExamIdById(paperParam.getExamId(), examTaskDetail.getId());
            }
        } else if(examTaskDetailOld.getSerialNumber().equals(paperParam.getRelatePaperType())) {
            return true;
        } else {
            this.updateExposedExamIdById(null, examTaskDetailOld.getId());
            ExamTaskDetail examTaskDetail = examDetailCourseList.stream().filter(m -> m.getSerialNumber().toString().equals(paperParam.getRelatePaperType())).findFirst().orElse(null);
            if(examTaskDetail == null){
                throw ExceptionResultEnum.ERROR.exception("备用卷不存在");
            } else if(examTaskDetail.getExposedExamId() != null){
                throw ExceptionResultEnum.ERROR.exception("备用卷已在其它考试中使用");
            } else {
                this.updateExposedExamIdById(paperParam.getExamId(), examTaskDetail.getId());
            }
        }

        // 更新选中的试卷编号、卷型
        String examDetailCourseIdStr = paperParam.getExamDetailCourseIds();
        List<Long> examDetailCourseIds = Arrays.asList(examDetailCourseIdStr.split(",")).stream().map(m -> Long.valueOf(m)).collect(Collectors.toList());
        examDetailCourseService.updatePaperNumber(examDetailCourseIds, paperParam.getPaperNumber(), paperParam.getRelatePaperType());

        // 初始化PDF生成任务数据
        for (ExamDetail examDetail : examDetails) {
            examDetailService.resetExamDetail(examDetail.getId(), CreatePdfTypeEnum.ALL, ExamDetailStatusEnum.NEW, false);
            tbTaskPdfService.resetRunningCreatePdf(examDetail.getId(), CreatePdfTypeEnum.ALL);
        }
        return true;
    }

    @Override
    public List<ExamTaskDetailDto> list(Long semesterId, Long examId, String relateType, List<Long> printPlanIdList, Long courseId, String paperNumber, String userName) {
        userName = SystemConstant.translateSpecificSign(userName);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        List<ExamTaskDetailDto> examDetailDtoIPage = this.baseMapper.listPage(schoolId, semesterId, examId, relateType, printPlanIdList, courseId, paperNumber, userName, dpr);
        examDetailDtoIPage.stream().map(m -> {
            List<String> examDetailCourseIds = this.baseMapper.listExamDetailCourse(schoolId, m.getRelatePaperType(), Long.valueOf(m.getPrintPlanId()), m.getCourseId(), m.getPaperNumber());
            m.setExamDetailCourseIds(String.join(",", examDetailCourseIds));
            return m;
        }).collect(Collectors.toList());
        return examDetailDtoIPage;
    }

    @Override
    public Map listTotal(Long semesterId, Long examId, String relateType, List<Long> printPlanIdList, Long courseId, String paperNumber, String userName) {
        userName = SystemConstant.translateSpecificSign(userName);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        List<ExamTaskDetailDto> examDetailDtoList = this.baseMapper.listPage(schoolId, semesterId, examId, relateType, printPlanIdList, courseId, paperNumber, userName, dpr);
        Map<String, Object> map = new HashMap<>();
        // 课程数量
        long courseCodeCount = examDetailDtoList.stream().map(m -> m.getCourseCode()).distinct().count();
        map.put("courseTotal", courseCodeCount);
        // 命题任务数量
        long paperNumberCount = examDetailDtoList.stream().map(m -> m.getPaperNumber()).distinct().count();
        map.put("taskTotal", paperNumberCount);
        // 科次数量
        int subjectTotalCount = examDetailDtoList.stream().mapToInt(m -> m.getTotalSubjects()).sum();
        map.put("subjectTotal", subjectTotalCount);
        return map;
    }

    @Override
    public void resetCardId(Long id) {
        QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
        examTaskDetailQueryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, id);
        List<ExamTaskDetail> examTaskDetails = this.list(examTaskDetailQueryWrapper);
        List<Long> cardIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(examTaskDetails)) {
            for (ExamTaskDetail e : examTaskDetails) {
                if (Objects.nonNull(e.getPaperAttachmentIds())) {
                    List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(e.getPaperAttachmentIds());
                    for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                        cardIds.add(Long.valueOf(paperInfoVo.getCardId()));
                        paperInfoVo.setCardId(null);
                    }
                    e.setPaperAttachmentIds(JSON.toJSONString(paperInfoVoList));
                }
            }
            this.updateBatchById(examTaskDetails);

            // 更新题卡使用状态
            examCardService.updateUsedById(cardIds);
        }
    }

    /**
     * 重启流程
     *
     * @param examTaskDetail
     * @return
     */
    @Override
    public Map<String, Object> taskRestart(ExamTaskDetail examTaskDetail) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        SysOrg sysOrg = commonCacheService.orgCache(requestUser.getOrgId());

        Map<String, Object> mapVar = new HashMap<>();
        ExamTask examTask = examTaskService.getById(examTaskDetail.getExamTaskId());
        List<ExamDetail> examDetailList = examDetailService.listExamDetailByExamIdAndCourseIdAndPaperNumber(examTask.getExamId(), examTask.getCourseId(), examTask.getPaperNumber());
        if (CollectionUtils.isNotEmpty(examDetailList)) {
            List<Long> examDetailIds = examDetailList.stream().map(ExamDetail::getId).collect(Collectors.toList());
            QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(ExamDetailCourse::getExamDetailId, examDetailIds);
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.list(queryWrapper);
            Set<String> paperTypeSet = new HashSet<>();
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                // 考场内任一课程有扫描数据，不允许打回
                boolean isScanPaper = scanPaperService.isScanByExamIdAndPaperNumber(examTask.getExamId(), examDetailCourse.getPaperNumber());
                if (isScanPaper) {
                    throw ExceptionResultEnum.ERROR.exception("课程已开始扫描，不允许打回");
                }
                if (StringUtils.isNotBlank(examDetailCourse.getPaperType())) {
                    paperTypeSet.addAll(Arrays.asList(examDetailCourse.getPaperType().split(",")));
                }
            }

            // 已打印任务，必须先作废，未打印任务，必须先撤回。才允许打回
            long count = examDetailList.stream().filter(m -> ExamDetailStatusEnum.FINISH.equals(m.getStatus()) || ExamDetailStatusEnum.WAITING.equals(m.getStatus())).count();
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("印刷任务有已打印或待印刷数据，不允许打回");
            }

            // 修改已曝光
            UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTaskDetail::getExposed, false)
                    .set(ExamTaskDetail::getExposedExamId, null)
                    .eq(ExamTaskDetail::getExamTaskId, examTask.getId())
                    .eq(ExamTaskDetail::getExposedExamId, examTask.getExamId());
            this.update(updateWrapper);

            StringJoiner stringJoiner = new StringJoiner("_");
            stringJoiner.add(String.valueOf(examTask.getExamId())).add(examTask.getPaperNumber()).add(String.valueOf(examDetailList.get(0).getExamStartTime())).add(String.valueOf(examDetailList.get(0).getExamEndTime()));
            String key = stringJoiner.toString();
            CreatePdfCacheUtil.deletePaperType(key);
            for (ExamDetail examDetail : examDetailList) {
                examDetailService.resetExamDetail(examDetail.getId(), CreatePdfTypeEnum.ALL, ExamDetailStatusEnum.REJECT, true);
            }
        }

        // 不需要审核，直接更新，否则加入待审核列表(状态为不需要审核的或者为管理员身份的都不重新走审核流程)
        if ((Objects.nonNull(examTask.getReview()) && examTask.getReview())) {
            // 发起新流程
            Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(),
                    examTask,
                    TFCustomTypeEnum.ELECTRON_FLOW,
                    examTask.getUserId(),
                    false);
            map.computeIfAbsent(SystemConstant.OLD_FLOW_ID, v -> examTask.getFlowId());
            map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.RESTART);
            String remark = Objects.nonNull(sysOrg) ? sysOrg.getName() : "无机构";
            map.computeIfAbsent(SystemConstant.APPROVE_REMARK, v -> requestUser.getRealName() + "(" + remark + ")" + FlowApprovePassEnum.RESTART.getTitle());
            map = activitiService.customFlowStart(map);
            Long flowId = SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID));
            UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTask::getFlowId, flowId).eq(ExamTask::getId, examTaskDetail.getExamTaskId());
            examTaskService.update(updateWrapper);
            mapVar.putAll(map);
            mapVar.put(SystemConstant.SEND_FLOW_START_MQ, true);

            tfFlowJoinService.saveOrUpdate(map);
//            if (Objects.nonNull(flowId)) {
//                // 审核一级
//                Task task = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).singleResult();
//                if (Objects.nonNull(task)) {
//                    Map<String, Object> approverMap = new HashMap<>();
//                    approverMap.computeIfAbsent(SystemConstant.FLOW_TASK_ID, v -> task.getId());
//                    approverMap.computeIfAbsent(SystemConstant.PAPAER_ATTACHMENT_IDS, v -> examTaskDetail.getPaperAttachmentIds());
//                    approverMap.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.PASS);
////                    map.computeIfAbsent(SystemConstant.APPROVE_USER_IDS, v -> Arrays.asList(examTask.getUserId()));
//                    String remark = Objects.nonNull(sysOrg) ? sysOrg.getName() : "无机构";
//                    approverMap.computeIfAbsent(SystemConstant.APPROVE_REMARK, v -> requestUser.getRealName() + "(" + remark + ")" + FlowApprovePassEnum.RESTART.getTitle());
//                    activitiService.taskApprove(approverMap);
//
//                    try {
//                        Thread.sleep(1000);
//                    } catch (InterruptedException e) {
//                        e.printStackTrace();
//                    }
//                    Task taskReject = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).singleResult();
//                    Map<String, Object> approverRejectMap = new HashMap<>();
//                    approverRejectMap.computeIfAbsent(SystemConstant.FLOW_TASK_ID, v -> taskReject.getId());
//                    approverRejectMap.computeIfAbsent(SystemConstant.PAPAER_ATTACHMENT_IDS, v -> examTaskDetail.getPaperAttachmentIds());
//                    approverRejectMap.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.REJECT);
//                    approverRejectMap.computeIfAbsent(SystemConstant.APPROVE_SETUP, v -> FlowApproveSetupEnum.SUBMIT.getSetup());
//                    activitiService.taskApprove(approverRejectMap);
//                    mapVar.putAll(approverMap);
//                    mapVar.remove(SystemConstant.SEND_FLOW_START_MQ);
//                    mapVar.put(SystemConstant.SEND_FLOW_MQ, true);
//                }
//            }
        } else {
            // 不需要审核
            UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTask::getStatus, ExamStatusEnum.STAGE).eq(ExamTask::getId, examTaskDetail.getExamTaskId());
            examTaskService.update(updateWrapper);
        }

        // 删除阅卷数据
        markService.deleteInitMarkData(examTask.getExamId(), examTask.getPaperNumber());

        mapVar.put(SystemConstant.SUCCESS, true);
        return mapVar;
    }

    /**
     * 根据examTaskId下载pdf
     *
     * @param examTaskId
     * @return
     */
    @Override
    public ExamTaskDetailPdfDownloadDto findPdfDownload(Long examTaskId) {
        return examTaskDetailMapper.findPdfDownload(examTaskId);
    }

    @Override
    public List<String> listRelateCommonTypes(Long semesterId, Long examId, String relateType, List<Long> idList, Long courseId, String paperNumber, String userName) {
        userName = SystemConstant.translateSpecificSign(userName);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        List<ExamTaskDetailDto> examDetailDtoList = this.baseMapper.listPage(schoolId, semesterId, examId, relateType, idList, courseId, paperNumber, userName, dpr);
        List<String> commonPaperTypes = new ArrayList<>();
        if (!examDetailDtoList.isEmpty()) {
            // 所有未曝光类型不为空的数据
            List<String> unposedPaperList = examDetailDtoList.stream().filter(m -> StringUtils.isNotBlank(m.getUnexposedPaperType())).map(m -> m.getUnexposedPaperType()).collect(Collectors.toList());
            if (examDetailDtoList.size() == unposedPaperList.size()) {
                // 取所有命题任务未曝光类型的交集
                List<String> basePaperTypes = Arrays.asList(unposedPaperList.get(0).split(","));
                for (String basePaperType : basePaperTypes) {
                    for (String s : unposedPaperList) {
                        List<String> particPaperTypes = Arrays.asList(s.split(","));
                        if (!particPaperTypes.contains(basePaperType)) {
                            break;
                        }
                    }
                    commonPaperTypes.add(basePaperType);
                }
            }
        }
        return commonPaperTypes;
    }

    @Transactional
    @Override
    public boolean cancel(Long id, String cancelRemark) {
        ExamTask examTask = examTaskService.getById(id);
        String paperNumber = examTask.getPaperNumber();

        List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamIdAndPaperNumber(examTask.getExamId(), examTask.getPaperNumber());
        if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                // 考场内任一课程有扫描数据，不允许打回
                if (scanPaperService.isScanByExamIdAndPaperNumber(examTask.getExamId(), examDetailCourse.getPaperNumber())) {
                    throw ExceptionResultEnum.ERROR.exception("课程已开始扫描，不允许作废");
                }
            }

            // 已打印任务，必须先作废，未打印任务，必须先撤回。才允许打回
            List<Long> examDetailList = examDetailCourseList.stream().map(ExamDetailCourse::getExamDetailId).collect(Collectors.toList());
            long count = examDetailService.listByIds(examDetailList).stream().filter(m -> (ExamDetailStatusEnum.FINISH.equals(m.getStatus()) && m.getNormal()) || ExamDetailStatusEnum.WAITING.equals(m.getStatus())).count();
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("印刷任务有已打印且未作废或者未撤回数据，不允许作废");
            }

//            StringJoiner stringJoiner = new StringJoiner("_");
//            stringJoiner.add(String.valueOf(examTask.getExamId())).add(examTask.getPaperNumber()).add(String.valueOf(examDetailList.get(0).getExamStartTime())).add(String.valueOf(examDetailList.get(0).getExamEndTime()));
//            String key = stringJoiner.toString();
//            CreatePdfCacheUtil.deletePaperType(key);
            for (Long examDetailId : examDetailList) {
                examDetailService.resetExamDetail(examDetailId, CreatePdfTypeEnum.ALL, ExamDetailStatusEnum.REJECT, true);
            }
        }

        String paperNumberCancel = paperNumber + "-作废";
        examTask.setPaperNumber(paperNumberCancel);
        examTask.setStatus(ExamStatusEnum.CANCEL);
        examTask.setCancelRemark(cancelRemark);
        examTaskService.updateById(examTask);

        // 考场状态改为作废
        if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
                examDetailUpdateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.CANCEL)
                        .set(ExamDetail::getNormal, false)
                        .eq(ExamDetail::getId, examDetailCourse.getExamDetailId());
                examDetailService.update(examDetailUpdateWrapper);

                UpdateWrapper<ExamDetailCourse> examDetailCourseUpdateWrapper = new UpdateWrapper<>();
                examDetailCourseUpdateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumberCancel)
                        .eq(ExamDetailCourse::getId, examDetailCourse.getId());
                examDetailCourseService.update(examDetailCourseUpdateWrapper);
            }
        }

        // 任务数据改为作废
        UpdateWrapper<TBTaskPdf> tbTaskUpdateWrapper = new UpdateWrapper<>();
        tbTaskUpdateWrapper.lambda().set(TBTaskPdf::getPaperNumber, paperNumberCancel)
                .eq(TBTaskPdf::getSchoolId, examTask.getSchoolId())
                .eq(TBTaskPdf::getExamId, examTask.getExamId())
                .eq(TBTaskPdf::getPaperNumber, paperNumber);
        tbTaskPdfService.update(tbTaskUpdateWrapper);

        // 考生表数据改为作废
        UpdateWrapper<ExamStudent> examStudentUpdateWrapper = new UpdateWrapper<>();
        examStudentUpdateWrapper.lambda().set(ExamStudent::getPaperNumber, paperNumberCancel)
                .eq(ExamStudent::getSchoolId, examTask.getSchoolId())
                .eq(ExamStudent::getExamId, examTask.getExamId())
                .eq(ExamStudent::getPaperNumber, paperNumber);
        examStudentService.update(examStudentUpdateWrapper);

        // 同步表数据改为作废
        UpdateWrapper<TBSyncTask> tbSyncTaskUpdateWrapper = new UpdateWrapper<>();
        tbSyncTaskUpdateWrapper.lambda().set(TBSyncTask::getPaperNumber, paperNumberCancel)
                .eq(TBSyncTask::getSchoolId, examTask.getSchoolId())
                .eq(TBSyncTask::getExamId, examTask.getExamId())
                .eq(TBSyncTask::getPaperNumber, paperNumber);
        tbSyncTaskService.update(tbSyncTaskUpdateWrapper);

        // 删除阅卷数据
        markService.deleteInitMarkData(examTask.getExamId(), examTask.getPaperNumber());

        return true;
    }

    @Override
    public ExamTaskDetail getByExamIdAndPaperNumberAndSerialNumber(Long examId, String paperNumber, Integer serialNumber) {
        List<ExamTaskDetail> examTaskDetailList = this.listByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isEmpty(examTaskDetailList)) {
            return null;
        }
        return examTaskDetailList.stream().filter(m -> m.getSerialNumber().equals(serialNumber)).findFirst().orElse(null);
    }

    @Override
    public ExamTaskDetail getByExamTaskIdAndSerialNumber(Long examTaskId, Integer serialNumber) {
        List<ExamTaskDetail> examTaskDetailList = this.listByExamTaskId(examTaskId);
        if (CollectionUtils.isEmpty(examTaskDetailList)) {
            return null;
        }
        return examTaskDetailList.stream().filter(m -> m.getSerialNumber().equals(serialNumber)).findFirst().orElse(null);
    }

    @Override
    public void updateExposePaperType(Long examId, List<ExamDetailCourse> examDetailCourses) {
        // 更新曝光卷型、未曝光卷型
        for (ExamDetailCourse examDetailCourse : examDetailCourses) {
            ExamTaskDetail examTaskDetail = this.getByExamIdAndPaperNumberAndSerialNumber(examId, examDetailCourse.getPaperNumber(), examDetailCourse.getSerialNumber());
            // 更新曝光卷型
            UpdateWrapper<ExamTaskDetail> examTaskDetailUpdateWrapper = new UpdateWrapper<>();
            examTaskDetailUpdateWrapper.lambda()
                    .set(ExamTaskDetail::getExposed, true)
                    .set(ExamTaskDetail::getExposedExamId, examId)
                    .eq(ExamTaskDetail::getId, examTaskDetail.getId());
            this.update(examTaskDetailUpdateWrapper);

        }
    }

    @Override
    public void updateUnexposePaperType(Long examId, String paperNumber, Integer serialNumber) {
        if (examTaskService.getByExamIdAndPaperNumber(examId, paperNumber) == null) {
            return;
        }
        // 更新曝光卷型、未曝光卷型
        ExamTaskDetail examTaskDetail = this.getByExamIdAndPaperNumberAndSerialNumber(examId, paperNumber, serialNumber);
        UpdateWrapper<ExamTaskDetail> examTaskDetailUpdateWrapper = new UpdateWrapper<>();
        examTaskDetailUpdateWrapper.lambda()
                .set(ExamTaskDetail::getExposed, false)
                .set(ExamTaskDetail::getExposedExamId, null)
                .eq(ExamTaskDetail::getId, examTaskDetail.getId());
        this.update(examTaskDetailUpdateWrapper);
    }

    @Override
    public int countByCardId(Long cardId) {
        return examTaskDetailMapper.countByCardId(cardId);
    }

    @Override
    public List<ExamTaskDetail> listByExamIdAndPaperNumber(Long examId, String paperNumber) {
        ExamTask examTask = examTaskService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (examTask == null) {
            return Collections.emptyList();
        }
        return this.listByExamTaskId(examTask.getId());
    }

    @Override
    public List<ExamTaskDetail> listByExamTaskId(Long examTaskId) {
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId)
                .orderByAsc(ExamTaskDetail::getSerialNumber);
        return this.list(queryWrapper);
    }

    @Override
    public void updateExposedExamIdById(Long examId, Long id) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getExposedExamId, examId)
                .eq(ExamTaskDetail::getId, id);
        this.update(updateWrapper);
    }

    @Override
    public ExamTaskDetail findByExamIdAndPaperNumberAndSerialNumber(Long examId, String paperNumber, Integer serialNumber) {
        List<ExamTaskDetail> examTaskDetailList = this.listByExamIdAndPaperNumber(examId, paperNumber);
        return examTaskDetailList.stream().filter(m -> m.getSerialNumber() != null && m.getSerialNumber().equals(serialNumber)).findFirst().orElse(null);
    }

    @Override
    public void deleteByExamTaskId(Long examTaskId) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId);
        this.remove(updateWrapper);
    }
}
