package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.dto.approvalForm.*;
import com.qmth.distributed.print.business.bean.dto.examObject.ExamObjectDto;
import com.qmth.distributed.print.business.bean.params.*;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentQuery;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.bean.result.WorkResult;
import com.qmth.distributed.print.business.bean.result.examTaskStudent.ExamTaskStudentObjectResult;
import com.qmth.distributed.print.business.bean.result.statistics.PrintCountResult;
import com.qmth.distributed.print.business.bean.result.statistics.ScanCalculateResult;
import com.qmth.distributed.print.business.bean.result.statistics.ScanCountResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.mapper.ExamTaskMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncPaperReviewPdfExportService;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.distributed.print.business.util.ExamTaskUtil;
import com.qmth.distributed.print.business.util.HtmlToJpgUtil;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.teachcloud.common.bean.dto.AssignTeacherDto;
import com.qmth.teachcloud.common.bean.dto.BlurryUserDto;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.marking.ConvertJpgStorage;
import com.qmth.teachcloud.common.bean.result.SysUserResult;
import com.qmth.teachcloud.common.bean.result.TbTaskDetailResult;
import com.qmth.teachcloud.common.bean.tiku.TikuPaperInfo;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.kit.TikuUtils;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.common.util.excel.ExcelError;
import org.activiti.engine.ActivitiObjectNotFoundException;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 命题任务 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskServiceImpl extends ServiceImpl<ExamTaskMapper, ExamTask> implements ExamTaskService {

    private static final Logger logger = LoggerFactory.getLogger(ExamTaskServiceImpl.class);

    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private ExamTaskTempService examTaskTempService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private BasicAttachmentService basicAttachmentService;
    @Resource
    @Lazy
    private PrintCommonService printCommonService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysUserRoleService sysUserRoleService;
    @Resource
    private ExamCardService examCardService;
    @Resource
    private ExamDetailCourseService examDetailCourseService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private HtmlToJpgUtil htmlToJpgUtil;
    @Resource
    TBTaskService tbTaskService;
    @Resource
    private TBTaskPdfService tbTaskPdfService;
    @Resource
    AsyncPaperReviewPdfExportService asyncPaperReviewPdfExportService;
    @Resource
    TeachcloudCommonService teachcloudCommonService;
    @Resource
    ActivitiService activitiService;
    @Resource
    TFFlowApproveService tfFlowApproveService;
    @Resource
    TaskService taskService;
    @Resource
    ExamPrintPlanService examPrintPlanService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    SysOrgService sysOrgService;
    @Resource
    FileStoreUtil fileStoreUtil;
    @Resource
    ExamTaskService examTaskService;
    @Resource
    ExamTaskPaperDataService examTaskPaperDataService;
    @Resource
    ExamStudentService examStudentService;
    @Resource
    TFFlowJoinService tfFlowJoinService;
    @Resource
    CreatePdfUtil createPdfUtil;
    @Resource
    BasicPrintConfigService basicPrintConfigService;
    @Resource
    TFFlowLogService tfFlowLogService;
    @Resource
    CommonCacheService commonCacheService;
    @Resource
    TFCustomFlowService tfCustomFlowService;
    @Resource
    BasicExamService basicExamService;
    @Resource
    BasicSemesterService basicSemesterService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private BasicOperationLogService basicOperationLogService;
    @Resource
    private RedisCounterUtil redisCounterUtil;
    @Resource
    private FileUploadService fileUploadService;
    @Resource
    private TikuUtils tikuUtils;

    @Override
    public IPage<ExamTaskDto> list(Long semesterId, Long examId, Boolean enable, String status, Long openCollegeId, Long courseId, String paperNumber, Long startTime, Long endTime, String userName, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        userName = SystemConstant.translateSpecificSign(userName);
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(),
                ServletUtil.getRequest().getServletPath());
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listPage(page, schoolId, semesterId, examId, enable,
                status, openCollegeId, courseId, paperNumber, startTime, endTime, userName, dpr);
        for (ExamTaskDto record : examTaskDtoIPage.getRecords()) {
            // 判断审批表状态
            String cellPaperNumber = record.getPaperNumber();
            List<PrintPlanStatusEnum> printPlanStatusEnumList = new ArrayList<>();
            printPlanStatusEnumList.add(PrintPlanStatusEnum.PRINTING);
            printPlanStatusEnumList.add(PrintPlanStatusEnum.FINISH);
            List<ExamPackageDetail> examPackageDetailDatasource = this.baseMapper.findExamPackageDetailByPaperNumber(
                    schoolId, examId, record.getCourseId(), cellPaperNumber, printPlanStatusEnumList);
            record.setApproveFormStatus(examPackageDetailDatasource.size() > 0);
        }
        return examTaskDtoIPage;
    }

    @Override
    public List<BlurryUserDto> listUser(Long courseId, String param) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        return this.baseMapper.listUser(schoolId, courseId, SystemConstant.translateSpecificSign(param), orgIds);
    }

    @Transactional
    @Override
    public ExamTask assignUser(ExamTask examTask) {
        ExamStatusEnum examStatusEnum = ExamStatusEnum.DRAFT;
        Long flowId = null;
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        ExamTask task = this.getById(examTask.getId());
        // 指派命题老师
        if (task.getStatus().name().equals(ExamStatusEnum.DRAFT.name())) {
            if (!task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务已禁用，不能指派命题老师");
            }

            if (Objects.nonNull(examTask.getUserId())) {
                examStatusEnum = ExamStatusEnum.STAGE;
                Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(),
                        examTask,
                        TFCustomTypeEnum.ELECTRON_FLOW,
                        examTask.getUserId(),
                        true);
                map = activitiService.customFlowStart(map);
                flowId = SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID));
                map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.NEW);
                tfFlowJoinService.saveOrUpdate(map);
                examTask.setFlowId(flowId);
            }
        }
        // 更改命题老师
        else if (task.getStatus().name().equals(ExamStatusEnum.STAGE.name())) {
            if (task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务禁用后，才能更改命题老师");
            }

            if (examTask.getUserId() == null) {
                examStatusEnum = ExamStatusEnum.DRAFT;
            } else {
                examStatusEnum = ExamStatusEnum.STAGE;

                if (examTask.getUserId().longValue() != task.getUserId().longValue()) {
                    Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(),
                            examTask,
                            TFCustomTypeEnum.ELECTRON_FLOW,
                            examTask.getUserId(),
                            true);
                    map = activitiService.customFlowStart(map);
                    flowId = SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID));
                    map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.NEW);

                    tfFlowJoinService.deleteByObjectIdAndFlowId(examTask.getId(), null);
                    tfFlowJoinService.saveOrUpdate(map);
                    examTask.setFlowId(flowId);
                } else {
                    flowId = task.getFlowId();
                }
            }
        }

        updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId())
                .set(ExamTask::getStatus, examStatusEnum)
                .set(ExamTask::getFlowId, flowId)
                .eq(ExamTask::getId, examTask.getId());
        this.update(updateWrapper);
        return examTask;
    }

    @Transactional
    @Override
    public boolean enable(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }

        ExamTask task = this.getById(examTask.getId());
        if (Objects.nonNull(task.getFlowId())) {
            TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(task.getFlowId());
            if (tfFlowApprove.getStatus().equals(FlowStatusEnum.FINISH) ||
                    tfFlowApprove.getStatus().equals(FlowStatusEnum.END)) {
                throw ExceptionResultEnum.ERROR.exception("流程状态为已结束或已终止，不能启用/禁用操作");
            }
        }

        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getEnable, examTask.getEnable()).eq(ExamTask::getId, task.getId());
        // 启用时，后续状态清空
        if (!examTask.getEnable()) {//解除题卡绑定
            examTaskDetailService.resetCardId(examTask.getId());
        }
        return this.update(updateWrapper);
    }

    /**
     * 保存命题任务新
     *
     * @param examTask
     * @return
     */
    @Override
    @Transactional
    public ExamTask saveExamTaskNew(ExamTask examTask) {
        try {
            Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
            Long examId = examTask.getExamId();
            BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(examId);
            examTask.setSchoolId(schoolId);
            examTask.setCardRuleId(basicPrintConfig.getCardRuleId());
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (StringUtils.isNotBlank(examTask.getPaperNumber())) {
                QueryWrapper<ExamTask> taskQueryWrapper = new QueryWrapper<>();
                taskQueryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getExamId, examId).eq(ExamTask::getPaperNumber, examTask.getPaperNumber());
                ExamTask task = this.getOne(taskQueryWrapper);
                if (task != null) {
                    BasicExam basicExam = basicExamService.getById(examId);
                    throw ExceptionResultEnum.ERROR.exception("试卷编号[" + examTask.getPaperNumber() + "]在考试[" + basicExam.getName() + "]下已存在");
                }
            } else {
                examTask.setPaperNumber(printCommonService.autoCreatePaperNumber(examId));
            }

            examTask.insertInfo(sysUser.getId());
            examTask.updateInfo(sysUser.getId());

            examTask.setReview(basicPrintConfig.getReview());
            // 取课程所在机构
            BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
            if (basicCourse == null) {
                throw ExceptionResultEnum.ERROR.exception("课程不存在");
            }
            examTask.setTeachingRoomId(basicCourse.getTeachingRoomId());
            if (examTask.getUserId() == null) {
                examTask.setStatus(ExamStatusEnum.DRAFT);
            } else if (Objects.nonNull(examTask.getReview()) && examTask.getReview()) {
                examTask.setStatus(ExamStatusEnum.STAGE);
                Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(),
                        examTask,
                        TFCustomTypeEnum.ELECTRON_FLOW,
                        examTask.getUserId(),
                        true);
                map = activitiService.customFlowStart(map);
                examTask.setFlowId(SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID)));
                map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.NEW);
                tfFlowJoinService.saveOrUpdate(map);
            } else if (Objects.nonNull(examTask.getReview()) && !examTask.getReview()) {
                examTask.setStatus(ExamStatusEnum.STAGE);
            }
            examTask.setSource(ExamTaskSourceEnum.ASSIGN);
            this.save(examTask);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ActivitiObjectNotFoundException) {
                ResultUtil.error("流程不存在");
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return examTask;
    }

    @Override
    public ExamTaskImportResultDto importFile(MultipartFile file) throws IOException, NoSuchFieldException {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(ExamTaskImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            // 课程代码与试卷有相同数据map
            Map<String, String> courseCodePaperNumberMap = new HashMap<>();
            // 试卷编号重复set
            Set<String> paperNubmerSet = new HashSet<>();
            List<ExcelError> excelErrorTemp = new ArrayList<>();
            // 只允许导入一个sheet
            if (finalExcelList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("excel中只允许有一个sheet");
            }
            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> examTaskTempList = excelMap.get(i);
                for (int y = 0; y < examTaskTempList.size(); y++) {
                    ExamTaskImportDto userImportDto = (ExamTaskImportDto) examTaskTempList.get(y);

                    String excelCourseCode = userImportDto.getCourseCode();
                    String teachingRoomName = userImportDto.getTeachingRoomName();
                    String excelPaperNumber = userImportDto.getPaperNumber();
                    String excelTeacherAccount = userImportDto.getUserAccount();
                    String excelTeacherName = userImportDto.getUserName();

                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, teachingRoomName);
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行开课学院[" + teachingRoomName + "]在组织架构中不存在"));
                    } else {
                        // 课程是否存在并且与名称对应
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrgList.get(0).getId(), excelCourseCode);
                        // 校验课程名称和课程代码
                        if (basicCourse == null) {
                            excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行开课学院[" + teachingRoomName + "]下无课程代码[" + excelCourseCode + "]的课程"));
                        }
                        if (courseCodePaperNumberMap.containsKey(excelCourseCode)) {
                            String paperNumber = courseCodePaperNumberMap.get(excelCourseCode);
                            if (paperNumber != null && paperNumber.equals(excelPaperNumber)) {
                                excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行课程代码[" + excelCourseCode + "]与试卷编号[" + excelPaperNumber + "]有相同数据"));
                            }
                        } else {
                            courseCodePaperNumberMap.put(excelCourseCode, excelPaperNumber);
                        }

                        // 校验试卷编号重复
                        if (StringUtils.isNotBlank(excelPaperNumber)) {
                            if (paperNubmerSet.contains(excelPaperNumber)) {
                                excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "试卷编号[" + excelPaperNumber + "]有相同数据"));
                            } else {
                                paperNubmerSet.add(excelPaperNumber);
                            }
                        }

                        // 校验命题老师
                        AssignTeacherDto assignTeacherDto = sysUserService.listUser(basicCourse.getId(), null);
                        List<SysUserResult> listUser = Objects.nonNull(assignTeacherDto) ? assignTeacherDto.getAllUserList() : new ArrayList<>();
                        Map<String, String> userMap = listUser.stream().collect(Collectors.toMap(SysUserResult::getLoginName, SysUserResult::getRealName));
                        if (userMap == null) {
                            excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行课程代码[" + excelCourseCode + "]所属机构下未找到命题老师"));
                        } else {
                            List<SysUserResult> courseUserList = Objects.nonNull(assignTeacherDto) ? assignTeacherDto.getCourseUserList() : new ArrayList<>();
                            Map<String, String> courseUserMap = courseUserList.stream().collect(Collectors.toMap(SysUserResult::getLoginName, SysUserResult::getRealName));
                            if (!courseUserMap.isEmpty()) {
                                userMap.putAll(courseUserMap);
                            }
                            if (StringUtils.isNotBlank(excelTeacherAccount)) {
                                if (StringUtils.isBlank(excelTeacherName)) {
                                    excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行命题老师姓名不能为空"));
                                } else {
                                    if (StringUtils.isBlank(userMap.get(excelTeacherAccount))) {
                                        excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行命题老师工号[" + excelTeacherAccount + "]系统中不存在"));
                                    } else {
                                        String teacherName = userMap.get(excelTeacherAccount);
                                        if (!excelTeacherName.equals(teacherName)) {
                                            excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行命题老师姓名与系统不一致。文件中为[" + excelTeacherName + "]，用户管理中为[" + teacherName + "]"));
                                        }
                                    }
                                }
                            } else {
                                if (StringUtils.isNotBlank(excelTeacherName)) {
                                    excelErrorTemp.add(new ExcelError(y + 1, "第" + (y + 1) + "行命题老师工号必填"));
                                }
                            }
                        }
                    }
                }
            }
            if (excelErrorTemp.size() > 0) {
                List<String> errors = excelErrorTemp.stream().map(m -> m.getExcelErrorType()).collect(Collectors.toList());
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(errors));
            }
            return finalExcelList;
        }, 2);
        return executeImportExamTaskBatch(finalList);
    }

    @Transactional
    ExamTaskImportResultDto executeImportExamTaskBatch(List<LinkedMultiValueMap<Integer, Object>> finalList) {
        // 学校ID
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 用户
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 批次号
        Gson gson = new Gson();
        String batchNo = String.valueOf(System.nanoTime());
        ExamTaskImportResultDto examTaskImportResultDto = new ExamTaskImportResultDto();
        examTaskImportResultDto.setBatchNo(batchNo);
        List<ExamTaskDto> tasks = new ArrayList<>();

        // 错误信息
        StringJoiner stringJoiner = new StringJoiner(";");
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> examTaskTempList = excelMap.get(i);
            for (int y = 0; y < examTaskTempList.size(); y++) {
                ExamTaskImportDto examTaskImportDto = (ExamTaskImportDto) examTaskTempList.get(y);
                ExamTaskTemp examTaskTemp = gson.fromJson(gson.toJson(examTaskImportDto), ExamTaskTemp.class);
                // 校验该用户是否有科目
                List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, examTaskTemp.getTeachingRoomName());
                // 课程是否存在并且与名称对应
                BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrgList.get(0).getId(), examTaskTemp.getCourseCode());
                examTaskTemp.setId(SystemConstant.getDbUuid());
                if (basicCourse == null) {
                    throw ExceptionResultEnum.ERROR.exception("开课学院[" + examTaskTemp.getTeachingRoomName() + "]没有课程代码[" + examTaskTemp.getCourseCode() + "]的课程");
                }
                examTaskTemp.setCourseId(basicCourse.getId());
                examTaskTemp.setBatchNo(batchNo);
                examTaskTemp.setSchoolId(schoolId);
                examTaskTemp.setCreateId(sysUser.getId());
                examTaskTemp.setCreateTime(System.currentTimeMillis());
                //保存
                examTaskTempService.save(examTaskTemp);

                ExamTaskDto examTaskDto = new ExamTaskDto();
                examTaskDto.setCourseId(basicCourse.getId());
                examTaskDto.setCourseCode(examTaskTemp.getCourseCode());
                examTaskDto.setCourseName(examTaskTemp.getCourseName());
                examTaskDto.setPaperNumber(examTaskTemp.getPaperNumber());

                QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysUser::getSchoolId, schoolId).eq(SysUser::getLoginName, examTaskTemp.getUserAccount());
                SysUser sysUser1 = sysUserService.getOne(queryWrapper);
                AssignTeacherDto assignTeacherDto;
                if (sysUser1 != null) {
                    assignTeacherDto = sysUserService.listUser(basicCourse.getId(), String.valueOf(sysUser1.getId()));
                } else {
                    assignTeacherDto = sysUserService.listUser(basicCourse.getId(), null);
                }
                examTaskDto.setUsers(Objects.nonNull(assignTeacherDto) ? assignTeacherDto.getAllUserList() : new ArrayList<>());
                String userName = examTaskImportDto.getUserName();
                String userAccount = examTaskImportDto.getUserAccount();
                if (SystemConstant.strNotNull(userName) && SystemConstant.strNotNull(userAccount)) {
                    List<SysUserResult> match = assignTeacherDto.getAllUserList().stream()
                            .filter(e -> e.getLoginName().equals(userAccount) && e.getRealName().equals(userName))
                            .collect(Collectors.toList());
                    if (match.size() > 1) {
                        throw ExceptionResultEnum.ERROR.exception("数据异常");
                    }
                    if (match.size() == 1) {
                        SysUserResult tmp = match.get(0);
                        examTaskDto.setUserId(String.valueOf(tmp.getId()));
                        examTaskDto.setUserName(tmp.getRealName());
                    }

                    if (Objects.nonNull(assignTeacherDto) && !CollectionUtils.isEmpty(assignTeacherDto.getCourseUserList())) {
                        List<SysUserResult> matchCourseUser = assignTeacherDto.getCourseUserList().stream()
                                .filter(e -> e.getLoginName().equals(userAccount) && e.getRealName().equals(userName))
                                .collect(Collectors.toList());
                        if (matchCourseUser.size() > 1) {
                            throw ExceptionResultEnum.ERROR.exception("数据异常");
                        }
                        if (matchCourseUser.size() == 1) {
                            SysUserResult tmp = matchCourseUser.get(0);
                            examTaskDto.setUserId(String.valueOf(tmp.getId()));
                            examTaskDto.setUserName(tmp.getRealName());
                        }
                    }
                }
                tasks.add(examTaskDto);
            }
        }

        // 异常抛出 by 20230214
        String errorMsg = stringJoiner.toString();
        if (errorMsg.length() > 0) {
            throw ExceptionResultEnum.ERROR.exception(errorMsg);
        }

        // 设置重复信息，页面做二次校验
        examTaskImportResultDto.setErrorMsg(stringJoiner.toString());
        examTaskImportResultDto.setTasks(tasks);
        return examTaskImportResultDto;
    }

    @Override
    public ExamTaskImportResultDto importFile1(MultipartFile file) throws IOException {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
        List<ExamTaskTemp> examTaskImportDtoList;
        try {
            examTaskImportDtoList = excelReader.getObjectList(ExamTaskTemp.class);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("读取excel内容失败");
        }

        // 单次导入限制不超过150条数据，否则页面渲染会导致页面崩溃（2024-11-11 add）
        if (examTaskImportDtoList.size() > 150) {
            throw ExceptionResultEnum.ERROR.exception("单次导入命题任务数量不超过150条");
        }

        AtomicInteger i = new AtomicInteger(3);
        List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, null);
        if (CollectionUtils.isEmpty(sysOrgList)) {
            throw ExceptionResultEnum.ERROR.exception("请先导入机构数据");
        }
        List<BasicCourse> basicCourseList = basicCourseService.listBySchoolId(schoolId);
        if (CollectionUtils.isEmpty(basicCourseList)) {
            throw ExceptionResultEnum.ERROR.exception("请先导入课程数据");
        }
        List<SysUserResult> sysUserList = sysUserService.listBySchoolId(schoolId);
        if (CollectionUtils.isEmpty(sysUserList)) {
            throw ExceptionResultEnum.ERROR.exception("请先导入用户数据");
        }

        // 课程与名称map
        Map<String, String> courseCodeAndNameMap = new HashMap<>();
        // 试卷编号与课程代码map
        Map<String, String> paperNumberAndCourseCodeMap = new HashMap<>();

        ExamTaskImportResultDto examTaskImportResultDto = new ExamTaskImportResultDto();
        examTaskImportResultDto.setBatchNo(String.valueOf(System.nanoTime()));
        List<ExamTaskTemp> examTaskTempList = new ArrayList<>();
        List<ExamTaskDto> examTaskDtos = new ArrayList<>();

        for (ExamTaskTemp examTaskTemp : examTaskImportDtoList) {
            StringJoiner errorMsg = new StringJoiner("");
            errorMsg.add("第" + i.getAndIncrement() + "行");
            String courseCode = examTaskTemp.getCourseCode();
            String courseName = examTaskTemp.getCourseName();
            String teachingRoomName = examTaskTemp.getTeachingRoomName();
            String paperNumber = examTaskTemp.getPaperNumber();
            String userAccount = examTaskTemp.getUserAccount();
            String userName = examTaskTemp.getUserName();
            if (StringUtils.isBlank(courseCode)) {
                errorMsg.add("课程代码" + courseCode + "不能为空");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            if (StringUtils.isBlank(courseName)) {
                errorMsg.add("课程名称[" + courseName + "]不能为空");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            if (StringUtils.isBlank(teachingRoomName)) {
                errorMsg.add("开课学院[" + teachingRoomName + "]不能为空");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            SysOrg sysOrg = sysOrgList.stream().filter(m -> m.getName().equals(teachingRoomName)).findFirst().orElse(null);
            if (sysOrg == null) {
                errorMsg.add("开课学院[" + teachingRoomName + "]不存在");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            BasicCourse basicCourse = basicCourseList.stream().filter(m -> m.getTeachingRoomId().equals(sysOrg.getId()) && m.getCode().equals(courseCode)).findFirst().orElse(null);
            if (basicCourse == null) {
                errorMsg.add("开课学院[" + teachingRoomName + "]未找到课程代码[" + courseCode + "]");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            // 课程是否重复
            if (courseCodeAndNameMap.containsKey(courseCode + teachingRoomName)) {
                if (courseCodeAndNameMap.get(courseCode + teachingRoomName).equals(paperNumber)) {
                    errorMsg.add("课程代码[" + courseCode + "]、开课学院[" + teachingRoomName + "]有重复数据");
                    throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
                }
            } else {
                courseCodeAndNameMap.put(courseCode + teachingRoomName, paperNumber);
            }
            // 试卷编号是否重复
            if (StringUtils.isNotBlank(paperNumber)) {
                if (paperNumberAndCourseCodeMap.containsKey(paperNumber)) {
                    errorMsg.add("试卷编号[" + paperNumber + "]有重复数据");
                    throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
                } else {
                    courseCodeAndNameMap.put(paperNumber, "1");
                }
            }
            if ((StringUtils.isNotBlank(userAccount) && StringUtils.isBlank(userName)) ||
                    (StringUtils.isBlank(userAccount) && StringUtils.isNotBlank(userName))) {
                errorMsg.add("命题老师工号、命题老师姓名必须都为空或都不为空");
                throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
            }
            SysUserResult sysUser = null;
            if (!StringUtils.isAllBlank(userAccount, userName)) {
                // 校验命题老师
                sysUser = sysUserList.stream().filter(m -> m.getLoginName().equals(userAccount)).findFirst().orElse(null);
                if (sysUser != null && !userName.equals(sysUser.getRealName())) {
                    errorMsg.add("命题老师姓名[" + userName + "]与用户管理中不一致");
                    throw ExceptionResultEnum.ERROR.exception(errorMsg.toString());
                }
            }

            examTaskTemp.setId(SystemConstant.getDbUuid());
            examTaskTemp.setSchoolId(schoolId);
            examTaskTemp.setBatchNo(examTaskImportResultDto.getBatchNo());
            examTaskTemp.setCourseId(basicCourse.getId());
            examTaskTempList.add(examTaskTemp);

            ExamTaskDto examTaskDto = new ExamTaskDto();
            examTaskDto.setCourseId(basicCourse.getId());
            examTaskDto.setCourseCode(examTaskTemp.getCourseCode());
            examTaskDto.setCourseName(examTaskTemp.getCourseName());
            examTaskDto.setPaperNumber(examTaskTemp.getPaperNumber());
            if (sysUser != null) {
                examTaskDto.setUserId(String.valueOf(sysUser.getId()));
                examTaskDto.setUserName(sysUser.getRealName());
                examTaskDto.setUsers(Arrays.asList(sysUser));
            } else {
                examTaskDto.setUsers(sysUserList);
            }
            examTaskDtos.add(examTaskDto);

        }
        examTaskImportResultDto.setTasks(examTaskDtos);
        if (CollectionUtils.isNotEmpty(examTaskTempList)) {
            examTaskTempService.saveBatch(examTaskTempList);
        }
        return examTaskImportResultDto;
    }

    @Override
    public List<ExamTask> saveBatch(ExamTask task) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long examId = task.getExamId();
        BasicExam basicExam = basicExamService.getById(examId);

        task.setSchoolId(schoolId);
        if (StringUtils.isBlank(task.getBatchNo())) {
            throw ExceptionResultEnum.ERROR.exception("导入批次号不能为空，请重新导入");
        }

        List<ExamTask> list = new ArrayList<>();
        List<ExamTaskImportTaskUserDto> userList = task.getUsers();

        StringJoiner stringJoiner = new StringJoiner(",");
        for (ExamTaskImportTaskUserDto dto : userList) {
            // 配置信息
            BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(examId);
            Long cardRuleId = basicPrintConfig.getCardRuleId();

            ExamTask examTask = new ExamTask();
            examTask.setSchoolId(task.getSchoolId());
            examTask.setCourseId(dto.getCourseId());
            examTask.setExamId(examId);
            examTask.setCardRuleId(cardRuleId);

            String paperNumber = dto.getPaperNumber();
            if (StringUtils.isNotBlank(paperNumber)) {
                QueryWrapper<ExamTask> taskQueryWrapper = new QueryWrapper<>();
                taskQueryWrapper.lambda().eq(ExamTask::getExamId, examId).eq(ExamTask::getPaperNumber, paperNumber);
                ExamTask task1 = this.getOne(taskQueryWrapper);
                if (task1 != null) {
                    stringJoiner.add(String.format("试卷编号[%s]在考试[%s]下已使用", paperNumber, basicExam.getName()));
//                    // 试卷编号已存在，直接跳过
                    continue;
                }
            } else {
                paperNumber = printCommonService.autoCreatePaperNumber(examId);
            }
            examTask.setPaperNumber(paperNumber);
            examTask.setStartTime(task.getStartTime());
            examTask.setEndTime(task.getEndTime());
            examTask.setEnable(true);
            examTask.setBatchNo(task.getBatchNo());
            examTask.insertInfo(sysUser.getId());

            // 取课程所在机构
            BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
            examTask.setTeachingRoomId(basicCourse.getTeachingRoomId());

            Long userId = dto.getUserId();
            if (userId != null) {
                examTask.setUserId(userId);
                examTask.setStatus(ExamStatusEnum.STAGE);
            } else {
                examTask.setStatus(ExamStatusEnum.DRAFT);
            }
            examTask.setReview(basicPrintConfig.getReview());
            examTask.setSource(ExamTaskSourceEnum.ASSIGN);
            list.add(examTask);
        }

        // 有异常，抛出
        if (stringJoiner.toString().length() > 0) {
            throw ExceptionResultEnum.ERROR.exception(stringJoiner.toString());
        }
        this.saveBatch(list);
        for (ExamTask examTask : list) {
            if (Objects.nonNull(examTask.getReview()) && examTask.getReview() && Objects.nonNull(examTask.getUserId()) && Objects.isNull(examTask.getFlowId())) {
                Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(),
                        examTask,
                        TFCustomTypeEnum.ELECTRON_FLOW,
                        examTask.getUserId(),
                        true);
                map = activitiService.customFlowStart(map);
                examTask.setFlowId(SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID)));
                map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.NEW);
                tfFlowJoinService.saveOrUpdate(map);
                examTask.updateInfo(sysUser.getId());
                this.updateById(examTask);
                activitiService.sendFlowStartMsg(examTask.getFlowId());
            }
        }
        return list;
    }

    @Override
    public IPage<ExamTaskDto> listTaskApply(Long semesterId, Long examId, String auditStatus, Long openCollegeId, Long courseId, String paperNumber, Long startTime, Long endTime, String userName, Integer pageNumber, Integer pageSize) {
        userName = SystemConstant.translateSpecificSign(userName);
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskApply(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId, auditStatus, openCollegeId, courseId, paperNumber, startTime, endTime, sysUser.getId(), userName);
        for (ExamTaskDto record : examTaskDtoIPage.getRecords()) {
            if ((Objects.nonNull(record.getReview()) && record.getReview() && FlowStatusEnum.FINISH.name().equals(record.getAuditStatus()) && !ExamModelEnum.MODEL3.equals(record.getExamModel()))) {
                record.setApproveFormStatus(true);
            } else {
                record.setApproveFormStatus(false);
            }
        }
        return examTaskDtoIPage;
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewUnaudited(Long semesterId, Long examId, Long openCollegeId, Long courseId, String paperNumber, Long userId, Long startTime, Long endTime, Long startCreateTime, Long endCreateTime, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.listTaskReviewUnaudited(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId,
                openCollegeId, courseId, paperNumber, userId, startTime, endTime, startCreateTime, endCreateTime,
                sysUser.getId());
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewAudited(Long semesterId, Long examId, Long openCollegeId, Long courseId, String
            paperNumber, Long userId, Long startTime, Long endTime, Long startCreateTime, Long endCreateTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return this.baseMapper.listTaskReviewAudited(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId,
                openCollegeId, courseId, paperNumber, userId, startTime, endTime, orgIds, startCreateTime, endCreateTime, sysUser.getId());
    }

    @Override
    public IPage<ExamTaskDetailDto> listTaskPaper(Long semesterId, Long examId, Long openCollegeId, Long courseId, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        boolean containsQuestionTeacher = false;
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(),
                ServletUtil.getRequest().getServletPath());
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examTaskDetailDtoIPage = this.baseMapper.listTaskPaper(page, schoolId, semesterId, examId, openCollegeId, courseId, paperNumber,
                startTime, endTime, dpr, containsQuestionTeacher, sysUser.getId());
        for (ExamTaskDetailDto record : examTaskDetailDtoIPage.getRecords()) {
            List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(record.getId());
            record.setPaperType(examTaskDetailList.stream().map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
            record.setExposedPaperType(examTaskDetailList.stream().filter(ExamTaskDetail::getExposed).map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
//            record.setRelatePaperType(examTaskDetailList.stream().filter(m->m.getExposedExamId() != null).map(m->m.getSerialNumber().toString()).collect(Collectors.joining(",")));
            record.setUnexposedPaperType(examTaskDetailList.stream().filter(m -> !m.getExposed()).map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
        }
        return examTaskDetailDtoIPage;
    }

    @Override
    public List<RelatePaperDto> listPaperTypes(Long examTaskId, Long printPlanId, Long courseId) {
        List<RelatePaperDto> list = new ArrayList<>();
        ExamTask examTask = this.getById(examTaskId);
        RelatePaperDto relatePaperDto = new RelatePaperDto();
        relatePaperDto.setId(examTask.getId());
        relatePaperDto.setSchoolId(examTask.getSchoolId());
        relatePaperDto.setCourseId(examTask.getCourseId());
        relatePaperDto.setPaperNumber(examTask.getPaperNumber());

        List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(examTaskId);
        if (CollectionUtils.isNotEmpty(examTaskDetailList)) {
            // 所有卷型
            relatePaperDto.setPaperTypes(examTaskDetailList.stream().map(ExamTaskDetail::getSerialNumber).collect(Collectors.toList()));
        }
        list.add(relatePaperDto);
        return list;
    }

    @Transactional
    @Override
    public Map<String, Object> saveExamTaskDetail(ExamTaskApplyPram examTaskApplyPram) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Optional.ofNullable(examTaskApplyPram.getExamTaskId()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("命题任务ID不能为空"));
        // 更新examTask状态status
        ExamTask examTask = this.getById(examTaskApplyPram.getExamTaskId());

        // 提交时，校验
        BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(examTask.getExamId());
        if (basicPrintConfig == null) {
            throw ExceptionResultEnum.ERROR.exception("印品配置未设置");
        }

        Map<String, Object> map = new HashMap<>();
        // 审核一级
        Task task = taskService.createTaskQuery().processInstanceId(String.valueOf(examTask.getFlowId())).singleResult();
        if (Objects.nonNull(task)) {
            map.computeIfAbsent(SystemConstant.FLOW_TASK_ID, v -> task.getId());
//                map.computeIfAbsent(SystemConstant.PAPAER_ATTACHMENT_IDS, v -> examTaskDetail.getPaperAttachmentIds());
            map.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.PASS);
            map.computeIfAbsent(SystemConstant.APPROVE_USER_IDS, v -> examTaskApplyPram.getApproveUserIds());
            map = activitiService.taskApprove(map);
            map.put(SystemConstant.SEND_FLOW_MQ, true);
        }

        examTask.setStatus(ExamStatusEnum.SUBMIT);
        examTask.setOpenAb(examTaskApplyPram.getOpenAb());
        examTask.setPaperConfirmAttachmentIds(examTaskApplyPram.getPaperConfirmAttachmentIds());
        examTask.setRemark(examTaskApplyPram.getRemark());
        examTask.setUpdateId(sysUser.getId());
        examTask.setUpdateTime(System.currentTimeMillis());
        examTask.setExamTaskContent(examTaskApplyPram.getExamTaskContent());
        this.updateById(examTask);

        examTaskDetailService.deleteByExamTaskId(examTaskApplyPram.getExamTaskId());
        // 新建试卷
        List<ExamTaskDetail> examTaskDetailList = examTaskApplyPram.getExamTaskDetailList();
        for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
            validSubmitParam(examTaskDetail, basicPrintConfig.getPrintContent(), examTaskApplyPram.getOpenAb());
            // 已曝光试卷和未曝光试卷赋值（新增时，已曝光试卷为null，未曝光试卷为paper_type）
            examTaskDetail.setId(SystemConstant.getDbUuid());
            examTaskDetail.setUnexposedPaperType(examTaskDetail.getPaperType());
            examTaskDetail.setExamTaskId(examTask.getId());
            examTaskDetail.insertInfo(sysUser.getId());
            List<PaperInfoVo> filePathVoList = examTaskDetail.getPaperInfoVoList();
            for (PaperInfoVo paperInfoVo : filePathVoList) {
                BasicAttachment basicAttachment = basicAttachmentService.getById(paperInfoVo.getAttachmentId());
                if (basicAttachment != null) {
                    FilePathVo filePathVo = JSON.parseObject(basicAttachment.getPath(), FilePathVo.class);
                    InputStream inputStream = fileUploadService.downloadFile(filePathVo.getPath(), filePathVo.getUploadType(), filePathVo.getType());
                    List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertPdfToJpg(Long.valueOf(paperInfoVo.getAttachmentId()), inputStream, sysUser.getId());
                    if (convertJpgStorageList.size() > 0) {
                        paperInfoVo.setJpgAttachmentId(JSON.toJSONString(convertJpgStorageList));
                    } else {
                        paperInfoVo.setJpgAttachmentId(null);
                    }
                }
            }
            examTaskDetail.setPaperAttachmentIds(JSON.toJSONString(filePathVoList));
            examTaskDetailService.save(examTaskDetail);

            String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
            List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(paperAttachmentIds);
            List<Long> cardIds = paperInfoVoList.stream().filter(m -> StringUtils.isNotBlank(m.getCardId())).map(m -> Long.parseLong(m.getCardId())).collect(Collectors.toList());
            for (Long cardId : cardIds) {
                ExamCard examCard = examCardService.getById(cardId);
                if (!examTask.getOpenAb().equals(examCard.getOpenAb())) {
                    throw ExceptionResultEnum.ERROR.exception(examTask.getOpenAb() && !examCard.getOpenAb() ? "命题任务开启了AB卷，请选择AB卷题卡或者编辑题卡开启AB卷" : "命题任务未开启了AB卷，请选择单题卡或者编辑题卡关闭AB卷");
                }
            }
            // 将题卡更新为已绑定
            examCardService.updateUsedById(cardIds);
            // 更新题库试卷表
            for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                if (StringUtils.isNotBlank(paperInfoVo.getUuid())) {
                    UpdateWrapper<ExamTaskPaperData> updateWrapper1 = new UpdateWrapper<>();
                    updateWrapper1.lambda().set(ExamTaskPaperData::getPaperNumber, examTask.getPaperNumber())
                            .eq(ExamTaskPaperData::getExamId, examTask.getExamId())
                            .eq(ExamTaskPaperData::getPaperNumber, paperInfoVo.getUuid())
                            .eq(ExamTaskPaperData::getPaperId, paperInfoVo.getPaperId())
                            .eq(ExamTaskPaperData::getUuid, paperInfoVo.getUuid());
                    examTaskPaperDataService.update(updateWrapper1);

                    // 删除临时数据
                    QueryWrapper<ExamTaskPaperData> deleteWrapper = new QueryWrapper<>();
                    deleteWrapper.lambda().eq(ExamTaskPaperData::getExamId, examTask.getExamId())
                            .eq(ExamTaskPaperData::getPaperNumber, paperInfoVo.getUuid())
                            .ne(ExamTaskPaperData::getPaperId, paperInfoVo.getPaperId());
                    examTaskPaperDataService.remove(deleteWrapper);
                }
            }
        }
        ExamTaskSubmitContent examTaskSubmitContent = JSON.parseObject(examTaskApplyPram.getExamTaskContent(), ExamTaskSubmitContent.class);
        if (examTaskSubmitContent != null) {
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamIdAndPaperNumber(examTask.getExamId(), examTask.getPaperNumber());
            if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
                Set<Long> examDetailIds = examDetailCourseList.stream().map(ExamDetailCourse::getExamDetailId).collect(Collectors.toSet());
                List<ExamDetail> examDetailList = examDetailService.listByIds(examDetailIds);
                if (CollectionUtils.isNotEmpty(examDetailList)) {
                    // 删除原考场
                    examDetailService.removeByIds(examDetailIds);
                }
                examDetailCourseService.removeByIds(examDetailCourseList.stream().map(ExamDetailCourse::getId).collect(Collectors.toList()));
                // 删除原考生
                examDetailCourseList.forEach(m -> examStudentService.removeByExamDetailCourseId(m.getId()));

                BasicExam basicExam = basicExamService.getById(examTask.getExamId());
                ExamPrintPlan examPrintPlan = examPrintPlanService.getById(examDetailList.get(0).getPrintPlanId());
                // 新增考场、考生
                this.addExamDetailAndStudent(examTask, basicExam, examTaskSubmitContent.getExamDetail(), examPrintPlan, sysUser);
            }
        }

        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> status(ExamTask examTask) {
        Optional.ofNullable(examTask.getId()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("参数有误"));
        ExamTask task = this.getById(examTask.getId());
        if (Objects.nonNull(task.getReview()) && !task.getReview()) {
            throw ExceptionResultEnum.ERROR.exception("命题任务不需要审核，不能撤回");
        }
        Optional.ofNullable(task.getFlowId()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("审核流程有误，不能撤回"));

        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(task.getFlowId());
        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("审核流程进度丢失，不能撤回"));

        if (tfFlowApprove.getSetup().intValue() != 2) {
            throw ExceptionResultEnum.ERROR.exception("当前审核节点不能撤回");
        }

        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        SysOrg sysOrg = commonCacheService.orgCache(sysUser.getOrgId());
        List<Task> flowTaskList = taskService.createTaskQuery().processInstanceId(String.valueOf(task.getFlowId())).list();
        Map<String, Object> map = new HashMap<>();
        map.computeIfAbsent(SystemConstant.FLOW_TASK_ID, v -> flowTaskList.get(0).getId());
        map.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.CANCEL);
        String remark = Objects.nonNull(sysOrg) ? sysOrg.getName() : "无机构";
        map.computeIfAbsent(SystemConstant.APPROVE_REMARK, v -> sysUser.getRealName() + "(" + remark + ")" + FlowApprovePassEnum.CANCEL.getTitle());
        map.computeIfAbsent(SystemConstant.APPROVE_SETUP, v -> FlowApproveSetupEnum.SUBMIT.getSetup());
        activitiService.taskApprove(map);

        return map;
    }

    @Override
    public ExamTaskDetailCardDto applyGetOne(Long examTaskId, String source) {
        ExamTaskDetailCardDto detailCardDto = this.baseMapper.applyGetOne(examTaskId, source);
        if (detailCardDto != null) {
            BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(Long.valueOf(detailCardDto.getExamId()));
            if (basicPrintConfig != null) {
                detailCardDto.setPrintContent(basicPrintConfig.getPrintContent());
                detailCardDto.setReview(basicPrintConfig.getReview());
                detailCardDto.setAuditContent(basicPrintConfig.getAuditContent());
            }
            detailCardDto.setExamTaskDetailList(examTaskDetailService.listByExamTaskId(examTaskId));
        }
        return detailCardDto;
    }

    @Override
    public void paperDownload(HttpServletResponse response, Long examTaskId) {
        ExamTaskDetailPdfDownloadDto examTaskDetailPdfDownloadDto = examTaskDetailService.findPdfDownload(examTaskId);
        Optional.ofNullable(examTaskDetailPdfDownloadDto).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("数据异常"));

        String rootPath = null;
        try {
            // 本地保存目录
            File tempFile = SystemConstant.getFileTempParentDirVar(SystemConstant.TEMP_PREFIX);
            rootPath = tempFile.getPath() + File.separator + SystemConstant.getNanoId();

            // 下载文件保存目录（用uuid命名）
            String downloadFilePath = rootPath + File.separator + examTaskDetailPdfDownloadDto.getCourseNameCode().replaceAll(" ", "") + "试卷包";
            File downloadPathFile = new File(downloadFilePath);
            if (!downloadPathFile.exists()) {
                downloadPathFile.mkdirs();
            }

            StringJoiner dirPath = new StringJoiner("");
            dirPath = dirPath.add(downloadFilePath).add(File.separator)
                    .add(examTaskDetailPdfDownloadDto.getSemesterName()).add(File.separator)
                    .add(examTaskDetailPdfDownloadDto.getExamName()).add(File.separator)
                    .add(examTaskDetailPdfDownloadDto.getCourseNameCode().replaceAll(" ", "")).add(File.separator)
                    .add(examTaskDetailPdfDownloadDto.getPaperNumber()).add(File.separator);

            List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(examTaskId);
            for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds());
                // 试卷
                for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                    Long attachmentId = Long.valueOf(paperInfoVo.getAttachmentId());
                    String name = paperInfoVo.getName();
                    if (Objects.nonNull(attachmentId)) {
                        BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
                        Optional.ofNullable(attachment).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("附件数据异常"));
                        String fileName = dirPath + "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "试卷" + SystemConstant.HYPHEN + name + attachment.getType();
                        fileUploadService.downloadFile(attachment, fileName);
                    }

                    // 题卡
                    Long cardId = Long.valueOf(paperInfoVo.getCardId());
                    if (cardId != null) {
                        ExamCard examCard = examCardService.getById(cardId);

                        String cardHtmlPath = dirPath + "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "题卡" + SystemConstant.HYPHEN + name + SystemConstant.HTML_PREFIX;
                        String cardPdfPath = dirPath + "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "题卡" + SystemConstant.HYPHEN + name + SystemConstant.PDF_PREFIX;

                        String htmlContent = createPdfUtil.replaceBlankHtmlContent(examCard.getHtmlContent(), examCard.getCourseId());
                        // html
                        File htmlFile = new File(cardHtmlPath);
                        if (!htmlFile.exists()) {
                            htmlFile.getParentFile().mkdirs();
                            htmlFile.createNewFile();
                        }
                        // 生成html文件
                        FileUtil.writeContent(htmlFile, htmlContent);
                        // 转pdf文件
                        File pdfFile = new File(cardPdfPath);
                        if (!pdfFile.exists()) {
                            pdfFile.getParentFile().mkdirs();
                            pdfFile.createNewFile();
                        }
                        HtmlToPdfUtil.convert(cardHtmlPath, cardPdfPath, "8K".equals(examCard.getPageSize()) ? PageSizeEnum.K8 : PageSizeEnum.valueOf(examCard.getPageSize()));
                    }
                }
            }
            String zipFileName = downloadPathFile.getName() + SystemConstant.ZIP_PREFIX;
            SysUser sysUser = sysUserService.getById(ServletUtil.getRequestUserId());
            FileUtil.downloadEncryptZip(response, downloadPathFile, zipFileName, new String(Base64Util.decode(sysUser.getPassword())));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("文件下载失败");
        } finally {
            if (Objects.nonNull(rootPath)) {
                FileUtil.deleteDirectory(rootPath);
            }
        }
    }

    @Override
    public void checkDataByExamination(Long printPlanId, List<Long> examDetailIdList, SysUser user) {
        Set<String> stringSet = new HashSet<>();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);
        List<ExamDetail> examDetailList = new ArrayList<>();
        for (Long examDetailId : examDetailIdList) {
            ExamDetail examDetail = examDetailService.getById(examDetailId);
            if (Objects.isNull(examDetail)) {
                stringSet.add(String.format("印刷计划[%s]没有考场ID[%s]的数据", examPrintPlan.getName(), examDetailId));
            }
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda()
                    .eq(ExamDetailCourse::getSchoolId, examDetail.getSchoolId())
                    .eq(ExamDetailCourse::getExamDetailId, examDetailId));
            if (CollectionUtils.isEmpty(examDetailCourseList)) {
                stringSet.add(String.format("印刷计划[%s]没有考点[%s]、考场[%s]的课程数据", examPrintPlan.getName(), examDetail.getExamPlace(), examDetail.getExamRoom()));
            }

            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                // 校验命题任务是否存在并审核通过
                this.validExamTaskAuditPass(examDetailCourse.getSchoolId(), examPrintPlan.getExamId(), examDetailCourse.getCourseId(), examDetailCourse.getPaperNumber());
            }

            examDetailList.add(examDetail);
        }

        // 有异常抛出
        if (!CollectionUtils.isEmpty(stringSet)) {
            throw ExceptionResultEnum.ERROR.exception(String.join("；\r\n", stringSet));
        }

        for (ExamDetail examDetail : examDetailList) {
            TbTaskDetailResult tbTaskDetailResult = examDetailCourseService.getByExamDetailId(examDetail.getId());
            tbTaskPdfService.saveTask(tbTaskDetailResult, examDetail.getPrintPlanId(), user, examDetail.getId());
        }
    }

    @Override
    public List<ExamTaskDetailDto> listTaskPaper(Long semesterId, Long examId, Long openCollegeId, Long courseId, String paperNumber, Long startTime, Long
            endTime) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 查询用户角色是否包含命题老师
        List<SysRole> list = sysUserRoleService.listRoleByUserId(sysUser.getId());
        boolean containsQuestionTeacher = false;
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.listTaskPaper(schoolId, semesterId, examId, openCollegeId, courseId, paperNumber, startTime, endTime, dpr, orgIds, containsQuestionTeacher, sysUser.getId());
    }

    @Override
    public TBTask taskDownloadPdf(Long semesterId, Long examId, Long openCollegeId, Long courseId, String paperNumber, Long startTime, Long endTime) {
        List<ExamTaskDetailDto> examTasks = this.listTaskPaper(semesterId, examId, openCollegeId, courseId, paperNumber, startTime, endTime);
        if (CollectionUtils.isEmpty(examTasks)) {
            throw ExceptionResultEnum.ERROR.exception("没有可导出数据");
        }
        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.PAPER_AND_CARD_PDF_DOWNLOAD, semesterId, examId, (SysUser) ServletUtil.getRequestUser());
        map.put("examTasks", examTasks);
        asyncPaperReviewPdfExportService.exportTask(map);
        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
        return tbTask;
    }

    /**
     * 获取待办任务
     *
     * @param iPage
     * @param schoolId
     * @param orgId
     * @param userId
     * @return
     */
    @Override
    public IPage<WorkResult> getFlowTaskReadyList(IPage<Map> iPage, Long schoolId, Long orgId, Long userId) {
        return this.baseMapper.getFlowTaskReadyList(iPage, schoolId, orgId, String.valueOf(userId));
    }

    /**
     * 根据流程id和任务节点id获取待办
     *
     * @param flowId
     * @param taskId
     * @return
     */
    @Override
    public WorkResult getFlowInfo(Long flowId, Long taskId) {
        return this.baseMapper.getFlowInfo(flowId, taskId);
    }

    /**
     * 根据流程状态查找命题任务
     */
    @Override
    public ExamTask findExamTaskByFlowStatus(Long schoolId, Long examId, Long courseId, String paperNumber, FlowStatusEnum flowStatus) {
        List<ExamTask> examTaskList = this.baseMapper.findExamTaskByFlowStatus(schoolId, examId, courseId, paperNumber, Objects.nonNull(flowStatus) ? flowStatus.name() : null);
        if (CollectionUtils.isEmpty(examTaskList)) {
            throw ExceptionResultEnum.ERROR.exception("试卷编号[" + paperNumber + "]的" + ExceptionResultEnum.EXAM_TASK_IS_NOT_EXIST.getMessage());
        } else if (examTaskList.size() != 1) {
            throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]", paperNumber) + ExceptionResultEnum.DATA_COUNT_EXCEPTION.getMessage());
        }
        return examTaskList.get(0);
    }

    @Transactional
    @Override
    public void removeByExamTaskId(Long examTaskId) {
        List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(examTaskId);
        for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
            examTaskDetailService.removeById(examTaskDetail.getId());

            // 题卡更新为未使用
            String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
            if (StringUtils.isNotBlank(paperAttachmentIds)) {
                List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(paperAttachmentIds);
                examCardService.updateUsedById(paperInfoVoList.stream().filter(m -> StringUtils.isNotBlank(m.getCardId())).map(m -> Long.parseLong(m.getCardId())).collect(Collectors.toList()));
            }
        }
        this.removeById(examTaskId);
    }

    /**
     * 校验输入的试卷编号是否使用
     */
    public void validPaperNumberExists(ExamTask examTask) {
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_PAPER_NUMBER_PREFIX + examTask.getExamId() + examTask.getPaperNumber(), SystemConstant.REDIS_LOCK_PAPER_NUMBER_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在校验试卷编号，请稍后再试");
        }
        try {
            ExamTask task = this.getByExamIdAndPaperNumber(examTask.getExamId(), examTask.getPaperNumber());
            if (task != null) {
                throw ExceptionResultEnum.ERROR.exception("试卷编号已使用");
            }
        } finally {
            redisUtil.releaseLock(SystemConstant.REDIS_LOCK_PAPER_NUMBER_PREFIX + examTask.getExamId() + examTask.getPaperNumber());
        }
    }

    @Transactional
    @Override
    public Map<String, Object> submitTaskApply(ExamTaskSubmitPram examTaskSubmitPram) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamTaskSubmitContent examTaskSubmitContent = JSON.parseObject(examTaskSubmitPram.getExamTaskContent(), ExamTaskSubmitContent.class);
        // 新建命题任务
        ExamTask examTask = examTaskSubmitContent.getExamTask();
        BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(examTask.getExamId());

        // 开启了入库审核
        if (basicPrintConfig.getReview()) {
            Long customFlowId = examTaskSubmitPram.getCustomFlowId();
            Optional.ofNullable(customFlowId).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程ID为空"));

            if (Objects.nonNull(examTaskSubmitPram.getVersion())) {
                Integer version = examTaskSubmitPram.getVersion();
                Optional.ofNullable(version).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程版本号为空"));

                TFCustomFlow tfCustomFlow = tfCustomFlowService.getById(customFlowId);
                Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到自定义流程数据"));

                if (tfCustomFlow.getVersion().intValue() != version.intValue()) {
                    throw ExceptionResultEnum.ERROR.exception("自定义流程数据已发生改变，请刷新重试");
                }
            }
        }

        Map<String, Object> mapVar = new HashMap<>();
        try {
            if (StringUtils.isNotBlank(examTask.getPaperNumber())) {
                validPaperNumberExists(examTask);
            } else {
                examTask.setPaperNumber(printCommonService.autoCreatePaperNumber(examTask.getExamId()));
            }

            examTask.setId(SystemConstant.getDbUuid());
            examTask.setSchoolId(schoolId);
            examTask.setReview(basicPrintConfig.getReview());
            examTask.setUserId(sysUser.getId());
            examTask.setStatus(ExamStatusEnum.SUBMIT);
            examTask.setSource(ExamTaskSourceEnum.SELF);
            examTask.setPaperConfirmAttachmentIds(examTaskSubmitPram.getPaperConfirmAttachmentIds());
            examTask.setRemark(examTaskSubmitPram.getRemark());
            examTask.insertInfo(sysUser.getId());
            examTask.setExamTaskContent(examTaskSubmitPram.getExamTaskContent());

            if (Objects.isNull(examTask.getFlowId())) {
                if (Objects.nonNull(basicPrintConfig.getReview()) && basicPrintConfig.getReview()) {
                    Map<String, Object> map = SystemConstant.buildFlowVar(examTask.getId(), examTask, TFCustomTypeEnum.ELECTRON_FLOW, examTask.getUserId(), false);
                    map = activitiService.customFlowStart(map);
                    examTask.setFlowId(SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID)));
                    map.computeIfAbsent(SystemConstant.FLOW_JOIN_TYPE, v -> FlowJoinTypeEnum.NEW);
                    tfFlowJoinService.saveOrUpdate(map);
                    mapVar.putAll(map);
                    mapVar.put(SystemConstant.SEND_FLOW_START_MQ, true);
                }
            }
            this.save(examTask);

            // 新建试卷
            List<ExamTaskDetail> examTaskDetailList = examTaskSubmitContent.getExamTaskDetailList();
            for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                validSubmitParam(examTaskDetail, basicPrintConfig.getPrintContent(), examTask.getOpenAb());
                // 已曝光试卷和未曝光试卷赋值（新增时，已曝光试卷为null，未曝光试卷为paper_type）
                examTaskDetail.setId(SystemConstant.getDbUuid());
                examTaskDetail.setUnexposedPaperType(examTaskDetail.getPaperType());
                examTaskDetail.setExamTaskId(examTask.getId());
                examTaskDetail.insertInfo(sysUser.getId());
                List<PaperInfoVo> filePathVoList = examTaskDetail.getPaperInfoVoList();
                for (PaperInfoVo paperInfoVo : filePathVoList) {
                    BasicAttachment basicAttachment = basicAttachmentService.getById(paperInfoVo.getAttachmentId());
                    FilePathVo filePathVo = JSON.parseObject(basicAttachment.getPath(), FilePathVo.class);
                    InputStream inputStream = fileUploadService.downloadFile(filePathVo.getPath(), filePathVo.getUploadType(), filePathVo.getType());
                    List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertPdfToJpg(Long.valueOf(paperInfoVo.getAttachmentId()), inputStream, sysUser.getId());
                    if (convertJpgStorageList.size() > 0) {
                        paperInfoVo.setJpgAttachmentId(JSON.toJSONString(convertJpgStorageList));
                    } else {
                        paperInfoVo.setJpgAttachmentId(null);
                    }
                }
                examTaskDetail.setPaperAttachmentIds(JSON.toJSONString(filePathVoList));
                examTaskDetailService.save(examTaskDetail);

                String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
                List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(paperAttachmentIds);

                List<Long> cardIds = paperInfoVoList.stream().filter(m -> StringUtils.isNotBlank(m.getCardId())).map(m -> Long.parseLong(m.getCardId())).collect(Collectors.toList());
                for (Long cardId : cardIds) {
                    ExamCard examCard = examCardService.getById(cardId);
                    if (!examTask.getOpenAb().equals(examCard.getOpenAb())) {
                        throw ExceptionResultEnum.ERROR.exception(examTask.getOpenAb() && !examCard.getOpenAb() ? "命题任务开启了AB卷，请选择AB卷题卡或者编辑题卡开启AB卷" : "命题任务未开启了AB卷，请选择单题卡或者编辑题卡关闭AB卷");
                    }
                }
                // 将题卡更新为已绑定
                examCardService.updateUsedById(cardIds);
                // 更新题库试卷表
                for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                    if (StringUtils.isNotBlank(paperInfoVo.getUuid())) {
                        UpdateWrapper<ExamTaskPaperData> updateWrapper = new UpdateWrapper<>();
                        updateWrapper.lambda().set(ExamTaskPaperData::getPaperNumber, examTask.getPaperNumber())
                                .eq(ExamTaskPaperData::getExamId, examTask.getExamId())
                                .eq(ExamTaskPaperData::getPaperNumber, paperInfoVo.getUuid())
                                .eq(ExamTaskPaperData::getPaperId, paperInfoVo.getPaperId())
                                .eq(ExamTaskPaperData::getUuid, paperInfoVo.getUuid());
                        examTaskPaperDataService.update(updateWrapper);
                    }
                }

                if (StringUtils.isNotBlank(paperInfoVoList.get(0).getUuid())) {
                    // 删除临时数据
                    List<Long> paperIdList = paperInfoVoList.stream().filter(m -> m.getPaperId() != null).map(PaperInfoVo::getPaperId).collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(paperIdList)) {
                        QueryWrapper<ExamTaskPaperData> deleteWrapper = new QueryWrapper<>();
                        deleteWrapper.lambda().eq(ExamTaskPaperData::getExamId, examTask.getExamId())
                                .eq(ExamTaskPaperData::getPaperNumber, paperInfoVoList.get(0).getUuid())
                                .notIn(ExamTaskPaperData::getPaperId, paperIdList);
                        examTaskPaperDataService.remove(deleteWrapper);
                    }
                }
            }

            BasicExam basicExam = basicExamService.getById(examTask.getExamId());
            if (ExamModelEnum.MODEL1.equals(basicExam.getExamModel()) || ExamModelEnum.MODEL2.equals(basicExam.getExamModel())) {
                // 创建印刷计划
                ExamPrintPlan examPrintPlan = new ExamPrintPlan();
                examPrintPlan.insertInfo(sysUser.getId());
                examPrintPlan.setSchoolId(schoolId);
                examPrintPlan.setCategory(ExamCategoryEnum.FORMAL);
                examPrintPlan.setExamId(examTask.getExamId());
                examPrintPlan.setOrgId(sysUser.getOrgId());
                BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
                examPrintPlan.setName((basicCourse != null ? basicCourse.getName() : "") + examTask.getPaperNumber());
                if (!ExamCategoryEnum.REBUILD.equals(basicExam.getCategory())) {
//                    ConvertUtil.analyzeAndCompareDateAndTime(examDetailParams.getExamStartTime(), examDetailParams.getExamEndTime());
//                    examPrintPlan.setExamStartTime(examDetailParams.getExamStartTime());
//                    examPrintPlan.setExamEndTime(examDetailParams.getExamEndTime());
                }
//                List<String> stringList = JSONObject.parseArray(basicPrintConfig.getPrintContent(), String.class);
//                if (!stringList.contains("PAPER")) {
//                    long count = paperInfoVoList.stream().filter(m -> Objects.isNull(m.getAttachmentId())).count();
//                    if (count == 0) {
//                        stringList.add("PAPER");
//                    }
//                }
                examPrintPlan.setPrintContent(basicPrintConfig.getPrintContent());
                examPrintPlan.setBackupMethod(BackupMethodEnum.valueOf(basicPrintConfig.getBackupMethod()));
                examPrintPlan.setBackupCount(basicPrintConfig.getBackupCount());
                examPrintPlan.setDrawRule(basicPrintConfig.getDrawRule());
                examPrintPlan.setVariableContent(basicPrintConfig.getVariableContent());
                examPrintPlan.setOrdinaryContent(basicPrintConfig.getOrdinaryContent());
                examPrintPlan.setStatus(PrintPlanStatusEnum.READY);
                examPrintPlanService.save(examPrintPlan);

                // 保存考场、考生
                this.addExamDetailAndStudent(examTask, basicExam, examTaskSubmitContent.getExamDetail(), examPrintPlan, sysUser);
            }

            if (Objects.nonNull(examTask.getFlowId())) {
                // 审核一级
                Task task = taskService.createTaskQuery().processInstanceId(String.valueOf(examTask.getFlowId())).singleResult();
                if (Objects.nonNull(task)) {
                    Map<String, Object> map = new HashMap<>();
                    map.computeIfAbsent(SystemConstant.FLOW_TASK_ID, v -> task.getId());
                    map.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.PASS);
                    map.computeIfAbsent(SystemConstant.PAPAER_ATTACHMENT_IDS, v -> examTaskDetailList.get(0).getPaperAttachmentIds());
                    map.computeIfAbsent(SystemConstant.APPROVE_USER_IDS, v -> examTaskSubmitPram.getApproveUserIds());
                    activitiService.taskApprove(map);
                    mapVar.putAll(map);
                    mapVar.remove(SystemConstant.SEND_FLOW_START_MQ);
                    mapVar.put(SystemConstant.SEND_FLOW_MQ, true);
                }
            }
            mapVar.put(SystemConstant.EXAM_TASK, examTask);
            return mapVar;
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ActivitiObjectNotFoundException) {
                ResultUtil.error("流程不存在");
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return null;
    }

    private void addExamDetailAndStudent(ExamTask examTask, BasicExam basicExam, ExamDetailParams examDetailParams, ExamPrintPlan examPrintPlan, SysUser sysUser) {
        Long schoolId = sysUser.getSchoolId();
        List<ExamDetailCourseDto> examDetailCourseList = examDetailService.listByExamIdAndCourseId(schoolId, examTask.getExamId(), examTask.getCourseId());

        // 查询考试+课程下已用过的班级
        Map<Long, String> basicStudentIdUserMap = new HashMap<>();

        String paperNumber = examTask.getPaperNumber();
        if (ExamModelEnum.MODEL1.equals(basicExam.getExamModel())) {
            for (ExamDetailList examDetailList : examDetailParams.getList()) {
                int count = examDetailList.getStudentCount();
                String className = examDetailList.getClassName();
                if (count == 0) {
                    throw ExceptionResultEnum.ERROR.exception("选择的考试对象[" + className + "]下没有考生");
                }

                if (!CollectionUtils.isEmpty(examDetailCourseList)) {
                    examDetailCourseList.stream().filter(m -> StringUtils.isNotBlank(m.getClazzName())).forEach(m -> {
                        examStudentService.listStudentByExamDetailCourseId(m.getId()).stream().filter(t -> t.getBasicStudentId() != null).forEach(s -> basicStudentIdUserMap.put(s.getBasicStudentId(), String.format("%s(%s)", m.getRealName(), m.getLoginName())));
                    });
                }

                // 校验班级或班级内考生是否已命题
                List<Long> examTaskStudentObjectParamList = examDetailList.getExamTaskStudentObjectParamListIds();
                if (MapUtils.isNotEmpty(basicStudentIdUserMap)) {
                    examTaskStudentObjectParamList.forEach(i -> {
                        if (basicStudentIdUserMap.containsKey(i)) {
                            BasicExamStudent basicExamStudent = basicExamStudentService.getById(i);
                            if (basicExamStudent != null) {
                                throw ExceptionResultEnum.ERROR.exception("教学班[" + basicExamStudent.getTeachClassName() + "]内有考生已被[" + basicStudentIdUserMap.get(i) + "]选择");
                            }
                        }
                    });
                }

                ExamDetail examDetail = new ExamDetail();
                examDetail.setId(SystemConstant.getDbUuid());
                examDetail.setPackageCode("1" + redisCounterUtil.getCounter(schoolId, 6, "packageCode"));
                examDetail.setSchoolId(schoolId);
                examDetail.setOrgId(sysUser.getOrgId());
                examDetail.setPrintPlanId(examPrintPlan.getId());
                examDetail.setExamId(examPrintPlan.getExamId());
                examDetail.setPrintPlanName(examPrintPlan.getName());
                examDetail.setExamPlace(examDetailList.getExamPlace()); // 无默认值
                examDetail.setExamRoom(examDetailList.getExamRoom()); // 无默认值
                examDetail.setStatus(ExamDetailStatusEnum.NEW);
                if (!ExamCategoryEnum.REBUILD.equals(basicExam.getCategory())) {
                    examDetail.setExamStartTime(examDetailList.getExamStartTime());
                    examDetail.setExamEndTime(examDetailList.getExamEndTime());
                }
                examDetail.setTotalSubjects(count);
                examDetail.setExamDataSource(ExamDataSourceEnum.EXAM_TASK);
                examDetail.setNormal(true);
                examDetail.setPrintHouseId(examDetailList.getPrintHouseId());
                examDetail.setBackupCount(examDetailList.getBackupCount());
                examDetail.setCreateId(sysUser.getId());
                examDetail.setCreateTime(System.currentTimeMillis());
                examDetailService.save(examDetail);

                ExamDetailCourse examDetailCourse = new ExamDetailCourse();
                examDetailCourse.setId(SystemConstant.getDbUuid());
                examDetailCourse.setSchoolId(schoolId);
                examDetailCourse.setExamDetailId(examDetail.getId());
                examDetailCourse.setCourseId(examTask.getCourseId());
                examDetailCourse.setPaperNumber(paperNumber);
                examDetailCourse.setCoursePaperId(String.valueOf(examTask.getId()));
                examDetailCourse.setClazzName(examDetailList.getClassName());
                examDetailCourse.setTotalSubjects(examDetailList.getStudentCount());
                examDetailCourse.setCreateId(sysUser.getId());
                examDetailCourseService.save(examDetailCourse);

                // 更新实际考生数量
                List<ExamStudent> examStudents = examStudentService.saveStudentForExamApply(examTask, examDetailList.getExtendFields(), examTaskStudentObjectParamList, examDetailCourse.getId(), sysUser);

                if (!CollectionUtils.isEmpty(examStudents)) {
                    examStudentService.saveBatch(examStudents);
                }
                int studentCount = examStudents.size();

                if (studentCount - examDetail.getTotalSubjects() != 0) {
                    UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(ExamDetail::getTotalSubjects, studentCount).eq(ExamDetail::getId, examDetail.getId());
                    examDetailService.update(updateWrapper);
                }
                if (studentCount - examDetailCourse.getTotalSubjects() != 0) {
                    UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(ExamDetailCourse::getTotalSubjects, studentCount).eq(ExamDetailCourse::getId, examDetailCourse.getId());
                    examDetailCourseService.update(updateWrapper);
                }
            }
        } else if (ExamModelEnum.MODEL2.equals(basicExam.getExamModel())) {
            // 查询考试+课程下已用过的班级
            Map<String, String> usedClassNameMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(examDetailCourseList)) {
                examDetailCourseList.stream().filter(m -> StringUtils.isNotBlank(m.getClazzName())).forEach(m -> {
                    for (String className : m.getClazzName().split(",")) {
                        usedClassNameMap.put(className, String.format("%s(%s)", m.getRealName(), m.getLoginName()));
                    }
                });
            }

            List<String> errorClassNames = new ArrayList<>();
            if (StringUtils.isNotBlank(examDetailParams.getClassName()) && MapUtils.isNotEmpty(usedClassNameMap)) {
                for (String s : examDetailParams.getClassName().split(",")) {
                    if (usedClassNameMap.containsKey(s)) {
                        errorClassNames.add(s);
                    }
                }
            }

            // 选择学生时，不校验
            boolean containsPrivilege = teachcloudCommonService.containsPrivilege(sysUser.getId(), "button", "SelectStudent");
            if (!containsPrivilege || ExamCategoryEnum.REBUILD.equals(basicExam.getCategory())) {
                Map<String, Set<String>> finalMap = new HashMap<>();
                StringJoiner stringJoiner = new StringJoiner(",");
                for (String errorClassName : errorClassNames) {
                    String key = usedClassNameMap.get(errorClassName);
                    Set<String> value;
                    if (finalMap.containsKey(key)) {
                        value = finalMap.get(key);
                    } else {
                        value = new HashSet<>();
                    }
                    value.add(errorClassName);
                    finalMap.put(key, value);
                }
                for (Map.Entry<String, Set<String>> entry : finalMap.entrySet()) {
                    stringJoiner.add("考试对象[" + String.join(",", entry.getValue()) + "]已被[" + entry.getKey() + "]选择");
                }
                if (StringUtils.isNotBlank(stringJoiner.toString())) {
                    throw ExceptionResultEnum.ERROR.exception(stringJoiner.toString());
                }
            }

            ExamDetail examDetail = new ExamDetail();
            examDetail.setId(SystemConstant.getDbUuid());
            examDetail.setPackageCode("1" + redisCounterUtil.getCounter(schoolId, 6, "packageCode"));
            examDetail.setSchoolId(schoolId);
            examDetail.setOrgId(sysUser.getOrgId());
            examDetail.setPrintPlanId(examPrintPlan.getId());
            examDetail.setPrintPlanName(examPrintPlan.getName());
            examDetail.setExamId(examPrintPlan.getExamId());
            examDetail.setStatus(ExamDetailStatusEnum.NEW);
            if (!ExamCategoryEnum.REBUILD.equals(basicExam.getCategory())) {
                examDetail.setExamStartTime(examDetailParams.getExamStartTime());
                examDetail.setExamEndTime(examDetailParams.getExamEndTime());
            }
            examDetail.setTotalSubjects(examDetailParams.getTotalSubjects());
            examDetail.setExamDataSource(ExamDataSourceEnum.EXAM_TASK);
            examDetail.setNormal(true);
            examDetail.setPrintHouseId(examDetailParams.getPrintHouseId());
            examDetail.setBackupCount(examDetailParams.getBackupCount());
            examDetail.setCreateId(sysUser.getId());
            examDetail.setCreateTime(System.currentTimeMillis());
            examDetailService.save(examDetail);

            ExamDetailCourse examDetailCourse = new ExamDetailCourse();
            examDetailCourse.setId(SystemConstant.getDbUuid());
            examDetailCourse.setSchoolId(schoolId);
            examDetailCourse.setExamDetailId(examDetail.getId());
            examDetailCourse.setCourseId(examTask.getCourseId());
            examDetailCourse.setPaperNumber(paperNumber);
            examDetailCourse.setCoursePaperId(String.valueOf(examTask.getId()));
            examDetailCourse.setClazzName(examDetailParams.getClassName());
            examDetailCourse.setTotalSubjects(examDetailParams.getTotalSubjects());
            examDetailCourse.setCreateId(sysUser.getId());
            examDetailCourseService.save(examDetailCourse);
        }
    }

    @Override
    public ExamTaskApprovalFormDto findExamTaskApprovalForm(Long examTaskId, Long examId, String paperNumber) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 1.basicInfo 部分
        ExamTask examTask = null;
        if (examTaskId != null) {
            examTask = this.getById(examTaskId);
        } else if (examId != null && StringUtils.isNotBlank(paperNumber)) {
            examTask = this.getByExamIdAndCourseIdAndPaperNumber(examId, null, paperNumber);
        } else {
            throw ExceptionResultEnum.ERROR.exception("传入参数有误");
        }
        if (Objects.isNull(examTask)) {
            throw ExceptionResultEnum.ERROR.exception("命题任务不存在");
        }
        String paperName = basicSemesterService.getById(basicExamService.getById(examTask.getExamId()).getSemesterId()).getName();
        Long teachingRoomId = examTask.getTeachingRoomId();
        SysOrg teachingRoom = sysOrgService.getOne(new QueryWrapper<SysOrg>().lambda().eq(SysOrg::getId, teachingRoomId));
        if (Objects.isNull(teachingRoom)) {
            throw ExceptionResultEnum.ERROR.exception("找不到教研室");
        }
        String teachingRoomName = teachingRoom.getName();

        // 查询教研室所在学院
        List<SysOrg> orgList = sysOrgService.findParentsByOrgId(teachingRoom.getId());
        List<SysOrg> schoolOrgList = orgList.stream().filter(e -> OrgTypeEnum.SCHOOL.equals(e.getType())).collect(Collectors.toList());
        if (schoolOrgList.size() != 1) {
            throw ExceptionResultEnum.ERROR.exception("学校级机构信息异常");
        }
        SysOrg schoolOrg = schoolOrgList.get(0);

        // 流程id
        Long flowId = examTask.getFlowId();

        // 3.ApprovalInfo部分
        Boolean review = examTask.getReview();
        List<ApprovalInfo> approvalInfoList = new ArrayList<>();
        if (review) {
            approvalInfoList = tfFlowLogService.findApprovalInfoByFlowId(flowId);
        }

        // 2.ExamRoomInfo 部分
        List<ExamPackageDetail> examPackageDetailDatasource = this.baseMapper.findExamPackageDetailByPaperNumber(schoolId, examTask.getExamId(), examTask.getCourseId(), examTask.getPaperNumber(), null);
        if (examPackageDetailDatasource.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("缺少该命题任务对应的考务数据");
        }
        ExamObjectDto examObjectDto = this.buildExamObjectDtoByPackageDatasource(examPackageDetailDatasource);

        // ---- 组装Dto
        ExamTaskApprovalFormDto examTaskApprovalFormDto = new ExamTaskApprovalFormDto();
        BasicInfo basicInfo = new BasicInfo();
        basicInfo.setTeachingRoomId(teachingRoomId);
        basicInfo.setTeachingRoomName(teachingRoomName);
        basicInfo.setPaperName(paperName);
        basicInfo.setPaperNumber(examTask.getPaperNumber());
        basicInfo.setTeacherName(examTask.getTeacherName());
        basicInfo.setExamStartTime(examObjectDto.getExamStartTime());
        basicInfo.setExamEndTime(examObjectDto.getExamEndTime());
        BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());

        if (basicCourse != null) {
            basicInfo.setCourseName(basicCourse.getName());
            SysOrg sysOrg = sysOrgService.getById(basicCourse.getTeachingRoomId());
            basicInfo.setCollegeId(sysOrg != null ? sysOrg.getId() : null);
            basicInfo.setCollegeName(sysOrg != null ? sysOrg.getName() : null);
        }

        examTaskApprovalFormDto.setBasicInfo(basicInfo);
        examTaskApprovalFormDto.setExamRoomInfoList(examObjectDto.getExamRoomInfoList());
        examTaskApprovalFormDto.setApprovalInfoList(approvalInfoList);
        return examTaskApprovalFormDto;
    }

    @Transactional
    @Override
    public void findExamTaskApprovalPdf(HttpServletResponse response, Long examTaskId, String htmlContext) throws Exception {
        ExamTask examTask = this.getById(examTaskId);
        if (Objects.isNull(examTask)) {
            throw ExceptionResultEnum.ERROR.exception("未找到命题任务");
        }
        String fileName = examTask.getPaperNumber() + "审批表";
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        Long approveFormAttachmentId = examTask.getApproveFormAttachmentId();
        BasicAttachment basicAttachment = basicAttachmentService.getById(approveFormAttachmentId);
        if (Objects.isNull(basicAttachment)) {
            // 命题任务试卷审批表字段为空 -> 保存html并生成pdf附件
            basicAttachment = printCommonService.saveAttachmentHtmlAndPdf(fileName, htmlContext, sysUser.getId());
            examTask.setApproveFormAttachmentId(basicAttachment.getId());
            this.updateById(examTask);
        }
        JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
        String pdfPath = (String) jsonObject.get(SystemConstant.PDF_PATH);
        String type = (String) jsonObject.get(SystemConstant.TYPE);
        InputStream inputStream = null;
        if (Objects.equals(type, SystemConstant.OSS)) {
            inputStream = fileStoreUtil.ossDownloadIs(pdfPath, fileStoreUtil.getUploadEnumByPath(pdfPath).getFssType());
        } else {
            inputStream = new FileInputStream(new File(pdfPath));
        }
        ConvertUtil.outputFile(response, inputStream, fileName + SystemConstant.PDF_PREFIX);
    }

    @Transactional
    @Override
    public ExamObjectDto findExamObjectDtoByPaperNumber(Long examId, String paperNumber) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        ExamTask examTask = examTaskService.getOne(new QueryWrapper<ExamTask>().lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getExamId, examId).eq(ExamTask::getPaperNumber, paperNumber));
        if (Objects.isNull(examTask)) {
            throw ExceptionResultEnum.ERROR.exception("未找到命题任务");
        }
        List<ExamPackageDetail> examPackageDetailDatasource = this.baseMapper.findExamPackageDetailByPaperNumber(schoolId, examTask.getExamId(), examTask.getCourseId(), paperNumber, null);
        ExamObjectDto examObjectDto = this.buildExamObjectDtoByPackageDatasource(examPackageDetailDatasource);

        examObjectDto.setPaperName(examTask.getPaperName());
        examObjectDto.setPaperNumber(examTask.getPaperNumber());
        return examObjectDto;
    }

    /**
     * 根据流程id查找examtask
     *
     * @param flowId
     * @return
     */
    @Override
    public ExamTask findByFlowId(Long flowId) {
        ExamTask examTask = this.baseMapper.findByFlowId(flowId);
        Optional.ofNullable(examTask).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("命题计划为空"));
        return examTask;
    }

    @Transactional
    @Override
    public Map<String, Object> applyResave(ExamTaskReApplyParam examTaskReApplyParam) {
        ExamTaskDetail examTaskDetail = examTaskReApplyParam.getExamTaskDetail();
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getPaperAttachmentIds, examTaskDetail.getPaperAttachmentIds()).eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
        examTaskDetailService.update(updateWrapper);

        String flowTaskId = examTaskReApplyParam.getFlowTaskId();

        Map<String, Object> map = new HashMap<>();
        map.putIfAbsent(SystemConstant.FLOW_TASK_ID, flowTaskId);
        map.computeIfAbsent(SystemConstant.PAPAER_ATTACHMENT_IDS, v -> examTaskDetail.getPaperAttachmentIds());
        map.computeIfAbsent(SystemConstant.APPROVE_OPERATION, v -> FlowApprovePassEnum.PASS);
        activitiService.taskApprove(map);
        return map;
    }

    @Override
    public ExamTaskDto getByExamTaskId(Long examTaskId) {
        return this.baseMapper.getByExamTaskId(examTaskId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<ExamTaskStudentObjectResult> findExamTaskStudentObject(Long examId, Long courseId, String teacher, String teachClassName, String className, String paperNumber) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<ExamTaskStudentObjectResult> result = new ArrayList<>();
        // 基础班级对象
        BasicExamStudentQuery basicExamStudentQuery = new BasicExamStudentQuery();
        basicExamStudentQuery.setSchoolId(schoolId);
        basicExamStudentQuery.setExamId(examId);
        basicExamStudentQuery.setCourseId(courseId);
        basicExamStudentQuery.setTeacherName(teacher);
        basicExamStudentQuery.setTeachClassName(teachClassName);
        basicExamStudentQuery.setClassName(className);
        basicExamStudentQuery.setPaperNumber(paperNumber);
        List<BasicExamStudentResult> basicStudentResultList = basicExamStudentService.listByParams(basicExamStudentQuery);
        if (!CollectionUtils.isEmpty(basicStudentResultList)) {
            LinkedMultiValueMap<String, BasicExamStudentResult> studentInfoLinkedMultiValueMap = new LinkedMultiValueMap<>();
            Map<String, ExamTaskStudentObjectResult> examTaskStudentObjectResultMap = new LinkedHashMap<>();
            for (BasicExamStudentResult e : basicStudentResultList) {
                String studentTeachClassName = StringUtils.isNotBlank(e.getTeachClassName()) ? e.getTeachClassName() : "空";
                // 没有班级的考生，不返回
                studentInfoLinkedMultiValueMap.add(studentTeachClassName, e);

                ExamTaskStudentObjectResult examTaskStudentObjectResult;
                if (!examTaskStudentObjectResultMap.containsKey(studentTeachClassName)) {
                    examTaskStudentObjectResult = new ExamTaskStudentObjectResult();
                    examTaskStudentObjectResult.setTeachClassName(studentTeachClassName);
                } else {
                    examTaskStudentObjectResult = examTaskStudentObjectResultMap.get(studentTeachClassName);
                }
                examTaskStudentObjectResult.setStudentInfoList(studentInfoLinkedMultiValueMap.get(studentTeachClassName));
                examTaskStudentObjectResultMap.put(studentTeachClassName, examTaskStudentObjectResult);
            }
            result.addAll(examTaskStudentObjectResultMap.values());
        }
        // 按教学班排序
        result.sort(Comparator.comparing(ExamTaskStudentObjectResult::getTeachClassName));
        return result;
    }

    @Override
    public void delete(List<Long> idList) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        List<SysRole> sysRoleList = sysUserRoleService.listRoleByUserId(sysUser.getId());
        boolean isSchoolAdmin = false;
        for (SysRole sysRole : sysRoleList) {
            if (RoleTypeEnum.SCHOOL_ADMIN.equals(sysRole.getType())) {
                isSchoolAdmin = true;
                break;
            }
        }
        List<String> errorMsg = new ArrayList<>();
        for (Long id : idList) {
            ExamTask examTask = this.getById(id);
            if (Objects.isNull(examTask)) {
                throw ExceptionResultEnum.ERROR.exception("系统异常： 未找到命题任务");
            }
            String paperNumber = examTask.getPaperNumber();
            String prefix = String.format("试卷编号为[%s]的命题任务删除失败, ", paperNumber);
            if (isSchoolAdmin) {
                // 学校管理员，可以删除
                QueryWrapper<TFFlowApprove> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(TFFlowApprove::getFlowId, examTask.getFlowId());
                TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(queryWrapper);
                //审核状态
                if (tfFlowApprove != null && !tfFlowApprove.getStatus().equals(FlowStatusEnum.START) && !tfFlowApprove.getStatus().equals(FlowStatusEnum.REJECT) && !tfFlowApprove.getStatus()
                        .equals(FlowStatusEnum.CANCEL)) {
                    errorMsg.add(prefix + tfFlowApprove.getStatus().getTitle() + "状态无法删除");
                    continue;
                }
                // 删除命题任务及试卷信息
                this.removeByExamTaskId(id);
                // 删除考场信息
                examDetailService.deleteByExamIdAndPaperNumber(examTask.getSchoolId(), examTask.getExamId(), examTask.getPaperNumber());
            } else {
                // 命题任务生成，命题老师未提交前允许删除
                if (examTask.getStatus().equals(ExamStatusEnum.SUBMIT)) {
                    errorMsg.add(prefix + "命题老师已提交，不能删除");
                    continue;
                }
                this.removeByExamTaskId(id);
            }
        }
        if (CollectionUtils.isNotEmpty(errorMsg)) {
            // 报错
            throw ExceptionResultEnum.ERROR.exception(String.join(";\n", errorMsg));
        }
    }

    @Transactional
    @Override
    public void applyDelete(Long id) {
        ExamTask examTask = this.getById(id);
        QueryWrapper<TFFlowApprove> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TFFlowApprove::getFlowId, examTask.getFlowId());
        TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(queryWrapper);
        //审核状态
        if (tfFlowApprove != null && !tfFlowApprove.getStatus().equals(FlowStatusEnum.START) && !tfFlowApprove.getStatus().equals(FlowStatusEnum.REJECT) && !tfFlowApprove.getStatus().equals(FlowStatusEnum.CANCEL)) {
            throw ExceptionResultEnum.ERROR.exception(tfFlowApprove.getStatus().getTitle() + "状态无法删除");
        }
        // 删除命题任务及试卷信息
        this.removeByExamTaskId(id);
        // 删除题库试卷包信息
        examTaskPaperDataService.deleteByExamIdAndPaperNumber(examTask.getExamId(), examTask.getPaperNumber());
        // 删除考场信息
        examDetailService.deleteByExamIdAndPaperNumber(examTask.getSchoolId(), examTask.getExamId(), examTask.getPaperNumber());
    }

    @Override
    public boolean savePreviewLog(Long examTaskId, String paperType, String type) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        ExamTask examTask = this.getById(examTaskId);
        if (examTask != null) {
            BasicOperationLog basicOperationLog = new BasicOperationLog();
            basicOperationLog.setId(SystemConstant.getDbUuid());
            basicOperationLog.setSchoolId(sysUser.getSchoolId());
            BasicExam basicExam = basicExamService.getById(examTask.getExamId());
            String examName = basicExam == null ? "-" : basicExam.getName();
            BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
            String courseName = basicCourse != null ? basicCourse.getName() + "(" + basicCourse.getCode() + ")" : "";
            basicOperationLog.setDetail(String.format("考试[%s]，课程[%s]，试卷编号[%s]，题卡/试卷[%s]，卷型[%s]", examName, courseName, examTask.getPaperNumber(), type.equals("paper") ? "试卷" : "题卡", paperType));
            basicOperationLog.setOperationType(OperationTypeEnum.PREVIEW);
            basicOperationLog.setPrivilegeId(ServletUtil.getCurrentPrivilegeId());
            basicOperationLog.setRunStatus("成功");
            basicOperationLog.setUserId(sysUser.getId());
            basicOperationLog.setIp(ServletUtil.getIP());
            basicOperationLog.setCreateTime(System.currentTimeMillis());
            basicOperationLogService.save(basicOperationLog);
        }
        return true;
    }

    @Override
    public List<String> listPaperNumber(String param, List<Long> printPlanIdList) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        SysOrg sysOrg = sysOrgService.getById(sysUser.getOrgId());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        if (printPlanIdList.size() == 0) {
            // 查询命题任务中的试卷编号
            QueryWrapper<ExamTask> queryWrapperExamTask = new QueryWrapper<>();
            queryWrapperExamTask.lambda().eq(ExamTask::getSchoolId, schoolId);
            if (sysOrg != null && !OrgTypeEnum.PRINTING_HOUSE.equals(sysOrg.getType()) && orgIds.size() > 0) {
                queryWrapperExamTask.lambda().in(ExamTask::getOrgId, orgIds);
            }
            List<String> list = new ArrayList<>();
            if (StringUtils.isNotBlank(param)) {
                queryWrapperExamTask.lambda().and(i -> i.like(ExamTask::getCourseCode, SystemConstant.translateSpecificSign(param)).or().like(ExamTask::getCourseName, SystemConstant.translateSpecificSign(param)));
            }
            List<ExamTask> examTasks = this.list(queryWrapperExamTask);
            if (examTasks != null && examTasks.size() > 0) {
                list = examTasks.stream().map(ExamTask::getPaperNumber).distinct().collect(Collectors.toList());
            }

            // 查询考务-科目中的试卷编号
            QueryWrapper<ExamDetailCourse> queryWrapperExamDetailCourse = new QueryWrapper<>();
            queryWrapperExamDetailCourse.lambda().eq(ExamDetailCourse::getSchoolId, schoolId);
            List<String> list2 = new ArrayList<>();
            if (StringUtils.isNotBlank(param)) {
                queryWrapperExamDetailCourse.lambda().and(i -> i.like(ExamDetailCourse::getCourseCode, SystemConstant.translateSpecificSign(param))
                        .or().like(ExamDetailCourse::getCourseName, SystemConstant.translateSpecificSign(param)));
            }
            List<ExamDetailCourse> ExamDetailCourses = examDetailCourseService.list(queryWrapperExamDetailCourse);
            if (ExamDetailCourses != null && ExamDetailCourses.size() > 0) {
                list2 = ExamDetailCourses.stream().map(ExamDetailCourse::getPaperNumber).distinct().collect(Collectors.toList());
            }
            list.addAll(list2);
            return list.stream().filter(m -> !m.endsWith("作废")).distinct().collect(Collectors.toList());
        } else {
            List<String> paperNumberList = examDetailCourseService.listPaperNumberByPrintPlanId(param, printPlanIdList, orgIds);
            return paperNumberList.stream().filter(m -> !m.endsWith("作废")).distinct().collect(Collectors.toList());
        }
    }

    @Override
    public ExamTask getByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamTask> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(ExamTask::getExamId, examId).eq(ExamTask::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public ExamTask getByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamTask> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(ExamTask::getExamId, examId);
        if (courseId != null) {
            lambdaQueryWrapper.eq(ExamTask::getCourseId, courseId);
        }
        lambdaQueryWrapper.eq(ExamTask::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public IPage<MakeupExamTaskDto> listMakeupExamTask(Long printPlanId, Long semesterId, Long examId, Long collegeId, Long courseId, String paperNumber, String userName, String campusName, String className, Integer pageNumber, Integer pageSize) {
        if (semesterId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择学期");
        }
        if (examId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择考试");
        }
        Page<MakeupExamTaskDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listMakeupExamTask(page, printPlanId, semesterId, examId, collegeId, courseId, paperNumber, userName, campusName, className);
    }

    @Override
    public void saveMakeupTask(MakeupExamTaskTotalParam makeupExamTaskTotalParam) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(makeupExamTaskTotalParam.getPrintPlanId());

        for (MakeupExamTaskParam makeupExamTaskParam : makeupExamTaskTotalParam.getList()) {
            ExamDetail examDetail;
            if (makeupExamTaskParam.getExamDetailId() == null) {
                // 课程是否在其它补考计划已创建任务
                int count = examDetailService.countMakeupUsedByExamIdAndPaperNumber(makeupExamTaskParam.getExamId(), makeupExamTaskParam.getPaperNumber());
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception("试卷编号[" + makeupExamTaskParam.getPaperNumber() + "]已在其它计划中提交");
                }

                examDetail = new ExamDetail();
                examDetail.setId(SystemConstant.getDbUuid());
                examDetail.setPackageCode("1" + redisCounterUtil.getCounter(schoolId, 6, "packageCode"));
                examDetail.setSchoolId(schoolId);
                examDetail.setOrgId(sysUser.getOrgId());
                examDetail.setPrintPlanId(examPrintPlan.getId());
                examDetail.setPrintPlanName(examPrintPlan.getName());
                examDetail.setExamId(makeupExamTaskParam.getExamId());
                examDetail.setStatus(ExamDetailStatusEnum.NEW);
                examDetail.setExamStartTime(DateDisposeUtils.parseDate(DateDisposeUtils.getDate() + " 00:00:00").getTime());
                examDetail.setExamEndTime(DateDisposeUtils.parseDate(DateDisposeUtils.getDate() + " 23:59:00").getTime());
                examDetail.setTotalSubjects(makeupExamTaskParam.getTotalSubjects());
                examDetail.setExamDataSource(ExamDataSourceEnum.EXAM_TASK);
                examDetail.setNormal(true);
                examDetail.setBackupCount(makeupExamTaskParam.getBackupCount());
                examDetail.setCreateId(sysUser.getId());
                examDetail.setCreateTime(System.currentTimeMillis());
                examDetailService.save(examDetail);

                ExamDetailCourse examDetailCourse = new ExamDetailCourse();
                examDetailCourse.setId(SystemConstant.getDbUuid());
                examDetailCourse.setSchoolId(schoolId);
                examDetailCourse.setExamDetailId(examDetail.getId());
                examDetailCourse.setCourseId(makeupExamTaskParam.getCourseId());
                examDetailCourse.setPaperNumber(makeupExamTaskParam.getPaperNumber());
                examDetailCourse.setTotalSubjects(makeupExamTaskParam.getTotalSubjects());
                examDetailCourse.setCreateId(sysUser.getId());
                examDetailCourseService.save(examDetailCourse);
            } else {
                examDetail = examDetailService.getById(makeupExamTaskParam.getExamDetailId());
                if (Objects.nonNull(examDetail.getTotalSubjects()) && Objects.nonNull(makeupExamTaskParam.getTotalSubjects()) && examDetail.getTotalSubjects().intValue() != makeupExamTaskParam.getTotalSubjects().intValue()) {
                    examDetail.setTotalSubjects(makeupExamTaskParam.getTotalSubjects());
                    examDetail.setStatus(ExamDetailStatusEnum.NEW);
                    examDetailService.updateById(examDetail);

                    UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(ExamDetailCourse::getTotalSubjects, makeupExamTaskParam.getTotalSubjects())
                            .eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
                    examDetailCourseService.update(updateWrapper);
                }
            }

            // 生成pdf
            printCommonService.checkDataMakeup(schoolId, examDetail, makeupExamTaskParam.getCourseId(), makeupExamTaskParam.getPaperNumber(), sysUser);
        }
    }

    @Override
    public IPage<TikuPaperInfo> pageTikuPaper(Long courseId, String paperName, String account, Integer pageNumber, Integer pageSize) {
        if (courseId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择课程");
        }
        BasicCourse basicCourse = basicCourseService.getById(courseId);
        if (basicCourse == null) {
            throw ExceptionResultEnum.ERROR.exception("课程不存在");
        }
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return tikuUtils.pagePaperInfo(schoolId, basicCourse.getCode(), paperName, account, pageNumber, pageSize);
    }

    @Override
    public Map<String, Object> getTikuPaperData(Long examId, Long paperId, String uuid) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Map<String, Object> map = new HashMap<>();
        try {
            File paperDataFile = tikuUtils.getTikuPaperData(schoolId, paperId);
            ExamTaskPaperData examTaskPaperData = saveTikuPaperData(examId, paperId, uuid, paperDataFile);
            map.put("paperJson", examTaskPaperData.getPaperJson());
            map.put("answerJson", examTaskPaperData.getAnswerJson());
            map.put("attachmentId", String.valueOf(examTaskPaperData.getPaperPdfAttachmentId()));
            map.put("paperUrl", examTaskPaperData.getPaperPdfAttachmentId() != null ? fileUploadService.filePreview(examTaskPaperData.getPaperPdfAttachmentId()) : null);
            return map;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    @Transactional
    @Override
    public List<ExamDetail> matchBasicExamStudent(BasicExam basicExam, ExamTask examTask, SysUser sysUser) {
        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.listByExamIdAndCourseIdAndPaperNumber(examTask.getExamId(), examTask.getCourseId(), examTask.getPaperNumber());
        if (CollectionUtils.isEmpty(basicExamStudentList)) {
            return null;
        }
        // 创建印刷计划
        QueryWrapper<ExamPrintPlan> printPlanQueryWrapper = new QueryWrapper<>();
        printPlanQueryWrapper.lambda().eq(ExamPrintPlan::getExamId, basicExam.getId()).eq(ExamPrintPlan::getName, examTask.getCourseName() + examTask.getPaperNumber());
        ExamPrintPlan examPrintPlan = examPrintPlanService.getOne(printPlanQueryWrapper);
        if (examPrintPlan != null) {
            return null;
        }

        examPrintPlan = new ExamPrintPlan();
        examPrintPlan.insertInfo(sysUser.getId());
        examPrintPlan.setSchoolId(sysUser.getSchoolId());
        examPrintPlan.setExamId(examTask.getExamId());
        examPrintPlan.setOrgId(sysUser.getOrgId());
        BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
        examPrintPlan.setName((basicCourse != null ? basicCourse.getName() : "") + examTask.getPaperNumber());
        BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(basicExam.getId());
        List<String> stringList = JSONObject.parseArray(basicPrintConfig.getPrintContent(), String.class);
        if (!stringList.contains("PAPER")) {
            List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetailService.listByExamTaskId(examTask.getId()));
            long count = paperInfoVoList.stream().filter(m -> Objects.isNull(m.getAttachmentId())).count();
            if (count == 0) {
                stringList.add("PAPER");
            }
        }
        examPrintPlan.setPrintContent(String.join(",", stringList));
        examPrintPlan.setBackupMethod(BackupMethodEnum.valueOf(basicPrintConfig.getBackupMethod()));
        examPrintPlan.setBackupCount(basicPrintConfig.getBackupCount());
        examPrintPlan.setDrawRule(basicPrintConfig.getDrawRule());
        examPrintPlan.setVariableContent(basicPrintConfig.getVariableContent());
        examPrintPlan.setOrdinaryContent(basicPrintConfig.getOrdinaryContent());
        examPrintPlan.setCategory(ExamCategoryEnum.FORMAL);
        examPrintPlan.setStatus(PrintPlanStatusEnum.READY);
        examPrintPlanService.save(examPrintPlan);

        List<Map<String, Object>> examDetailKeyList = basicExamStudentList.stream().flatMap(e -> {
            Long schoolIdId = e.getSchoolId();
            Map<String, Object> map = new HashMap<>();
            map.put("schoolId", schoolIdId);
            map.put("examPlace", e.getExamPlace());
            map.put("examRoom", e.getExamRoom());
            map.put("examStartTime", e.getExamStartTime());
            map.put("examEndTime", e.getExamEndTime());
            return Stream.of(map);
        }).distinct().collect(Collectors.toList());

        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(sysUser.getSchoolId());
        Optional<EnumResult> optional = basicExamRule.getRequiredFieldList().stream().filter(m -> RequiredFieldsEnum.EXAM_PLACE.getCode().equals(m.getCode())).findFirst();

        List<ExamDetail> examDetailList = new ArrayList<>();
        for (Map<String, Object> map : examDetailKeyList) {
            Long schoolId = Long.valueOf(String.valueOf(map.get("schoolId")));
            String examPlace = String.valueOf(map.get("examPlace"));
            String examRoom = String.valueOf(map.get("examRoom"));
            Long examStartTime = Long.valueOf(String.valueOf(map.get("examStartTime")));
            Long examEndTime = Long.valueOf(String.valueOf(map.get("examEndTime")));

            List<BasicExamStudent> examDetailStudentList;
            if (optional.isPresent() && optional.get().getEnable()) {
                examDetailStudentList = basicExamStudentList.stream().filter(e -> e.getExamPlace().equals(examPlace) &&
                                e.getExamRoom().equals(examRoom) && e.getExamStartTime().equals(examStartTime) && e.getExamEndTime().equals(examEndTime))
                        .collect(Collectors.toList());
            } else {
                examDetailStudentList = basicExamStudentList.stream().filter(e ->
                                e.getExamRoom().equals(examRoom) && e.getExamStartTime().equals(examStartTime) && e.getExamEndTime().equals(examEndTime))
                        .collect(Collectors.toList());
            }

            ExamDetail examDetail = new ExamDetail();
            examDetail.setId(SystemConstant.getDbUuid());
            examDetail.setPackageCode("1" + redisCounterUtil.getCounter(schoolId, 6, "packageCode"));
            examDetail.setSchoolId(schoolId);
            examDetail.setOrgId(sysUser.getOrgId());
            examDetail.setPrintPlanId(examPrintPlan.getId());
            examDetail.setPrintPlanName(examPrintPlan.getName());
            examDetail.setExamId(examPrintPlan.getExamId());
            examDetail.setExamPlace(examPlace);
            examDetail.setExamRoom(examRoom);
            examDetail.setStatus(ExamDetailStatusEnum.NEW);
            examDetail.setTotalSubjects(examDetailStudentList.size());
            examDetail.setExamStartTime(examStartTime);
            examDetail.setExamEndTime(examEndTime);
            examDetail.setExamDataSource(ExamDataSourceEnum.FILE_IMPORT);
            examDetail.setNormal(true);
            examDetail.setCreateId(sysUser.getId());
            examDetail.setBackupCount(examPrintPlan.getBackupCount());
            examDetailService.save(examDetail);

            //组装exam_detail_course数据
            List<Map<String, Object>> examDetailCourseKeyList = examDetailStudentList.stream().flatMap(e -> {
                Map<String, Object> examDetailCourseMap = new HashMap<>();
                examDetailCourseMap.put("courseId", e.getCourseId());
                examDetailCourseMap.put("paperNumber", e.getPaperNumber());
                return Stream.of(examDetailCourseMap);
            }).distinct().collect(Collectors.toList());


            for (Map<String, Object> examDetailCourseMap : examDetailCourseKeyList) {
                Long courseId = Long.valueOf(String.valueOf(examDetailCourseMap.get("courseId")));
                String paperNumber = String.valueOf(examDetailCourseMap.get("paperNumber"));

                // 获取该科目下的考生人数
                List<BasicExamStudent> examDetailCourseStudentList = examDetailStudentList.stream().filter(e -> e.getCourseId().equals(courseId) &&
                        e.getPaperNumber().equals(paperNumber)).collect(Collectors.toList());

                ExamDetailCourse examDetailCourse = new ExamDetailCourse();
                examDetailCourse.setId(SystemConstant.getDbUuid());
                examDetailCourse.setSchoolId(schoolId);
                examDetailCourse.setExamDetailId(examDetail.getId());
                examDetailCourse.setCourseId(courseId);
                examDetailCourse.setCourseName(examTask.getCourseName());
                examDetailCourse.setPaperNumber(paperNumber);
                examDetailCourse.setCoursePaperId(String.valueOf(examTask.getId()));
                examDetailCourse.setTotalSubjects(examDetailCourseStudentList.size());
                examDetailCourse.setCreateId(sysUser.getId());
                examDetailCourseService.save(examDetailCourse);

                //组装exam_student数据
                List<ExamStudent> examStudentList = new ArrayList<>();
                for (BasicExamStudent basicExamStudent : examDetailCourseStudentList) {
                    ExamStudent examStudent = new ExamStudent();
                    examStudent.setId(SystemConstant.getDbUuid());
                    examStudent.setSchoolId(schoolId);
                    examStudent.setExamId(examTask.getExamId());
                    examStudent.setBasicStudentId(basicExamStudent.getId());
                    examStudent.setStudentCode(basicExamStudent.getStudentCode());
                    examStudent.setExamDetailCourseId(examDetailCourse.getId());
                    examStudent.setPaperNumber(paperNumber);
                    examStudent.setCoursePaperId(examDetailCourse.getCoursePaperId());
                    examStudent.setCreateId(sysUser.getId());
                    examStudentList.add(examStudent);
                }
                examStudentService.saveBatch(examStudentList);
                examDetailList.add(examDetail);
            }
        }
        return examDetailList;
    }

    @Override
    public List<TaskPrintHouseDto> listPrintHouse() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<SysOrg> sysOrgList = sysOrgService.listPrintHouse(schoolId, OrgTypeEnum.PRINTING_HOUSE);
        return sysOrgList.stream().map(m -> {
            TaskPrintHouseDto taskPrintHouseDto = new TaskPrintHouseDto();
            taskPrintHouseDto.setPrintHouseId(m.getId());
            taskPrintHouseDto.setPrintHouseName(m.getName());
            return taskPrintHouseDto;
        }).collect(Collectors.toList());
    }

    @Override
    public ExamTask getByIdIncludeCourse(Long examTaskId) {
        ExamTask examTask = this.getById(examTaskId);
        if (examTask != null && examTask.getCourseId() != null) {
            BasicCourse basicCourse = basicCourseService.getById(examTask.getCourseId());
            if (basicCourse != null) {
                examTask.setCourseCode(basicCourse.getCode());
                examTask.setCourseName(basicCourse.getName());
            }
        }
        return examTask;
    }

    @Override
    public List<ExamTask> listByExamId(Long examId) {
        return this.list(new QueryWrapper<ExamTask>().lambda().eq(ExamTask::getExamId, examId));
    }

    @Override
    public ExamTask validExamTaskAuditPass(Long schoolId, Long examId, Long courseId, String paperNumber) {
        //查询命题任务绑定的试卷和题卡
        ExamTask examTask = examTaskService.getByExamIdAndCourseIdAndPaperNumber(examId, courseId, paperNumber);
        if (examTask == null) {
            throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]的" + ExceptionResultEnum.EXAM_TASK_IS_NOT_EXIST.getMessage(), paperNumber));
        } else {
            if (!examTask.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]的" + ExceptionResultEnum.EXAM_TASK_ENABLE.getMessage(), paperNumber));
            } else if (examTask.getReview()) {
                List<ExamTask> examTaskList = this.baseMapper.findExamTaskByFlowStatus(schoolId, examId, courseId, paperNumber, FlowStatusEnum.FINISH.name());
                if (CollectionUtils.isEmpty(examTaskList)) {
                    throw ExceptionResultEnum.ERROR.exception("试卷编号[" + paperNumber + "]的" + ExceptionResultEnum.EXAM_TASK_IS_NOT_EXIST.getMessage());
                } else if (examTaskList.size() != 1) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]", paperNumber) + ExceptionResultEnum.DATA_COUNT_EXCEPTION.getMessage());
                }
            } else {
                if (!ExamStatusEnum.SUBMIT.equals(examTask.getStatus())) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]的" + ExceptionResultEnum.EXAM_TASK_NOT_SUBMIT.getMessage(), paperNumber));
                }
            }
        }
        return examTask;
    }

    /**
     * 命题任务统计查询
     *
     * @param iPage
     * @param semesterId
     * @param examId
     * @param printPlanId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param finishStatus
     * @return
     */
    @Override
    public IPage<PrintCountResult> printCountList(IPage<Map> iPage, Long semesterId, Long examId, Long printPlanId, Long collegeId, Long courseId, String paperNumber, CardTypeEnum cardType, StatisticsStatusEnum finishStatus) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.printCountList(iPage, schoolId, semesterId, examId, printPlanId, collegeId, courseId, paperNumber, Objects.nonNull(cardType) ? cardType.name() : null, Objects.nonNull(finishStatus) ? finishStatus.name() : null, dpr);
    }

    /**
     * 命题任务统计导出
     *
     * @param semesterId
     * @param examId
     * @param printPlanId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @param finishStatus
     * @return
     */
    @Override
    public List<PrintCountResult> printCountListExport(Long semesterId, Long examId, Long printPlanId, Long collegeId, Long courseId, String paperNumber, CardTypeEnum cardType, StatisticsStatusEnum finishStatus) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.printCountListExport(schoolId, semesterId, examId, printPlanId, collegeId, courseId, paperNumber, Objects.nonNull(cardType) ? cardType.name() : null, Objects.nonNull(finishStatus) ? finishStatus.name() : null, dpr);
    }

    /**
     * 印刷任务统计查询
     *
     * @param iPage
     * @param semesterId
     * @param examId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @return
     */
    @Override
    public IPage<ScanCountResult> scanCountList(IPage<Map> iPage, Long semesterId, Long examId, Long collegeId, Long courseId, String paperNumber, CardTypeEnum cardType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ScanCountResult> scanCountResultIPage = this.baseMapper.scanCountList(iPage, schoolId, semesterId, examId, collegeId, courseId, paperNumber, Objects.nonNull(cardType) ? cardType.name() : null, dpr);
        if (Objects.nonNull(scanCountResultIPage) && !CollectionUtils.isEmpty(scanCountResultIPage.getRecords())) {
            for (ScanCountResult s : scanCountResultIPage.getRecords()) {
                ScanCalculateResult scanCalculateResult = this.baseMapper.scanCalculate(s.getExamId(), s.getCourseId(), s.getPaperNumber(), s.getStudentCount());
                if (Objects.nonNull(scanCalculateResult)) {
                    s.updateRateAndSheetCount(scanCalculateResult);
                }
            }
        }
        return scanCountResultIPage;
    }

    /**
     * 印刷任务统计导出
     *
     * @param semesterId
     * @param examId
     * @param collegeId
     * @param courseId
     * @param paperNumber
     * @param cardType
     * @return
     */
    @Override
    public List<ScanCountResult> scanCountListExport(Long semesterId, Long examId, Long collegeId, Long courseId, String paperNumber, CardTypeEnum cardType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        List<ScanCountResult> scanCountResultList = this.baseMapper.scanCountListExport(schoolId, semesterId, examId, collegeId, courseId, paperNumber, Objects.nonNull(cardType) ? cardType.name() : null, dpr);
        if (!CollectionUtils.isEmpty(scanCountResultList)) {
            for (ScanCountResult s : scanCountResultList) {
                ScanCalculateResult scanCalculateResult = this.baseMapper.scanCalculate(s.getExamId(), s.getCourseId(), s.getPaperNumber(), s.getStudentCount());
                if (Objects.nonNull(scanCalculateResult)) {
                    s.updateRateAndSheetCount(scanCalculateResult);
                }
            }
        }
        return scanCountResultList;
    }

    private ExamTaskPaperData saveTikuPaperData(Long examId, Long paperId, String uuid, File zipFile) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String zipDestPath = null;
        try {
            zipDestPath = zipFile.getParent() + File.separator + System.nanoTime();
            Zip4jUtil.unzip(zipFile.getPath(), zipDestPath);

            ExamTaskPaperData examTaskPaperData = examTaskPaperDataService.selectByMultiId(new ExamTaskPaperData(examId, uuid, paperId));
            if (examTaskPaperData == null) {
                examTaskPaperData = new ExamTaskPaperData();
                examTaskPaperData.setExamId(examId);
                // 唯一码，命题任务提交时，根据唯一码反向更新真实的试卷编号
                examTaskPaperData.setPaperNumber(uuid);
                examTaskPaperData.setUuid(uuid);
                examTaskPaperData.setPaperId(paperId);
            }
            // data.zip
            String zipPdfName = PdfUtil.md5FileName(examTaskPaperData.getUuid() + "data" + paperId) + SystemConstant.ZIP_PREFIX;
            FilePathVo filePathVoZip = fileUploadService.uploadFile(zipFile, UploadFileEnum.FILE, zipPdfName);
            examTaskPaperData.setDataPacketPath(JSON.toJSONString(filePathVoZip));
            // paper.json
            String paperJson = FileUtils.readFileToString(new File(zipDestPath + File.separator + "paper.json"), StandardCharsets.UTF_8);
            examTaskPaperData.setPaperJson(paperJson);
            // answer.json
            String answerJson = FileUtils.readFileToString(new File(zipDestPath + File.separator + "answer.json"), StandardCharsets.UTF_8);
            examTaskPaperData.setAnswerJson(answerJson);
            // paper.pdf
            String paperPdfName = PdfUtil.md5FileName(examTaskPaperData.getUuid() + "paper" + paperId) + SystemConstant.PDF_PREFIX;
            File paperPdfFile = new File(zipDestPath + File.separator + "paper.pdf");
            FilePathVo filePathVoPaper = fileUploadService.uploadFile(paperPdfFile, UploadFileEnum.PDF, paperPdfName);
            BasicAttachment paperPdfBasicAttachment = new BasicAttachment(JSON.toJSONString(filePathVoPaper), paperPdfName, SystemConstant.PDF_PREFIX, new BigDecimal(paperPdfFile.length()), filePathVoPaper.getMd5(), sysUser.getId());
            basicAttachmentService.save(paperPdfBasicAttachment);
            examTaskPaperData.setPaperPdfAttachmentId(paperPdfBasicAttachment.getId());
            // answer.pdf
            String answerPdfName = PdfUtil.md5FileName(examTaskPaperData.getUuid() + "answer" + paperId) + SystemConstant.PDF_PREFIX;
            File answerPdfFile = new File(zipDestPath + File.separator + "answer.pdf");
            FilePathVo filePathVoAnswer = fileUploadService.uploadFile(answerPdfFile, UploadFileEnum.PDF, answerPdfName);
            BasicAttachment answerBasicAttachment = new BasicAttachment(JSON.toJSONString(filePathVoAnswer), answerPdfName, SystemConstant.PDF_PREFIX, new BigDecimal(answerPdfFile.length()), filePathVoAnswer.getMd5(), sysUser.getId());
            basicAttachmentService.save(answerBasicAttachment);
            examTaskPaperData.setAnswerPdfAttachmentId(answerBasicAttachment.getId());
            examTaskPaperDataService.saveOrUpdateByMultiId(examTaskPaperData);
            return examTaskPaperData;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("获取试卷包数据失败");
        } finally {
            if (zipFile.exists()) {
                FileUtil.deleteFile(zipFile);
            }
            if (zipDestPath != null) {
                FileUtil.deleteDirectory(zipDestPath);
            }
        }

    }

    /**
     * 简单校验提交参数
     *
     * @param examTaskDetail
     * @param openAb
     */
    void validSubmitParam(ExamTaskDetail examTaskDetail, String printContent, Boolean openAb) {
        List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds());
        if (StringUtils.isBlank(examTaskDetail.getPaperType()) || CollectionUtils.isEmpty(paperInfoVoList)) {
            throw ExceptionResultEnum.ERROR.exception("请添加卷型并上传试卷或题卡");
        }

        for (PaperInfoVo paperInfoVo : paperInfoVoList) {
            // 校验试卷必须绑定题卡
            Long cardId = Long.valueOf(paperInfoVo.getCardId());
            if (printContent.indexOf("PAPER") > 0 && Objects.isNull(cardId)) {
                throw ExceptionResultEnum.ERROR.exception("卷型[" + paperInfoVo.getName() + "]没有绑定题卡");
            }
            // 校验题卡是否提交
            ExamCard examCard = examCardService.getById(cardId);
            if (!ExamCardStatusEnum.SUBMIT.equals(examCard.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("卷型[" + paperInfoVo.getName() + "]绑定的题卡未提交");
            }
        }

        String[] paperTypes = examTaskDetail.getPaperType().split(",");
        int paperCount = openAb != null && openAb ? 2 : 1;
        if (paperCount - paperTypes.length != 0) {
            throw ExceptionResultEnum.ERROR.exception(openAb ? "备用卷内必须有AB两个卷型" : "备用卷内只能有A卷");
        }

        // 是否强制包含试卷
        // 未上传试卷的类型个数
        long count = paperInfoVoList.stream().filter(m -> Objects.isNull(m.getAttachmentId())).count();
        if (printContent.contains("PAPER")) {
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("有部分卷型未上传试卷文件");
            }
        } else {
            // count == 0为全部上传，paperAttachmentIds.size() == count 为全部未上传
            if (count > 0 && paperInfoVoList.size() != count) {
                throw ExceptionResultEnum.ERROR.exception("所有卷型的试卷文件必须全部上传或全部不上传");
            }
        }
    }

    /**
     * 根据试卷袋信息构建考试对象信息
     *
     * @param examPackageDetailDatasource 试卷袋数据源
     * @return 考试对象
     */
    private ExamObjectDto buildExamObjectDtoByPackageDatasource(List<ExamPackageDetail> examPackageDetailDatasource) {
        Long examStartTime = null;
        Long examEndTime = null;
        int printTotalCount = 0;
        List<ExamRoomInfo> examRoomInfoList = new ArrayList<>();
        if (examPackageDetailDatasource.size() > 0) {
            Set<Long> examDetailIdSet = examPackageDetailDatasource.stream().map(ExamPackageDetail::getExamDetailId).collect(Collectors.toSet());
            for (Long examDetailId : examDetailIdSet) {
                // 一个考场下
                // 班级
                int studentCount = 0;
                String printCount = "";

                List<ExamPackageDetail> examPackageDetailCell = examPackageDetailDatasource.stream()
                        .filter(e -> examDetailId.equals(e.getExamDetailId()))
                        .collect(Collectors.toList());
                List<String> packageCodeList = examPackageDetailCell.stream().map(ExamPackageDetail::getPackageCode).distinct().collect(Collectors.toList());
                String packageCode = "";
                if (packageCodeList.size() > 0) {
                    packageCode = packageCodeList.get(0);
                }

                List<String> printHourseNameList = examPackageDetailCell.stream().map(ExamPackageDetail::getPrintHouseName).distinct().collect(Collectors.toList());
                String printHourseName = "";
                if (printHourseNameList.size() > 0) {
                    printHourseName = printHourseNameList.get(0);
                }

                List<BackupMethodEnum> backupMethodEnums = examPackageDetailCell.stream().map(ExamPackageDetail::getBackupMethod).distinct().collect(Collectors.toList());
                if (backupMethodEnums.size() != 1 && backupMethodEnums.get(0) != BackupMethodEnum.ROOM) {
                    throw ExceptionResultEnum.ERROR.exception("印刷计划试卷备份方式必须按考场");
                }
                List<Double> backupCountList = examPackageDetailCell.stream().map(ExamPackageDetail::getBackupCount).distinct().collect(Collectors.toList());
                if (backupCountList.size() != 1) {
                    throw ExceptionResultEnum.ERROR.exception("印刷计划试卷备份数量异常");
                }

                // 班级主键','隔开
                List<String> clazzNames = new ArrayList<>();
                String separator = ",";
                for (ExamPackageDetail examPackageDetail : examPackageDetailCell) {
                    // 处理考试开始结束时间
                    Long cellStartTime = examPackageDetail.getExamStartTime();
                    Long cellEndTime = examPackageDetail.getExamEndTime();
                    if (examStartTime == null || cellStartTime < examStartTime) {
                        examStartTime = cellStartTime;
                    }
                    if (examEndTime == null || cellEndTime > examEndTime) {
                        examEndTime = cellEndTime;
                    }
                    // 处理班级和数量
                    Integer studentCellCount = examPackageDetail.getExamDetailCourseTotalSubjects();
                    studentCount = studentCount + studentCellCount;
                    String clazzIdCell = examPackageDetail.getClassNames();
                    if (StringUtils.isNotBlank(clazzIdCell)) {
                        for (String s : clazzIdCell.split(separator)) {
                            clazzNames.add(s);
                        }
                    }
                }
                // 试卷该考场备份数量
                Integer backupCount = SystemConstant.calcBackupCount(backupCountList.get(0), studentCount, 1);
                printCount = studentCount + " + " + backupCount;
                printTotalCount = printTotalCount + studentCount + backupCount;

                // 组装examRoom
                ExamRoomInfo examRoomInfo = new ExamRoomInfo();
                examRoomInfo.setPackageCode(packageCode);
                examRoomInfo.setClazzNames(CollectionUtils.isNotEmpty(clazzNames) ? String.join(",", clazzNames) : "");
                examRoomInfo.setStudentCount(studentCount);
                examRoomInfo.setPrintCount(printCount);
                examRoomInfo.setPrintHouseName(printHourseName);
                examRoomInfoList.add(examRoomInfo);
            }
        }
        ExamObjectDto examObjectDto = new ExamObjectDto();
        examObjectDto.setExamEndTime(examEndTime);
        examObjectDto.setExamStartTime(examStartTime);
        examObjectDto.setExamRoomInfoList(examRoomInfoList);
        examObjectDto.setPrintTotalCount(printTotalCount);
        return examObjectDto;
    }
}
