package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamTaskWhole;
import com.qmth.distributed.print.business.enums.ExamTaskWholeStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskWholeMapper;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.business.service.ExamTaskWholeService;
import com.qmth.distributed.print.business.templete.execute.AsyncCreateWholePdfTemplateService;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.bean.vo.PrintPathVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.enums.CreatePdfTypeEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.util.FileUtil;
import com.qmth.teachcloud.mark.enums.LockType;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static java.lang.String.valueOf;

@Service
public class ExamTaskWholeServiceImpl extends ServiceImpl<ExamTaskWholeMapper, ExamTaskWhole> implements ExamTaskWholeService {

    private List<String> examTaskWholeList = new ArrayList<>();
    @Resource
    private BasicAttachmentService basicAttachmentService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private FileUploadService fileUploadService;
    @Resource
    private ConcurrentService concurrentService;
    @Resource
    private AsyncCreateWholePdfTemplateService asyncCreateWholePdfTemplateService;

    @Override
    public void checkAndCreateTask(Long examId, Long courseId, String paperNumber) {
        String key = examId + valueOf(courseId) + paperNumber;
        concurrentService.getReadWriteLock(LockType.EXAM_TASK_WHOLE + "-" + key).writeLock().lock();
        try {
            if (examTaskWholeList.contains(key)) {
                return;
            }
            examTaskWholeList.add(key);
            if (this.countByExamIdAndCourseIdAndPaperNumber(examId, paperNumber) > 1) {
                this.deleteByExamIdAndCourseIdAndPaperNumber(examId, paperNumber);
            }
            boolean isAllPrintFinish = examDetailService.allExamDetailIsExist(examId, courseId, paperNumber);
            ExamTaskWhole examTaskWhole = this.getByExamIdAndCourseIdAndPaperNumber(examId, paperNumber);
            if (isAllPrintFinish) {
                if (examTaskWhole != null) {
                    if (ExamTaskWholeStatusEnum.RUNNING.equals(examTaskWhole.getStatus())) {
                        throw ExceptionResultEnum.ERROR.exception("文件生成中，请稍后再试");
                    }
                    examTaskWhole.setAttachmentPath(null);
                    examTaskWhole.setStatus(ExamTaskWholeStatusEnum.INIT);
                    this.updateById(examTaskWhole);
                } else {
                    examTaskWhole = new ExamTaskWhole();
                    examTaskWhole.setId(SystemConstant.getDbUuid());
                    examTaskWhole.setExamId(examId);
                    examTaskWhole.setCourseId(courseId);
                    examTaskWhole.setPaperNumber(paperNumber);
                    examTaskWhole.setStatus(ExamTaskWholeStatusEnum.INIT);
                    examTaskWhole.setCreateTime(System.currentTimeMillis());
                    this.save(examTaskWhole);
                }
                // 开始生成
                asyncCreateWholePdfTemplateService.createWholePdf(examTaskWhole);

            } else {
                this.removeById(examTaskWhole.getId());
                throw ExceptionResultEnum.ERROR.exception("部分考场未生成PDF，无法合并文件");
            }
        } catch (ApiException e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        } finally {
            examTaskWholeList.remove(key);
            concurrentService.getReadWriteLock(LockType.EXAM_TASK_WHOLE + "-" + key).writeLock().unlock();
        }
    }

    @Override
    public ExamTaskWhole getByExamIdAndCourseIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<ExamTaskWhole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskWhole::getExamId, examId)
                .eq(ExamTaskWhole::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public int countByExamIdAndCourseIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<ExamTaskWhole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskWhole::getExamId, examId)
                .eq(ExamTaskWhole::getPaperNumber, paperNumber);
        return this.count(queryWrapper);
    }

    @Override
    public boolean deleteByExamIdAndCourseIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<ExamTaskWhole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskWhole::getExamId, examId)
                .eq(ExamTaskWhole::getPaperNumber, paperNumber);
        return this.remove(queryWrapper);
    }

    @Override
    public Map getWholePdf(Long examId, String paperNumber) {
        if (this.countByExamIdAndCourseIdAndPaperNumber(examId, paperNumber) > 1) {
            this.deleteByExamIdAndCourseIdAndPaperNumber(examId, paperNumber);
        }
        ExamTaskWhole examTaskWhole = this.getByExamIdAndCourseIdAndPaperNumber(examId, paperNumber);
        Map map = new HashMap();
        map.put("status", examTaskWhole != null ? examTaskWhole.getStatus() : ExamTaskWholeStatusEnum.INIT);
        map.put("url", examTaskWhole != null && StringUtils.isNotBlank(examTaskWhole.getAttachmentPath()) ? fileUploadService.filePreview(examTaskWhole.getAttachmentPath()) : null);
        return map;
    }

    @Override
    public void mergeWholePdf(ExamTaskWhole examTaskWhole) {
        List<ExamDetail> examDetailList = examDetailService.listExamDetailByExamIdAndCourseIdAndPaperNumber(examTaskWhole.getExamId(), examTaskWhole.getCourseId(), examTaskWhole.getPaperNumber());
        if (CollectionUtils.isNotEmpty(examDetailList)) {
            List<InputStream> inputStreamList = new ArrayList<>();
            for (ExamDetail examDetail : examDetailList) {
                String attachmentPath = examDetail.getAttachmentPath();
                if (StringUtils.isNotBlank(attachmentPath)) {
                    JSONObject jsonObject = JSONObject.parseObject(attachmentPath);
                    List<PrintPathVo> pathList = JSONObject.parseArray(jsonObject.getString("path"), PrintPathVo.class);
                    for (PrintPathVo printPathVo : pathList) {
                        InputStream inputStream = fileUploadService.downloadFile(printPathVo.getPdfPath(), printPathVo.getUploadType(), printPathVo.getType());
                        if (inputStream == null) {
                            throw ExceptionResultEnum.ERROR.exception("读取印品文件失败:" + printPathVo.getPdfPath());
                        }
                        inputStream = PdfUtil.addPdfEmptyPageForWholePdf(inputStream);
                        inputStreamList.add(inputStream);
                    }
                }

                // 试卷（试卷+备用试卷）
                Long attachmentId = examDetail.getAttachmentId();
                if (attachmentId != null) {
                    BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                    if (basicAttachment == null) {
                        throw ExceptionResultEnum.ERROR.exception("未找到试卷，附件ID：" + attachmentId);
                    }
                    if (StringUtils.isNotBlank(basicAttachment.getPath())) {
                        FilePathVo filePathVo = JSON.parseObject(basicAttachment.getPath(), FilePathVo.class);
                        InputStream inputStream = fileUploadService.downloadFile(filePathVo.getPath(), filePathVo.getUploadType(), filePathVo.getType());
                        if (inputStream == null) {
                            throw ExceptionResultEnum.ERROR.exception("读取试卷文件失败:" + filePathVo.getPath());
                        }
                        inputStreamList.add(inputStream);
                    }
                }

                // 题卡（题卡+备用题卡）
                Long cardAttachmentId = examDetail.getCardAttachmentId();
                if (cardAttachmentId != null) {
                    BasicAttachment basicAttachment = basicAttachmentService.getById(cardAttachmentId);
                    if (basicAttachment == null) {
                        throw ExceptionResultEnum.ERROR.exception("未找到题卡，附件ID：" + cardAttachmentId);
                    }
                    if (StringUtils.isNotBlank(basicAttachment.getPath())) {
                        FilePathVo filePathVo = JSON.parseObject(basicAttachment.getPath(), FilePathVo.class);
                        InputStream inputStream = fileUploadService.downloadFile(filePathVo.getPath(), filePathVo.getUploadType(), filePathVo.getType());
                        if (inputStream == null) {
                            throw ExceptionResultEnum.ERROR.exception("读取题卡文件失败:" + filePathVo.getPath());
                        }
                        inputStreamList.add(inputStream);
                    }
                }
            }
            File pdfFileTemp = null;
            try {
                pdfFileTemp = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
                String mergePdf = PdfUtil.mergeWholePdf(inputStreamList, pdfFileTemp.getAbsolutePath());
                if (StringUtils.isNotBlank(mergePdf)) {
                    File localPaperPdfFile = new File(FileUtil.replaceSplit(mergePdf));
                    String uploadFileName = PdfUtil.md5FileName(CreatePdfTypeEnum.ALL.name() + examTaskWhole.getId()) + SystemConstant.PDF_PREFIX;
                    FilePathVo filePathVo = fileUploadService.uploadFile(localPaperPdfFile, UploadFileEnum.PDF, uploadFileName);
                    examTaskWhole.setAttachmentPath(JSON.toJSONString(filePathVo));
                }
            } catch (IOException e) {
                throw ExceptionResultEnum.ERROR.exception("合并pdf失败");
            } finally {
                if (pdfFileTemp != null) {
                    pdfFileTemp.delete();
                }
            }
        }
    }
}
