package com.qmth.distributed.print.business.templete.execute;

import cn.hutool.core.date.DateUtil;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.TBTaskPdf;
import com.qmth.distributed.print.business.service.ExamTaskWholeService;
import com.qmth.distributed.print.business.service.TBTaskPdfService;
import com.qmth.distributed.print.business.templete.callback.CallbackCreatePdf;
import com.qmth.distributed.print.business.templete.create.AsyncCreateTaskTemplete;
import com.qmth.distributed.print.business.templete.service.PdfTaskLogicService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

/**
 * @Description: 创建pdf模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/9
 */
@Service
public class AsyncCreatePdfTemplateService extends AsyncCreateTaskTemplete {
    private final static Logger log = LoggerFactory.getLogger(AsyncCreatePdfTemplateService.class);

    static final String OBJ_TITLE = "pdf数据";

    /**
     * 创建pdf
     *
     * @param tbTaskPdf
     * @param callbackCreatePdf
     * @return
     * @throws IOException
     */
    @Override
    public Result createPdf(TBTaskPdf tbTaskPdf, CallbackCreatePdf callbackCreatePdf) {
        StringJoiner stringJoinerSummary = new StringJoiner("\n").add(MessageFormat.format("{0}{1}{2}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), BEGIN_TITLE, OBJ_TITLE));
        SystemConstant.addSummary(stringJoinerSummary, "开始处理PDF文件生成数据");
        TBTaskPdfService tbTaskPdfService = SpringContextHolder.getBean(TBTaskPdfService.class);
        tbTaskPdfService.updateStatus(TaskStatusEnum.RUNNING, TaskStatusEnum.INIT, tbTaskPdf.getId());
        List<ExamDetailCourse> examDetailCourseList = null;
        try {
            PdfTaskLogicService pdfTaskLogicService = SpringContextHolder.getBean(PdfTaskLogicService.class);
            examDetailCourseList = pdfTaskLogicService.executeCreatePdfLogic(tbTaskPdf, stringJoinerSummary);
            tbTaskPdf.setResult(TaskResultEnum.SUCCESS);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "数据处理完成且发生异常。异常信息为[" + e.getMessage() + "]");
            tbTaskPdf.setResult(TaskResultEnum.ERROR);
        } finally {
            tbTaskPdf.setStatus(TaskStatusEnum.FINISH);
            tbTaskPdf.setSummary(stringJoinerSummary.toString());
            tbTaskPdf.setUpdateTime(System.currentTimeMillis());
            tbTaskPdfService.updateById(tbTaskPdf);

            if (TaskResultEnum.SUCCESS.equals(tbTaskPdf.getResult()) && CollectionUtils.isNotEmpty(examDetailCourseList)) {
                ExamTaskWholeService examTaskWholeService = SpringContextHolder.getBean(ExamTaskWholeService.class);
                for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                    // 生成合并文件
                    examTaskWholeService.checkAndCreateTask(tbTaskPdf.getExamId(), examDetailCourse.getCourseId(), examDetailCourse.getPaperNumber());
                }
            }
        }
        return ResultUtil.ok(tbTaskPdf);
    }

    @Override
    public Result importTask(Map<String, Object> map) throws IOException {
        return null;
    }
}
