package com.qmth.distributed.print.business.templete.execute;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentImport;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.importData.AsyncImportTaskTemplete;
import com.qmth.distributed.print.business.templete.service.ImportLogicService;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.*;

/**
 * @Description: 考生字典导入同步任务
 * @Author: CaoZixuan
 * @Date: 2024-02-26
 */
@Service
public class AsyncExamStudentImportService extends AsyncImportTaskTemplete {

    private final static Logger log = LoggerFactory.getLogger(AsyncExamStudentImportService.class);

    public static final String BEGIN_TITLE = "->开始处理导入文件";
    public static final String EXCEPTION_TITLE = "->数据处理发生异常!";
    public static final String EXCEPTION_DATA = "错误信息:";
    public static final String FINISH_TITLE = "->数据处理结束，共处理了";
    public static final String FINISH_TOTAL_SIZE = "条数据，成功";
    public static final String FINISH_SUCCESS_SIZE = "条数据，失败";
    public static final String FINISH_ERROR_SIZE = "条数据";

    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private FileUploadService fileUploadService;

    @Override
    public Result importTask(Map<String, Object> map) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        InputStream inputStream = super.getUploadFileInputStream(tbTask);
        map.put("inputStream", inputStream);

        StringJoiner stringJoinerSummary = new StringJoiner("\n");
        SystemConstant.addSummary(stringJoinerSummary, BEGIN_TITLE);
        tbTask.setStatus(TaskStatusEnum.RUNNING);
        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        tbTaskService.updateById(tbTask);
        int dataCount = 0;
        int successCount = 0;
        int errorCount = 0;
        boolean hasError;
        List<BasicExamStudentImport> errorDataList;

        try {
            ImportLogicService importLogicService = SpringContextHolder.getBean(ImportLogicService.class);

            // 执行导入考务数据
            Map<String, Object> result = importLogicService.executeImportExamStudentLogic(map, tbTask, stringJoinerSummary);
            dataCount = Integer.parseInt(String.valueOf(result.get(SystemConstant.DATA_COUNT)));
            successCount = Integer.parseInt(String.valueOf(result.get(SystemConstant.SUCCESS_DATA_COUNT)));
            errorCount = Integer.parseInt(String.valueOf(result.get(SystemConstant.ERROR_DATA_COUNT)));
            hasError = Boolean.parseBoolean(String.valueOf(result.get(SystemConstant.HAS_ERROR_DATA)));

            if (hasError) {
                SystemConstant.addSummary(stringJoinerSummary, "有异常数据，开始生成错误文件");
                tbTask.setResult(TaskResultEnum.ERROR);
                // 有异常数据
                errorDataList = JSON.parseArray(JSON.toJSONString(result.get(SystemConstant.ERROR_DATA_LIST)),
                        BasicExamStudentImport.class);
                try {
                    String[] columnNames = (String[]) result.get(SystemConstant.COLUMN_NAMES);
                    File excelFileTemp = SystemConstant.getFileTempVar(SystemConstant.EXCEL_PREFIX);
                    this.createErrorExcelFile(columnNames, errorDataList, excelFileTemp);
                    String fileName = SystemConstant.getNanoId() + SystemConstant.EXCEL_PREFIX;
                    FilePathVo filePathVo = fileUploadService.uploadFile(excelFileTemp, UploadFileEnum.FILE, fileName);
                    tbTask.setErrorFilePath(JSON.toJSONString(filePathVo));
                    SystemConstant.addSummary(stringJoinerSummary, "错误文件生成生成，请到\"导入结果查询\"中，下载错误文件");
                } catch (Exception e) {
                    SystemConstant.addSummary(stringJoinerSummary, "错误文件生成失败，" + e.getMessage());
                }
            } else {
                SystemConstant.addSummary(stringJoinerSummary, "数据校验通过，开始校验命题任务是否审核通过");

                StringJoiner paperNumberValidError = new StringJoiner("\r\n");
                Map<String, Long> paperNumberCourseIdInMap = (Map<String, Long>) result.get("paperNumberCourseIdInMap");
                paperNumberCourseIdInMap.forEach((k, v)->{
                    try {
                        examTaskService.validExamTaskAuditPass(tbTask.getSchoolId(), tbTask.getExamId(), v, k);
                    } catch (Exception e) {
                        paperNumberValidError.add(e.getMessage());
                    }
                });

                if(paperNumberValidError.toString().length() > 0){
                    throw ExceptionResultEnum.ERROR.exception(paperNumberValidError.toString());
                }

                SystemConstant.addSummary(stringJoinerSummary, "命题任务校验通过，开始保存基础考生数据");

                List<BasicExamStudent> basicExamStudentList = JSON.parseArray(JSON.toJSONString(result.get(SystemConstant.DATASOURCE)), BasicExamStudent.class);
                basicExamStudentService.saveOrUpdateBatch(basicExamStudentList);
                SystemConstant.addSummary(stringJoinerSummary, "保存基础考生数据完成，开始保存印刷任务数据");

                ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
                // 组装exam_detail数据
                List<Long> examDetailIdList = examDetailService.saveImportExamStudent(examPrintPlan, basicExamStudentList, tbTask.getCreateId());

                SystemConstant.addSummary(stringJoinerSummary, "保存印刷任务数据完成，提交生成PDF");
                // 更改印刷计划状态
                examPrintPlan.setStatus(PrintPlanStatusEnum.READY);
                examPrintPlanService.updateById(examPrintPlan);
                // 按照考场检验命题任务是否全部完成，完成生成pdf
                SysUser user = (SysUser) map.get(SystemConstant.USER);

                examTaskService.checkDataByExamination(tbTask.getPrintPlanId(), examDetailIdList, user);
                tbTask.setResult(TaskResultEnum.SUCCESS);

                stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}{4}{5}{6}{7}",
                        DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), FINISH_TITLE, dataCount,
                        FINISH_TOTAL_SIZE, successCount, FINISH_SUCCESS_SIZE, errorCount, FINISH_ERROR_SIZE));
            }
        } catch (Exception e) {
            tbTask.setResult(TaskResultEnum.ERROR);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), EXCEPTION_TITLE, EXCEPTION_DATA, e.getMessage()));
        } finally {//生成文件
            tbTask.setStatus(TaskStatusEnum.FINISH);
            tbTask.setSummary(stringJoinerSummary.toString());
            tbTaskService.updateById(tbTask);
        }
        return ResultUtil.ok();
    }

    private void createErrorExcelFile(String[] columnName, List<BasicExamStudentImport> basicExamStudentDtoList, File excelFileTemp) {
        try {
            List<String> columnNameList = new ArrayList<>(Arrays.asList(columnName));
            columnNameList.add("错误信息");
            String[] columnNames = columnNameList.toArray(new String[0]);
            List<String[]> columnValues = new ArrayList<>();
            for (BasicExamStudentImport basicExamStudentImport : basicExamStudentDtoList) {
                List<String> valueList = new ArrayList<>();
                for (String title : columnNameList) {
                    if (title.equals("错误信息")) {
                        valueList.add(basicExamStudentImport.getErrorMsg());
                        continue;
                    }
                    String value = "";
                    for (CodeNameEnableValue codeNameEnableValue : basicExamStudentImport.getRequiredFieldList()) {
                        if (codeNameEnableValue.getName().equals(title)) {
                            value = codeNameEnableValue.getValue();
                        }
                    }
                    if (StringUtils.isBlank(value)) {
                        for (CodeNameEnableValue codeNameEnableValue : basicExamStudentImport.getExtendFieldList()) {
                            if (codeNameEnableValue.getName().equals(title)) {
                                value = codeNameEnableValue.getValue();
                            }
                        }
                    }
                    valueList.add(value);
                }
                // 错误信息
                String[] columnValue = valueList.toArray(new String[valueList.size()]);
                columnValues.add(columnValue);
            }

            FileOutputStream outputStream = new FileOutputStream(excelFileTemp);
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeStringArrays("考生数据", null, columnNames, columnValues.listIterator());
            writer.output(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (Exception e) {

        }
    }

}
