package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.distributed.print.business.bean.dto.ObjectiveStructDto;
import com.qmth.distributed.print.business.bean.dto.SubjectiveStructDto;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentImport;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentParseDto;
import com.qmth.distributed.print.business.entity.BasicExamRule;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.ImportLogicService;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableDisabledValue;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.params.UserSaveParams;
import com.qmth.teachcloud.common.bean.result.ExcelResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.DateDisposeUtils;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupSingleDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionAnswerService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class ImportLogicServiceImpl implements ImportLogicService {
    private final static Logger log = LoggerFactory.getLogger(ImportLogicServiceImpl.class);

    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private BasicExamService basicExamService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private SysRoleService sysRoleService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private TBTaskPdfService tbTaskPdfService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private MarkQuestionAnswerService markQuestionAnswerService;

    @Transactional
    @Override
    public Map<String, Object> executeImportBasicExamStudentLogic(Map<String, Object> map, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        SystemConstant.addSummary(stringJoinerSummary, "开始解析导入文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            // 解析excel文件内容
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getPaperType, BasicExamStudent::getExamStartTime, BasicExamStudent::getExamEndTime)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        // 学校设置强制校验卷型或者模式4
        boolean needPaperNumber = (basicSchool.getHasPaperNumber() != null && basicSchool.getHasPaperNumber());
        // 试卷编号->课程
        Map<String, Long> paperNumberCourseIdMap = new HashMap<>();
        // 试卷编号->考试时间
        Map<String, String> paperNumberExamTimeMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.toMap(BasicExamStudent::getPaperNumber, m -> m.getExamStartTime() + "-" + m.getExamEndTime(), (v1, v2) -> v1));
        Map<String, Long> basicCourseIdMap = new HashMap<>();
        // 课程id+教学班名称->任课老师Id
        Map<String, Long> courseTeachClassTeacherMap = new HashMap<>();
        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 课程+考生在数据库中已存在的试卷编号和导入文件中不致的数据
        List<String> oldPaperNumberList = new ArrayList<>();
        // 是否有错误提示
        boolean hasError = false;

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            BeanUtils.copyProperties(basicExamStudentImport, basicExamStudent);
            basicExamStudent.setSchoolId(schoolId);
            basicExamStudent.setSemesterId(semesterId);
            basicExamStudent.setExamId(examId);

            StringJoiner stringJoiner = new StringJoiner(";");
            // 校验excel内容
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                // 模式4必填字段校验
                if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                    if (StringUtils.isBlank(basicExamStudentImport.getPaperNumber())) {
                        stringJoiner.add(RequiredFieldsEnum.PAPER_NUMBER.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getStudentCode())) {
                        stringJoiner.add(RequiredFieldsEnum.STUDENT_CODE.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getStudentName())) {
                        stringJoiner.add(RequiredFieldsEnum.STUDENT_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getCourseCode())) {
                        stringJoiner.add(RequiredFieldsEnum.COURSE_CODE.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getCourseName())) {
                        stringJoiner.add(RequiredFieldsEnum.COURSE_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getTeachingRoomName())) {
                        stringJoiner.add(RequiredFieldsEnum.TEACHING_ROOM_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudent.getTeachClassName())) {
                        stringJoiner.add(RequiredFieldsEnum.TEACH_CLASS_NAME.getName() + "必填");
                    }
                } else {
                    // 通用规则中基础字段默认不可取消的，必填
                    if (m.getDisabled() && StringUtils.isBlank(m.getValue())) {
                        stringJoiner.add(m.getName() + "必填");
                    }
                    if (m.getCode().equals(RequiredFieldsEnum.PAPER_NUMBER.getCode()) && needPaperNumber) {
                        if (StringUtils.isBlank(m.getValue())) {
                            stringJoiner.add(m.getName() + "必填");
                        }
                    }
                }
//                // 试卷编号不为空时，校区（考点）必填
//                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber()) && m.getCode().equals(RequiredFieldsEnum.EXAM_PLACE.getCode()) && m.getEnable() && StringUtils.isBlank(basicExamStudent.getExamPlace())) {
//                    stringJoiner.add(RequiredFieldsEnum.EXAM_PLACE.getName() + "必填");
//                }
            });

            // 学号校验
            if (StringUtils.isNotBlank(basicExamStudent.getStudentCode()) && !basicExamStudent.getStudentCode().matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
                stringJoiner.add("学号只能由字母、数字组成");
            }

            if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                if (basicExamStudent.getPaperType() == null) {
                    basicExamStudent.setPaperType(SystemConstant.DEFAULT_PAPER_TYPE_A);
                }
            }
            // 任课老师和任课老师工号都有值或都没值
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode()) && StringUtils.isBlank(basicExamStudentImport.getTeacherName())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_NAME.getName() + "必填");
            }
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherName()) && StringUtils.isBlank(basicExamStudentImport.getTeacherCode())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_CODE.getName() + "必填");
            }

            // 解析考试日期和考试时间
            if (StringUtils.isNoneBlank(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime())) {
                try {
                    Map<String, Object> timeMap = ConvertUtil.analyzeExamTime(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime());
                    basicExamStudent.setExamStartTime(Long.valueOf(String.valueOf(timeMap.get("startTime"))));
                    basicExamStudent.setExamEndTime(Long.valueOf(String.valueOf(timeMap.get("endTime"))));

                    // 反向更新examDate为标准日期格式，
                    for (CodeNameEnableDisabledValue codeNameEnableDisabledValue : basicExamStudentImport.getRequiredFieldList()) {
                        if (codeNameEnableDisabledValue.getCode().equals(RequiredFieldsEnum.EXAM_DATE.getCode())) {
                            codeNameEnableDisabledValue.setValue(String.valueOf(timeMap.get("dateStr")));
                        }
                    }
                } catch (Exception e) {
                    stringJoiner.add(e.getMessage());
                }
            }

            basicExamStudent.setRequiredFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getRequiredFieldList()) ? JSON.toJSONString(basicExamStudentImport.getRequiredFieldList()) : null);
            basicExamStudent.setExtendFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getExtendFieldList()) ? JSON.toJSONString(basicExamStudentImport.getExtendFieldList()) : null);

            // 校验课程开课学院和课程代码
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else if (sysOrgList.size() == 1) {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), basicExamStudentImport.getCourseCode());
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    } else {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]存在多个相同值");
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            // 校验任课老师工号和姓名
            if (StringUtils.isNoneBlank(basicExamStudentImport.getTeacherCode(), basicExamStudentImport.getTeacherName())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }

                // 校验同课程下，同一教学班只能有一个任课老师
                String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
                if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
                    Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
                    if ((teacherId == null && basicExamStudent.getTeacherId() != null) || (teacherId != null && basicExamStudent.getTeacherId() == null) || (teacherId != null && !teacherId.equals(basicExamStudent.getTeacherId()))) {
                        stringJoiner.add("同课程同教学班，只允许设置一个任课老师");
                    }
                } else {
                    courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
                }
            }

            if (basicExamStudent.getCourseId() != null) {
                if (courseIdStudentCodeMap.containsKey(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode())) {
                    BasicExamStudent student = courseIdStudentCodeMap.get(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode());
                    if (StringUtils.isNotBlank(student.getPaperNumber()) && !student.getPaperNumber().equals(basicExamStudent.getPaperNumber())) {
                        throw ExceptionResultEnum.ERROR.exception("不允许修改试卷编号");
                    } else {
                        basicExamStudent.setId(student.getId());
                        basicExamStudent.setPaperType(student.getPaperType());
                    }
                }

                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                    if (paperNumberCourseIdMap.containsKey(basicExamStudent.getPaperNumber())) {
                        if (!paperNumberCourseIdMap.get(basicExamStudent.getPaperNumber()).equals(basicExamStudent.getCourseId())) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]不能绑定多个课程");
                        }
                    } else {
                        long count = basicExamStudentList.stream().filter(m -> m.getPaperNumber() != null && m.getPaperNumber().equals(basicExamStudent.getPaperNumber()) && !Objects.equals(m.getCourseId(), basicExamStudent.getCourseId())).count();
                        if (count > 0) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]已绑定其它课程");
                        } else {
                            paperNumberCourseIdMap.put(basicExamStudent.getPaperNumber(), basicExamStudent.getCourseId());
                        }
                    }

                    // 同一试卷编号只能对应同一考试时间
                    if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
                        String examTimeValue = basicExamStudent.getExamStartTime() + "-" + basicExamStudent.getExamEndTime();
                        if (paperNumberExamTimeMap.containsKey(basicExamStudent.getPaperNumber())) {
                            if (!paperNumberExamTimeMap.get(basicExamStudent.getPaperNumber()).equals(examTimeValue)) {
                                stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]只能有相同考试时间");
                            }
                        } else {
                            paperNumberExamTimeMap.put(basicExamStudent.getPaperNumber(), examTimeValue);
                        }
                    }
                }

                // 校验课程学号唯一
                String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudentImport.getStudentCode();
                if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                    stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]存在重复考生", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                } else {
                    courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());
        map.put("oldPaperNumberList", oldPaperNumberList);

        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportBasicExamStudentTeacherLogic(Map<String, Object> map, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }

        SystemConstant.addSummary(stringJoinerSummary, "开始解析导入文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getTeachClassName, BasicExamStudent::getTeacherId, BasicExamStudent::getRequiredFields)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        Map<String, Long> basicCourseIdMap = new HashMap<>();
        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 课程id+教学班名称->任课老师Id
        Map<String, Long> courseTeachClassTeacherMap = new HashMap<>();
        // 是否有错误提示
        boolean hasError = false;

        // 必填字段为课程代码、开课学院、学号、教学班、任课老师工号、任课老师
        List<String> requiredCodes = Arrays.asList(RequiredFieldsEnum.STUDENT_CODE.getCode(), RequiredFieldsEnum.COURSE_CODE.getCode(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getCode(), RequiredFieldsEnum.TEACH_CLASS_NAME.getCode(), RequiredFieldsEnum.TEACHER_NAME.getCode(), RequiredFieldsEnum.TEACHER_CODE.getCode());

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            StringJoiner stringJoiner = new StringJoiner(";");

            // 校验excel内容
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                if (requiredCodes.contains(m.getCode()) && StringUtils.isBlank(m.getValue())) {
                    stringJoiner.add(m.getName() + "必填");
                }
            });

            // 校验课程
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), basicExamStudentImport.getCourseCode());
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            //校验任课老师
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }
            }

            if (basicExamStudent.getCourseId() != null) {
                // 校验同课程下，同一教学班只能有一个任课老师
                if (StringUtils.isNotBlank(basicExamStudent.getTeachClassName())) {
                    String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
                    if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
                        Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
                        if (teacherId != null && basicExamStudent.getTeacherId() != null && !teacherId.equals(basicExamStudent.getTeacherId())) {
                            stringJoiner.add("同课程同教学班，只允许设置一个任课老师");
                        }
                    } else {
                        courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
                    }
                }

                // 校验课程+学号是否存在
                if (StringUtils.isNotBlank(basicExamStudentImport.getStudentCode())) {
                    String studentKey = basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudentImport.getStudentCode();
                    if (courseIdStudentCodeMap.containsKey(studentKey)) {
                        BasicExamStudent student = courseIdStudentCodeMap.get(studentKey);
                        basicExamStudent.setId(student.getId());
                        basicExamStudent.setPaperNumber(student.getPaperNumber());
                        basicExamStudent.setPaperType(student.getPaperType());
                        List<CodeNameEnableValue> requiredFieldList = StringUtils.isNotBlank(student.getRequiredFields()) ? JSON.parseArray(student.getRequiredFields(), CodeNameEnableValue.class) : Collections.emptyList();
                        // 更新requiredFields中的任课老师和任课老师工号
                        for (CodeNameEnableValue codeNameEnableValue : requiredFieldList) {
                            if (codeNameEnableValue.getCode().equals(RequiredFieldsEnum.TEACHER_CODE.getCode())) {
                                codeNameEnableValue.setValue(basicExamStudentImport.getTeacherCode());
                            }
                            if (codeNameEnableValue.getCode().equals(RequiredFieldsEnum.TEACHER_NAME.getCode())) {
                                codeNameEnableValue.setValue(basicExamStudentImport.getTeacherName());
                            }
                        }
                        basicExamStudent.setRequiredFields(JSON.toJSONString(requiredFieldList));
                    } else {
                        stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]考生管理中不存在", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                    }
                }

                // 校验课程学号唯一
                String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudentImport.getStudentCode();
                if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                    stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]存在重复考生", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                } else {
                    courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());

        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportExamStudentLogic(Map<String, Object> map, TBTask tbTask, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long schoolId = requestUser.getSchoolId();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划不存在");
        }

        String printPlanName = examPrintPlan.getName();
        if (!tbTaskPdfService.countByPrintPlanIdAndEntityId(schoolId, tbTask.getPrintPlanId())) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划[" + printPlanName + "]有数据正在生成PDF文件，无法导入考务数据");
        }
        PrintPlanStatusEnum printPlanStatus = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != printPlanStatus && PrintPlanStatusEnum.READY != printPlanStatus) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划状态为[" + PrintPlanStatusEnum.NEW.getName() + "、" + PrintPlanStatusEnum.READY.getName() + "]时才可导入，当前状态[" + printPlanStatus.getName() + "]");
        }

        Long examId = examPrintPlan.getExamId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        // 解析excel文件
        SystemConstant.addSummary(stringJoinerSummary, "开始解析文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getExamStartTime, BasicExamStudent::getExamEndTime)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        // 文件中试卷编号对应多个课程代码
        Map<String, Long> paperNumberCourseIdInMap = new HashMap<>();
        // 试卷编号->考试时间
        Map<String, String> paperNumberExamTimeMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.toMap(BasicExamStudent::getPaperNumber, m -> m.getExamStartTime() + "-" + m.getExamEndTime(), (v1, v2) -> v1));
        Map<String, Long> basicCourseIdMap = new HashMap<>();

        // 课程管理中课程代码-课程名
        Map<String, BasicCourse> courseCodeNameInBasicCourseMap = new HashMap<>();
        List<BasicCourse> basicCourseList = basicCourseService.list(new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId));
        basicCourseList.forEach(e -> courseCodeNameInBasicCourseMap.put(e.getTeachingRoomId().toString().concat(e.getCode()), e));

        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 是否有错误提示
        boolean hasError = false;

        // 命题任务列表
        List<ExamTask> examTaskList = examTaskService.listByExamId(examId);
        Map<String, List<ExamTask>> paperNumberMap = examTaskList.stream().collect(Collectors.groupingBy(m -> m.getPaperNumber()));

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            BeanUtils.copyProperties(basicExamStudentImport, basicExamStudent);
            basicExamStudent.setSchoolId(schoolId);
            basicExamStudent.setSemesterId(semesterId);
            basicExamStudent.setExamId(examId);
            basicExamStudent.setRequiredFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getRequiredFieldList()) ? JSON.toJSONString(basicExamStudentImport.getRequiredFieldList()) : null);
            basicExamStudent.setExtendFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getExtendFieldList()) ? JSON.toJSONString(basicExamStudentImport.getExtendFieldList()) : null);

            StringJoiner stringJoiner = new StringJoiner(";");
            // 校验必填项
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                if (m.getEnable() && StringUtils.isBlank(m.getValue())) {
                    stringJoiner.add(m.getName() + "必填");
                }
            });

            // 学号校验
            if (StringUtils.isNotBlank(basicExamStudent.getStudentCode()) && !basicExamStudent.getStudentCode().matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
                stringJoiner.add("学号只能由字母、数字组成");
            }

            // 校验任课老师，都填或者都不填
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode()) && StringUtils.isBlank(basicExamStudentImport.getTeacherName())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_NAME.getName() + "必填");
            }
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherName()) && StringUtils.isBlank(basicExamStudentImport.getTeacherCode())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_CODE.getName() + "必填");
            }

            // 解析并校验考试日期、考试时间
            if (StringUtils.isNoneBlank(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime())) {
                try {
                    Map<String, Object> timeMap = ConvertUtil.analyzeExamTime(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime());
                    basicExamStudent.setExamStartTime(Long.valueOf(String.valueOf(timeMap.get("startTime"))));
                    basicExamStudent.setExamEndTime(Long.valueOf(String.valueOf(timeMap.get("endTime"))));

                    // 反向更新examDate为标准日期格式，
                    basicExamStudentImport.setExamDate(String.valueOf(timeMap.get("dateStr")));
                } catch (Exception e) {
                    stringJoiner.add(e.getMessage());
                }
            }

            // 校验开课学院是否存在该课程代码
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else if (sysOrgList.size() == 1) {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = courseCodeNameInBasicCourseMap.get(sysOrg.getId().toString().concat(basicExamStudentImport.getCourseCode()));
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    } else {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]存在多个相同值");
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            // 校验试卷编号和课程代码对应关系，试卷编号只能绑定一个课程
            if (basicExamStudent.getCourseId() != null) {
                if (paperNumberCourseIdInMap.containsKey(basicExamStudent.getPaperNumber())) {
                    Long courseId = paperNumberCourseIdInMap.get(basicExamStudent.getPaperNumber());
                    if (!basicExamStudent.getCourseId().equals(courseId)) {
                        stringJoiner.add(String.format("试卷编号[%s]对应多个不同的课程代码", basicExamStudent.getPaperNumber()));
                    }
                } else {
                    paperNumberCourseIdInMap.put(basicExamStudent.getPaperNumber(), basicExamStudent.getCourseId());
                }

                // 校验试卷编号课程代码是否已命题且命题任务中课程代码是否一致
                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                    if (paperNumberMap.containsKey(basicExamStudent.getPaperNumber())) {
                        List<ExamTask> examTasks = paperNumberMap.get(basicExamStudent.getPaperNumber());
                        if (examTasks.size() > 1) {
                            stringJoiner.add(String.format("试卷编号[%s]在考试[%s]下有多条命题任务，请联系管理员处理", basicExamStudent.getPaperNumber(), basicExam.getName()));
                        } else {
                            Long courseId = paperNumberCourseIdInMap.get(basicExamStudent.getPaperNumber());
                            ExamTask examTask = examTasks.get(0);
                            if (!examTask.getCourseId().equals(courseId)) {
                                stringJoiner.add(String.format("试卷编号[%s]对应课程代码与命题任务中对应课程代码不一致", basicExamStudent.getPaperNumber()));
                            } else {
                                basicExamStudent.setCoursePaperId(String.valueOf(examTask.getId()));
                            }
                        }
                    } else {
                        stringJoiner.add(String.format("试卷编号[%s]未命题", basicExamStudent.getPaperNumber()));
                    }
                }
            }

            //校验任课老师是否存在，且工号和姓名是否与系统一致
            if (StringUtils.isNoneBlank(basicExamStudentImport.getTeacherCode(), basicExamStudentImport.getTeacherName())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }

                // 校验同课程下，同一教学班只能有一个任课老师
//                String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
//                if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
//                    Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
//                    if ((teacherId == null && basicExamStudent.getTeacherId() != null) || (teacherId != null && basicExamStudent.getTeacherId() == null) || (teacherId != null && !teacherId.equals(basicExamStudent.getTeacherId()))) {
//                        stringJoiner.add("同课程下相同教学班，只能有一位任课老师");
//                    }
//                } else {
//                    courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
//                }
            }

            // 校验试卷编号学号是否已生成印刷任务
            if (StringUtils.isNoneBlank(basicExamStudent.getPaperNumber(), basicExamStudent.getStudentCode())) {
                int studentCount = examStudentService.countByExamIdAndPaperNumberAndStudentCode(examId, basicExamStudent.getPaperNumber(), basicExamStudent.getStudentCode());
                if (studentCount > 0) {
                    stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]、学号[" + basicExamStudent.getStudentCode() + "]已有印刷任务，不允许重复导入");
                } else {
                    if (basicExamStudent.getCourseId() != null) {
                        // 校验课程学号唯一
                        String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudent.getStudentCode();
                        if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]已存在学号[" + basicExamStudentImport.getStudentCode() + "]的考生数据");
                        } else {
                            courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                        }

                        if (courseIdStudentCodeMap.containsKey(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode())) {
                            BasicExamStudent basicStudent = courseIdStudentCodeMap.get(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode());
                            basicExamStudent.setId(basicStudent.getId());
                        }
                    }
                }
            }

            // 校验试卷编号、考试日期、考试时间、考点、考场是否已命题
            if (basicExamStudent.getCourseId() != null && StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                List<ExamDetail> examDetailList = examDetailService.listExamDetailByExamIdAndCourseIdAndPaperNumber(examId, basicExamStudent.getCourseId(), basicExamStudent.getPaperNumber());
                if (CollectionUtils.isNotEmpty(examDetailList)) {
                    long count = examDetailList.stream().filter(m -> basicExamStudent.getExamStartTime().equals(m.getExamStartTime()) && basicExamStudent.getExamEndTime().equals(m.getExamEndTime()) && basicExamStudent.getExamPlace().equals(m.getExamPlace()) && basicExamStudent.getExamRoom().equals(m.getExamRoom())).count();
                    if (count != 0) {
                        stringJoiner.add("试卷编号[" + basicExamStudentImport.getPaperNumber() + "]、考试日期[" + basicExamStudentImport.getExamDate() + "]、考试时间[" + basicExamStudentImport.getExamTime() + "]、考场[" + basicExamStudentImport.getExamPlace() + "]、考点[" + basicExamStudentImport.getExamRoom() + "]已有印刷任务");
                    }
                }

                // 同一试卷编号只能对应同一考试时间
                if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
                    String examTimeValue = basicExamStudent.getExamStartTime() + "-" + basicExamStudent.getExamEndTime();
                    if (paperNumberExamTimeMap.containsKey(basicExamStudent.getPaperNumber())) {
                        if (!paperNumberExamTimeMap.get(basicExamStudent.getPaperNumber()).equals(examTimeValue)) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]只能存在相同考试时间");
                        }
                    } else {
                        paperNumberExamTimeMap.put(basicExamStudent.getPaperNumber(), examTimeValue);
                    }
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        //校验命题任务是否存在并审核通过

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());
        map.put("paperNumberCourseIdInMap", paperNumberCourseIdInMap);

        return map;
    }

    private BasicExamStudentParseDto parseImportExamStudentData(Long schoolId, InputStream inputStream) {
        ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 1);
        List<DataMap> dataMapList;
        try {
            dataMapList = excelReader.getDataMapList();
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("读取excel内容失败");
        }
        BasicExamStudentParseDto basicExamStudentParseDto = new BasicExamStudentParseDto();
        List<BasicExamStudentImport> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dataMapList)) {
            String[] columnNames = excelReader.getColumnNames();

            BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
            if (basicExamRule == null) {
                throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
            }

            // 必填字段
            Map<String, EnumResult> requiredMap = basicExamRule.getRequiredFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getName, e -> e));
            // 扩展字段
            Map<String, EnumResult> extendMap = basicExamRule.getExtendFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getName, e -> e));

            // 通用规则表头
            List<String> actualTitleList = Stream.concat(requiredMap.keySet().stream(), extendMap.keySet().stream()).collect(Collectors.toList());
            if (!CollectionUtils.isEqualCollection(Arrays.asList(columnNames), actualTitleList)) {
                throw ExceptionResultEnum.ERROR.exception("表头名称错误，正确表头为【" + String.join(",", actualTitleList) + "】");
            }

            basicExamStudentParseDto.setColumnNames(columnNames);

            AtomicInteger i = new AtomicInteger(2);
            dataMapList.forEach(m -> {
                if (m.size() > 0) {
                    BasicExamStudentImport basicExamStudentImport = new BasicExamStudentImport();
                    basicExamStudentImport.setRowIndex(i.getAndIncrement());
                    Class<BasicExamStudentImport> aClass = (Class<BasicExamStudentImport>) basicExamStudentImport.getClass();
                    for (Map.Entry<String, String> entry : m.entrySet()) {
                        // 必填字段
                        if (requiredMap.containsKey(entry.getKey())) {
                            if (entry.getValue() == null) {
                                continue;
                            }
                            EnumResult enumResult = requiredMap.get(entry.getKey());
                            try {
                                Field declaredField = aClass.getDeclaredField(enumResult.getCode());
                                declaredField.setAccessible(true);
                                declaredField.set(basicExamStudentImport, RequiredFieldsEnum.TEACHER_NAME.getCode().equals(enumResult.getCode()) ? entry.getValue().trim() : StringUtils.deleteWhitespace(entry.getValue()));
                            } catch (NoSuchFieldException e) {
                                throw ExceptionResultEnum.ERROR.exception("未获取到表头为[" + entry.getKey() + "]的属性值");
                            } catch (IllegalAccessException e) {
                                throw ExceptionResultEnum.ERROR.exception("表头为[" + entry.getKey() + "]的值赋值失败");
                            }
                            basicExamStudentImport.getRequiredFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), StringUtils.isNotBlank(entry.getValue()) ? entry.getValue() : null));
                        }
                        // 扩展字段
                        if (extendMap.containsKey(entry.getKey())) {
                            EnumResult enumResult = extendMap.get(entry.getKey());
                            basicExamStudentImport.getExtendFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), StringUtils.isNotBlank(entry.getValue()) ? entry.getValue() : null));
                        }
                    }

                    List<CodeNameEnableDisabledValue> requiredFieldList = basicExamStudentImport.getRequiredFieldList();
                    // 必填字段
                    if (requiredMap.keySet().size() > requiredFieldList.size()) {
                        for (String s : requiredMap.keySet()) {
                            Optional<CodeNameEnableDisabledValue> optional = basicExamStudentImport.getRequiredFieldList().stream().filter(r -> s.equals(r.getName())).findFirst();
                            if (!optional.isPresent()) {
                                EnumResult enumResult = requiredMap.get(s);
                                basicExamStudentImport.getRequiredFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), null));
                            }
                        }
                    }
                    List<CodeNameEnableDisabledValue> extendFieldList = basicExamStudentImport.getExtendFieldList();
                    // 扩展字段
                    if (extendMap.keySet().size() > extendFieldList.size()) {
                        for (String s : extendMap.keySet()) {
                            Optional<CodeNameEnableDisabledValue> optional = basicExamStudentImport.getExtendFieldList().stream().filter(r -> s.equals(r.getName())).findFirst();
                            if (!optional.isPresent()) {
                                EnumResult enumResult = extendMap.get(s);
                                basicExamStudentImport.getExtendFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), null));
                            }
                        }
                    }
                    list.add(basicExamStudentImport);
                }
            });
        }
        basicExamStudentParseDto.setBasicExamStudentImportList(list);
        return basicExamStudentParseDto;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportObjectiveStructLogic(Map<String, Object> map) throws Exception {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            throw ExceptionResultEnum.ERROR.exception("仅针对模式4的考试可以导入");
        }
        // 课程编号名称校验
        Map<String, Set<String>> courseCheckMap = basicCourseService.list(
                new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId)).stream().collect(
                Collectors.toMap(BasicCourse::getCode, v -> new HashSet<>(Collections.singletonList(v.getName())), (Set<String> v1, Set<String> v2) -> {
                    v1.addAll(v2);
                    return v1;
                }));

        List<MarkPaper> markPaperList = markPaperService.findMarkPaperListByExamId(examId);
        // 题号重复校验
        // 导入的客观题题号集合
        Set<String> questionNumberCheckSet = new HashSet<>();
        // 客观题选项个数在大题下一致性校验
        Map<String, Integer> questionOptionCountMap = new HashMap<>();
        // 该试卷已存在的主观题题号集合
        Set<String> dbSubjectiveQuestionNumberSet = markQuestionService.list(
                        new QueryWrapper<MarkQuestion>().lambda().eq(MarkQuestion::getExamId, examId)
                                .eq(MarkQuestion::getObjective, false)).stream()
                .map(e -> e.getPaperNumber() + SystemConstant.HYPHEN + e.getMainNumber() + SystemConstant.HYPHEN + e.getSubNumber()).collect(Collectors.toSet());

        ExcelResult<ObjectiveStructDto> excelResult = ConvertUtil.analyzeExcel(inputStream, ObjectiveStructDto.class,
                true, 0);
        List<ObjectiveStructDto> objectiveStructDtoList = excelResult.getDatasource();
        List<MarkQuestion> markQuestionList = new ArrayList<>();
        for (ObjectiveStructDto objectiveStructDto : objectiveStructDtoList) {
            List<String> logicErrorList = new ArrayList<>();
            // 公共异常提示
            String logicErrorCommonNotice = "";

            String courseCode = objectiveStructDto.getCourseCode();
            String courseName = objectiveStructDto.getCourseName();
            String paperNumber = objectiveStructDto.getPaperNumber();
            String mainTitle = objectiveStructDto.getMainTitle();
            Integer mainNumber = objectiveStructDto.getMainNumber();
            Integer subNumber = objectiveStructDto.getSubNumber();
            String answer = objectiveStructDto.getAnswer();
            Integer optionCount = objectiveStructDto.getOptionCount();
            Double totalScore = objectiveStructDto.getTotalScore();
            Integer questionType = objectiveStructDto.getQuestionType();
            String errorMsg = objectiveStructDto.getErrorMsg();

            // 1.题号重复校验
            if (SystemConstant.strNotNull(paperNumber) && mainNumber != null && subNumber != null) {
                logicErrorCommonNotice = String.format("课程代码为[%s],试卷编号为[%s],大题号为[%s],小题号为[%s]的数据异常 : ", courseCode,
                        paperNumber, mainNumber, subNumber);
                String questionNumberKey =
                        courseCode + SystemConstant.HYPHEN + paperNumber + SystemConstant.HYPHEN + mainNumber + SystemConstant.HYPHEN + subNumber;
                if (questionNumberCheckSet.contains(questionNumberKey)) {
                    logicErrorList.add("excel中题号重复");
                } else {
                    questionNumberCheckSet.add(questionNumberKey);
                }
                if (dbSubjectiveQuestionNumberSet.contains(questionNumberKey)) {
                    logicErrorList.add("该题号已存在主观题中");
                }
                String mainQuestionKey = paperNumber + SystemConstant.HYPHEN + mainNumber;
                if (questionOptionCountMap.containsKey(mainQuestionKey)) {
                    if (!Objects.equals(questionOptionCountMap.get(mainQuestionKey), optionCount)) {
                        logicErrorList.add("选项个数和该大题中的其他小题不一致");
                    }
                } else {
                    questionOptionCountMap.put(mainQuestionKey, optionCount);
                }
            }

            // 2.校验课程编号,课程名称,试卷在考试下是否存在
            if (SystemConstant.strNotNull(courseCode) && SystemConstant.strNotNull(courseName) && SystemConstant.strNotNull(paperNumber)) {
                if (!courseCheckMap.containsKey(courseCode)) {
                    logicErrorList.add("课程编号不存在");
                } else if (!courseCheckMap.get(courseCode).contains(courseName)) {
                    logicErrorList.add("课程编号名称不匹配");
                }
                if (markPaperList.stream().noneMatch(
                        m -> m.getPaperNumber().equals(paperNumber) && m.getCourseCode().equals(courseCode) && m.getCourseName().equals(courseName))) {
                    logicErrorList.add("考试课程中不存在该试卷，请先创建试卷");
                }
            }
            // 3.选项越界校验
            List<String> optionScope = new ArrayList<>();
            try {
                for (int j = 1; j <= optionCount; j++) {
                    String code = OptionsEnum.getCodeByIndex(j);
                    optionScope.add(code);
                }
            } catch (Exception e) {
                throw ExceptionResultEnum.ERROR.exception(e.getMessage());
            }

            // 答案选项越界校验
            List<String> answerList = Arrays.asList(answer.split(""));
            if (CollectionUtils.isNotEmpty(answerList)) {
                for (String cell : answerList) {
                    String outOfBoundsException = String.format("答案[%s]错误。答案只能从[%s]中选择", cell, String.join("", optionScope));
                    try {
                        OptionsEnum optionsEnum = OptionsEnum.getByCode(cell);
                        int optionIndex = optionsEnum.getIndex();
                        if (optionIndex > optionCount) {
                            // 答案选项超出范围
                            logicErrorList.add(outOfBoundsException);
                        }
                    } catch (Exception e) {
                        logicErrorList.add(outOfBoundsException);
                    }
                }
            } else {
                logicErrorList.add("答案必填");
            }

            // 4.判断答案是否符合题型
            if (questionType != null && SystemConstant.strNotNull(answer)) {
                switch (questionType) {
                    case 1:
                        if (answer.length() > 1) {
                            logicErrorList.add("单选题答案只能有一个");
                        }
                        break;
                    case 2:
                        break;
                    case 3:
                        if (optionCount != 2) {
                            logicErrorList.add("判断题[选项个数]只能为2");
                        }
                        break;
                    default:
                        logicErrorList.add("[题型(1-单选，2-多选，3-判断)]必须从1、2、3中填写");
                }
            }

            // 整理异常信息
            if (CollectionUtils.isNotEmpty(logicErrorList)) {
                if (SystemConstant.strNotNull(errorMsg)) {
                    errorMsg = errorMsg + "、" + logicErrorCommonNotice + String.join("、", logicErrorList);
                } else {
                    errorMsg = logicErrorCommonNotice + String.join("、", logicErrorList);
                }
                objectiveStructDto.setErrorMsg(errorMsg);
            } else {
                MarkQuestion markQuestion = new MarkQuestion();
                markQuestion.setExamId(examId);
                markQuestion.setPaperNumber(paperNumber);
                markQuestion.setPaperType(OptionsEnum.A.getCode());
                markQuestion.setObjective(true);
                markQuestion.setMainNumber(mainNumber);
                markQuestion.setSubNumber(subNumber);
                markQuestion.setMainTitle(mainTitle);
                // todo 2024-10-16
                markQuestion.setAnswer(answer);
                markQuestion.setOptionCount(optionCount);
                markQuestion.setTotalScore(totalScore);
                // todo 2024-10-16
//                markQuestion.setObjectivePolicy(ObjectivePolicy.NONE);
                markQuestion.setQuestionType(questionType);
                markQuestionList.add(markQuestion);
            }
        }

        if (objectiveStructDtoList.stream().anyMatch(e -> SystemConstant.strNotNull(e.getErrorMsg()))) {
            // 抛出excel解析异常
            throw ExceptionResultEnum.ERROR.exception(
                    objectiveStructDtoList.stream().map(ObjectiveStructDto::getErrorMsg).filter(SystemConstant::strNotNull).collect(Collectors.joining(System.lineSeparator())));
        } else {
            // 保存客观题结构
            Map<String, List<MarkQuestion>> markQuestionMap = markQuestionList.stream().collect(Collectors.groupingBy(MarkQuestion::getPaperNumber));
            markQuestionMap.forEach((k, v) -> {
                // 优先删除该试卷编号下的客观题
                markQuestionService.deleteByExamIdAndPaperNumberAndObjective(examId, k, true);
                // 保存客观题
                markQuestionService.saveBatch(v);
                markQuestionService.updateMarkPaperScore(examId, k);

                // 增加客观题
                markQuestionAnswerService.deleteByExamIdAndPaperNumber(examId, k);
                markQuestionAnswerService.saveByMarkQuestion(examId, k, v, true);
            });
            // 更改试卷结构状态为已提交
            Set<String> paperNumberSet = markQuestionMap.keySet();
            if (CollectionUtils.isNotEmpty(paperNumberSet)) {
                markPaperService.update(new UpdateWrapper<MarkPaper>().lambda().eq(MarkPaper::getExamId, examId)
                        .in(MarkPaper::getPaperNumber, paperNumberSet).set(MarkPaper::getQuestionStatus, true));
            }
        }
        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportSubjectiveStructLogic(Map<String, Object> map) throws Exception {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            throw ExceptionResultEnum.ERROR.exception("仅针对模式4的考试可以导入");
        }
        SysRole markerRole = sysRoleService.getOne(
                new QueryWrapper<SysRole>().lambda().eq(SysRole::getType, RoleTypeEnum.MARKER));
        if (Objects.isNull(markerRole)) {
            throw ExceptionResultEnum.ERROR.exception("缺少评卷员角色");
        }

        // 课程编号名称校验
        Map<String, Set<String>> courseCheckMap = basicCourseService.list(
                new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId)).stream().collect(
                Collectors.toMap(BasicCourse::getCode, v -> new HashSet<>(Collections.singletonList(v.getName())),
                        (Set<String> v1, Set<String> v2) -> {
                            v1.addAll(v2);
                            return v1;
                        }));
        List<MarkPaper> markPaperList = markPaperService.findMarkPaperListByExamId(examId);
        // 题号重复校验
        // 导入的客观题题号集合
        Set<String> questionNumberCheckSet = new HashSet<>();
        // 该试卷已存在的客观题题号集合
        Set<String> dbObjectiveQuestionNumberSet = markQuestionService.list(
                        new QueryWrapper<MarkQuestion>().lambda().eq(MarkQuestion::getExamId, examId)
                                .eq(MarkQuestion::getObjective, true)).stream()
                .map(e -> e.getPaperNumber() + SystemConstant.HYPHEN + e.getMainNumber() + SystemConstant.HYPHEN + e.getSubNumber()).collect(Collectors.toSet());

        ExcelResult<SubjectiveStructDto> excelResult = ConvertUtil.analyzeExcel(inputStream, SubjectiveStructDto.class,
                true, 0);
        List<SubjectiveStructDto> subjectiveStructDtoList = excelResult.getDatasource();
        for (SubjectiveStructDto subjectiveStructDto : subjectiveStructDtoList) {
            List<String> excelErrorList = new ArrayList<>();
            // 公共异常提示
            String excelErrorCommonNotice = "";

            String courseCode = subjectiveStructDto.getCourseCode();
            String courseName = subjectiveStructDto.getCourseName();
            String paperNumber = subjectiveStructDto.getPaperNumber();
            Integer mainNumber = subjectiveStructDto.getMainNumber();
            Integer subNumber = subjectiveStructDto.getSubNumber();
            Integer markerCount = subjectiveStructDto.getMarkerCount();
            String markerCodes = subjectiveStructDto.getMarkerCodes();
            String errorMsg = subjectiveStructDto.getErrorMsg();

            // 1.题号重复校验
            if (SystemConstant.strNotNull(paperNumber) && mainNumber != null && subNumber != null) {
                excelErrorCommonNotice = String.format("课程代码为[%s],试卷编号为[%s],大题号为[%s],小题号为[%s]的数据异常 : ", courseCode,
                        paperNumber, mainNumber, subNumber);
                String questionNumberKey =
                        courseCode + SystemConstant.HYPHEN + paperNumber + SystemConstant.HYPHEN + mainNumber + SystemConstant.HYPHEN + subNumber;
                if (questionNumberCheckSet.contains(questionNumberKey)) {
                    excelErrorList.add("excel中题号重复");
                } else {
                    questionNumberCheckSet.add(questionNumberKey);
                }
                if (dbObjectiveQuestionNumberSet.contains(questionNumberKey)) {
                    excelErrorList.add("该题号已存在客观题中");
                }
            }

            // 2.校验课程代码,课程名称,试卷在考试下是否存在
            if (SystemConstant.strNotNull(courseCode) && SystemConstant.strNotNull(courseName) && SystemConstant.strNotNull(paperNumber)) {
                if (!courseCheckMap.containsKey(courseCode)) {
                    excelErrorList.add("课程代码不存在");
                } else if (!courseCheckMap.get(courseCode).contains(courseName)) {
                    excelErrorList.add("课程代码名称不匹配");
                }
                if (markPaperList.stream().noneMatch(
                        m -> m.getPaperNumber().equals(paperNumber) && m.getCourseCode().equals(courseCode) && m.getCourseName().equals(courseName))) {
                    excelErrorList.add("考试课程中不存在该试卷，请先创建试卷");
                }
            }

            // 3.评卷员校验
            if (Objects.isNull(markerCount) && Objects.isNull(markerCodes)) {
                excelErrorList.add("[评卷员数量（系统生成）]和[绑定工号（英文逗号分隔）]不能同时为空");
            }

            // 整理异常信息
            if (CollectionUtils.isNotEmpty(excelErrorList)) {
                if (SystemConstant.strNotNull(errorMsg)) {
                    errorMsg = errorMsg + "、" + excelErrorCommonNotice + String.join("、", excelErrorList);
                } else {
                    errorMsg = excelErrorCommonNotice + String.join("、", excelErrorList);
                }
                subjectiveStructDto.setErrorMsg(errorMsg);
            }
        }
        if (subjectiveStructDtoList.stream().anyMatch(e -> SystemConstant.strNotNull(e.getErrorMsg()))) {
            // 抛出excel解析异常
            throw ExceptionResultEnum.ERROR.exception(
                    subjectiveStructDtoList.stream().map(SubjectiveStructDto::getErrorMsg).filter(SystemConstant::strNotNull).collect(Collectors.joining(System.lineSeparator())));
        } else {
            List<String> paperErrorList = new ArrayList<>();
            // 试卷编号-> 题目集合 Map
            Map<String, List<SubjectiveStructDto>> paperQuestionsMap = subjectiveStructDtoList.stream().collect(Collectors.groupingBy(SubjectiveStructDto::getPaperNumber));
            paperQuestionsMap.forEach((paperNumber, paperQuestionList) -> {
                if (markGroupService.countByExamIdAndPaperNumber(examId, paperNumber) > 0) {
                    paperErrorList.add(String.format("[试卷编号]%s,已存在分组,请先删除该试卷的所有分组再导入试题", paperNumber));
                } else {
                    MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
                    Long courseId = markPaper.getCourseId();
                    BasicCourse basicCourse = basicCourseService.getById(courseId);
                    String orgName;
                    Long orgId;
                    if (Objects.isNull(basicCourse)) {
                        throw ExceptionResultEnum.ERROR.exception("无法找到课程");
                    } else {
                        Long teachingRoomId = basicCourse.getTeachingRoomId();
                        SysOrg sysOrg = sysOrgService.getById(teachingRoomId);
                        if (Objects.isNull(sysOrg)) {
                            throw ExceptionResultEnum.ERROR.exception("无法找到机构");
                        }
                        orgName = sysOrg.getName();
                        orgId = sysOrg.getId();
                    }

                    // 优先删除该试卷的主观题
                    markQuestionService.remove(
                            new QueryWrapper<MarkQuestion>().lambda().eq(MarkQuestion::getExamId, examId)
                                    .eq(MarkQuestion::getPaperNumber, paperNumber).eq(MarkQuestion::getObjective, false));

                    // 试卷下分组集合
                    List<Integer> groupNumberList = paperQuestionList.stream().map(SubjectiveStructDto::getGroupNumber)
                            .distinct().sorted().collect(Collectors.toList());

                    // 评卷员序号
                    AtomicInteger markerNumber = new AtomicInteger(0);
                    for (Integer groupNumber : groupNumberList) {
                        List<String> groupErrorList = new ArrayList<>();
                        // 分组下题目集合
                        List<SubjectiveStructDto> groupQuestionList = paperQuestionList.stream()
                                .filter(e -> e.getGroupNumber().equals(groupNumber)).collect(Collectors.toList());

                        String paperNumberGroupNumberNotice = String.format("[试卷编号]%s,[评卷分组(只能用小写数字)]%s,的", paperNumber,
                                groupNumber);

                        Integer markerCount = null;
                        String markerCodes = null;
                        List<Integer> markerCountList = groupQuestionList.stream().map(SubjectiveStructDto::getMarkerCount).distinct().collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(markerCountList)) {
                            if (markerCountList.size() > 1) {
                                groupErrorList.add("[评卷员数量（系统生成）]不一致");
                            } else if (markerCountList.size() == 1) {
                                markerCount = markerCountList.get(0);
                            }
                        }

                        List<String> markerCodesList = groupQuestionList.stream().map(SubjectiveStructDto::getMarkerCodes).distinct().collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(markerCodesList)) {
                            if (markerCodesList.size() > 1) {
                                groupErrorList.add("[绑定工号（英文逗号分隔）]不一致");
                            } else if (markerCodesList.size() == 1) {
                                markerCodes = markerCodesList.get(0);
                            }
                        }

                        // 组装评卷员信息
                        List<MarkUser> markUsers = new ArrayList<>();
                        if (SystemConstant.strNotNull(markerCodes)) {
                            // 优先以绑定工号为准
                            String[] markerCodeArray = markerCodes.split(SystemConstant.COMMA);
                            for (String markerCode : markerCodeArray) {
                                SysUser sysUser = sysUserService.getOne(
                                        new QueryWrapper<SysUser>().lambda().eq(SysUser::getSchoolId, schoolId).eq(SysUser::getLoginName, markerCode).last(SystemConstant.LIMIT1));
                                if (Objects.isNull(sysUser)) {
                                    groupErrorList.add(paperNumberGroupNumberNotice + String.format(
                                            "[绑定工号（英文逗号分隔）]中的%s,在系统中不存在请先创建用户", markerCode));
                                } else {
                                    // 添加评卷员信息
                                    MarkUser markUser = new MarkUser();
                                    markUser.setUserId(sysUser.getId());
                                    markUser.setLoginName(sysUser.getLoginName());
                                    markUser.setName(sysUser.getRealName());
                                    markUser.setOrgName(orgName);
                                    markUsers.add(markUser);
                                }
                            }
                        } else if (Objects.nonNull(markerCount) && markerCount > 0) {
                            // 没有绑定工号且评卷员生成数量大于0,生成评卷员（或使用之前工号相同的评卷员）
                            for (int i = 0; i < markerCount; i++) {
                                int num = markerNumber.incrementAndGet();
                                if (num > 99) {
                                    paperErrorList.add("[试卷编号]%s的[评卷员数量（系统生成）]总有效数量不能超过99");
                                    break;
                                }
                                String markerCode = paperNumber + String.format("%02d", num);

                                SysUser sysUser = sysUserService.getOne(
                                        new QueryWrapper<SysUser>().lambda().eq(SysUser::getSchoolId, schoolId).eq(SysUser::getLoginName, markerCode).last(SystemConstant.LIMIT1));

                                MarkUser markUser = new MarkUser();
                                markUser.setLoginName(markerCode);
                                markUser.setOrgName(orgName);
                                markUsers.add(markUser);
                                if (Objects.nonNull(sysUser)) {
                                    markUser.setUserId(sysUser.getId());
                                    markUser.setName(sysUser.getRealName());
                                } else {
                                    // 创建用户
                                    UserSaveParams userSaveParams = new UserSaveParams();
                                    userSaveParams.setSchoolId(schoolId);
                                    userSaveParams.setLoginName(markerCode);
                                    userSaveParams.setRealName(markerCode);
                                    userSaveParams.setCode(markerCode);
                                    userSaveParams.setPassword(SystemConstant.DEFAULT_PASSWORD);
                                    userSaveParams.setOrgId(orgId);
                                    userSaveParams.setEnable(true);
                                    userSaveParams.setRoleIds(new Long[]{markerRole.getId()});
                                    try {
                                        markUser.setUserId(sysUserService.saveUser(userSaveParams, requestUser.getId()));
                                        markUser.setName(markerCode);
                                    } catch (IllegalAccessException e) {
                                        e.printStackTrace();
                                    }
                                }
                            }
                        }
                        if (CollectionUtils.isEmpty(groupErrorList) && CollectionUtils.isEmpty(paperErrorList)) {
                            // 创建评卷试题
                            List<MarkQuestion> markQuestionList = paperQuestionList.stream().filter(e -> groupNumber.equals(e.getGroupNumber())).flatMap(e -> {
                                MarkQuestion markQuestion = new MarkQuestion();
                                markQuestion.setExamId(examId);
                                markQuestion.setPaperNumber(paperNumber);
                                markQuestion.setPaperType(OptionsEnum.A.getCode());
                                markQuestion.setObjective(false);
                                markQuestion.setMainNumber(e.getMainNumber());
                                markQuestion.setSubNumber(e.getSubNumber());
                                markQuestion.setMainTitle(e.getMainTitle());
                                markQuestion.setTotalScore(e.getTotalScore());
                                markQuestion.setIntervalScore(e.getIntervalScore());
                                markQuestion.setQuestionType(QuestionType.QUESTION_ANSWER.getValue());
                                return Stream.of(markQuestion);
                            }).collect(Collectors.toList());
                            markQuestionService.saveBatch(markQuestionList);

                            MarkGroupDto markGroupDto = new MarkGroupDto();
                            markGroupDto.setDoubleEnable(false);
                            markGroupDto.setGroupNumber(groupNumber);
                            markGroupDto.setMarkers(markUsers);
                            markGroupDto.setQuestions(markQuestionList);
                            markGroupDto.setPictureConfigs(new ArrayList<>());
                            MarkGroupSingleDto markGroupSingleDto = new MarkGroupSingleDto();
                            markGroupSingleDto.setExamId(examId);
                            markGroupSingleDto.setPaperNumber(paperNumber);
                            markGroupSingleDto.setGroupInfo(markGroupDto);
                            markGroupService.saveGroup(markGroupSingleDto);
                        }
                        if (CollectionUtils.isNotEmpty(groupErrorList)) {
                            String groupErrorMsg = paperNumberGroupNumberNotice + String.join(",", groupErrorList);
                            paperErrorList.add(groupErrorMsg);
                        }
                    }
                }
            });
            if (CollectionUtils.isNotEmpty(paperErrorList)) {
                throw ExceptionResultEnum.ERROR.exception(String.join(System.lineSeparator(), paperErrorList));
            }
            // 更改试卷结构状态为已提交
            Set<String> paperNumberSet = paperQuestionsMap.keySet();
            if (CollectionUtils.isNotEmpty(paperNumberSet)) {
                markPaperService.update(new UpdateWrapper<MarkPaper>().lambda().eq(MarkPaper::getExamId, examId)
                        .in(MarkPaper::getPaperNumber, paperNumberSet).set(MarkPaper::getQuestionStatus, true));
            }
        }
        return map;
    }
}
