package com.qmth.distributed.print.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.params.BasicExamStudentParam;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentPageQuery;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.templete.execute.AsyncBasicExamStudentTeacherImportService;
import com.qmth.distributed.print.business.templete.execute.AsyncBasicExamStudentImportService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.BasicExamStudentStatusEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.TaskTypeEnum;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 考生字典表 前端控制器
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@Api(tags = "考生管理controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_BASIC + "/exam_student")
public class BasicExamStudentController {

    @Resource
    private BasicExamStudentService basicExamStudentService;

    @Resource
    private PrintCommonService printCommonService;

    @Resource
    private AsyncBasicExamStudentImportService syncBasicExamStudentImportService;
    @Resource
    private AsyncBasicExamStudentTeacherImportService asyncBasicExamStudentTeacherImportService;

    @ApiOperation(value = "分页查询")
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = BasicExamStudentResult.class)})
    public Result findBasicExamStudentPage(@ApiParam(value = "查询条件") BasicExamStudentPageQuery query) {
        if (query.getSemesterId() == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择学期");
        }
        if (query.getExamId() == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择考试");
        }
        return ResultUtil.ok(basicExamStudentService.queryPage(query));
    }

    @ApiOperation(value = "新增/编辑")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.SAVE)
    public Result saveBasicExamStudent(@Valid @RequestBody BasicExamStudentParam basicExamStudentParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(basicExamStudentService.save(basicExamStudentParam, sysUser));
    }

    @ApiOperation(value = "状态更新")
    @RequestMapping(value = "/status", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "状态更新操作，id：{{id}}、状态：{{status}}。备注：N(\"正常\"),D(\"缓考\"),F(\"免考\"),M(\"缺考\"),B(\"违纪\")")
    public Result status(@ApiParam(value = "考生ID", required = true) @RequestParam Long id,
                         @ApiParam(value = "状态", required = true) @RequestParam BasicExamStudentStatusEnum status) {
        return ResultUtil.ok(basicExamStudentService.updateStatus(id, status));
    }

    @ApiOperation(value = "导入")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.IMPORT, detail = "导入考生操作")
    public Result basicExamStudentImportSync(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                             @ApiParam(value = "考试id", required = true) @RequestParam String examId) throws Exception {
        Map<String, Object> map = printCommonService.saveTask(file, SystemConstant.convertIdToLong(examId), TaskTypeEnum.BASIC_EXAM_STUDENT_IMPORT);
        map.put("examId", SystemConstant.convertIdToLong(examId));
        syncBasicExamStudentImportService.importTask(map);
        return ResultUtil.success(true);
    }

    @ApiOperation(value = "任课老师导入")
    @RequestMapping(value = "/import_teacher", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.IMPORT, detail = "任课老师导入操作")
    public Result importTeacher(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                @ApiParam(value = "考试id", required = true) @RequestParam String examId) throws Exception {
        Map<String, Object> map = printCommonService.saveTask(file, SystemConstant.convertIdToLong(examId), TaskTypeEnum.BASIC_EXAM_STUDENT_TEACHER_IMPORT);
        map.put("examId", SystemConstant.convertIdToLong(examId));
        asyncBasicExamStudentTeacherImportService.importTask(map);
        return ResultUtil.success(true);
    }

    @ApiOperation(value = "导出")
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.EXPORT)
    public void export(@ApiParam(value = "查询条件") BasicExamStudentPageQuery query) throws Exception {
        basicExamStudentService.exportLogic(query);
    }

    @ApiOperation(value = "删除时查询考生扫描阅卷状态")
    @RequestMapping(value = "/mark_status", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result status(@ApiParam(value = "考生id集合", required = true) @RequestParam List<Long> idList) {
        return ResultUtil.ok(basicExamStudentService.getStudentMarkStatus(idList));
    }

    @ApiOperation(value = "删除")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = Result.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.DELETE, detail = "删除操作，考生ID集合：{{idList}}")
    public Result deleteBasicExamStudent(@ApiParam(value = "考生id集合", required = true) @RequestParam List<Long> idList) {
        basicExamStudentService.deleteBatch(idList);
        return ResultUtil.ok();
    }
}