package com.qmth.distributed.print.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.enums.MakeMethodEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.MessageEnum;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 卷库查询 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "卷库查询Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_EXAM + "/task")
@Validated
public class ExamTaskQueryController {

    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private PrintCommonService printCommonService;
    @Resource
    private BasicMessageService basicMessageService;
    @Resource
    private ActivitiService activitiService;

    @ApiOperation(value = "卷库查询-分页查询")
    @RequestMapping(value = "/paper_list", method = RequestMethod.POST)
    public Result taskPaperList(@ApiParam(value = "学期ID") @RequestParam(value = "semesterId", required = false) Long semesterId,
                                @ApiParam(value = "考试ID") @RequestParam(value = "examId", required = false) Long examId,
                                @ApiParam(value = "开课学院id") @RequestParam(value = "openCollegeId", required = false) Long openCollegeId,
                                @ApiParam(value = "课程ID") @RequestParam(value = "courseId", required = false) Long courseId,
                                @ApiParam(value = "试卷编号") @RequestParam(value = "paperNumber", required = false) String paperNumber,
                                @ApiParam(value = "入库时间（开始）") @RequestParam(value = "startTime", required = false) Long startTime,
                                @ApiParam(value = "入库时间（结束）") @RequestParam(value = "endTime", required = false) Long endTime,
                                @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                @ApiParam(value = "分页参数") @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        if (semesterId == null) {
            throw ExceptionResultEnum.SEMESTER_ID_IS_NULL.exception();
        }
        if (examId == null) {
            throw ExceptionResultEnum.EXAM_ID_IS_NULL.exception();
        }
        return ResultUtil.ok(
                examTaskService.listTaskPaper(semesterId, examId, openCollegeId, courseId, paperNumber, startTime,
                        endTime, pageNumber, pageSize));
    }

    /**
     * 下载
     *
     * @param examTaskId 命题任务ID
     */
    @ApiOperation(value = "下载")
    @RequestMapping(value = "/paper_download", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.EXPORT, detail = "下载操作，命题任务ID：{{examTaskId}}")
    public void taskPaperDownload(HttpServletResponse response, @RequestParam Long examTaskId) throws Exception {
        examTaskService.paperDownload(response, examTaskId);
    }

    /**
     * 卷库查询-启用/禁用
     *
     * @param examTask 命题任务详情对象
     * @return Result
     */
    @ApiOperation(value = "卷库查询-启用/禁用")
    @RequestMapping(value = "/paper_enable", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.ENABLE, detail = "启用/禁用操作，命题任务ID：{{examTaskDetail.examTaskId}}")
    public Result taskPaperEnable(@RequestBody ExamTask examTask) throws IOException {
        boolean isSuccess = examTaskDetailService.enable(examTask);
        ExamTask examTaskById = examTaskService.getById(examTask.getId());
        // 启用，触发考场生成pdf
        if (examTaskById.getExamTaskDetailEnable()) {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

            // 校验课程关联考场，是否已经提交打印
            Boolean isCreate = printCommonService.checkExamDetailStatus(examTaskById.getSchoolId(), null, examTaskById.getExamId(), examTaskById.getCourseId(), examTaskById.getPaperNumber());
            // 校验是否可以提交打印状态
            if (isCreate) {
                printCommonService.checkExamDataAndCreatePdfTask(examTaskById.getSchoolId(), examTaskById.getExamId(), examTaskById.getCourseId(), examTaskById.getPaperNumber(), sysUser);
            }
        }
        return ResultUtil.ok(isSuccess);
    }

    @ApiOperation(value = "卷库查询-批量下载试卷PDF、题卡")
    @RequestMapping(value = "/paper_card_download_pdf", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.EXPORT, detail = "批量下载试卷PDF、题卡操作，考试ID：{{examId}}、课程代码：{{courseCode}}、试卷编号：{{paperNumber}}")
    public Result paperCardDownloadPdf(@ApiParam(value = "学期ID") @RequestParam(value = "semesterId", required = false) Long semesterId,
                                       @ApiParam(value = "考试ID") @RequestParam(value = "examId", required = false) Long examId,
                                       @ApiParam(value = "考试ID") @RequestParam(value = "openCollegeId", required = false) Long openCollegeId,
                                       @ApiParam(value = "课程ID") @RequestParam(value = "courseCode", required = false) Long courseId,
                                       @ApiParam(value = "试卷编号") @RequestParam(value = "paperNumber", required = false) String paperNumber,
                                       @ApiParam(value = "入库时间（开始）") @RequestParam(value = "startTime", required = false) Long startTime,
                                       @ApiParam(value = "入库时间（结束）") @RequestParam(value = "endTime", required = false) Long endTime) {
        if (semesterId == null) {
            throw ExceptionResultEnum.SEMESTER_ID_IS_NULL.exception();
        }
        if (examId == null) {
            throw ExceptionResultEnum.EXAM_ID_IS_NULL.exception();
        }
        TBTask tbTask = examTaskService.taskDownloadPdf(semesterId, examId, openCollegeId, courseId, paperNumber, startTime, endTime);
        return Objects.nonNull(tbTask) ? ResultUtil.ok(new EditResult(tbTask.getId())) : ResultUtil.error("创建任务失败");
    }

    /**
     * 卷库查询-作废
     *
     * @param id 命题任务ID
     * @return Result
     */
    @ApiOperation(value = "卷库查询-作废")
    @RequestMapping(value = "/cancel", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE)
    public Result cancel(@RequestParam Long id,
                         @RequestParam String cancelRemark) {
        boolean isSuccess = examTaskDetailService.cancel(id, cancelRemark);
        return ResultUtil.ok(isSuccess);
    }

    @ApiOperation(value = "卷库查询-打回")
    @RequestMapping(value = "/restart", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "打回操作，命题任务ID：{{examTaskDetail.examTaskId}}")
    public Result taskFlowRestart(@RequestBody ExamTaskDetail examTaskDetail) {
        Map<String, Object> map = examTaskDetailService.taskRestart(examTaskDetail);
        Boolean isSuccess = (Boolean) map.get(SystemConstant.SUCCESS);
        if (isSuccess) {
            ExamTask examTask = examTaskService.getById(examTaskDetail.getExamTaskId());
            if (Objects.nonNull(examTask.getReview()) && examTask.getReview()) {
                // 发送短信
                basicMessageService.sendNoticeTaskAuditCreateOrReview(examTask, MessageEnum.NOTICE_OF_AUDIT_REVIEW);
            } else {
               /* SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                // 校验课程关联考场，是否已经提交打印
                Boolean isCreate = printCommonService.checkExamDetailStatus(examTask.getSchoolId(), null, examTask.getCourseCode(), examTask.getPaperNumber());
                // 校验是否可以提交打印状态
                if (isCreate) {
                    printCommonService.checkData(examTask.getSchoolId(), examTask.getExamId(), examTask.getCourseCode(), examTask.getPaperNumber(), sysUser);
                }*/
            }
            Boolean sendFlowStartMq = (Boolean) map.get(SystemConstant.SEND_FLOW_START_MQ);
            Boolean sendFlowMq = (Boolean) map.get(SystemConstant.SEND_FLOW_MQ);
            if (Objects.nonNull(sendFlowStartMq) && sendFlowStartMq) {
                activitiService.sendFlowStartMsg(examTask.getFlowId());
            } else if (Objects.nonNull(sendFlowMq) && sendFlowMq) {
                activitiService.sendFlowTaskApproveMsg(map);
            }
        }
        return ResultUtil.ok(isSuccess);
    }
}

