package com.qmth.distributed.print.api;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.result.statistics.ScanCountResult;
import com.qmth.distributed.print.business.bean.result.statistics.PrintCountResult;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.distributed.print.business.enums.StatisticsStatusEnum;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * <p>
 * 统计管理 前端控制器
 * </p>
 */
@Api(tags = "统计管理Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_STATISTICS)
@Validated
public class StatisticalManagementController {

    @Resource
    ExamTaskService examTaskService;

    @ApiOperation(value = "印刷任务统计查询")
    @RequestMapping(value = "/print/count", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = PrintCountResult.class)})
    public Result printCount(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                                 @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "印刷计划ID") @RequestParam(required = false) Long printPlanId,
                                 @ApiParam(value = "开课学院id") @RequestParam(required = false) Long openCollegeId,
                                 @ApiParam(value = "课程ID") @RequestParam(required = false) Long courseId,
                                 @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                 @ApiParam(value = "题卡类型，GENERIC：通卡，CUSTOM：自定义") @RequestParam(required = false) CardTypeEnum cardType,
                                 @ApiParam(value = "状态，FINISH：已完成，UN_FINISH：未完成") @RequestParam(required = false) StatisticsStatusEnum finishStatus,
                                 @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                 @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(examTaskService.printCountList(new Page<>(pageNumber, pageSize), semesterId, examId, printPlanId, openCollegeId, courseId, paperNumber, cardType, finishStatus));
    }

    @ApiOperation(value = "印刷任务统计导出")
    @RequestMapping(value = "/print/count/export", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "导出成功", response = PrintCountResult.class)})
    public void printCountExport(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                                     @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                     @ApiParam(value = "印刷计划ID") @RequestParam(required = false) Long printPlanId,
                                     @ApiParam(value = "开课学院id") @RequestParam(required = false) Long openCollegeId,
                                     @ApiParam(value = "课程ID") @RequestParam(required = false) Long courseId,
                                     @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                     @ApiParam(value = "题卡类型，GENERIC：通卡，CUSTOM：自定义") @RequestParam(required = false) CardTypeEnum cardType,
                                     @ApiParam(value = "状态，FINISH：已完成，UN_FINISH：未完成") @RequestParam(required = false) StatisticsStatusEnum finishStatus) throws Exception {
        List<PrintCountResult> statementResultList = examTaskService.printCountListExport(semesterId, examId, printPlanId, openCollegeId, courseId, paperNumber, cardType, finishStatus);
        ExcelUtil.excelExport("印刷进度", PrintCountResult.class, statementResultList, ServletUtil.getResponse());
    }

    @ApiOperation(value = "扫描任务统计查询")
    @RequestMapping(value = "/scan/count", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = ScanCountResult.class)})
    public Result scanCount(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                                 @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "开课学院id") @RequestParam(required = false) Long openCollegeId,
                                 @ApiParam(value = "课程ID") @RequestParam(required = false) Long courseId,
                                 @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                 @ApiParam(value = "题卡类型，GENERIC：通卡，CUSTOM：自定义") @RequestParam(required = false) CardTypeEnum cardType,
                                 @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                 @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(examTaskService.scanCountList(new Page<>(pageNumber, pageSize), semesterId, examId, openCollegeId, courseId, paperNumber, cardType));
    }

    @ApiOperation(value = "扫描任务统计导出")
    @RequestMapping(value = "/scan/count/export", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "导出成功", response = ScanCountResult.class)})
    public void scanCountExport(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                                     @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                     @ApiParam(value = "开课学院id") @RequestParam(required = false) Long openCollegeId,
                                     @ApiParam(value = "课程ID") @RequestParam(required = false) Long courseId,
                                     @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                     @ApiParam(value = "题卡类型，GENERIC：通卡，CUSTOM：自定义") @RequestParam(required = false) CardTypeEnum cardType) throws Exception {
        List<ScanCountResult> printTaskResultList = examTaskService.scanCountListExport(semesterId, examId, openCollegeId, courseId, paperNumber, cardType);
        ExcelUtil.excelExport("扫描进度", ScanCountResult.class, printTaskResultList, ServletUtil.getResponse());
    }
}

