package com.qmth.distributed.print.listener;

import cn.hutool.core.date.DateUtil;
import com.qmth.distributed.print.business.entity.TSJobLog;
import com.qmth.distributed.print.business.service.TSJobLogService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.JobListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: job监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/9/22
 */
public class MyJobListener implements JobListener {

    private final static Logger log = LoggerFactory.getLogger(MyJobListener.class);

    @Override
    public String getName() {
        return "myJobListener";
    }

    @Override
    public void jobToBeExecuted(JobExecutionContext context) {
        log.info("定时任务:{}-开始执行", context.getTrigger().getJobKey());
    }

    @Override
    public void jobExecutionVetoed(JobExecutionContext context) {

    }

    @Override
    @Transactional
    public void jobWasExecuted(JobExecutionContext context, JobExecutionException jobException) {
        String jobEnum = context.getTrigger().getJobKey().getGroup();
        switch (jobEnum) {
            case "CREATE_PDF_JOB_GROUP":
                StringJoiner stringJoiner = new StringJoiner("，");
                stringJoiner.add("当前执行时间：" + DateUtil.format(context.getFireTime(), SystemConstant.DEFAULT_DATE_PATTERN));
                if (Objects.nonNull(context.getPreviousFireTime())) {
                    stringJoiner.add("上次执行时间：" + DateUtil.format(context.getPreviousFireTime(), SystemConstant.DEFAULT_DATE_PATTERN));
                }
                if (Objects.nonNull(context.getNextFireTime())) {
                    stringJoiner.add("下次执行时间：" + DateUtil.format(context.getNextFireTime(), SystemConstant.DEFAULT_DATE_PATTERN));
                }
                TSJobLogService tsJobLogService = SpringContextHolder.getBean(TSJobLogService.class);
                TSJobLog tsJobLog = new TSJobLog(jobEnum, context.getResult().toString(), stringJoiner.toString());
                tsJobLogService.save(tsJobLog);
                break;
            default:
                break;
        }
        log.info("定时任务:{}-执行结束", context.getTrigger().getJobKey());
    }
}