package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.itextpdf.text.Document;
import com.itextpdf.text.Image;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfWriter;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.CellValue;
import com.qmth.boot.tools.io.ZipWriter;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.mark.bean.StringResult;
import com.qmth.teachcloud.mark.bean.UpdateTimeVo;
import com.qmth.teachcloud.mark.bean.archivescore.*;
import com.qmth.teachcloud.mark.bean.omredit.OmrEditDomain;
import com.qmth.teachcloud.mark.bean.omredit.OmrEditPaper;
import com.qmth.teachcloud.mark.bean.scananswer.*;
import com.qmth.teachcloud.mark.bean.scanexaminfo.CheckTask;
import com.qmth.teachcloud.mark.bean.scanexaminfo.ScanExamCheckInfoVo;
import com.qmth.teachcloud.mark.bean.scanexaminfo.ScanExamInfoVo;
import com.qmth.teachcloud.mark.bean.student.AbsentManualUpdateVo;
import com.qmth.teachcloud.mark.bean.student.MarkStudentQuery;
import com.qmth.teachcloud.mark.bean.student.StudentQuery;
import com.qmth.teachcloud.mark.bean.student.StudentVo;
import com.qmth.teachcloud.mark.dto.ScanOmrTaskStatusDto;
import com.qmth.teachcloud.mark.dto.UnexistStudentDto;
import com.qmth.teachcloud.mark.dto.mark.*;
import com.qmth.teachcloud.mark.dto.mark.manage.Task;
import com.qmth.teachcloud.mark.dto.mark.score.*;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.*;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.mapper.MarkStudentMapper;
import com.qmth.teachcloud.mark.params.MarkHeaderResult;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.BatchGetDataUtil;
import com.qmth.teachcloud.mark.utils.Calculator;
import com.qmth.teachcloud.mark.utils.ScoreCalculateUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.awt.*;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.io.*;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.List;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试考生库 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkStudentServiceImpl extends ServiceImpl<MarkStudentMapper, MarkStudent> implements MarkStudentService {
    @Resource
    private SysUserService sysUserService;

    @Resource
    private MarkPaperService markPaperService;

    @Resource
    private MarkPaperPackageService markPaperPackageService;

    @Resource
    private ScanPackageService scanPackageService;

    @Resource
    private ScanPaperService scanPaperService;

    @Resource
    private ScanPaperPageService scanPaperPageService;

    @Resource
    private ScanOmrTaskService scanOmrTaskService;

    @Resource
    private ScanAnswerCardService answerCardService;

    @Resource
    private ScanStudentPaperService studentPaperService;

    @Resource
    private MarkQuestionService markQuestionService;

    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Resource
    private ConcurrentService concurrentService;

    @Resource
    private MarkService markService;

    @Resource
    private LockService lockService;

    @Resource
    private TaskService taskService;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    private MarkSubjectiveScoreService markSubjectiveScoreService;

    @Resource
    private MarkSyncService markSyncService;

    @Resource
    private FileUploadService fileUploadService;

    @Resource
    private MarkTrackService markTrackService;

    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private ScanStudentPaperService scanStudentPaperService;
    @Resource
    private ScanBatchPaperService scanBatchPaperService;
    @Resource
    private BasicCourseService basicCourseService;

    @Override
    public List<String> listClassByExamIdAndCourseCode(Long examId, String paperNumber) {
        MarkStudentQuery markStudentQuery = new MarkStudentQuery();
        markStudentQuery.setExamId(examId);
        markStudentQuery.setPaperNumber(paperNumber);
        List<MarkStudentVo> markStudentList = this.listMarkStudentVo(markStudentQuery);

        List<String> classNameList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(markStudentList)) {
            classNameList = markStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getTeachClassName()))
                    .map(MarkStudentVo::getTeachClassName).distinct().collect(Collectors.toList());
        }
        return classNameList;
    }

    @Override
    public void updateSubjectiveStatusAndScore(Long studentId, SubjectiveStatus status, Double score,
                                               String scoreList) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getSubjectiveStatus, status).set(MarkStudent::getSubjectiveScore, score)
                .set(MarkStudent::getSubjectiveScoreList, scoreList).eq(MarkStudent::getId, studentId);
        this.update(updateWrapper);
    }

    @Override
    public void updateSubjectiveStatusAndScore(Long examId, String paperNumber, SubjectiveStatus status, Double score,
                                               String scoreList) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getSubjectiveStatus, status).set(MarkStudent::getSubjectiveScore, score)
                .set(MarkStudent::getSubjectiveScoreList, scoreList)
                .set(MarkStudent::getCheckUserId, null)
                .set(MarkStudent::getCheckTime, null).eq(MarkStudent::getExamId, examId)
                .eq(MarkStudent::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public ScanExamInfoVo getScanExamInfo(BasicExam exam, Long courseId, String coursePaperId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(),
                sysUser.getId(), ServletUtil.getRequest().getServletPath());
        ScanExamInfoVo vo = new ScanExamInfoVo();
        vo.setId(exam.getId());
        vo.setName(exam.getName());
        vo.getAnswerScan().setCourseCount(
                markPaperService.getCourseCount(exam.getId(), courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        vo.getAnswerScan().setPaperNumberCount(markPaperService.getPaperNumberCount(exam.getId(), courseId,
                coursePaperId, MarkPaperStatus.FORMAL, dpr));
        vo.getAnswerScan()
                .setTotalCount(getCount(exam.getId(), null, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        vo.getAnswerScan().setScannedCount(
                getCount(exam.getId(), ScanStatus.SCANNED, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        vo.getPackageScan().setTotalCount(markPaperPackageService.getPackageCountByExamId(exam.getId(), courseId,
                coursePaperId, MarkPaperStatus.FORMAL, dpr));
        vo.getPackageScan().setScannedCount(
                scanPackageService.getCount(exam.getId(), courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        return vo;
    }

    @Override
    public IPage<StudentScoreDetailDto> pageStudentScore(Long examId, String paperNumber, String college, String majorName, String teachClassName,
                                                         String className, String teacher, Integer filter, String status, Boolean breach, Double startScore,
                                                         Double endScore, Double subjectiveStartScore, Double subjectiveEndScore, Double objectiveStartScore, Double objectiveEndScore,
                                                         Double subScore, Integer objectiveScoreRateLt, String studentName, String studentCode, String secretNumber,
                                                         String orderType, String orderField, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (startScore != null && endScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入试卷总分结束分数值");
        }
        if (objectiveStartScore != null && objectiveEndScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入客观题总分结束分数值");
        }
        if (subjectiveStartScore != null && subjectiveEndScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入主观题总分结束分数值");
        }
        college = SystemConstant.translateSpecificSign(college);
        majorName = SystemConstant.translateSpecificSign(majorName);
        className = SystemConstant.translateSpecificSign(className);
        teacher = SystemConstant.translateSpecificSign(teacher);

        Page<StudentScoreDetailDto> page = new Page<>(pageNumber, pageSize);
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        Double objectiveScoreLt = objectiveScoreRateLt == null ?
                null :
                Calculator.round(Calculator.divide(Calculator.multiply(markPaper.getObjectiveScore(), Double.parseDouble(String.valueOf(objectiveScoreRateLt))), 100), 2);
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/setting/scoreList");
        IPage<StudentScoreDetailDto> studentScoreDetailDtoIPage = this.baseMapper.pageStudentScore(page, examId,
                paperNumber, college, majorName, teachClassName, className, teacher, filter, status, breach, startScore, endScore,
                subjectiveStartScore, subjectiveEndScore, objectiveStartScore, objectiveEndScore, subScore,
                objectiveScoreLt, studentName, studentCode, secretNumber, orderType, orderField, dpr);
        for (StudentScoreDetailDto scoreDetailDto : studentScoreDetailDtoIPage.getRecords()) {
            // 原图
            scoreDetailDto.setSheetUrls(buildSheetUrls(scoreDetailDto.getStudentId()));
            scoreDetailDto.setSheetPath(null);
            // 状态
//            if (ScanStatus.UNEXIST.equals(scoreDetailDto.getScanStatus())) {
//                scoreDetailDto.setStatusDisplay("未扫描");
//            } else if (ScanStatus.MANUAL_ABSENT.equals(scoreDetailDto.getScanStatus()) || scoreDetailDto.getAbsent() || scoreDetailDto.getOmrAbsent()) {
//                scoreDetailDto.setStatusDisplay("缺考");
//            } else if (!scoreDetailDto.getAbsent() && !scoreDetailDto.getOmrAbsent() && scoreDetailDto.getUpload() && ScanStatus.SCANNED.equals(scoreDetailDto.getScanStatus())) {
//                scoreDetailDto.setStatusDisplay("正常");
//            }

            // 扫描状态
            if (BasicExamStudentStatusEnum.M.equals(scoreDetailDto.getStudentStatus()) || scoreDetailDto.getAbsent() || scoreDetailDto.getOmrAbsent() || scoreDetailDto.getManualAbsent()) {
                scoreDetailDto.setStudentStatusDisplay("缺考");
            } else if (ScanStatus.UNEXIST.equals(scoreDetailDto.getScanStatus())) {
                scoreDetailDto.setStudentStatusDisplay(ScanStatus.UNEXIST.getName());
            } else {
                scoreDetailDto.setStudentStatusDisplay("正常");
            }

            // 违纪状态
            if (BasicExamStudentStatusEnum.B.equals(scoreDetailDto.getStudentStatus()) || scoreDetailDto.getBreach() || scoreDetailDto.getOmrBreach() || scoreDetailDto.getManualBreach()) {
                scoreDetailDto.setBreachDisplay("违纪");
            }

            // 主观题检查标记
            scoreDetailDto.setSubjectiveCheckFlag(
                    !scoreDetailDto.getAbsent() && !scoreDetailDto.getOmrAbsent() && scoreDetailDto.getUpload() && ScanStatus.SCANNED.equals(scoreDetailDto.getScanStatus())
                            && SubjectiveStatus.MARKED.equals(scoreDetailDto.getSubjectiveStatus()) && StringUtils.isNotBlank(scoreDetailDto.getSubjectiveScore()) && StringUtils.isNotBlank(
                            scoreDetailDto.getSubjectiveScoreList()));
            // 客观题检查标记
            scoreDetailDto.setObjectiveCheckFlag(
                    !scoreDetailDto.getAbsent() && !scoreDetailDto.getManualAbsent() && !scoreDetailDto.getOmrAbsent() && scoreDetailDto.getUpload() && ScanStatus.SCANNED.equals(scoreDetailDto.getScanStatus()) && StringUtils.isNotBlank(
                            scoreDetailDto.getObjectiveScore()) && StringUtils.isNotBlank(scoreDetailDto.getObjectiveScoreList()));

            // 格式化分数
            scoreDetailDto.setObjectiveScore(Calculator.round(scoreDetailDto.getObjectiveScore(), 1));
            scoreDetailDto.setSubjectiveScore(Calculator.round(scoreDetailDto.getSubjectiveScore(), 1));
            scoreDetailDto.setTotalScore(Calculator.round(scoreDetailDto.getTotalScore(), 1));
        }
        return studentScoreDetailDtoIPage;
    }

    /**
     * 评卷管理/成绩检查/成绩详情导出
     */
    @Override
    public List<StudentScoreDetailDto> pageStudentScoreExport(Long examId, String paperNumber, String college, String majorName, String teachClassName, String className, String teacher, Integer filter, String status, Boolean breach, Double startScore, Double endScore, Double subjectiveStartScore, Double subjectiveEndScore, Double objectiveStartScore, Double objectiveEndScore, Double subScore, Integer objectiveScoreRateLt, String studentName, String studentCode, String secretNumber, String orderType, String orderField) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (startScore != null && endScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入试卷总分结束分数值");
        }
        if (objectiveStartScore != null && objectiveEndScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入客观题总分结束分数值");
        }
        if (subjectiveStartScore != null && subjectiveEndScore == null) {
            throw ExceptionResultEnum.ERROR.exception("请输入主观题总分结束分数值");
        }
        college = SystemConstant.translateSpecificSign(college);
        majorName = SystemConstant.translateSpecificSign(majorName);
        className = SystemConstant.translateSpecificSign(className);
        teacher = SystemConstant.translateSpecificSign(teacher);

        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        Double objectiveScoreLt = objectiveScoreRateLt == null ?
                null :
                Calculator.round(Calculator.divide(Calculator.multiply(markPaper.getObjectiveScore(), Double.parseDouble(String.valueOf(objectiveScoreRateLt))), 100), 2);
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/setting/scoreList");
        List<StudentScoreDetailDto> studentScoreDetailDtoList = this.baseMapper.pageStudentScoreExport(examId,
                paperNumber, college, majorName, teachClassName, className, teacher, filter, status, breach, startScore, endScore,
                subjectiveStartScore, subjectiveEndScore, objectiveStartScore, objectiveEndScore, subScore,
                objectiveScoreLt, studentName, studentCode, secretNumber, orderType, orderField, dpr);
        Set<StudentScoreDetailExportDto> studentScoreDetailExportDtoSet = new HashSet<>();
        for (StudentScoreDetailDto scoreDetailDto : studentScoreDetailDtoList) {
            studentScoreDetailExportDtoSet.add(new StudentScoreDetailExportDto(scoreDetailDto));

            List<MarkTask> markTaskList = markTaskService.list(new QueryWrapper<MarkTask>().lambda().eq(MarkTask::getStudentId, scoreDetailDto.getStudentId()));
            if (!CollectionUtils.isEmpty(markTaskList)) {
                for (MarkTask m : markTaskList) {
                    if (Objects.nonNull(m.getStatus()) && m.getStatus() != MarkTaskStatus.MARKED) {
                        throw ExceptionResultEnum.ERROR.exception("[" + scoreDetailDto.getStudentName() + "]评卷任务未完成，请完成评卷任务并检查成绩后再导出");
                    }
                }
            }

            // 格式化分数
            scoreDetailDto.setObjectiveScore(Calculator.round(scoreDetailDto.getObjectiveScore(), 1));
            scoreDetailDto.setSubjectiveScore(Calculator.round(scoreDetailDto.getSubjectiveScore(), 1));
            scoreDetailDto.setTotalScore(Calculator.round(scoreDetailDto.getTotalScore(), 1));

            // 扫描状态
            if (BasicExamStudentStatusEnum.M.equals(scoreDetailDto.getStudentStatus()) || scoreDetailDto.getAbsent() || scoreDetailDto.getOmrAbsent() || scoreDetailDto.getManualAbsent()) {
                scoreDetailDto.setStudentStatusDisplay("缺考");
            } else if (ScanStatus.UNEXIST.equals(scoreDetailDto.getScanStatus())) {
                scoreDetailDto.setStudentStatusDisplay(ScanStatus.UNEXIST.getName());
            } else {
                scoreDetailDto.setStudentStatusDisplay("正常");
            }

            // 违纪状态
            if (BasicExamStudentStatusEnum.B.equals(scoreDetailDto.getStudentStatus()) || scoreDetailDto.getBreach() || scoreDetailDto.getOmrBreach() || scoreDetailDto.getManualBreach()) {
                scoreDetailDto.setBreachDisplay("违纪");
            }
        }

        if (!CollectionUtils.isEmpty(studentScoreDetailExportDtoSet)) {
            for (StudentScoreDetailExportDto s : studentScoreDetailExportDtoSet) {
                markPaperService.finishPaper(s.getExamId(), s.getPaperNumber(), s.getTeachClassName());
            }
        }

        HttpServletResponse response = ServletUtil.getResponse();
        //生成表头
        String[] columnName = new String[]{"姓名", "学号", "学院", "教学班", "行政班", "课程代码", "课程名称", "状态", "违纪状态", "客观分", "主观分", "成绩"};
        List<MarkQuestion> oQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, true);
        List<MarkQuestion> sQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, null, false);
        List<String> columnNameList = new ArrayList<>(Arrays.asList(columnName));
        for (MarkQuestion question : oQuestionList) {
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "选项");
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "得分");
        }
        for (MarkQuestion question : sQuestionList) {
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber());
        }
        String[] columnNames = columnNameList.toArray(new String[0]);
        //生成动态内容
        List<String[]> columnValues = new ArrayList<>();
        for (StudentScoreDetailDto s : studentScoreDetailDtoList) {
            List<String> valueList = new ArrayList<>();
            valueList.add(s.getStudentName());
            valueList.add(s.getStudentCode());
            valueList.add(s.getCollegeName());
            valueList.add(s.getTeachClassName());
            valueList.add(s.getClassName());
            valueList.add(s.getCourseCode());
            valueList.add(s.getCourseName());
            // 扫描状态
            valueList.add(s.getStudentStatusDisplay());

            // 违纪状态
            valueList.add(s.getBreachDisplay());
            valueList.add(s.getObjectiveScore() == null ? "" : s.getObjectiveScore().toString());
            valueList.add(s.getSubjectiveScore() == null ? "" : s.getSubjectiveScore().toString());
            valueList.add(s.getTotalScore() == null ? "" : s.getTotalScore().toString());
            for (ScoreItem item : s.getScoreList(true, oQuestionList)) {
                valueList.add(item.getAnswer());
                valueList.add(item.getScore() == null ? "" : item.getScore().toString());
            }
            for (ScoreItem item : s.getScoreList(false, sQuestionList)) {
                valueList.add(item.getScore().toString());
            }
            String[] columnValue = valueList.toArray(new String[valueList.size()]);
            columnValues.add(columnValue);
        }
        try {
            log.debug("导出Excel开始...");
            response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode("成绩导出", SystemConstant.CHARSET_NAME) + ".xlsx");
            response.setContentType("application/vnd.ms-excel");
            ServletOutputStream outputStream = response.getOutputStream();
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeStringArrays("成绩导出", null, columnNames, columnValues.listIterator());
            writer.output(outputStream);
            outputStream.flush();
            outputStream.close();
            log.debug("导出Excel结束");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return studentScoreDetailDtoList;
    }

    @Override
    public List<SheetUrlDto> buildSheetUrls(Long studentId) {
        // 原图
        List<SheetUrlDto> sheetUrls = new ArrayList<>();
        List<StudentPaperDetailDto> studentPaperDetailDtoList = scanPaperService.listStudentPaperDetail(studentId);
        for (int i = 0; i < studentPaperDetailDtoList.size(); i++) {
            StudentPaperDetailDto studentPaperDetailDto = studentPaperDetailDtoList.get(i);
            sheetUrls.add(new SheetUrlDto(
                    2 * (studentPaperDetailDto.getPaperIndex() - 1) + studentPaperDetailDto.getPageIndex(),
                    teachcloudCommonService.filePreview(studentPaperDetailDto.getSheetPath()),
                    studentPaperDetailDto.getRecogData()));
        }
        return sheetUrls;
    }

    private int getCount(Long examId, ScanStatus status, Long courseId, String coursePaperId,
                         MarkPaperStatus markPaperStatus, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setScanStatus(status);
        if (ScanStatus.UNEXIST.equals(status)) {
            markStudent.setManualAbsent(false);
        }
        markStudent.setMarkPaperStatus(markPaperStatus.name());
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    private int getManualAbsentCount(Long examId, Long courseId, String coursePaperId,
                                     MarkPaperStatus markPaperStatus, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setScanStatus(ScanStatus.UNEXIST);
        markStudent.setManualAbsent(true);
        markStudent.setMarkPaperStatus(markPaperStatus.name());
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    private int getMissScanCount(Long examId, Long courseId, String coursePaperId,
                                 MarkPaperStatus markPaperStatus, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setMissScan(true);
        markStudent.setMarkPaperStatus(markPaperStatus.name());
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    private int getOmrAbsentCount(Long examId, Boolean checked, Long courseId, String coursePaperId,
                                  MarkPaperStatus status, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setOmrAbsent(true);
        markStudent.setOmrAbsentChecked(checked);
        markStudent.setMarkPaperStatus(status.name());
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    private int getIncompleteCount(Long examId, Long courseId, String coursePaperId, MarkPaperStatus status,
                                   DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setIncomplete(true);
        markStudent.setMarkPaperStatus(status.name());
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    @Override
    public int getPaperTypeCheckCount(Long examId, Long courseId, String coursePaperId, PaperTypeCheckStatus status, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setScanStatus(ScanStatus.SCANNED);
        markStudent.setPaperTypeCheckStatus(status);
        return baseMapper.selectCountByQuery(markStudent, dpr);
    }

    @Override
    public ScanExamCheckInfoVo checkInfo(BasicExam exam, Long courseId, String coursePaperId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(),
                sysUser.getId(), ServletUtil.getRequest().getServletPath());
        Long examId = exam.getId();
        ScanExamCheckInfoVo vo = new ScanExamCheckInfoVo();
        vo.setId(exam.getId());
        vo.setName(exam.getName());
        CheckTask ct = vo.getCheckTask();
        ct.setUnexistCount(getCount(examId, ScanStatus.UNEXIST, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setUnexistCheckedCount(getManualAbsentCount(examId, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setMissScanCount(getMissScanCount(examId, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setAssignedCount(getAssignedCount(examId, false, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setAssignedCheckedCount(
                getAssignedCount(examId, true, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setAbsentCheckCount(getOmrAbsentCount(examId, false, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setAbsentCheckedCount(getOmrAbsentCount(examId, true, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));
        ct.setObjectiveCheckCount(
                scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, courseId, coursePaperId, OmrTaskStatus.WAITING.name(), dpr));
        ct.setObjectiveCheckedCount(
                scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, courseId, coursePaperId, OmrTaskStatus.PROCESSED.name(), dpr));
        ct.setIncompleteCount(getIncompleteCount(examId, courseId, coursePaperId, MarkPaperStatus.FORMAL, dpr));

        ct.setPaperTypeCheckCount(getPaperTypeCheckCount(examId, courseId, coursePaperId, PaperTypeCheckStatus.WAITING, dpr));
        ct.setPaperTypeCheckedCount(getPaperTypeCheckCount(examId, courseId, coursePaperId, PaperTypeCheckStatus.PROCESSED, dpr));
        return vo;
    }

    /**
     * 根据考生当前绑定的paper刷新考生状态，需要在外部调用处对考生上锁
     */
    @Override
    @Transactional
    public void updateStudentByPaper(@NotNull Long userId, @NotNull Long studentId, @NotNull boolean updateOmrTask, @NotNull boolean omrEdit, @NotNull boolean omrAssigned) {
        MarkStudent student = this.getById(studentId);
        if (student == null) {
            throw new ParameterException("找不到对应的考生");
        }

        Long examId = student.getExamId();
        String paperNumber = student.getPaperNumber();
        AbInfoVo abInfoVo = this.findExamTaskPaperTypeOpenStatus(examId, paperNumber);

        // 重置状态
        student.setIncomplete(false);
        student.setInvalid(false);
        student.setQuestionFilled(false);
        int paperCount = 0;
        List<ScanStudentPaper> studentPaperList = studentPaperService.findByStudentId(studentId);
        List<String> objectiveAnswers = new ArrayList<>();
        for (ScanStudentPaper studentPaper : studentPaperList) {
            paperCount++;
            // 获取paper详情更新考生状态
            ScanPaper paper = scanPaperService.getById(studentPaper.getPaperId());
            // 二次识别不进人工绑定检查
            if (omrAssigned) {
                student.setAssigned(paper.getAssigned());
                student.setAssignConfirmed(false);
            }
            student.setInvalid(student.getInvalid() || paper.getInvalid());
            student.setQuestionFilled(student.getQuestionFilled() || paper.getQuestionFilled());
            student.setCardNumber(paper.getCardNumber());
            // 单独判断首张纸正面的识别结果
            if (studentPaper.getPaperIndex() == 1) {
                // 根据识别结果更新考生属性
                ScanPaperPage page = scanPaperPageService.findPaperIdAndIndex(paper.getId(), 1);
                if (!omrEdit) {
                    student.setOmrAbsent(page.getAbsent() != null && page.getAbsent().getResult());
                    student.setOmrBreach(page.getBreach() != null && page.getBreach().getResult());
                    student.setOmrAbsentChecked(false);
                    student.setManualAbsent(false);
                    student.setManualBreach(false);
                    if (student.getOmrAbsent()) {
                        student.setObjectiveScore(null);
                        student.setObjectiveScoreList(null);
                    }
                } else {
                    // 缺考时，校验缺考检查任务是否做完
                    if (student.getOmrAbsent() && !student.getOmrAbsentChecked()) {
                        throw ExceptionResultEnum.ERROR.exception("缺考检查任务未完成，不能更新为缺考");
                    }
                    // 缺考时，校验人工绑定任务是否做完
                    if (student.getAssigned() && !student.getAssignConfirmed()) {
                        throw ExceptionResultEnum.ERROR.exception("人工绑定任务未完成，不能更新为缺考");
                    }
                    // 缺考时，校验卷型检查任务是否做完
                    if (PaperTypeCheckStatus.WAITING.equals(student.getPaperTypeCheckStatus())) {
                        throw ExceptionResultEnum.ERROR.exception("卷型检查任务未完成，不能更新为缺考");
                    }
                    // 缺考时，校验客观题检查任务是否全部做完
                    if (page.getAbsent() != null && page.getAbsent().getResult()) {
                        if (scanOmrTaskService.getStudentTaskCount(studentId, OmrTaskStatus.WAITING) > 0) {
                            throw ExceptionResultEnum.ERROR.exception("客观题检查任务未完成，不能更新为缺考");
                        }
                    }

                    // 人工指定后，识别缺考未处理的数据，不再处理，已处理的，不变
                    student.setOmrAbsent(false);
                    student.setManualAbsent(page.getAbsent() != null && page.getAbsent().getResult());
                }
                student.setAbsent(student.getOmrAbsent() || (student.getManualAbsent() != null && student.getManualAbsent()));
                student.setBreach(student.getOmrBreach() || (student.getManualBreach() != null && student.getManualBreach()));

                // 根据卷型识别结果更新考生卷型，如果卷型识别结果异常则添加到卷型检查中
                StringResult paperTypeResult = page.getPaperType();

                if (abInfoVo != null && abInfoVo.getOpenAb() && PaperTypeEnum.FILL.equals(abInfoVo.getPaperTypeStyle())) {
                    if (Objects.isNull(paperTypeResult)) {
                        // 卷型为空 (正常不会出现这种情况)
                        throw ExceptionResultEnum.ERROR.exception("开启卷型的试卷未检测到识别的卷型");
                    }
                    String paperType = paperTypeResult.getResult();
                    if (!SystemConstant.strNotNull(paperType)) {
                        // 卷型为空 (正常不会出现这种情况)
                        throw ExceptionResultEnum.ERROR.exception("开启卷型的试卷未检测到识别的卷型");
                    }
                    if (!paperType.matches("[A-Z#]+")) {
                        // 未识别到卷型 (正常不会出现这种情况)
                        throw ExceptionResultEnum.ERROR.exception("卷型识别错误");
                    }
                    student.setPaperType(paperType);
                    if (!omrEdit) {
                        if (paperType.length() > 1 || "#".equals(paperType)) {
                            // 多卷型、没有识别到卷型
                            student.setPaperTypeCheckStatus(PaperTypeCheckStatus.WAITING);
                        } else {
                            student.setPaperTypeCheckStatus(PaperTypeCheckStatus.NORMAL);
                        }
                    } else {
                        // 卷型检查，待检查的数据，不再处理，已处理的，不变
                        if (PaperTypeCheckStatus.WAITING.equals(student.getPaperTypeCheckStatus())) {
                            student.setPaperTypeCheckStatus(PaperTypeCheckStatus.NORMAL);
                        }
                    }
                }

            }

            List<ScanPaperPage> scanPaperPages = scanPaperPageService.listByPaperId(studentPaper.getPaperId());
            for (ScanPaperPage scanPaperPage : scanPaperPages) {
                if (scanPaperPage.getQuestion() != null && CollectionUtils.isNotEmpty(scanPaperPage.getQuestion().getResult())) {
                    for (String s : scanPaperPage.getQuestion().getResult()) {
                        if (s.startsWith("?")) {
                            s = s.replace("?", "");
                        }
                        objectiveAnswers.add(s);
                    }
                }
            }
        }
        student.setAnswers(JSON.toJSONString(objectiveAnswers));
        // 更新考生状态
        if (paperCount > 0) {
            ScanAnswerCard answerCard = answerCardService.findByExamAndNumber(student.getExamId(),
                    student.getCardNumber());
            student.setUpload(true);
            student.setIncomplete(paperCount != answerCard.getPaperCount());
            student.setScanStatus(ScanStatus.SCANNED);
            student.setMissScan(false);
            // 更新图片数量和图片地址
            updateStudentSheetInfo(student, studentPaperList);
        } else {
            student.setScanStatus(ScanStatus.UNEXIST);
        }
//        if (omrEdit) {
//            boolean absent = student.getOmrAbsent();
//            student.setAbsent(absent);
//            student.setBreach(student.getOmrBreach());
//            if (absent) {
//                // 缺考的缺考检查改为已处理
//                student.setOmrAbsentChecked(true);
//            }
//            student.setPaperTypeCheckStatus(PaperTypeCheckStatus.PROCESSED);
//        }
        // student.setUpdaterId(userId);
        // student.setUpdateTime(System.currentTimeMillis());
        this.saveOrUpdate(student);
        // 客观题统分
        this.calculateObjectiveScore(student);

        // 更新课程表中上传人数
        markPaperService.updateUploadCount(student.getExamId(), student.getPaperNumber(),
                this.countUploadedByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber()));
        markPaperService.updateAbsentCount(student.getExamId(), student.getPaperNumber(),
                this.countAbsentByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber()));
        if (updateOmrTask) {
            // 清除识别对照任务
            scanOmrTaskService.deleteByStudentId(student.getExamId(), student.getId());
            // 重新生成识别对照任务
            if (!student.getManualAbsent()) {
                scanOmrTaskService.saveTask(student.getId());
            }
        }
    }

    private MarkStudent updateStudentSheetInfo(MarkStudent student, List<ScanStudentPaper> studentPaperList) {
        List<FilePathVo> filePathVoList = new ArrayList<>();
        for (ScanStudentPaper scanStudentPaper : studentPaperList) {
            List<ScanPaperPage> scanPaperPages = scanPaperPageService.listByPaperId(scanStudentPaper.getPaperId());
            for (ScanPaperPage scanPaperPage : scanPaperPages) {
                String sheetPath = scanPaperPage.getSheetPath();
                if (StringUtils.isNotBlank(sheetPath)) {
                    filePathVoList.add(JSON.parseObject(sheetPath, FilePathVo.class));
                }
            }
        }
        student.setSheetCount(filePathVoList.size());
        student.setSheetPath(JSON.toJSONString(filePathVoList));
        return student;
    }

    @Override
    public MarkStudent findByExamIdAndCoursePaperIdAndStudentCode(Long examId, String coursePaperId, String studentCode) {
        return baseMapper.selectByExamIdAndCoursePaperIdAndStudentCode(examId, coursePaperId, studentCode);
    }

    @Override
    public MarkStudent findByExamIdAndPaperNumberAndStudentCode(Long examId, String paperNumber, String studentCode) {
        QueryWrapper<MarkStudent> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkStudent> lw = wrapper.lambda();
        lw.eq(MarkStudent::getExamId, examId);
        lw.eq(MarkStudent::getPaperNumber, paperNumber);
        if (StringUtils.isNotBlank(studentCode)) {
            lw.eq(MarkStudent::getStudentCode, studentCode);
        } else {
            lw.last("limit 1");
        }
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<MarkStudent> listByStudentId(Long studentId) {
        QueryWrapper<MarkStudent> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(MarkStudent::getBasicStudentId, studentId);
        return this.list(wrapper);
    }

    @Override
    public StudentObjectiveDetailDto getObjectiveInspectedTask(Long studentId) {
        MarkStudentVo markStudent = this.getMarkStudentVoByStudentId(studentId);
        StudentObjectiveDetailDto studentObjectiveDetailDto = new StudentObjectiveDetailDto();
        if (markStudent != null) {
            studentObjectiveDetailDto.setStudentId(markStudent.getId());
            studentObjectiveDetailDto.setStudentName(markStudent.getStudentName());
            studentObjectiveDetailDto.setStudentCode(markStudent.getStudentCode());
            studentObjectiveDetailDto.setExamPlace(markStudent.getExamPlace());
            studentObjectiveDetailDto.setExamRoom(markStudent.getExamRoom());
            studentObjectiveDetailDto.setExamId(markStudent.getExamId());
            studentObjectiveDetailDto.setCourseCode(markStudent.getCourseCode());
            studentObjectiveDetailDto.setCourseName(markStudent.getCourseName());
            studentObjectiveDetailDto.setPaperNumber(markStudent.getPaperNumber());
            studentObjectiveDetailDto
                    .setObjectiveScore(markStudent.getObjectiveScore() != null ? markStudent.getObjectiveScore() : 0);
            studentObjectiveDetailDto.setSubjectiveScore(
                    markStudent.getSubjectiveScore() != null ? markStudent.getSubjectiveScore() : 0);
            studentObjectiveDetailDto.setUpload(markStudent.getUpload());
            studentObjectiveDetailDto.setAbsent(markStudent.getAbsent() || (markStudent.getManualAbsent() != null && markStudent.getManualAbsent()) || markStudent.getOmrAbsent());
            studentObjectiveDetailDto.setSheetUrls(this.buildSheetUrls(studentId));

            List<MarkQuestionAnswerVo> questions = markQuestionService.listQuestionAnswerByExamIdAndPaperNumberAndPaperType(
                    markStudent.getExamId(), markStudent.getPaperNumber(), markStudent.getPaperType(), true);
            List<String> answers = markStudent.getAnswerList();
            List<String> answerScores = markStudent.getAnswerScoreList();
            int questionCount = questions.size();
            int answerCount = answers.size();
            int answerScoreCount = answerScores.size();

            List<StudentObjectiveAnswerDto> answerDtoList = new ArrayList<>();
            Map<Integer, String> titles = new HashMap<>();
            // 已设置客观题
            int maxCount = Math.max(questionCount, answerCount);
            for (int i = 0; i < maxCount; i++) {
                MarkQuestionAnswerVo q = questionCount > i ? questions.get(i) : null;
                String answer = answerCount > i ? answers.get(i) : "#";
                String answerScore = answerScoreCount > i ? answerScores.get(i) : "0";
                StudentObjectiveAnswerDto studentObjectiveAnswerDto = new StudentObjectiveAnswerDto();
                studentObjectiveAnswerDto.setMainNumber(q != null ? q.getMainNumber() : 0);
                studentObjectiveAnswerDto.setSubNumber(q != null ? q.getSubNumber() : 0);
                studentObjectiveAnswerDto.setStandardAnswer(q != null ? q.getAnswer() : null);
                studentObjectiveAnswerDto.setAnswer(answer);
                studentObjectiveAnswerDto.setScore(Double.valueOf(answerScore));
                studentObjectiveAnswerDto.setTotalScore(q != null && q.getTotalScore() != null ? q.getTotalScore() : 0);
                studentObjectiveAnswerDto.setExist(q != null && q.getTotalScore() != null && q.getTotalScore() > 0);
                studentObjectiveAnswerDto.setQuestionType(q != null ? q.getQuestionType() : 0);
                answerDtoList.add(studentObjectiveAnswerDto);

                if (q != null) {
                    titles.put(q.getMainNumber(), q.getMainTitle());
                }
            }
            studentObjectiveDetailDto.setAnswers(answerDtoList);
            studentObjectiveDetailDto.setTitles(titles);
        }

        return studentObjectiveDetailDto;
    }

    @Override
    public Boolean saveObjectiveInspectedTask(Long studentId, String answers) {
        Long userId = ServletUtil.getRequestUserId();
        MarkStudent student = this.getById(studentId);
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber());
        // 评卷是否结束
        if (markPaper == null || MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目已结束评卷");
        }
        answers = StringUtils.trimToEmpty(answers);
        if (student != null) {
            student.setAnswers(answers.toUpperCase());
            student.setCheckUserId(userId);
            student.setCheckTime(System.currentTimeMillis());
            boolean success = this.updateScanInfo(student);
            if (success) {
                calculateObjectiveScore(student);
            }
            return success;
        } else {
            return false;
        }
    }

    @Override
    public int countUploadedByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber)
                .eq(MarkStudent::getUpload, true)
                .eq(MarkStudent::getScanStatus, ScanStatus.SCANNED)
                .eq(MarkStudent::getAbsent, false)
                .eq(MarkStudent::getManualAbsent, false)
                .eq(MarkStudent::getOmrAbsent, false);
        return this.count(queryWrapper);
    }

    @Override
    public boolean updateScanInfo(MarkStudent student) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getAnswers, student.getAnswers())
                .set(MarkStudent::getCheckUserId, student.getCheckUserId())
                .set(MarkStudent::getCheckTime, student.getCheckTime())
                .eq(MarkStudent::getId, student.getId());
        return this.update(updateWrapper);

    }

    @Override
    public List<MarkStudent> listAbsentOrBreachMarkTaskStudent(Long examId, String paperNumber) {
        return this.baseMapper.listAbsentOrBreachMarkTaskStudent(examId, paperNumber);
    }

    @Override
    public List<MarkStudent> listUnMarkTaskStudent(Long examId, String paperNumber, Integer groupNumber,
                                                   int pageSize) {
        Page<MarkStudent> page = new Page<>(1, pageSize);
        IPage<MarkStudent> markStudentIPage = this.baseMapper.listUnMarkTaskStudent(page, examId, paperNumber,
                groupNumber);
        return markStudentIPage.getRecords();
    }

    /**
     * 客观题统分 统分场景： 1.后台-成绩检查-客观题检查 2.扫描端-考生图片上传 3.扫描端-客观题二次识别
     */
    @Override
    public void calculateObjectiveScore(MarkStudent student) {
        // 缺考状态不统分(人工指定缺考、识别缺考)
        if ((student.getManualAbsent() != null && !student.getManualAbsent()) && !student.getAbsent() && !student.getOmrAbsent()) {
            ScoreCalculateUtil util = ScoreCalculateUtil.instance(student);
            ScoreInfo info = util.calculate(markQuestionService.listQuestionAnswerByExamIdAndPaperNumberAndPaperType(
                    student.getExamId(), student.getPaperNumber(), student.getPaperType(), true), null);
            student.setObjectiveScore(info.getObjectiveScore());
            student.setScoreList(info.getScoreList(), true);
            this.updateObjectiveScoreAndScoreList(student);
        }
    }

    @Override
    @Transactional
    public void updateStudentAndPaper(@NotNull SysUser user, @NotNull Long id,
                                      @NotNull List<ScanStudentPaper> studentPaperList, boolean clearMarkTask) {
        if (CollectionUtils.isNotEmpty(studentPaperList)) {
            for (ScanStudentPaper studentPaper : studentPaperList) {
                studentPaper.setStudentId(id);
            }
        }
        // 清空原有绑定关系
        if (studentPaperService.removeByStudentId(id) > 0) {
            // 删除评卷任务
            if (clearMarkTask) {
                MarkStudent student = this.getById(id);
                markService.deleteMarkTaskByStudent(student);
            }
        }
        if (CollectionUtils.isNotEmpty(studentPaperList)) {
            // 保存绑定关系
            studentPaperService.saveOrUpdateBatch(studentPaperList);
        }
        // 更新考生状态
        updateStudentByPaper(user.getId(), id, true, false, true);
    }

    @Override
    public StudentVo findOne(StudentQuery query) {
        return baseMapper.findOne(query);
    }

    @Override
    public int countByExamIdAndSecretNumber(Long examId, String secretNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getSecretNumber, secretNumber);
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkStudent> listByExamIdAndCoursePaperId(Long examId, String coursePaperId) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getCoursePaperId, coursePaperId);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<AnswerQueryVo> query(AnswerQueryDomain query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(),
                sysUser.getId(), ServletUtil.getRequest().getServletPath());
        // 查询考生分页信息
        query.setMarkPaperStatus(MarkPaperStatus.FORMAL.name());
        IPage<AnswerQueryVo> iPage = baseMapper.queryPage(new Page<>(query.getPageNumber(), query.getPageSize()), query,
                dpr);
        if (CollectionUtils.isNotEmpty(iPage.getRecords())) {
            for (AnswerQueryVo vo : iPage.getRecords()) {
                if (vo.getIsAbsent() != null && vo.getIsAbsent()) {
                    vo.setExamStatus(ExamStatus.ABSENT);
                } else {
                    vo.setExamStatus(ExamStatus.OK);
                }
            }
        }
        if (CollectionUtils.isNotEmpty(iPage.getRecords()) && (query.getWithPaper() != null && query.getWithPaper())) {
            Map<Long, AnswerQueryVo> map = new HashMap<>();

            for (AnswerQueryVo vo : iPage.getRecords()) {
                List<AnswerPaperVo> papers = new ArrayList<>();
                vo.setPapers(papers);
                if (vo.getCardPaperCount() != null) {
                    for (int i = 1; i <= vo.getCardPaperCount(); i++) {
                        AnswerPaperVo pv = new AnswerPaperVo();
                        pv.setNumber(i);
                        papers.add(pv);
                    }
                }
                map.put(vo.getId(), vo);
            }
            // 根据考生id查找绑定paper
            List<Long> studentIds = iPage.getRecords().stream().map(p -> p.getId()).collect(Collectors.toList());
            List<StudentPaperVo> paperList = new BatchGetDataUtil<StudentPaperVo, Long>() {

                @Override
                public List<StudentPaperVo> getData(List<Long> paramList) {
                    return scanPaperService.listByStudentIds(paramList);
                }
            }.getDataForBatch(studentIds, 200);

            if (CollectionUtils.isNotEmpty(paperList)) {
                Map<Long, AnswerPaperVo> paperMap = new HashMap<>();
                for (StudentPaperVo p : paperList) {
                    AnswerQueryVo vo = map.get(p.getStudentId());
                    if (vo == null) {
                        continue;
                    }
                    List<AnswerPaperVo> papers = vo.getPapers();
                    if (papers == null) {
                        continue;
                    }
                    if (papers.size() < p.getNumber()) {
                        continue;
                    }
                    AnswerPaperVo pvo = papers.get(p.getNumber() - 1);
                    pvo.setId(p.getPaperId());
                    pvo.setNumber(p.getNumber());
                    pvo.setAssigned(p.getAssigned());
                    pvo.setInvalid(p.getInvalid());
                    paperMap.put(p.getPaperId(), pvo);
                }
                // 查找page
                List<Long> paperIds = paperList.stream().map(p -> p.getPaperId()).collect(Collectors.toList());
                List<ScanPaperPage> paperPageList = new BatchGetDataUtil<ScanPaperPage, Long>() {

                    @Override
                    public List<ScanPaperPage> getData(List<Long> paramList) {
                        return scanPaperPageService.listByPaperList(paramList);
                    }
                }.getDataForBatch(paperIds, 200);

                if (CollectionUtils.isNotEmpty(paperPageList)) {
                    for (ScanPaperPage p : paperPageList) {
                        AnswerPaperVo pvo = paperMap.get(p.getPaperId());
                        if (pvo == null) {
                            continue;
                        }
                        List<AnswerPageVo> pages = pvo.getPages();
                        if (pages == null) {
                            pages = new ArrayList<>();
                            pvo.setPages(pages);
                        }
                        AnswerPageVo pageVo = new AnswerPageVo();
                        pageVo.setIndex(p.getPageIndex());
                        pageVo.setSheetUri(teachcloudCommonService.filePreview(p.getSheetPath()));
                        if (query.getWithOmrDetail() != null && query.getWithOmrDetail()) {
                            pageVo.setAbsent(p.getAbsent() != null ? p.getAbsent().getResult() : null);
                            pageVo.setBreach(p.getBreach() != null ? p.getBreach().getResult() : null);
                            pageVo.setQuestion(p.getQuestion() != null ? p.getQuestion().getResult() : null);
                            pageVo.setRecogData(p.getRecogData());
                            pageVo.setPaperType(p.getPaperType() != null ? p.getPaperType().getResult() : null);
                        }
                        pages.add(pageVo);
                    }
                }
            }
        }
        return iPage;
    }

    @Override
    public List<String> summary(AnswerQueryDomain query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(),
                sysUser.getId(), ServletUtil.getRequest().getServletPath());

        // 不分页查询考生准考证号
        query.setMarkPaperStatus(MarkPaperStatus.FORMAL.name());
        return baseMapper.querySummary(query, dpr);
    }

    @Transactional
    @Override
    public UpdateTimeVo omrEdit(Long userId, OmrEditDomain domain) {
        MarkStudent student = findByExamIdAndCoursePaperIdAndStudentCode(domain.getExamId(), domain.getCoursePaperId(),
                domain.getStudentCode());
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            for (OmrEditPaper paperEdit : domain.getPapers()) {
                ScanStudentPaper sp = studentPaperService.findByStudentIdAndPaperNumber(student.getId(),
                        paperEdit.getNumber());
                if (sp == null) {
                    throw new ParameterException("未找到绑定扫描结果");
                }
                ScanPaper paperEntity = scanPaperService.getById(sp.getPaperId());
                if (paperEntity == null) {
                    throw new ParameterException("未找到paper信息结果");
                }
                paperEntity.setUpdaterId(userId);
                paperEntity.setUpdateTime(System.currentTimeMillis());
                List<ScanPaperPage> pages = scanPaperPageService.listByPaperId(paperEntity.getId());
                for (ScanPaperPage pageEntity : pages) {
                    paperEdit.updatePage(pageEntity);
                }
                scanPaperService.savePaperAndPages(paperEntity, pages);
            }
            updateStudentByPaper(userId, student.getId(), true, true, false);
            return UpdateTimeVo.create();
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    @Override
    public long countByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkStudent> lambda = queryWrapper.lambda();
        lambda.eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber);
        return this.count(queryWrapper);
    }

    @Override
    public long countByExamIdAndPaperNumberAndSerialNumber(Long examId, String paperNumber, Integer serialNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkStudent> lambda = queryWrapper.lambda();
        lambda.eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber)
                .eq(MarkStudent::getSerialNumber, serialNumber);
        return this.count(queryWrapper);
    }

    @Transactional
    @Override
    public AbsentManualUpdateVo absentManualUpdate(Long examId, String coursePaperId, String studentCode,
                                                   Boolean manualAbsent) {
        MarkStudent student = findByExamIdAndCoursePaperIdAndStudentCode(examId, coursePaperId, studentCode);
        if (student == null) {
            throw ExceptionResultEnum.ERROR.exception("考生未找到");
        }
        if (manualAbsent && !ScanStatus.UNEXIST.equals(student.getScanStatus())) {
            throw ExceptionResultEnum.ERROR.exception("考生不是未扫描状态");
        }
        LambdaUpdateWrapper<MarkStudent> lw = new LambdaUpdateWrapper<>();
        lw.set(MarkStudent::getScanStatus, ScanStatus.UNEXIST);
        lw.set(MarkStudent::getManualAbsent, manualAbsent);
        lw.set(MarkStudent::getAbsent, student.getOmrAbsent() || manualAbsent);
        lw.set(MarkStudent::getMissScan, false);
        lw.eq(MarkStudent::getId, student.getId());
        update(lw);
        // 更新课程表中上传人数
        markPaperService.updateUploadCount(student.getExamId(), student.getPaperNumber(),
                this.countUploadedByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber()));
        markPaperService.updateAbsentCount(student.getExamId(), student.getPaperNumber(),
                this.countAbsentByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber()));

        return AbsentManualUpdateVo.create(manualAbsent);
    }

    @Transactional
    @Override
    public UpdateTimeVo confirm(Long examId, String coursePaperId, String studentCode, Boolean omrAbsent) {
        MarkStudent student = findByExamIdAndCoursePaperIdAndStudentCode(examId, coursePaperId, studentCode);
        LambdaUpdateWrapper<MarkStudent> lw = new LambdaUpdateWrapper<>();

        lw.set(MarkStudent::getOmrAbsentChecked, true);
        lw.set(MarkStudent::getOmrAbsent, omrAbsent);
        lw.set(MarkStudent::getAbsent, omrAbsent || (student.getManualAbsent() != null && student.getManualAbsent()));
        if (omrAbsent) {
            lw.set(MarkStudent::getObjectiveScore, null);
            lw.set(MarkStudent::getObjectiveScoreList, null);
        }
        lw.eq(MarkStudent::getId, student.getId());
        update(lw);

        // 确认缺考清除客观题查检任务，取消缺考统计客观题分数、重新生成客观题检查任务
        if (!omrAbsent) {
            calculateObjectiveScore(student);
            scanOmrTaskService.saveTask(student.getId());
        } else {
            scanOmrTaskService.deleteByStudentId(student.getExamId(), student.getId());
        }
        // 更新识别缺考值
        updatePaperPageData(student.getId(), omrAbsent, null, null);

        markPaperService.updateUploadCount(examId, student.getPaperNumber(),
                this.countUploadedByExamIdAndPaperNumber(examId, student.getPaperNumber()));
        markPaperService.updateAbsentCount(examId, student.getPaperNumber(),
                this.countAbsentByExamIdAndPaperNumber(examId, student.getPaperNumber()));
        return UpdateTimeVo.create();
    }

    private void updatePaperPageData(Long studentId, Boolean omrAbsent, Boolean breach, String paperType) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 更改试卷和考生绑定关系的卷型
        ScanStudentPaper scanStudentPaper = scanStudentPaperService.findByStudentIdAndPaperNumber(studentId, 1);
        if (scanStudentPaper == null) {
            throw ExceptionResultEnum.ERROR.exception("缺少考生和试卷绑定关系");
        }
        Long paperId = scanStudentPaper.getPaperId();
        ScanPaperPage scanPaperPage = scanPaperPageService.findPaperIdAndIndex(paperId, 1);
        if (scanPaperPage == null) {
            throw ExceptionResultEnum.ERROR.exception("缺少试卷和页面绑定关系");
        }
        // 更新识别缺考结果
        if (omrAbsent != null) {
            if (scanPaperPage.getAbsent() != null) {
                scanPaperPage.getAbsent().setResult(omrAbsent);
            }
        }
        // 更新识别违纪结果
        if (breach != null) {
            if (scanPaperPage.getBreach() != null) {
                scanPaperPage.getBreach().setResult(breach);
            }
        }
        // 更新识别卷型结果
        if (StringUtils.isNotBlank(paperType)) {
            if (scanPaperPage.getPaperType() != null) {
                scanPaperPage.getPaperType().setResult(paperType);
            }
        }
        scanPaperPage.setUpdaterId(sysUser.getId());
        scanPaperPage.setUpdateTime(System.currentTimeMillis());
        scanPaperPageService.updateByMultiId(scanPaperPage);
    }

    @Override
    public List<Long> findIdByExamIdAndPaperNumber(Long examId, String paperNumber) {
        return this.baseMapper.findIdByExamIdAndPaperNumber(examId, paperNumber);
    }

    @Override
    public Task getSubjectiveInspectedTask(Long studentId) {
        Task task = null;
        if (studentId != null) {
            Long userId = ServletUtil.getRequestUserId();
            MarkStudent markStudent = this.getById(studentId);
            markService.releaseByStudent(markStudent);
            if (markService.applyStudent(markStudent, userId)) {
                task = taskService.build(studentId);
            }
        }
        return task;
    }

    @Transactional
    @Override
    public void saveSubjectiveInspectedTask(MarkHeaderResult markResult) {
        MarkStudent markStudent = this.getById(markResult.getStudentId());
        if (markStudent == null) {
            throw ExceptionResultEnum.ERROR.exception("考生不存在");
        }
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markStudent.getExamId(),
                markStudent.getPaperNumber());
        // 评卷是否结束
        if (markPaper == null || MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法打分");
        }
        try {
            lockService.watch(LockType.EXAM_SUBJECT, markStudent.getExamId(), markStudent.getPaperNumber());
            lockService.waitlock(LockType.STUDENT, markResult.getStudentId());
            markService.submitHeaderTask(markResult.getGroups(), markStudent);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        } finally {
            lockService.unlock(LockType.STUDENT, markResult.getStudentId());
            lockService.unwatch(LockType.EXAM_SUBJECT, markStudent.getExamId(), markStudent.getPaperNumber());
            markService.releaseByStudent(markStudent);
        }
    }

    @Override
    public IPage<ArchiveStudentVo> studentList(ArchiveStudentQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 使用成绩管理列表请求的数据权限
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/archive/score/list");
        Page<ArchiveStudentVo> page = new Page<>(query.getPageNumber(), query.getPageSize());
        IPage<ArchiveStudentVo> ret = baseMapper.studentList(page, sysUser.getSchoolId(), query, dpr);
        for (ArchiveStudentVo record : ret.getRecords()) {
            // 扫描状态
            if (BasicExamStudentStatusEnum.M.equals(record.getStudentStatus()) || record.getAbsent() || record.getOmrAbsent() || record.getManualAbsent()) {
                record.setStudentStatusDisplay("缺考");
            } else if (ScanStatus.UNEXIST.equals(record.getScanStatus())) {
                record.setStudentStatusDisplay(ScanStatus.UNEXIST.getName());
            } else {
                record.setStudentStatusDisplay("正常");
            }

            // 违纪状态
            if (BasicExamStudentStatusEnum.B.equals(record.getStudentStatus()) || record.getBreach() || record.getOmrBreach() || record.getManualBreach()) {
                record.setBreachDisplay("违纪");
            }

            List<String> list = new ArrayList<>();
            List<FilePathVo> vos = JSON.parseArray(StringUtils.trimToNull(record.getSheetPath()), FilePathVo.class);
            if (CollectionUtils.isEmpty(vos)) {
                continue;
            }
            for (FilePathVo filePathVo : vos) {
                list.add(JSON.toJSONString(filePathVo));
            }
            record.setSheetUrls(teachcloudCommonService.filePreview(list, SystemConstant.MARK_JPG_EXPIRE_MINUTE));
        }
        return ret;
    }

    @Override
    public void scoreExport(ArchiveStudentQuery query, HttpServletResponse response) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(query.getExamId(), query.getPaperNumber());
        BasicCourse basicCourse = basicCourseService.getById(markPaper.getCourseId());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        //生成表头
        String[] columnName = new String[]{"学生姓名", "学号", "学院", "教学班", "行政班", "课程代码", "课程名称", "状态", "违纪状态", "客观分", "主观分", "成绩"};
        List<MarkQuestion> oQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(query.getExamId(), query.getPaperNumber(), null, true);
        List<MarkQuestion> sQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(query.getExamId(), query.getPaperNumber(), null, false);
        List<String> columnNameList = new ArrayList<>(Arrays.asList(columnName));
        for (MarkQuestion question : oQuestionList) {
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "选项");
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "得分");
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "满分");
        }
        for (MarkQuestion question : sQuestionList) {
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber());
            columnNameList.add(question.getMainTitle() + " " + question.getMainNumber() + "-" + question.getSubNumber() + "满分");
        }
        String[] columnNames = columnNameList.toArray(new String[0]);
        // 使用成绩管理列表请求的数据权限
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/archive/score/list");
        List<ArchiveStudentVo> ret = baseMapper.studentList(sysUser.getSchoolId(), query, dpr);

        //生成动态内容
        List<CellValue[]> columnValues = new ArrayList<>();
        for (ArchiveStudentVo s : ret) {
            List<CellValue> valueList = new ArrayList<>();
            valueList.add(CellValue.of(s.getStudentName()));
            valueList.add(CellValue.of(s.getStudentCode()));
            valueList.add(CellValue.of(s.getCollegeName()));
            valueList.add(CellValue.of(s.getTeachClassName()));
            valueList.add(CellValue.of(s.getClassName()));
            valueList.add(CellValue.of(s.getCourseCode()));
            valueList.add(CellValue.of(s.getCourseName()));
            String studentStatusDisplay = "";
            // 扫描状态
            if (BasicExamStudentStatusEnum.M.equals(s.getStudentStatus()) || s.getAbsent() || s.getOmrAbsent() || s.getManualAbsent()) {
                studentStatusDisplay = "缺考";
            } else if (ScanStatus.UNEXIST.equals(s.getScanStatus())) {
                studentStatusDisplay = ScanStatus.UNEXIST.getName();
            } else {
                studentStatusDisplay = "正常";
            }
            s.setStudentStatusDisplay(studentStatusDisplay);
            valueList.add(CellValue.of(studentStatusDisplay));

            String breachDisplay = "";
            // 违纪状态
            if (BasicExamStudentStatusEnum.B.equals(s.getStudentStatus()) || s.getBreach() || s.getOmrBreach() || s.getManualBreach()) {
                breachDisplay = "违纪";
            }
            s.setBreachDisplay(breachDisplay);
            valueList.add(CellValue.of(breachDisplay));

            valueList.add(s.getObjectiveScore() == null ? CellValue.of("") : CellValue.of(s.getObjectiveScore()));
            valueList.add(s.getSubjectiveScore() == null ? CellValue.of("") : CellValue.of(s.getSubjectiveScore()));
            valueList.add(s.getTotalScore() == null ? CellValue.of("") : CellValue.of(s.getTotalScore()));
            for (ScoreItem item : s.getScoreList(true, oQuestionList)) {
                valueList.add(CellValue.of(item.getAnswer()));
                valueList.add(item.getScore() == null ? CellValue.of("") : CellValue.of(item.getScore()));
                valueList.add(item.getTotalScore() == null ? CellValue.of("") : CellValue.of(item.getTotalScore()));
            }
            for (ScoreItem item : s.getScoreList(false, sQuestionList)) {
                valueList.add(CellValue.of(item.getScore()));
                valueList.add(item.getTotalScore() == null ? CellValue.of("") : CellValue.of(item.getTotalScore()));
            }
            CellValue[] columnValue = valueList.toArray(new CellValue[0]);
            columnValues.add(columnValue);
        }

        String rootPath = null;
        try {
            // 本地保存目录
            File tempFile = SystemConstant.getFileTempParentDirVar(SystemConstant.TEMP_PREFIX);
            rootPath = tempFile.getPath() + File.separator + SystemConstant.getNanoId();

            // 下载文件保存目录（用uuid命名）
            String downloadFilePath = rootPath + File.separator + query.getPaperNumber();
            File downloadPathFile = new File(downloadFilePath);
            if (!downloadPathFile.exists()) {
                downloadPathFile.mkdirs();
            }

            log.debug("导出Excel开始...");
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeDataArrays("成绩单", null, columnNames, columnValues.listIterator());
            FileOutputStream fileOut = new FileOutputStream(downloadFilePath + File.separator + basicCourse.getName() + "-成绩单" + SystemConstant.EXCEL_PREFIX);
            writer.output(fileOut);
            fileOut.flush();
            fileOut.close();

            // 通过教学班聚合
            Map<Object, List<CellValue[]>> listMap = columnValues.stream().collect(Collectors.groupingBy(m -> getCellValueFieldValue("value", m[3])));
            Set<Map.Entry<Object, List<CellValue[]>>> entrySet = listMap.entrySet();
            if (entrySet.size() > 1) {
                for (Map.Entry<Object, List<CellValue[]>> entry : entrySet) {
                    writer = ExcelWriter.create(ExcelType.XLSX);
                    writer.writeDataArrays("成绩单", null, columnNames, entry.getValue().listIterator());
                    FileOutputStream classFileOut = new FileOutputStream(downloadFilePath + File.separator + basicCourse.getName() + "-" + entry.getKey() + "成绩单" + SystemConstant.EXCEL_PREFIX);
                    writer.output(classFileOut);
                    classFileOut.flush();
                    classFileOut.close();
                }
            }
            log.debug("导出Excel结束");

            String zipFileName = downloadPathFile.getName() + SystemConstant.ZIP_PREFIX;
            SysUser sysUser1 = sysUserService.getById(ServletUtil.getRequestUserId());
            FileUtil.downloadEncryptZip(response, downloadPathFile, zipFileName, new String(Base64Util.decode(sysUser1.getPassword())));
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            if (Objects.nonNull(rootPath)) {
                FileUtil.deleteDirectory(rootPath);
            }
        }
    }

    private static Object getCellValueFieldValue(String fieldName, CellValue cellValue) {
        try {
            Field declaredField = CellValue.class.getDeclaredField(fieldName);
            declaredField.setAccessible(true);
            return declaredField.get(cellValue);
        } catch (Exception e) {
            throw new RuntimeException("获取属性值失败: " + e.getMessage(), e);
        }
    }

    @Override
    public ScoreReportVo scoreReport(ArchiveStudentQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String postUrl = "/api/admin/mark/archive/score/list";
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), postUrl);
        ScoreReportVo ret = new ScoreReportVo();
        // 考试概况
        ret.setOverview(baseMapper.overview(sysUser.getSchoolId(), query, dpr));
        if (ret.getOverview() != null && ret.getOverview().getStudentCount() > 0) {
            double total = ret.getOverview().getStudentCount() - ret.getOverview().getAbsentCount();
            ret.getOverview().setPassRate(
                    Calculator.divide2String(Calculator.multiply(ret.getOverview().getPassCount(), 100), total, 2));
            ret.getOverview().setExcellentRate(
                    Calculator.divide2String(Calculator.multiply(ret.getOverview().getExcellentCount(), 100), total, 2));
            ret.getOverview().setAvgScore(Calculator.round(ret.getOverview().getAvgScore(), 2));
        }
        List<ArchiveStudentVo> studentList = baseMapper.studentList(sysUser.getSchoolId(), query, dpr);

        // 成绩分数段分析
        fillScoreRange(ret, studentList);

        // 学院成绩分析
        ret.setCollege(baseMapper.college(sysUser.getSchoolId(), query, dpr));
        if (CollectionUtils.isNotEmpty(ret.getCollege())) {
            for (CollegeVo vo : ret.getCollege()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(
                        Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
                vo.setMinScore(Calculator.round(vo.getMinScore(), 2));
                vo.setMaxScore(Calculator.round(vo.getMaxScore(), 2));
            }
        }

        // 班级成绩分析
        ret.setClassData(baseMapper.classData(sysUser.getSchoolId(), query, dpr));
        if (CollectionUtils.isNotEmpty(ret.getClassData())) {
            for (ClassVo vo : ret.getClassData()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(
                        Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
                vo.setMinScore(Calculator.round(vo.getMinScore(), 2));
                vo.setMaxScore(Calculator.round(vo.getMaxScore(), 2));
            }
        }

        // 任课老师分析
        ret.setTeacher(baseMapper.teacher(sysUser.getSchoolId(), query, dpr));
        if (CollectionUtils.isNotEmpty(ret.getTeacher())) {
            for (TeacherVo vo : ret.getTeacher()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(
                        Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
                vo.setMinScore(Calculator.round(vo.getMinScore(), 2));
                vo.setMaxScore(Calculator.round(vo.getMaxScore(), 2));
            }
        }

        // 任课老师班级分析
        ret.setTeacherClass(baseMapper.teacherClass(sysUser.getSchoolId(), query, dpr));
        if (CollectionUtils.isNotEmpty(ret.getTeacherClass())) {
            for (TeacherClassVo vo : ret.getTeacherClass()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(
                        Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
            }
        }

        // 客观题成绩分析
        fillObjective(ret, studentList, query.getExamId(), query.getPaperNumber());

        // 主观题成绩分析
        List<Long> studentIds = studentList.stream().map(ArchiveStudentVo::getStudentId).collect(Collectors.toList());
        ret.setSubjective(markSubjectiveScoreService.getSubjectiveVo(studentIds));
        if (CollectionUtils.isNotEmpty(ret.getSubjective())) {
            for (QuestionVo vo : ret.getSubjective()) {
                double total = vo.getStudentCount();
                vo.setScoreRate(Calculator.divide(vo.getScoreCount(), total, 2));
                vo.setFullScoreRate(Calculator.divide(vo.getFullScoreCount(), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
            }
        }

        // 大题分析
        fillMainQuestionAnalysis(ret, studentList, query.getExamId(), query.getPaperNumber());
        return ret;
    }

    @Override
    public void exportUnexist(Long examId, Long courseId, String coursePaperId, HttpServletResponse response) {
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(),
                    sysUser.getId(), ServletUtil.getRequest().getServletPath());
            List<UnexistStudentDto> unexistStudentDtoList = this.baseMapper.listUnexistStudentByExamIdAndCoursePaperId(
                    examId, courseId, coursePaperId, MarkPaperStatus.FORMAL.name(), dpr);
            ExcelUtil.excelExport("评卷员工作量", UnexistStudentDto.class, unexistStudentDtoList, response);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("导出评卷员工作量失败");
        }
    }

    @Override
    public int countByExamIdAndPaperNumberAndMarkStatus(Long examId, String paperNumber, SubjectiveStatus status) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkStudent> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber);
        if (status != null) {
            lambdaQueryWrapper.eq(MarkStudent::getSubjectiveStatus, status);
        }
        lambdaQueryWrapper.eq(MarkStudent::getUpload, true).eq(MarkStudent::getAbsent, false)
                .and(m -> m.eq(MarkStudent::getManualAbsent, false).or().isNull(MarkStudent::getManualAbsent)).eq(MarkStudent::getOmrAbsent, false);
        return this.count(queryWrapper);
    }

    @Override
    public void updateCheckInfo(Long studentId, Long userId) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getCheckUserId, userId)
                .set(MarkStudent::getCheckTime, System.currentTimeMillis()).eq(MarkStudent::getId, studentId);
        this.update(updateWrapper);
    }

    @Override
    public int countOmrAbsentStudent(Long examId, String paperNumber, boolean isOmrAbsentConfirm) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber)
                .eq(MarkStudent::getOmrAbsent, true)
                .eq(MarkStudent::getOmrAbsentChecked, isOmrAbsentConfirm);
        return this.count(queryWrapper);
    }

    @Override
    public int countOmrAbsentStudent(Long examId, String paperNumber, boolean isOmrAbsentConfirm, String
            teachClassName) {
        return this.baseMapper.countOmrAbsentStudent(examId, paperNumber, isOmrAbsentConfirm, teachClassName);
    }

    @Override
    public void scoreReportDownload(MultipartFile file, HttpServletResponse response) {
        String rootPath = null;
        File htmlFile;
        File pdfFile;
        try {
            // 本地保存目录
            File tempFile = SystemConstant.getFileTempDirVar(
                    System.currentTimeMillis() + File.separator + SystemConstant.getNanoId(),
                    SystemConstant.TEMP_PREFIX);
            rootPath = tempFile.getParent();

            // 删除临时目录中创建的临时文件
            if (Objects.nonNull(tempFile)) {
                tempFile.delete();
            }
            // html文件
            String cardHtmlPath = rootPath + File.separator + System.currentTimeMillis() + File.separator + "temp"
                    + SystemConstant.HTML_PREFIX;
            htmlFile = new File(cardHtmlPath);
            if (!htmlFile.exists()) {
                htmlFile.getParentFile().mkdirs();
                htmlFile.createNewFile();
            }
            // 生成html文件
            FileCopyUtils.copy(file.getBytes(), htmlFile);

            // pdf文件
            String cardPdfPath = rootPath + File.separator + System.currentTimeMillis() + File.separator + "temp"
                    + SystemConstant.PDF_PREFIX;
            pdfFile = new File(cardPdfPath);
            if (!pdfFile.exists()) {
                pdfFile.getParentFile().mkdirs();
                pdfFile.createNewFile();
            }
            HtmlToPdfUtil.convert(cardHtmlPath, cardPdfPath, PageSizeEnum.A4);
            FileUtil.outputFile(response, pdfFile, "报告" + SystemConstant.PDF_PREFIX);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(rootPath)) {
                ConvertUtil.delFolder(rootPath);
            }
        }
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public boolean calcObjectiveScore(Long examId, String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            if (lockService.trylock(LockType.SCORE_CALCULATE, markPaper.getId())) {
                markSyncService.calcObjectiveScore(markPaper);
                return true;
            } else {
                throw ExceptionResultEnum.ERROR.exception("客观题正在统分，请稍后再试");
            }
        }
        return true;
    }

    @Override
    public IPage<MarkStudent> pageByExamAndPaperNumber(Long examId, String paperNumber, int pageNumber, int pageSize) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber)
                .orderByAsc(MarkStudent::getId);
        return this.page(new Page<>(pageNumber, pageSize), queryWrapper);
    }

    @Override
    public void updateObjectiveScoreAndScoreList(MarkStudent markStudent) {
        UpdateWrapper<MarkStudent> objectiveUpdateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkStudent> lambdaUpdateWrapper = objectiveUpdateWrapper.lambda();
        lambdaUpdateWrapper.set(MarkStudent::getObjectiveScore, markStudent.getObjectiveScore())
                .set(MarkStudent::getObjectiveScoreList, markStudent.getObjectiveScoreList());
        if (markStudent.getCheckUserId() != null) {
            lambdaUpdateWrapper.set(MarkStudent::getCheckUserId, markStudent.getCheckUserId());
        }
        if (markStudent.getCheckTime() != null) {
            lambdaUpdateWrapper.set(MarkStudent::getCheckTime, markStudent.getCheckTime());
        }
        lambdaUpdateWrapper.eq(MarkStudent::getId, markStudent.getId());
        this.update(objectiveUpdateWrapper);
    }

    @Override
    public boolean updateAssignConfirm(Long studentId, boolean assignConfirm) {
        UpdateWrapper<MarkStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkStudent::getAssignConfirmed, assignConfirm).eq(MarkStudent::getId, studentId);
        return this.update(updateWrapper);
    }

    private void fillObjective(ScoreReportVo ret, List<ArchiveStudentVo> studentList, Long examId, String
            paperNumber) {
        List<MarkQuestion> qs = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, true);
        Map<String, List<ArchiveStudentVo>> collect = studentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperType())).collect(Collectors.groupingBy(ArchiveStudentVo::getPaperType));

        List<QuestionObjectiveVo> questionObjectiveVoList = new ArrayList<>();
        for (Map.Entry<String, List<ArchiveStudentVo>> entry : collect.entrySet()) {
            QuestionObjectiveVo questionObjectiveVo = new QuestionObjectiveVo();
            questionObjectiveVo.setPaperType(entry.getKey());
            Map<String, QuestionVo> map = new HashMap<>();
            List<QuestionVo> list = new ArrayList<>();
            for (ArchiveStudentVo s : entry.getValue()) {
                List<ScoreItem> sis = s.getScoreList(true, qs);
                if (CollectionUtils.isNotEmpty(sis)) {
                    for (ScoreItem si : sis) {
                        String key = si.getMainNumber() + "-" + si.getSubNumber();
                        QuestionVo vo = map.get(key);
                        if (vo == null) {
                            vo = new QuestionVo();
                            vo.setScoreCount(0);
                            vo.setFullScoreCount(0);
                            vo.setStudentCount(0);
                            vo.setScoreSum(0.0);
                            vo.setMainNumber(si.getMainNumber());
                            vo.setSubNumber(si.getSubNumber());
                            vo.setTitle(si.getTitle());
                            vo.setScore(si.getTotalScore());
                            map.put(key, vo);
                        }
                        vo.setStudentCount(vo.getStudentCount() + 1);
                        vo.setScoreSum(vo.getScoreSum() + si.getScore());
                        if (Objects.nonNull(si.getScore()) && Objects.nonNull(si.getTotalScore()) && si.getScore().doubleValue() == si.getTotalScore().doubleValue()) {
                            vo.setFullScoreCount(vo.getFullScoreCount() + 1);
                        }
                        if (si.getScore() > 0) {
                            vo.setScoreCount(vo.getScoreCount() + 1);
                        }
                    }
                }
            }
            if (map.isEmpty()) {
                questionObjectiveVo.setList(list);
                continue;
            }

            list = new ArrayList<>(map.values());
            for (QuestionVo questionVo : list) {
                // 平均分
                if (questionVo.getStudentCount() == null || questionVo.getStudentCount() == 0) {
                    questionVo.setAvgScore(0D);
                } else {
                    questionVo.setAvgScore(Calculator.round(Calculator.divide(questionVo.getScoreSum(), questionVo.getStudentCount()), 2));
                }
            }

            list.sort(Comparator.comparingInt(QuestionVo::getMainNumber).thenComparingInt(QuestionVo::getSubNumber));

            for (QuestionVo vo : list) {
                double total = vo.getStudentCount();
                vo.setScoreRate(Calculator.divide(vo.getScoreCount(), total, 2));
                vo.setFullScoreRate(Calculator.divide(vo.getFullScoreCount(), total, 2));
            }
            questionObjectiveVo.setList(list);
            questionObjectiveVoList.add(questionObjectiveVo);
        }
        ret.setObjective(questionObjectiveVoList);
    }

    /**
     * 填充大题分析
     *
     * @param ret         ret
     * @param studentList 涉及的考生集合
     * @param examId      考试id
     * @param paperNumber 试卷编号
     */
    private void fillMainQuestionAnalysis(ScoreReportVo ret, List<ArchiveStudentVo> studentList, Long
            examId, String paperNumber) {
        // 试卷结构数据
        Map<String, QuestionVo> mainQuestionMap = new HashMap<>();

        List<MarkQuestion> questionDatasource = markQuestionService.listByExamIdAndPaperNumber(examId, paperNumber);
        for (MarkQuestion question : questionDatasource) {
            Integer mainNumber = question.getMainNumber();
            String key = getMainQuestionMapKey(mainNumber, question.getObjective());
            if (mainQuestionMap.containsKey(key)) {
                QuestionVo questionVo = mainQuestionMap.get(key);
                questionVo.setScore(questionVo.getScore() + question.getTotalScore());
            } else {
                QuestionVo questionVo = new QuestionVo();
                questionVo.setMainNumber(mainNumber);
                questionVo.setScore(question.getTotalScore());
                questionVo.setTitle(question.getMainTitle());

                // 初始化
                questionVo.setStudentCount(0);
                questionVo.setScoreCount(0);
                questionVo.setFullScoreCount(0);
                questionVo.setScoreSum(0.0);
                questionVo.setZeroScoreCount(0);
                mainQuestionMap.put(key, questionVo);
            }
        }
        studentList = studentList.stream().filter(e -> !BasicExamStudentStatusEnum.M.equals(e.getStatus())).collect(Collectors.toList());
        List<Long> studentIdList = studentList.stream().map(ArchiveStudentVo::getStudentId).distinct().collect(Collectors.toList());
        List<MarkSubjectiveScore> subjectiveScoreDatasource = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(studentIdList)) {
            subjectiveScoreDatasource = markSubjectiveScoreService.list(
                    new QueryWrapper<MarkSubjectiveScore>().lambda().eq(MarkSubjectiveScore::getExamId, examId)
                            .eq(MarkSubjectiveScore::getPaperNumber, paperNumber).in(MarkSubjectiveScore::getStudentId, studentIdList));
        }

        List<MarkQuestion> objectiveQuestions = questionDatasource.stream().filter(MarkQuestion::getObjective).collect(Collectors.toList());
        for (ArchiveStudentVo studentVo : studentList) {
            List<ScoreItem> scoreItemList = studentVo.getScoreList(true, objectiveQuestions);

            for (String key : mainQuestionMap.keySet()) {
                String[] arr = key.split("-");
                int mainNumber = Integer.parseInt(arr[0]);
                boolean objective = Boolean.parseBoolean(arr[1]);
                QuestionVo questionVo = mainQuestionMap.get(key);
                // 考生总数
                questionVo.setStudentCount(questionVo.getStudentCount() + 1);
                // 考生在大题得分
                double studentScore = 0;
                if (objective) {
                    // 客观题处理
                    List<ScoreItem> mainScoreItemList = scoreItemList.stream()
                            .filter(e -> Objects.equals(mainNumber, e.getMainNumber())).collect(Collectors.toList());

                    for (ScoreItem scoreItem : mainScoreItemList) {
                        studentScore = studentScore + scoreItem.getScore();
                    }
                } else {
                    // 主观题处理
                    List<MarkSubjectiveScore> subjectiveScoreList = subjectiveScoreDatasource.stream()
                            .filter(e -> Objects.equals(mainNumber, e.getMainNumber()) && Objects.equals(studentVo.getStudentId(), e.getStudentId())).collect(Collectors.toList());

                    for (MarkSubjectiveScore markSubjectiveScore : subjectiveScoreList) {
                        studentScore = studentScore + markSubjectiveScore.getScore();
                    }
                }
                // 有得分总数
                if (studentScore > 0) {
                    questionVo.setScoreCount(questionVo.getScoreCount() + 1);
                }
                // 满分总数
                if (studentScore == questionVo.getScore()) {
                    questionVo.setFullScoreCount(questionVo.getFullScoreCount() + 1);
                }
                // 零分总数
                if (studentScore == 0) {
                    questionVo.setZeroScoreCount(questionVo.getZeroScoreCount() + 1);
                }
                // 考生总得分
                questionVo.setScoreSum(questionVo.getScoreSum() + studentScore);
                // 最高分
                if (Objects.isNull(questionVo.getMaxScore()) || studentScore > questionVo.getMaxScore()) {
                    questionVo.setMaxScore(studentScore);
                }
                // 最低分
                if (Objects.isNull(questionVo.getMinScore()) || studentScore < questionVo.getMinScore()) {
                    questionVo.setMinScore(studentScore);
                }
            }
        }
        List<QuestionVo> questionVoList = new ArrayList<>(mainQuestionMap.values());
        questionVoList.sort(Comparator.comparingInt(QuestionVo::getMainNumber));
        questionVoList.forEach(e -> {
            Integer total = e.getStudentCount();
            Double avgScore = 0D;
            double scoreRate = 0D;
            double fullScoreRate = 0D;
            if (total != null && total > 0) {
                avgScore = Calculator.round(Calculator.divide(e.getScoreSum(), total), 2);
                scoreRate = Calculator.divide(e.getScoreCount(), total, 2);
                fullScoreRate = Calculator.divide(e.getFullScoreCount(), total, 2);
            }
            e.setAvgScore(avgScore);
            e.setScoreRate(scoreRate);
            e.setFullScoreRate(fullScoreRate);
        });
        ret.setMainQuestionAnalysis(questionVoList);
    }

    private String getMainQuestionMapKey(Integer mainNumber, boolean objective) {
        return mainNumber + "-" + objective;
    }

    private List<MarkStudentVo> listByExamIdAndPaperNumberAndNotAbsent(Long examId, String paperNumber) {
        MarkStudentQuery markStudentQuery = new MarkStudentQuery();
        markStudentQuery.setExamId(examId);
        markStudentQuery.setPaperNumber(paperNumber);
        markStudentQuery.setUpload(true);
        markStudentQuery.setAbsent(false);
        markStudentQuery.setBreach(false);
        markStudentQuery.setOmrAbsent(false);
        return this.listMarkStudentVo(markStudentQuery);
    }

    private void fillScoreRange(ScoreReportVo ret, List<ArchiveStudentVo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<ScoreRangeVo> scoreRange = new ArrayList<>();
        ret.setScoreRange(scoreRange);
        int total = ret.getOverview().getStudentCount() - ret.getOverview().getAbsentCount();
        scoreRange.add(getScoreRangeVo(total, list, 1.0, 9.5));
        scoreRange.add(getScoreRangeVo(total, list, 10.0, 19.5));
        scoreRange.add(getScoreRangeVo(total, list, 20.0, 29.5));
        scoreRange.add(getScoreRangeVo(total, list, 30.0, 39.5));
        scoreRange.add(getScoreRangeVo(total, list, 40.0, 49.5));
        scoreRange.add(getScoreRangeVo(total, list, 50.0, 59.5));
        scoreRange.add(getScoreRangeVo(total, list, 60.0, 69.5));
        scoreRange.add(getScoreRangeVo(total, list, 70.0, 79.5));
        scoreRange.add(getScoreRangeVo(total, list, 80.0, 89.5));
        scoreRange.add(getScoreRangeVo(total, list, 90.0, 100.0));
    }

    private ScoreRangeVo getScoreRangeVo(int totalCount, List<ArchiveStudentVo> list, Double start, Double end) {
        int count = (int) list.stream().filter(s -> s.getTotalScore() >= start && s.getTotalScore() <= end).count();
        Double rate = 0d;
        if (list.size() != 0 && totalCount != 0) {
            rate = Calculator.divide(count, totalCount, 2);
        }
        ScoreRangeVo vo = new ScoreRangeVo(count, start, end, Calculator.multiply(rate, 100, 0));
        return vo;
    }

    private int getCountByPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkStudent> lw = wrapper.lambda();
        lw.eq(MarkStudent::getExamId, examId);
        lw.eq(MarkStudent::getPaperNumber, paperNumber);
        return this.count(wrapper);
    }

    @Override
    public int getAssignedCount(Long examId, Boolean checked, Long courseId, String coursePaperId,
                                MarkPaperStatus status, DataPermissionRule dpr) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setAssigned(true);
        markStudent.setAssignConfirmed(checked);
        markStudent.setMarkPaperStatus(status.name());
        return baseMapper.countAssigned(markStudent, dpr);
    }

    @Override
    public int getAssignedCount(Long examId, Boolean checked, Long courseId, String coursePaperId,
                                MarkPaperStatus status, DataPermissionRule dpr, String teachClassName) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setCourseId(courseId);
        markStudent.setCoursePaperId(coursePaperId);
        markStudent.setAssigned(true);
        markStudent.setAssignConfirmed(checked);
        markStudent.setMarkPaperStatus(status.name());
        return baseMapper.countAssignedNew(markStudent, dpr, teachClassName);
    }

    @Override
    public int countAbsentByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getExamId, examId).eq(MarkStudent::getPaperNumber, paperNumber)
                .and(o -> o.eq(MarkStudent::getAbsent, true).or().eq(MarkStudent::getManualAbsent, true).or().eq(MarkStudent::getOmrAbsent, true));
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkStudent> listScanCollegeByExamIdAndCourseCodeAndCoursePaperId(Long examId, Long courseId,
                                                                                  String coursePaperId, String status, DataPermissionRule dpr) {
        return this.baseMapper.listScanCollegeByExamIdAndCourseCodeAndCoursePaperId(examId, courseId, coursePaperId,
                status, dpr);
    }

    @Override
    public int countUnexistByExamIdAndPaperNumber(Long examId, String paperNumber) {
        MarkStudent markStudent = new MarkStudent();
        markStudent.setExamId(examId);
        markStudent.setPaperNumber(paperNumber);
        markStudent.setScanStatus(ScanStatus.UNEXIST);
        markStudent.setManualAbsent(false);
        markStudent.setMarkPaperStatus(MarkPaperStatus.FORMAL.name());
        return baseMapper.selectCountByQuery(markStudent, null);
    }

    @Override
    public void updateStudentAnswer(Long studentId) {
        List<String> objectiveAnswers = new ArrayList<>();
        MarkStudent student = this.getById(studentId);
        List<ScanStudentPaper> studentPaperList = studentPaperService.findByStudentId(studentId);
        for (ScanStudentPaper studentPaper : studentPaperList) {
            List<ScanPaperPage> scanPaperPages = scanPaperPageService.listByPaperId(studentPaper.getPaperId());
            for (ScanPaperPage scanPaperPage : scanPaperPages) {
                if (scanPaperPage.getQuestion() != null
                        && CollectionUtils.isNotEmpty(scanPaperPage.getQuestion().getResult())) {
                    for (String s : scanPaperPage.getQuestion().getResult()) {
                        if (s.startsWith("?")) {
                            s = s.replace("?", "");
                        }
                        objectiveAnswers.add(s);
                    }
                }
            }
        }
        student.setAnswers(JSON.toJSONString(objectiveAnswers));
        this.saveOrUpdate(student);
        // 客观题统分
        this.calculateObjectiveScore(student);
    }

    @Override
    public List<MarkStudentScoreVo> listMarkStudentScoreList(Long examId, String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (Objects.isNull(markPaper)) {
            return null;
        }
        if (!MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目未结束评卷，不能同步考生成绩");
        }

        List<MarkStudentVo> markStudentList = this.listByExamIdAndPaperNumberAndNotAbsent(examId, paperNumber);
        List<MarkQuestion> objectiveQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, true);
        List<MarkQuestion> subjectiveQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        List<MarkStudentScoreVo> markStudentScoreVoList = new ArrayList<>();
        for (MarkStudentVo markStudent : markStudentList) {
            MarkStudentScoreVo markStudentScoreVo = new MarkStudentScoreVo();
            markStudentScoreVo.setTeacherId(markStudent.getTeacherId());
            markStudentScoreVo.setTeachClassName(markStudent.getTeachClassName());
            markStudentScoreVo.setExamId(markStudent.getExamId());
            markStudentScoreVo.setCourseId(markStudent.getCourseId());
            markStudentScoreVo.setPaperNumber(markStudent.getPaperNumber());
            markStudentScoreVo.setStudentCode(markStudent.getStudentCode());
            markStudentScoreVo.setStudentName(markStudent.getStudentName());
            // 客观题
            markStudentScoreVo.setObjectiveScore(markStudent.getObjectiveScore());
            markStudentScoreVo.setObjectiveScoreList(markStudent.getScoreList(true, objectiveQuestionList));
            // 主观题
            markStudentScoreVo.setSubjectiveScore(markStudent.getSubjectiveScore());
            markStudentScoreVo.setSubjectiveScoreList(markStudent.getScoreList(false, subjectiveQuestionList));

            // 总分
            markStudentScoreVo.setTotalScore(markStudent.getTotalScore());
            List<ScoreItem> totalScoreItemList = new ArrayList<>();
            totalScoreItemList.addAll(markStudentScoreVo.getObjectiveScoreList());
            totalScoreItemList.addAll(markStudentScoreVo.getSubjectiveScoreList());
            totalScoreItemList.stream().sorted(Comparator.comparing(ScoreItem::getMainNumber).thenComparing(ScoreItem::getSubNumber));
            markStudentScoreVo.setTotalScoreList(totalScoreItemList);
            markStudentScoreVoList.add(markStudentScoreVo);
        }
        return markStudentScoreVoList;
    }

    @Override
    public void trackExport(Long examId, String coursePaperId, HttpServletResponse response) {
        {
            List<MarkStudent> list = this.listByExamIdAndCoursePaperId(examId, coursePaperId);
            if (CollectionUtils.isNotEmpty(list)) {
                try {
                    response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode("轨迹图.zip", "UTF-8"));
                    ZipWriter writer = ZipWriter.create(response.getOutputStream());
                    ByteArrayOutputStream os = new ByteArrayOutputStream();
                    for (MarkStudent s : list) {
                        if (!s.getUpload()) {
                            continue;
                        }
                        List<FilePathVo> vos = JSON.parseArray(s.getSheetPath(), FilePathVo.class);
                        List<MarkTrack> trackList = markTrackService.listByStudentId(s.getId());
                        Document document = new Document(new Rectangle(PageSize.A3.getHeight(), PageSize.A3.getWidth()));
                        // 本地保存目录
                        File tempFile = SystemConstant.getFileTempParentDirVar(SystemConstant.TEMP_PREFIX);
                        File file = new File(tempFile.getPath() + File.separator + s.getStudentCode() + ".pdf");
                        PdfWriter pdfWriter = PdfWriter.getInstance(document, new FileOutputStream(file));
                        document.open();
                        for (int i = 0; i < vos.size(); i++) {
                            FilePathVo vo = vos.get(i);
                            File sheet = fileUploadService.downloadFile(JSON.toJSONString(vo), vo.getPath());
                            File track = new File(tempFile.getPath() + File.separator + s.getStudentCode() + "-" + (i + 1) + ".jpg");
                            int offsetIndex = i + 1;
                            List<MarkTrack> tracks = trackList.stream().filter(t -> t.getOffsetIndex().equals(offsetIndex)).collect(Collectors.toList());
                            this.createTrack(sheet, track, tracks);
                            Image image = Image.getInstance(track.getPath());
                            image.scaleAbsolute(PageSize.A3.getHeight() - 100, PageSize.A3.getWidth() - 100);
                            if (i != 0) {
                                document.newPage();
                            }
                            document.add(image);
                            sheet.delete();
                            track.delete();
                        }
                        document.close();
                        pdfWriter.close();
                        writer.write(file, s.getStudentCode() + ".pdf");
                    }
                    writer.close();
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new ParameterException("文件下载失败", e);
                }
            }
        }
    }

    private void createTrack(File sheet, File track, List<MarkTrack> trackList) throws IOException {
        {
            FileOutputStream output = null;
            try {
                BufferedImage image = ImageIO.read(new FileInputStream(sheet));
                image = new ColorConvertOp(ColorSpace.getInstance(ColorSpace.CS_sRGB), null).filter(image, null);
                Graphics2D g = image.createGraphics();// 得到图形上下文
                g.setColor(Color.red); // 设置画笔颜色
                // 设置字体
                g.setFont(new Font("微软雅黑", Font.LAYOUT_LEFT_TO_RIGHT, 30));
                // 写入签名
                if (trackList != null && !trackList.isEmpty()) {
                    for (int i = 0; i < trackList.size(); i++) {
                        MarkTrack t = trackList.get(i);
                        BigDecimal x = new BigDecimal(t.getOffsetX());
                        BigDecimal y = new BigDecimal(t.getOffsetY());
                        g.drawString(t.getScore().toString(), x.intValue(), y.intValue());
                    }
                }
                g.dispose();

                output = new FileOutputStream(track);
                ImageIO.write(image, "jpg", output);
            } catch (Exception e) {
                log.error(SystemConstant.LOG_ERROR, e);
            } finally {
                if (Objects.nonNull(output)) {
                    output.flush();
                    output.close();
                }
            }
        }
    }

    @Override
    public void sheetExport(Long examId, String coursePaperId, HttpServletResponse response) {
        List<MarkStudent> list = this.listByExamIdAndCoursePaperId(examId, coursePaperId);
        if (CollectionUtils.isNotEmpty(list)) {
            try {
                response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode("原图.zip", "UTF-8"));
                ZipWriter writer = ZipWriter.create(response.getOutputStream());
                ByteArrayOutputStream os = new ByteArrayOutputStream();
                for (MarkStudent s : list) {
                    if (!s.getUpload()) {
                        continue;
                    }
                    List<FilePathVo> vos = JSON.parseArray(s.getSheetPath(), FilePathVo.class);
                    for (int i = 0; i < vos.size(); i++) {
                        FilePathVo vo = vos.get(i);
                        File file = fileUploadService.downloadFile(JSON.toJSONString(vo), vo.getPath());
                        String format = FilenameUtils.getExtension(file.getName());
                        writer.write(file, s.getStudentCode() + "-" + (i + 1) + "." + format);
                        file.delete();
                    }
                }
                writer.close();
            } catch (Exception e) {
                e.printStackTrace();
                throw new ParameterException("文件下载失败", e);
            }
        }
    }

    @Override
    public MarkStudent assembleByBasicExamStudent(BasicExamStudent basicExamStudent, MarkPaper
            markPaper, Set<String> secretNumberSet, boolean containPaperType) {
        MarkStudent markStudent = this.findByBasicStudentId(basicExamStudent.getId());
        if (markStudent == null) {
            if (MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("试卷编号[" + markPaper.getPaperNumber() + "]已结束阅卷，不允许新增考生");
            }
            if (containPaperType && StringUtils.isBlank(basicExamStudent.getPaperType())) {
                throw ExceptionResultEnum.ERROR.exception("卷型必须");
            } else {
                markStudent = new MarkStudent(SystemConstant.getDbUuid(), basicExamStudent.getExamId(), basicExamStudent.getId(), basicExamStudent.getCourseId(), basicExamStudent.getPaperNumber(), markPaper.getCoursePaperId(), markPaper.getSerialNumber(), basicExamStudent.getPaperType(), basicExamStudent.getStudentCode(), "", basicExamStudent.getExamPlace(), basicExamStudent.getExamRoom(), basicExamStudent.getExamStartTime(), basicExamStudent.getExamEndTime(), markPaper.getUserId());
                if (markStudent.getSecretNumber() == null) {
                    markStudent.randomSecretNumber();
                    while (secretNumberSet.contains(markStudent.getSecretNumber())
                            || this.countByExamIdAndSecretNumber(markStudent.getExamId(), markStudent.getSecretNumber()) > 0) {
                        markStudent.randomSecretNumber();
                    }
                }
                secretNumberSet.add(markStudent.getSecretNumber());
            }
        } else {
            markStudent.setPaperNumber(basicExamStudent.getPaperNumber());
            markStudent.setPaperType(basicExamStudent.getPaperType());
        }
        return markStudent;
    }

    @Override
    public void deleteByBasicStudentId(List<Long> basicExamStudentIds) {
        List<MarkStudent> markStudentList = this.listByBasicStudentIds(basicExamStudentIds);
        List<Long> markStudentIds = markStudentList.stream().map(MarkStudent::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(markStudentIds)) {
            return;
        }
        List<ScanStudentPaper> scanStudentPaperList = scanStudentPaperService.listByStudentIds(markStudentIds);
        List<Long> paperIds = scanStudentPaperList.stream().map(ScanStudentPaper::getPaperId).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(paperIds)) {
            scanPaperPageService.removeByPaperIds(paperIds);
            scanPaperService.removeByIds(paperIds);
        }

        scanStudentPaperService.removeByStudentIds(markStudentIds);
        scanBatchPaperService.removeByStudentIds(markStudentIds);
        scanOmrTaskService.deleteByStudentIds(markStudentList.get(0).getExamId(), markStudentIds);
        this.removeByIds(markStudentIds);
    }

    @Override
    public Integer maxCardNumber(Long examId, String paperNumber) {
        return this.baseMapper.maxCardNumber(examId, paperNumber);
    }

    @Override
    public void scoreDownload(ArchiveStudentQuery query, HttpServletResponse response) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        //生成表头
        String[] columnNames = new String[]{"序号", "班级", "学号", "姓名", "平时(必填)", "实验(必填)", "期末(必填)", "备注"};
        //生成动态内容
        List<String[]> columnValues = new ArrayList<>();
        // 使用成绩管理列表请求的数据权限
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/archive/score/list");
        List<ArchiveStudentVo> ret = baseMapper.studentList(sysUser.getSchoolId(), query, dpr);
        for (int i = 0; i < ret.size(); i++) {
            ArchiveStudentVo s = ret.get(i);
            List<String> valueList = new ArrayList<>();
            valueList.add(String.valueOf(i + 1));
            valueList.add(s.getTeachClassName());
            valueList.add(s.getStudentCode());
            valueList.add(s.getStudentName());
            valueList.add("");
            valueList.add("");
            valueList.add(s.getTotalScore() == null ? "" : s.getTotalScore().toString());
            valueList.add("");
            String[] columnValue = valueList.toArray(new String[valueList.size()]);
            columnValues.add(columnValue);
        }
        try {
            String sheetName = "成绩下载";
            log.debug("导出Excel开始...");
            response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode("成绩下载", SystemConstant.CHARSET_NAME) + ".xlsx");
            response.setContentType("application/vnd.ms-excel");
            ServletOutputStream outputStream = response.getOutputStream();
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);

            CellStyle cellStyle1 = writer.createCellStyle();
            org.apache.poi.ss.usermodel.Font font1 = writer.createFont();
            font1.setBold(true);
            font1.setFontHeightInPoints((short) 20);
            cellStyle1.setFont(font1);
            cellStyle1.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
            cellStyle1.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            writer.setCellStyle(sheetName, cellStyle1, 0, new int[]{0, 1, 2, 3, 7});

            CellStyle cellStyle2 = writer.createCellStyle();
            org.apache.poi.ss.usermodel.Font font2 = writer.createFont();
            font2.setBold(true);
            font2.setFontHeightInPoints((short) 15);
            font2.setColor(IndexedColors.RED.getIndex());
            cellStyle2.setFont(font2);
            cellStyle2.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
            cellStyle2.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            writer.setCellStyle(sheetName, cellStyle2, 0, new int[]{4, 5, 6});

            writer.writeStringArrays(sheetName, null, columnNames, columnValues.listIterator());

            writer.output(outputStream);
            outputStream.flush();
            outputStream.close();
            log.debug("导出Excel结束");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Transactional
    @Override
    public Boolean missScanUpdate(Long examId, String coursePaperId, String studentCode, Boolean missScan) {
        MarkStudent student = findByExamIdAndCoursePaperIdAndStudentCode(examId, coursePaperId, studentCode);
        if (student == null) {
            throw new ParameterException("考生未找到");
        }
        if (!ScanStatus.UNEXIST.equals(student.getScanStatus())) {
            throw new ParameterException("考生不是未扫描状态");
        }
        LambdaUpdateWrapper<MarkStudent> lw = new LambdaUpdateWrapper<>();
        lw.set(MarkStudent::getScanStatus, ScanStatus.UNEXIST);
        lw.set(MarkStudent::getManualAbsent, false);
        lw.set(MarkStudent::getAbsent, false);
        lw.set(MarkStudent::getMissScan, missScan);
        lw.eq(MarkStudent::getId, student.getId());
        this.update(lw);

        return true;
    }

    @Override
    public MarkStudentVo getMarkStudentVoByStudentId(Long studentId) {
        MarkStudentQuery markStudentQuery = new MarkStudentQuery();
        markStudentQuery.setId(studentId);
        List<MarkStudentVo> markStudentVoList = this.baseMapper.listMarkStudentVo(markStudentQuery);
        return CollectionUtils.isNotEmpty(markStudentVoList) ? markStudentVoList.get(0) : null;
    }

    @Override
    public List<MarkStudentVo> listMarkStudentVo(MarkStudentQuery markStudentQuery) {
        return this.baseMapper.listMarkStudentVo(markStudentQuery);
    }

    @Override
    public IPage<MarkStudentVo> pageMarkStudentVo(Page<MarkStudentVo> page, MarkStudentQuery markStudentQuery) {
        return this.baseMapper.listMarkStudentVo(page, markStudentQuery);
    }

    @Override
    public MarkStudent findByBasicStudentId(Long basicStudentId) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkStudent::getBasicStudentId, basicStudentId);
        return this.getOne(queryWrapper);
    }

    @Override
    public void updateBasicExamStudentPaperType(Long markStudentId) {
        MarkStudent markStudent = this.getById(markStudentId);
        if (markStudent != null && StringUtils.isNotBlank(markStudent.getPaperType()) && markStudent.getBasicStudentId() != null) {
            this.baseMapper.updateBasicExamStudentPaperType(markStudent.getPaperType(), markStudent.getBasicStudentId());
        }
    }

    @Override
    public boolean calcBatchObjectiveScore(List<Long> ids) {
        for (Long id : ids) {
            MarkPaper markPaper = markPaperService.getById(id);
            if (markPaper != null) {
                if (lockService.trylock(LockType.SCORE_CALCULATE, markPaper.getId())) {
                    markSyncService.calcObjectiveScore(markPaper);
                } else {
                    throw ExceptionResultEnum.ERROR.exception("客观题正在统分，请稍后再试");
                }
            }
        }
        return true;
    }

    @Override
    public List<MarkStudent> listByBasicStudentIds(List<Long> basicExamStudentIds) {
        QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(MarkStudent::getBasicStudentId, basicExamStudentIds);
        return this.list(queryWrapper);
    }

    @Override
    public void confirmSubjectiveInspectedTask(Long studentId) {
        Long userId = ServletUtil.getRequestUserId();
        MarkStudent markStudent = this.getById(studentId);
        if (markStudent == null) {
            throw ExceptionResultEnum.ERROR.exception("考生不存在");
        }
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markStudent.getExamId(),
                markStudent.getPaperNumber());
        // 评卷是否结束
        if (markPaper == null || MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法执行此操作");
        }
        try {
            lockService.watch(LockType.EXAM_SUBJECT, markStudent.getExamId(), markStudent.getPaperNumber());
            lockService.waitlock(LockType.STUDENT, studentId);
            this.updateCheckInfo(markStudent.getId(), userId);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        } finally {
            lockService.unlock(LockType.STUDENT, studentId);
            lockService.unwatch(LockType.EXAM_SUBJECT, markStudent.getExamId(), markStudent.getPaperNumber());
            markService.releaseByStudent(markStudent);
        }
    }

    @Override
    public MarkStudentVo getDetailById(Long studentId) {
        return this.baseMapper.getDetailById(studentId);
    }

    @Override
    public ScanOmrTaskStatusDto getStatus(Long examId, Long courseId, String coursePaperId) {
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission();
        Integer todoCount = this.getPaperTypeCheckCount(examId, courseId, coursePaperId, PaperTypeCheckStatus.WAITING,
                dpr);
        Integer finishCount = this.getPaperTypeCheckCount(examId, courseId, coursePaperId, PaperTypeCheckStatus.PROCESSED, dpr);
        ScanOmrTaskStatusDto result = new ScanOmrTaskStatusDto();
        result.setTodoCount(todoCount);
        result.setFinishCount(finishCount);
        return result;
    }

    @Transactional
    @Override
    public UpdateTimeVo disposePaperTypeCheck(Long studentId, String paperType) {
        // 更新识别缺考值
        updatePaperPageData(studentId, null, null, paperType);

        // 更改考生卷型和卷型检查状态
        UpdateWrapper<MarkStudent> markStudentUpdateWrapper = new UpdateWrapper<>();
        markStudentUpdateWrapper.lambda().eq(MarkStudent::getId, studentId).set(MarkStudent::getPaperType, paperType)
                .set(MarkStudent::getPaperTypeCheckStatus, PaperTypeCheckStatus.PROCESSED);
        this.update(markStudentUpdateWrapper);
        // 更新考生管理卷型
        updateBasicExamStudentPaperType(studentId);
        // 客观题统分
        this.calculateObjectiveScore(this.getById(studentId));
        return UpdateTimeVo.create();
    }

    @Override
    public AbInfoVo findExamTaskPaperTypeOpenStatus(Long examId, String paperNumber) {
        return this.baseMapper.findExamTaskPaperTypeOpenStatus(examId, paperNumber);
    }

    @Override
    public UpdateTimeVo breachUpdate(Long studentId, Boolean breach) {
        // 更新识别缺考值
        updatePaperPageData(studentId, null, breach, null);

        // 更改考生卷型和卷型检查状态（更新人工指定违纪状态时，识别违纪改为false）
        UpdateWrapper<MarkStudent> markStudentUpdateWrapper = new UpdateWrapper<>();
        markStudentUpdateWrapper.lambda().eq(MarkStudent::getId, studentId).set(MarkStudent::getManualBreach, breach)
                .set(MarkStudent::getOmrBreach, false)
                .set(MarkStudent::getBreach, breach);
        this.update(markStudentUpdateWrapper);
        return UpdateTimeVo.create();
    }

}
