package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.mark.MarkProblemStatus;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.dto.mark.manage.*;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkProblemHistory;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.mapper.MarkTaskMapper;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.Calculator;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 评卷任务表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkTaskServiceImpl extends ServiceImpl<MarkTaskMapper, MarkTask> implements MarkTaskService {

    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkProblemHistoryService markProblemHistoryService;
    @Resource
    private MarkService markService;
    @Resource
    private LockService lockService;
    @Resource
    private TaskService taskService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Override
    public IPage<MarkManageDto> listPaperManage(Long examId, Long openCollegeId, Long courseId, String paperNumber, Boolean progressStatus, Integer pageNumber, Integer pageSize) {
        Page<MarkManageDto> page = new Page<>(pageNumber, pageSize);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<MarkManageDto> markManageDtoIPage = this.baseMapper.listPaperManage(page, examId, openCollegeId, courseId,
                paperNumber, progressStatus, dpr);
        for (MarkManageDto record : markManageDtoIPage.getRecords()) {
            record.setStatusDisplay(record.getStatus() == null ? null : record.getStatus().getName());
            record.setPercent(Calculator.divide2String(Calculator.multiply(record.getMarkedCount(), 100), Double.valueOf(record.getTotalCount()), 2));
            // 问题卷数量
            record.setProblemCount(markProblemHistoryService.countByExamIdAndPaperNumberAndStatus(examId, record.getPaperNumber(), MarkProblemStatus.WAITING));
            // 正在评卷数量
            record.setCurrentCount(markGroupService.countCurrentCountByExamIdAndPaperNumber(examId, record.getPaperNumber()));
        }
        return markManageDtoIPage;
    }

    @Override
    public void exportMarker(Long examId, Long courseId, String paperNumber, HttpServletResponse response) {
        List<MarkerInfoDto> markerInfoDtoList = this.baseMapper.listUserMarkedCount(examId, courseId, paperNumber);
        for (MarkerInfoDto markerInfoDto : markerInfoDtoList) {
            MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber());
            markGroup.setQuestionList(markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber(), false));
            markerInfoDto.setGroupName(markGroup.getQuestionTitle());
            markerInfoDto.setTotalScore(markQuestionService.sumTotalScoreByGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber()));
        }
        try {
            ExcelUtil.excelExport("评卷员工作量", MarkerInfoDto.class, markerInfoDtoList, response);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("导出评卷员工作量失败");
        }
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndClassName(Long examId, String paperNumber, Integer groupNumber, Long userId, String className) {
        return this.baseMapper.listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndClassName(examId, paperNumber, groupNumber, userId, className);
    }

    @Override
    public Integer markedCount(List<MarkTask> markTask) {
        if (CollectionUtils.isEmpty(markTask)) {
            return 0;
        }
        long count = markTask.stream().filter(m -> m.getStatus().equals(MarkTaskStatus.MARKED) || m.getStatus().equals(MarkTaskStatus.ARBITRATED) || m.getStatus().equals(MarkTaskStatus.WAIT_ARBITRATE)).count();
        return (int) count;
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndUserId(Long examId, String paperNumber, Integer groupNumber, Long userId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkTask> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber);
        if (userId != null) {
            lambdaQueryWrapper.eq(MarkTask::getUserId, userId);
        }
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndStatusNotIn(Long examId, String paperNumber, Integer groupNumber, Long userId, List<MarkTaskStatus> statusList) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getUserId, userId)
                .notIn(MarkTask::getStatus, statusList);
        return this.list(queryWrapper);
    }

    @Override
    public boolean resetById(Long markTaskId, Long userId, String rejectReason, Long rejectId, Long date, MarkTaskStatus newStatus) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkTask::getStatus, newStatus)
                .set(MarkTask::getUserId, userId)
                .set(MarkTask::getMarkerTime, null)
                .set(MarkTask::getMarkerScore, null)
                .set(MarkTask::getMarkerScoreList, null)
                .set(MarkTask::getMarkerSpent, null)
                .set(MarkTask::getHeaderId, rejectId)
                .set(MarkTask::getHeaderTime, date)
                .set(MarkTask::getHeaderScore, null)
                .set(MarkTask::getHeaderScoreList, null)
                .set(MarkTask::getRejectReason, rejectReason)
                .eq(MarkTask::getId, markTaskId);
        return this.update(updateWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndStatusIn(Long examId, String paperNumber, Integer groupNumber, List<MarkTaskStatus> statusList) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .in(MarkTask::getStatus, statusList);
        return this.count(queryWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndStatusIn(Long examId, String paperNumber, List<MarkTaskStatus> statusList) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .in(MarkTask::getStatus, statusList);
        return this.count(queryWrapper);
    }

    @Override
    public MarkTask getLastOneByUserIdAndStatus(Long examId, String paperNumber, Integer groupNumber, Long userId, MarkTaskStatus status) {
        return this.baseMapper.getLastOneByUserIdAndStatus(examId, paperNumber, groupNumber, userId, status.name());
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndStudentId(Long examId, String paperNumber, Integer groupNumber, Long studentId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getStudentId, studentId);
        return this.list(queryWrapper);
    }

    @Override
    public List<MarkTask> listByStudentIdAndGroupNumber(Long studentId, Integer groupNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getStudentId, studentId)
                .eq(MarkTask::getGroupNumber, groupNumber);
        return this.list(queryWrapper);
    }

    @Override
    public void updateHeaderResult(Long examId, String paperNumber, Integer groupNumber, Long studentId, Long updateUserId, Double totalScore, String scoreList, Long updateTime, MarkTaskStatus arbitrated) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkTask::getHeaderId, updateUserId)
                .set(MarkTask::getHeaderScore, totalScore)
                .set(MarkTask::getHeaderScoreList, scoreList)
                .set(MarkTask::getHeaderTime, updateTime)
                .set(MarkTask::getStatus, arbitrated)
                .eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getStudentId, studentId);
        this.update(updateWrapper);
    }

    @Override
    public void deleteByStudentId(Long studentId) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkTask::getStudentId, studentId);
        this.remove(updateWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber);
        return this.count(queryWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndUserIdAndAndClassNameStatusIn(Long examId, String paperNumber, Integer groupNumber, Long userId, List<String> className, MarkTaskStatus... status) {
        return this.baseMapper.countByExamIdAndPaperNumberAndGroupNumberAndUserIdAndAndClassNameStatusIn(examId, paperNumber, groupNumber, userId, className, status);
    }

    @Override
    public void deleteByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber);
        this.remove(updateWrapper);
    }

    @Override
    public IPage<MarkTask> listPageHistory(Page<MarkTask> page, Long userId, Long examId, String paperNumber,
                                           Integer groupNumber, String secretNumber, Double markerScore) {
        return this.baseMapper.listPageHistory(page, userId, examId, paperNumber, groupNumber, secretNumber, markerScore);
    }

    @Override
    public List<MarkTask> findUnMarked(Page<MarkTask> page, Long examId, String paperNumber, Integer groupNumber,
                                       Long userId) {
        return this.baseMapper.findUnMarked(page, examId, paperNumber, groupNumber, userId);
    }

    @Override
    public List<MarkTask> findUnMarkedFilterClass(Page<MarkTask> page, Long examId, String paperNumber,
                                                  Integer groupNumber, Long userId) {
        return this.baseMapper.findUnMarkedFilterClass(page, examId, paperNumber, groupNumber, userId);
    }

    @Override
    public int countByIdAndStatus(Long id, MarkTaskStatus status) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getId, id)
                .eq(MarkTask::getStatus, status);
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkTask> findByStudentIdAndGroupNumberAndStatus(Long studentId, Integer groupNumber,
                                                                 MarkTaskStatus... status) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getStudentId, studentId)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .in(MarkTask::getStatus, Arrays.asList(status));
        return this.list(queryWrapper);
    }

    @Override
    public boolean updateProblemResult(Long taskId, Long userId, Long now, int spent) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkTask::getId, taskId)
                .in(MarkTask::getStatus, Arrays.asList(MarkTaskStatus.WAITING, MarkTaskStatus.REJECTED, MarkTaskStatus.MARKED))
                .set(MarkTask::getStatus, MarkTaskStatus.PROBLEM)
                .set(MarkTask::getUserId, userId)
                .set(MarkTask::getMarkerScore, null)
                .set(MarkTask::getMarkerScoreList, null)
                .set(MarkTask::getMarkerTime, now)
                .set(MarkTask::getMarkerSpent, spent);
        return this.update(updateWrapper);
    }

    @Override
    public boolean updateStatusByStudentIdAndGroupNumber(Long studentId, Integer groupNumber, MarkTaskStatus status) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkTask::getStudentId, studentId)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .set(MarkTask::getStatus, status);
        return this.update(updateWrapper);
    }

    @Override
    public int countByStudentIdAndMarkerIdAndIdNotEqual(Long studentId, Long examId, String paperNumber, Integer groupNumber, Long userId, Long taskId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getStudentId, studentId)
                .eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getUserId, userId)
                .ne(MarkTask::getId, taskId);
        return this.count(queryWrapper);
    }

    @Override
    public boolean updateMarkerResult(Long taskId, MarkTaskStatus status, Long userId, Double markerScore,
                                      String scoreList, Long now, int spent, MarkTaskStatus... inStatus) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkTask::getId, taskId)
                .in(MarkTask::getStatus, Arrays.asList(inStatus))
                .set(MarkTask::getStatus, status)
                .set(MarkTask::getUserId, userId)
                .set(MarkTask::getMarkerScore, markerScore)
                .set(MarkTask::getMarkerScoreList, scoreList)
                .set(MarkTask::getMarkerTime, now)
                .set(MarkTask::getMarkerSpent, spent)
                .set(MarkTask::getHeaderId, null)
                .set(MarkTask::getHeaderTime, null)
                .set(MarkTask::getRejectReason, null);
        return this.update(updateWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndTaskNumber(Long examId, String paperNumber, Integer groupNumber, int taskNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getTaskNumber, taskNumber);
        return this.count(queryWrapper);
    }

    @Override
    public int countByStudentId(Long studentId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getStudentId, studentId);
        return this.count(queryWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber);
        return this.count(queryWrapper);
    }

    @Override
    public IPage<MarkTaskDto> pageMarkTask(Long examId, String paperNumber, Integer groupNumber, String loginName, MarkTaskStatus status, String studentCode, String secretNumber, String teachClassName, Double subScore, Integer pageNumber, Integer pageSize) {
        IPage<MarkTaskDto> markTaskDtoIPage = this.baseMapper.pageMarkTask(new Page<>(pageNumber, pageSize), examId, paperNumber, groupNumber, loginName, status, studentCode, secretNumber, teachClassName, subScore);
        for (MarkTaskDto record : markTaskDtoIPage.getRecords()) {
            record.setStatusDisplay(record.getStatus().getName());
            // 分组题目
            record.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(record.getExamId(), record.getPaperNumber(), record.getGroupNumber()));
        }
        return markTaskDtoIPage;
    }

    @Override
    public void rejectMarkTask(Long id, String rejectReason) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        MarkTask markTask = this.getById(id);
        if (markTask == null) {
            throw ExceptionResultEnum.ERROR.exception("任务不存在");
        }
        try {
            lockService.watch(LockType.EXAM_SUBJECT, markTask.getExamId(), markTask.getPaperNumber());
            lockService.watch(LockType.GROUP, markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            lockService.waitlock(LockType.STUDENT, markTask.getStudentId());
            if (!markTask.getStatus().equals(MarkTaskStatus.MARKED)) {
                throw ExceptionResultEnum.ERROR.exception("当前任务不允许打回");
            }
            markService.rejectMarkTask(markTask, sysUser.getId(), rejectReason);
        } catch (Exception e) {
            log.error("打回失败", e);
        } finally {
            lockService.unlock(LockType.STUDENT, markTask.getStudentId());
            lockService.unwatch(LockType.GROUP, markTask.getExamId(), markTask.getPaperNumber(), markTask.getGroupNumber());
            lockService.unwatch(LockType.EXAM_SUBJECT, markTask.getExamId(), markTask.getPaperNumber());
        }
    }

    @Override
    public Task getMarkTrack(Long id) {
        MarkTask markTask = this.getById(id);
        if (markTask == null) {
            throw ExceptionResultEnum.ERROR.exception("任务不存在，请刷新列表");
        }
        if (!MarkTaskStatus.MARKED.equals(markTask.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("已评任务才允许打回");
        }
        Task dto = taskService.build(markTask);
        dto.setPrevious(true);
        return dto;
    }
}
