package com.qmth.teachcloud.mark.utils;

import java.util.LinkedList;
import java.util.List;

import com.qmth.teachcloud.mark.dto.mark.MarkQuestionAnswerVo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ObjectivePolicy;
import com.qmth.teachcloud.mark.dto.mark.ScoreInfo;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.entity.MarkStudent;

public class ScoreCalculateUtil {

    protected static Logger log = LoggerFactory.getLogger(ScoreCalculateUtil.class);

    private MarkStudent student;

    private ScoreCalculateUtil(MarkStudent student) {
        this.student = student;
    }

    public ScoreInfo calculate(List<MarkQuestionAnswerVo> oList, List<MarkQuestion> sList) {
        // log.debug("start calculate, examId=" + student.getExamId() +
        // ", examNumber=" + student.getExamNumber());

        ScoreInfo info = new ScoreInfo();
        if (oList != null && oList.size() > 0) {
            calculateObjective(oList, info);
        }
        if (sList != null && sList.size() > 0) {
            calculateSubjective(sList, info);
        }
        return info;
    }

    private void calculateObjective(List<MarkQuestionAnswerVo> questionList, ScoreInfo scoreInfo) {
        // log.debug("start calculate objective score");
        List<String> answerList = student.getAnswerList();
        // Map<String, Integer> config = exam.getObjectiveBlockMap();

        // log.debug("student answers: " + student.getAnswers());
        // log.debug("exam objective config: " + exam.getObjectiveBlock());

        // 客观题未设置统分策略，不统分
        long count = questionList.stream().filter(m -> m.getObjectivePolicy() == null).count();
        if (count > 0) {
            return;
        }

        int index = -1;
        int lastMainNumber = 0;
        for (MarkQuestionAnswerVo question : questionList) {
            if (StringUtils.isBlank(question.getAnswer())) {
                continue;
            }
            // log.debug("find objective block for " + block.getTitle() +
            // ", question start=" + start);
            if (question.getMainNumber() != lastMainNumber) {
                scoreInfo.incrMainNumber(question.getMainTitle());
            }

            index++;
            String answer = "#";
            double score = 0d;
            try {
                answer = answerList.get(index);
            } catch (Exception e) {
                continue;
            }

            boolean correct = true;
            int correctCount = 0;
            // 客观题判分策略
            // 任选给分
            if (ObjectivePolicy.ALL.equals(question.getObjectivePolicy())) {
                score = answer.length() == 0 || answer.equals("#") ? 0 : question.getTotalScore();
            } else {
                for (int i = 0; i < answer.length(); i++) {
                    if (!question.getAnswer().contains(String.valueOf(answer.charAt(i)))) {
                        correct = false;
                        break;
                    } else {
                        correctCount++;
                    }
                }
                if (correct & ObjectivePolicy.LEAK.equals(question.getObjectivePolicy())) {
                    score = answer.length() < question.getAnswer().length() ? question.getObjectivePolicyScore() * correctCount : question
                            .getTotalScore();
                }
                if (correct & ObjectivePolicy.LEAK_HALF.equals(question.getObjectivePolicy())) {
                    score = answer.length() < question.getAnswer().length() ? question.getTotalScore() / 2 : question
                            .getTotalScore();
                }
                if (correct
                        & (question.getObjectivePolicy() == null || ObjectivePolicy.NONE.equals(question
                        .getObjectivePolicy()))) {
                    score = answer.length() < question.getAnswer().length() ? 0 : question.getTotalScore();
                }
            }

            // log.debug("question result: " + question.getNumber() + ", " +
            // answer + ", " + score);
            scoreInfo.addScore(null, question.getSubNumber(), score, answer, true);
        }
    }

    private void calculateSubjective(List<MarkQuestion> sList, ScoreInfo scoreInfo) {

    }

    public static ScoreCalculateUtil instance(MarkStudent student) {
        return new ScoreCalculateUtil(student);
    }

    public static void main(String[] args) {
        MarkStudent student = new MarkStudent();
        student.setAnswers("A,B,AB,C,#,#,#,#");
        ScoreCalculateUtil util = ScoreCalculateUtil.instance(student);

        List<MarkQuestion> oList = new LinkedList<MarkQuestion>();
        // ExamQuestion q = new ExamQuestion();
        // q.setMainNumber(1);
        // q.setMainTitle("单选");
        // q.setSubNumber(1);
        // q.setAnswer("B");
        // q.setTotalScore(2d);
        // oList.add(q);
        //
        // q = new ExamQuestion();
        // q.setMainNumber(1);
        // q.setMainTitle("单选");
        // q.setSubNumber(2);
        // q.setAnswer("B");
        // q.setTotalScore(2d);
        // oList.add(q);
        //
        // q = new ExamQuestion();
        // q.setMainNumber(2);
        // q.setMainTitle("多选");
        // q.setSubNumber(1);
        // q.setAnswer("AC");
        // q.setTotalScore(3d);
        // oList.add(q);
        //
        // q = new ExamQuestion();
        // q.setMainNumber(2);
        // q.setMainTitle("多选");
        // q.setSubNumber(2);
        // q.setAnswer("C");
        // q.setTotalScore(3d);
        // oList.add(q);

//        ScoreInfo info = util.calculate(oList, null);
//        System.out.println(info.getObjectiveScore());
//        for (ScoreItem item : info.getScoreList()) {
//            System.out.println(item.getTitle() + ": " + item.toString());
//        }
    }
}
