package com.qmth.teachcloud.obe.been.report;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.obe.been.dto.CourseWeightDto;
import io.swagger.annotations.ApiModelProperty;
import org.jetbrains.annotations.NotNull;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * @Description: 课程目标word dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/2/26
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class CourseTargetWordDto implements Serializable, Comparable<CourseTargetWordDto> {

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "课程目标id")
    private Long targetId;

    @ApiModelProperty(value = "课程目标名称")
    private String targetName;

    @ApiModelProperty(value = "毕业要求")
    private String graduationRequirement;

    @ApiModelProperty(value = "毕业要求指标点")
    private String graduationRequirementPoint;

    @ApiModelProperty(value = "目标整体权重")
    private BigDecimal targetSumWeight = new BigDecimal(0);

    @ApiModelProperty(value = "换算平均分")
    private BigDecimal matrixAvgScore = new BigDecimal(0);

    @ApiModelProperty(value = "目标达成评价值")
    private BigDecimal evaluationValue = new BigDecimal(0);

    @ApiModelProperty(value = "平时成绩信息")
    private UsualScoreWordDto usualScoreDto;

    @ApiModelProperty(value = "期末成绩信息")
    private FinalScoreWordDto finalScoreDto;

    @ApiModelProperty(value = "知识点卷面总分")
    private BigDecimal dimensionScoreSum = new BigDecimal(0);

    @ApiModelProperty(value = "知识点集合")
    private String dimensionPoint;

    @ApiModelProperty(value = "期末成绩小题卷面总分")
    private BigDecimal finalScoreQuestionScoreSum = new BigDecimal(0);

    @ApiModelProperty("指标点名称")
    private String obeCultureProgramRequirementName;

    @ApiModelProperty("父指标点名称")
    private String parentRequirementName;

    public CourseTargetWordDto() {

    }

    public CourseTargetWordDto(CourseWeightDto courseWeightDto) {
        this.targetId = courseWeightDto.getCourseTargetId();
        this.targetName = courseWeightDto.getCourseTargetName();
        this.graduationRequirement = courseWeightDto.getDegreeRequirement();
        this.graduationRequirementPoint = courseWeightDto.getDimensionStr();
        this.targetSumWeight = courseWeightDto.getTotalWeight();
        this.obeCultureProgramRequirementName = courseWeightDto.getObeCultureProgramRequirementName();
        this.parentRequirementName = courseWeightDto.getParentRequirementName();
    }

    public String getObeCultureProgramRequirementName() {
        return obeCultureProgramRequirementName;
    }

    public void setObeCultureProgramRequirementName(String obeCultureProgramRequirementName) {
        this.obeCultureProgramRequirementName = obeCultureProgramRequirementName;
    }

    public String getParentRequirementName() {
        return parentRequirementName;
    }

    public void setParentRequirementName(String parentRequirementName) {
        this.parentRequirementName = parentRequirementName;
    }

    public String getDimensionPoint() {
        return dimensionPoint;
    }

    public void setDimensionPoint(String dimensionPoint) {
        this.dimensionPoint = dimensionPoint;
    }

    public void setMatrixAvgScore(BigDecimal matrixAvgScore) {
        this.matrixAvgScore = matrixAvgScore;
    }

    public BigDecimal getDimensionScoreSum() {
        return dimensionScoreSum;
    }

    public void setDimensionScoreSum(BigDecimal dimensionScoreSum) {
        this.dimensionScoreSum = dimensionScoreSum;
    }

    public BigDecimal getFinalScoreQuestionScoreSum() {
        return finalScoreQuestionScoreSum;
    }

    public void setFinalScoreQuestionScoreSum(BigDecimal finalScoreQuestionScoreSum) {
        this.finalScoreQuestionScoreSum = finalScoreQuestionScoreSum;
    }

    public BigDecimal getEvaluationValue() {
        return evaluationValue;
    }

    public void setEvaluationValue(BigDecimal evaluationValue) {
        this.evaluationValue = evaluationValue;
    }

    public String getGraduationRequirement() {
        return graduationRequirement;
    }

    public void setGraduationRequirement(String graduationRequirement) {
        this.graduationRequirement = graduationRequirement;
    }

    public String getGraduationRequirementPoint() {
        return graduationRequirementPoint;
    }

    public void setGraduationRequirementPoint(String graduationRequirementPoint) {
        this.graduationRequirementPoint = graduationRequirementPoint;
    }

    public BigDecimal getTargetSumWeight() {
        return targetSumWeight;
    }

    public void setTargetSumWeight(BigDecimal targetSumWeight) {
        this.targetSumWeight = targetSumWeight;
    }

    public Long getTargetId() {
        return targetId;
    }

    public void setTargetId(Long targetId) {
        this.targetId = targetId;
    }

    public String getTargetName() {
        return targetName;
    }

    public void setTargetName(String targetName) {
        this.targetName = targetName;
    }

    public UsualScoreWordDto getUsualScoreDto() {
        return usualScoreDto;
    }

    public void setUsualScoreDto(UsualScoreWordDto usualScoreDto) {
        this.usualScoreDto = usualScoreDto;
    }

    public FinalScoreWordDto getFinalScoreDto() {
        return finalScoreDto;
    }

    public void setFinalScoreDto(FinalScoreWordDto finalScoreDto) {
        this.finalScoreDto = finalScoreDto;
    }

    public BigDecimal getMatrixAvgScore() {
        return matrixAvgScore;
    }

    @Override
    public int compareTo(@NotNull CourseTargetWordDto o) {
        return this.getTargetId().compareTo(o.getTargetId());
    }
}
