package com.qmth.teachcloud.obe.been.report;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.qmth.teachcloud.obe.been.obe.CourseRequirementMatrixDto;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: obe课程毕业指标dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/6/12
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ObeCourseRequirementDto implements Serializable {

    @ApiModelProperty(value = "课程编码")
    private String courseCode;

    @ApiModelProperty(value = "课程名称")
    private String courseName;

    @ApiModelProperty(value = "学期")
    private String semesterName;

    @ApiModelProperty(value = "专业毕业指标达成值")
    private BigDecimal matrixDegree;

    @ApiModelProperty(value = "课程毕业指标集合")
    private List<ObeRequirementDto> obeRequirements;

    @ApiModelProperty(value = "课程指标达成值")
    private BigDecimal courseMatrixDegree;

    public ObeCourseRequirementDto() {

    }

    public ObeCourseRequirementDto(CourseRequirementMatrixDto courseRequirementMatrixDto, BigDecimal courseMatrixDegree) {
        this.courseCode = courseRequirementMatrixDto.getCourseCode();
        this.courseName = courseRequirementMatrixDto.getCourseName();
        this.semesterName = courseRequirementMatrixDto.getSemesterName();
        this.courseMatrixDegree = courseMatrixDegree;
    }

    public ObeCourseRequirementDto(ObeCourseRequirementDto obeCourseRequirementDto) {
        this.courseCode = obeCourseRequirementDto.getCourseCode();
        this.courseName = obeCourseRequirementDto.getCourseName();
        this.semesterName = obeCourseRequirementDto.getSemesterName();
        this.matrixDegree = obeCourseRequirementDto.getMatrixDegree();
        this.obeRequirements = obeCourseRequirementDto.getObeRequirements().stream().map(ObeRequirementDto::new).collect(Collectors.toList());
        this.courseMatrixDegree = obeCourseRequirementDto.getCourseMatrixDegree();
    }

    public void updateInfo() {
        this.courseCode = null;
        this.courseName = null;
    }

    public String getSemesterName() {
        return semesterName;
    }

    public void setSemesterName(String semesterName) {
        this.semesterName = semesterName;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public BigDecimal getMatrixDegree() {
        return matrixDegree;
    }

    public void setMatrixDegree(BigDecimal matrixDegree) {
        this.matrixDegree = matrixDegree;
    }

    public List<ObeRequirementDto> getObeRequirements() {
        return obeRequirements;
    }

    public void setObeRequirements(List<ObeRequirementDto> obeRequirements) {
        this.obeRequirements = obeRequirements;
    }

    public BigDecimal getCourseMatrixDegree() {
        return courseMatrixDegree;
    }

    public void setCourseMatrixDegree(BigDecimal courseMatrixDegree) {
        this.courseMatrixDegree = courseMatrixDegree;
    }
}
