package com.qmth.teachcloud.obe.been.result.report.word.custom;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qmth.teachcloud.obe.been.report.CourseTargetWebDto;
import com.qmth.teachcloud.obe.been.report.ReportCourseEvaluationResultDto;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description: 西安工业大学课程基本信息bean
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/10/11
 */
public class XagyuCourseBasicBean implements Serializable {

    @ApiModelProperty(name = "课程名称")
    String courseName;

    @ApiModelProperty(name = "课程编码")
    String courseCode;

    @ApiModelProperty(name = "学分")
    String credit;

    @ApiModelProperty(name = "学时")
    String period;

    @ApiModelProperty(name = "开课专业")
    String profession;

    @ApiModelProperty(value = "开课时间")
    private String openTime;

    @ApiModelProperty(value = "授课对象")
    private String teachingObject;

    @ApiModelProperty(value = "任课老师")
    private String teacher;

    @ApiModelProperty(value = "课程持续改进")
    private List<XagyuCourseSuggestDto> courseSuggestList;

//    @ApiModelProperty(name = "期末考试权重")
//    BigDecimal finalScoreWeight;

    public XagyuCourseBasicBean() {

    }

    public XagyuCourseBasicBean(TRBasicInfo trBasicInfo) {
        this.courseName = trBasicInfo.getCourseName();
        this.courseCode = trBasicInfo.getCourseCode();
        this.credit = trBasicInfo.getCredit();
        this.period = trBasicInfo.getPeriod();
        this.profession = trBasicInfo.getProfession();
        this.openTime = trBasicInfo.getOpenTime();
        this.teachingObject = trBasicInfo.getTeachingObject();
        this.teacher = trBasicInfo.getTeacher();
        if (Objects.nonNull(trBasicInfo.getCourseSuggest())) {
            List<XagyuCourseSuggestDto> courseSuggestDtoList = JSONArray.parseArray(trBasicInfo.getCourseSuggest(), XagyuCourseSuggestDto.class);
            this.courseSuggestList = courseSuggestDtoList;
        } else {
            ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class);
            List<CourseTargetWebDto> courseTargetWebDtoList = reportCourseEvaluationResultDto.getTargetList();
            List<XagyuCourseSuggestDto> courseSuggestDtoList = new ArrayList<>(courseTargetWebDtoList.size());
            courseTargetWebDtoList.stream().peek(s -> courseSuggestDtoList.add(new XagyuCourseSuggestDto(s))).collect(Collectors.toList());
            this.courseSuggestList = courseSuggestDtoList;
        }
    }

//    public BigDecimal getFinalScoreWeight() {
//        return finalScoreWeight;
//    }
//
//    public void setFinalScoreWeight(BigDecimal finalScoreWeight) {
//        this.finalScoreWeight = finalScoreWeight;
//    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCredit() {
        return credit;
    }

    public void setCredit(String credit) {
        this.credit = credit;
    }

    public String getPeriod() {
        return period;
    }

    public void setPeriod(String period) {
        this.period = period;
    }

    public String getProfession() {
        return profession;
    }

    public void setProfession(String profession) {
        this.profession = profession;
    }

    public String getOpenTime() {
        return openTime;
    }

    public void setOpenTime(String openTime) {
        this.openTime = openTime;
    }

    public String getTeachingObject() {
        return teachingObject;
    }

    public void setTeachingObject(String teachingObject) {
        this.teachingObject = teachingObject;
    }

    public String getTeacher() {
        return teacher;
    }

    public void setTeacher(String teacher) {
        this.teacher = teacher;
    }

    public List<XagyuCourseSuggestDto> getCourseSuggestList() {
        return courseSuggestList;
    }

    public void setCourseSuggestList(List<XagyuCourseSuggestDto> courseSuggestList) {
        this.courseSuggestList = courseSuggestList;
    }
}
