package com.qmth.teachcloud.obe.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.distributed.print.business.enums.SourceEnum;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.mark.bean.archivescore.MarkStudentScoreVo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Objects;

/**
 * <p>
 * 期末考试成绩表
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@ApiModel(value = "TCFinalScore对象", description = "期末考试成绩表")
public class TCFinalScore extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "培养方案id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long cultureProgramId;

    @ApiModelProperty(value = "课程id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "课程编码")
    private String courseCode;

    @ApiModelProperty(value = "课程名称")
    private String courseName;

    @ApiModelProperty(value = "试卷编码")
    private String paperNumber;

    @ApiModelProperty(value = "考生姓名")
    @NotNull(message = "考生姓名不能为空")
    private String name;

    @ApiModelProperty(value = "学号")
    private String studentCode;

    @ApiModelProperty(value = "分数")
    @NotNull(message = "分数不能为空")
    private Double score;

    @ApiModelProperty(value = "成绩明细")
    @NotNull(message = "成绩明细不能为空")
    private String scoreDetail;

    @ApiModelProperty(value = "是否启用，0：停用，1：启用")
    private Boolean enable;

    @ApiModelProperty(value = "来源，EXCEL_IMPORT：excel导入，SYNC：数据同步")
    private SourceEnum source;

    @ApiModelProperty(value = "教学班")
    private String teachClassName;

    public TCFinalScore() {

    }

    public TCFinalScore(Long cultureProgramId, Long courseId, String courseCode, String courseName, String paperNumber, SourceEnum source, Long userId, Long examId) {
        insertInfo(userId);
        this.cultureProgramId = cultureProgramId;
        this.courseId = courseId;
        this.courseCode = courseCode;
        this.courseName = courseName;
        this.paperNumber = paperNumber;
        this.source = source;
        this.enable = true;
        this.examId = examId;
    }

    public TCFinalScore(MarkStudentScoreVo markStudentScoreVo, String scoreDetail, SourceEnum source, Long userId, Long cultureProgramId, Long courseId) {
        insertInfo(Objects.nonNull(markStudentScoreVo.getTeacherId()) ? markStudentScoreVo.getTeacherId() : userId);
//        insertInfo(userId);
        this.examId = markStudentScoreVo.getExamId();
        this.courseCode = markStudentScoreVo.getCourseCode();
        this.courseName = markStudentScoreVo.getCourseName();
        this.paperNumber = markStudentScoreVo.getPaperNumber();
        this.name = markStudentScoreVo.getStudentName();
        this.studentCode = markStudentScoreVo.getStudentCode();
        this.score = markStudentScoreVo.getTotalScore();
        this.scoreDetail = scoreDetail;
        this.source = source;
        this.enable = true;
        this.cultureProgramId = cultureProgramId;
        this.courseId = courseId;
        this.teachClassName = markStudentScoreVo.getTeachClassName();
    }

    public TCFinalScore(BasicExamStudent basicExamStudent, BasicCourse basicCourse, SourceEnum source, Long userId, Long cultureProgramId, Long courseId) {
        insertInfo(Objects.nonNull(basicExamStudent.getTeacherId()) ? basicExamStudent.getTeacherId() : userId);
//        insertInfo(userId);
        this.examId = basicExamStudent.getExamId();
        this.courseCode = basicCourse.getCode();
        this.courseName = basicCourse.getName();
        this.paperNumber = basicExamStudent.getPaperNumber();
        this.name = basicExamStudent.getStudentName();
        this.studentCode = basicExamStudent.getStudentCode();
        this.source = source;
        this.enable = true;
        this.cultureProgramId = cultureProgramId;
        this.courseId = courseId;
        this.teachClassName = basicExamStudent.getTeachClassName();
    }

    public void updateInfo(TCFinalScore tcFinalScore, Long userId) {
        this.name = tcFinalScore.getName();
        this.score = tcFinalScore.getScore();
        this.scoreDetail = tcFinalScore.getScoreDetail();
        updateInfo(userId);
    }

    public void updateExamAndPaperNumberInfo(TCFinalScore tcFinalScore) {
        this.examId = tcFinalScore.getExamId();
        this.paperNumber = tcFinalScore.getPaperNumber();
        this.teachClassName = tcFinalScore.getTeachClassName();
    }

    public String getTeachClassName() {
        return teachClassName;
    }

    public void setTeachClassName(String teachClassName) {
        this.teachClassName = teachClassName;
    }

    public Long getCultureProgramId() {
        return cultureProgramId;
    }

    public void setCultureProgramId(Long cultureProgramId) {
        this.cultureProgramId = cultureProgramId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public SourceEnum getSource() {
        return source;
    }

    public void setSource(SourceEnum source) {
        this.source = source;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public Double getScore() {
        return score;
    }

    public void setScore(Double score) {
        this.score = score;
    }

    public String getScoreDetail() {
        return scoreDetail;
    }

    public void setScoreDetail(String scoreDetail) {
        this.scoreDetail = scoreDetail;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        TCFinalScore that = (TCFinalScore) o;
        return Objects.equals(source, that.source) && Objects.equals(teachClassName, that.teachClassName) && Objects.equals(examId, that.examId) && Objects.equals(getCreateId(), that.getCreateId()) && cultureProgramId.equals(that.cultureProgramId) && Objects.equals(courseCode, that.courseCode) && Objects.equals(courseName, that.courseName) && Objects.equals(paperNumber, that.paperNumber) && courseId.equals(that.courseId) && name.equals(that.name) && studentCode.equals(that.studentCode) && Objects.equals(score, that.score) && Objects.equals(scoreDetail, that.scoreDetail) && enable.equals(that.enable) && Objects.equals(getCreateId(), that.getCreateId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(source, teachClassName, examId, getCreateId(), cultureProgramId, courseCode, courseName, paperNumber, courseId, name, studentCode, score, scoreDetail, enable, getCreateId());
    }
}
