package com.qmth.teachcloud.obe.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.result.DictionaryResult;
import com.qmth.teachcloud.obe.been.result.ObeCourseBasicInfoResult;
import com.qmth.teachcloud.obe.been.result.ObeCourseOutlineResult;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 课程大纲表 Mapper 接口
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-06-05
 */
public interface ObeCourseOutlineMapper extends BaseMapper<ObeCourseOutline> {

    IPage<ObeCourseOutlineResult> findObeCourseOutlinePage(@Param("iPage") Page<ObeCourseOutlineResult> iPage, @Param("schoolId") Long schoolId,
                                                           @Param("professionalId") Long professionalId, @Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId,
                                                           @Param("courseOutlineName") String courseOutlineName, @Param("dpr") DataPermissionRule dpr);

    /**
     * 查询课程基本信息
     *
     * @param obeCourseOutlineId 课程大纲id
     * @return 课程基本信息
     */
    ObeCourseBasicInfoResult findCourseBasicInfo(@Param("obeCourseOutlineId") Long obeCourseOutlineId);

    /**
     * 查询可选培养方案
     *
     * @param schoolId  学校id
     * @param collegeId 学院id
     * @return 可选培养方案
     */
    List<DictionaryResult> findCanChooseCultureProgram(@Param("schoolId") Long schoolId, @Param("collegeId") Long collegeId);

    /**
     * 查询可选课程
     *
     * @param obeCultureProgramId 培养方案id
     * @return 可选课程
     */
    List<DictionaryResult> findCanChooseCourse(@Param("obeCultureProgramId") Long obeCultureProgramId);
}
