package com.qmth.teachcloud.obe.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.obe.been.dto.FinalScoreDto;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.entity.TCFinalScore;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 期末考试成绩表 Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TCFinalScoreMapper extends BaseMapper<TCFinalScore> {

    /**
     * 期末成绩列表（分页）
     *
     * @param iPage
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @param dpr
     * @return
     */
    public IPage<TCFinalScore> finalScoreList(IPage<Map> iPage, @Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId, @Param("examId") Long examId, @Param("paperNumber") String paperNumber, @Param("dpr") DataPermissionRule dpr);

    /**
     * 查找期末成绩分数图
     *
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @param userId
     * @return
     */
    public FinalScoreDto finalScoreScoreOverView(@Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId, @Param("examId") Long examId, @Param("paperNumber") String paperNumber, @Param("teachClassName") String teachClassName, @Param("userId") Long userId);

    /**
     * 查找期末成绩分数区间
     *
     * @param cultureProgramId
     * @param courseId
     * @param start
     * @param end
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @param userId
     * @return
     */
    public int getCountByScoreRange(@Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId, @Param("start") Double start, @Param("end") Double end, @Param("examId") Long examId, @Param("paperNumber") String paperNumber, @Param("teachClassName") String teachClassName, @Param("userId") Long userId);

    /**
     * 查找考生成绩
     *
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @param userId
     * @return
     */
    public List<FinalScoreResult> examStudentOverview(@Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId, @Param("examId") Long examId, @Param("paperNumber") String paperNumber, @Param("teachClassName") String teachClassName, @Param("userId") Long userId);

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param dpr
     * @return
     */
    public List<TCFinalScore> queryFinalScore(@Param("cultureProgramId") Long cultureProgramId, @Param("courseId") Long courseId, @Param("paperNumber") String paperNumber, @Param("examId") Long examId, @Param("dpr") DataPermissionRule dpr);
}
