package com.qmth.teachcloud.obe.service;

import com.qmth.teachcloud.obe.been.result.report.audit.AuditReport;
import com.qmth.teachcloud.obe.been.result.report.audit.CourseBasicInfo;
import com.qmth.teachcloud.obe.been.result.report.audit.CourseTargetInfo;
import com.qmth.teachcloud.obe.been.result.report.audit.RationalityEvaluation;
import com.qmth.teachcloud.obe.entity.ObeCourseOutlineAuditReport;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * <p>
 * 课程目标达成情况评价依据合理性审核表 服务类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-10-08
 */
public interface ObeCourseOutlineAuditReportService extends IService<ObeCourseOutlineAuditReport> {

    /**
     * 获取或生成审核报告
     *
     * @param courseOutlineId 课程大纲id
     * @param rationality     展示合理性评价部分
     * @return 审核报告
     */
    AuditReport getAuditReport(Long courseOutlineId, boolean rationality);

    /**
     * 获取课程基本信息
     *
     * @param courseOutlineId 课程大纲id
     * @return 课程基本信息
     */
    CourseBasicInfo getCourseBasicInfo(Long courseOutlineId);

    /**
     * 获取课程目标信息
     *
     * @param courseOutlineId 课程大纲id
     * @return 获取课程目标信息
     */
    CourseTargetInfo getCourseTargetInfo(Long courseOutlineId);

    /**
     * 获取课程目标合理性评价
     *
     * @param courseOutlineId 课程大纲id
     * @return 获取课程目标合理性评价
     */
    RationalityEvaluation getRationalityEvaluation(Long courseOutlineId);

    /**
     * 保存审核报告
     *
     * @param auditReport 审核报告
     * @return id
     */
    Long saveReport(AuditReport auditReport);
}
