package com.qmth.teachcloud.obe.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.obe.entity.TCPaperStruct;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Map;

/**
 * <p>
 * 试卷蓝图表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TCPaperStructService extends IService<TCPaperStruct> {

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 导入期末成绩-试卷结构excel
     *
     * @param file
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     * @throws IOException
     */
    public Map<String, String> paperStructExcelImport(MultipartFile file, Long cultureProgramId, Long courseId, String paperNumber, Long examId) throws IOException;

    /**
     * 同步试卷结构
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     * @throws IOException
     */
    public Map<String, String> paperStructSync(Long examId, Long courseId, String paperNumber, Long cultureProgramId) throws IOException;

    /**
     * 查询试卷数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public TCPaperStruct queryPaperStruct(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取试卷结构导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getPaperStructSyncLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取试卷结构导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getPaperStructImportLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);
}
