package com.qmth.teachcloud.obe.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.obe.entity.TCUsualScore;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平常作业成绩表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TCUsualScoreService extends IService<TCUsualScore> {

    /**
     * 平时成绩导入
     *
     * @param file
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @param usualScoreList
     * @return 结果
     */
    public Map<String, String> usualScoreExcelImport(MultipartFile file, Long cultureProgramId, Long courseId, Long examId, String paperNumber, String usualScoreList) throws IOException;

    /**
     * 平时成绩列表（分页）
     *
     * @param iPage
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public IPage<TCUsualScore> usualScoreList(IPage<Map> iPage, Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public List<TCUsualScore> queryUsualScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public List<TCUsualScore> queryUsualScoreAll(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取平时成绩导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getUsualScoreImportLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId);

    /**
     * 更新成绩数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public Boolean updateScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId);
}
