package com.qmth.teachcloud.obe.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.obe.been.dto.CourseWeightDto;
import com.qmth.teachcloud.obe.been.obe.CourseRequirementMatrixDto;
import com.qmth.teachcloud.obe.been.obe.MatrixRequirementDto;
import com.qmth.teachcloud.obe.been.obe.MatrixSubRequirementDto;
import com.qmth.teachcloud.obe.been.report.*;
import com.qmth.teachcloud.obe.been.result.ObeCourseRequirementListResult;
import com.qmth.teachcloud.obe.been.result.ObeCourseWeightResult;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import com.qmth.teachcloud.obe.entity.ObeCourseRequirementReport;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import com.qmth.teachcloud.obe.mapper.ObeCourseRequirementReportMapper;
import com.qmth.teachcloud.obe.service.ObeCourseOutlineService;
import com.qmth.teachcloud.obe.service.ObeCourseRequirementMatrixService;
import com.qmth.teachcloud.obe.service.ObeCourseRequirementReportService;
import com.qmth.teachcloud.obe.service.TRBasicInfoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程毕业要求报表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-06-13
 */
@Service
public class ObeCourseRequirementReportServiceImpl extends ServiceImpl<ObeCourseRequirementReportMapper, ObeCourseRequirementReport> implements ObeCourseRequirementReportService {
    private final static Logger log = LoggerFactory.getLogger(ObeCourseRequirementReportServiceImpl.class);

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    ObeCourseRequirementReportService obeCourseRequirementReportService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    ObeCourseOutlineService obeCourseOutlineService;

    @Resource
    ObeCourseRequirementMatrixService obeCourseRequirementMatrixService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    /**
     * 雷达报表计算方法
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public ObeCourseRequirementDto calculate(Long cultureProgramId) {
        ObeCourseRequirementDto obeCourseRequirementResultDto = null;
        List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList = obeCourseRequirementMatrixService.getMatrix(cultureProgramId);
        if (!CollectionUtils.isEmpty(courseRequirementMatrixDtoList)) {
            log.info("courseRequirementMatrixDto:{}", JacksonUtil.parseJson(courseRequirementMatrixDtoList));
            List<ObeCourseOutline> obeCourseOutlineList = obeCourseOutlineService.findByCultureProgramId(cultureProgramId);
            if (!CollectionUtils.isEmpty(obeCourseOutlineList)) {
                List<ObeCourseRequirementDto> obeCourseRequirementDtoList = new ArrayList<>();
                Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap = new LinkedHashMap<>();
                for (ObeCourseOutline obeCourseOutline : obeCourseOutlineList) {
                    obeCourseRequirementReportService.matrixCourseReportInfo(courseRequirementMatrixDtoList, obeCourseOutline.getId(),
                            null, obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
                }
                obeCourseRequirementResultDto = obeCourseRequirementReportService.matrixCourseReport(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);

                ObeCourseRequirementDto finalObeCourseRequirementResultDto = obeCourseRequirementResultDto;
                obeSubRequirementsDtoMap.forEach((k, v) -> {
                    List<ObeRequirementDto> obeRequirementDtoList = finalObeCourseRequirementResultDto.getObeRequirements();
                    for (ObeRequirementDto obeRequirementDto : obeRequirementDtoList) {
                        List<ObeSubRequirementsDto> obeSubRequirementsDtoList = obeRequirementDto.getObeSubRequirements();
                        for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                            if (obeSubRequirementsDto.getSubRequirementId().longValue() == k.longValue()) {
                                List<Double> obeMatrixDegreeList = v.stream().filter(x -> Objects.nonNull(x.getMatrixDegree())).map(x -> x.getMatrixDegree().doubleValue()).collect(Collectors.toList());
                                Optional<Double> degree = obeMatrixDegreeList.stream().reduce(Double::sum);
                                if (degree.isPresent()) {
                                    obeSubRequirementsDto.setMatrixDegree(new BigDecimal(degree.get()).setScale(2, BigDecimal.ROUND_HALF_UP));
                                }
                            }
                        }
                    }
                });
                ObeCourseRequirementReport obeCourseRequirementReport = new ObeCourseRequirementReport(
                        cultureProgramId, Objects.nonNull(obeCourseRequirementResultDto) ? JacksonUtil.parseJson(obeCourseRequirementResultDto) : null,
                        !CollectionUtils.isEmpty(obeCourseRequirementDtoList) ? JacksonUtil.parseJson(obeCourseRequirementDtoList) : null, Objects.nonNull(obeCourseRequirementResultDto) && Objects.nonNull(obeCourseRequirementResultDto.getMatrixDegree()) ? obeCourseRequirementResultDto.getMatrixDegree().doubleValue() : null);
                obeCourseRequirementReportService.removeReport(cultureProgramId);
                if (Objects.nonNull(obeCourseRequirementReport.getRequirementRadar()) &&
                        Objects.nonNull(obeCourseRequirementReport.getRequirementDetail())
                        && Objects.nonNull(obeCourseRequirementReport.getRequirementDegree())) {
                    obeCourseRequirementReportService.save(obeCourseRequirementReport);
                }
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("报告生成失败，请先完善培养方案");
        }
        return obeCourseRequirementResultDto;
    }

    /**
     * 雷达详情报表计算方法
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    public List<ObeCourseRequirementDto> calculateDetail(Long cultureProgramId) {
        List<ObeCourseRequirementDto> obeCourseRequirementDtoList = new ArrayList<>();
        List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList = obeCourseRequirementMatrixService.getMatrix(cultureProgramId);
        if (!CollectionUtils.isEmpty(courseRequirementMatrixDtoList)) {
            log.info("courseRequirementMatrixDto:{}", JacksonUtil.parseJson(courseRequirementMatrixDtoList));
            List<ObeCourseOutline> obeCourseOutlineList = obeCourseOutlineService.findByCultureProgramId(cultureProgramId);
            if (!CollectionUtils.isEmpty(obeCourseOutlineList)) {
                Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap = new LinkedHashMap<>();
                for (ObeCourseOutline obeCourseOutline : obeCourseOutlineList) {
                    obeCourseRequirementReportService.matrixCourseReportInfo(courseRequirementMatrixDtoList, obeCourseOutline.getId(),
                            null, obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
                }
            }
        }
        return obeCourseRequirementDtoList;
    }

    /**
     * 换算课程毕业达成度报表详情
     *
     * @param courseRequirementMatrixDtoList
     * @param obeCourseOutlineId
     * @param examId
     * @return
     */
    @Override
    public Map<String, Object> matrixCourseReportInfo(List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList,
                                                      Long obeCourseOutlineId, Long examId,
                                                      List<ObeCourseRequirementDto> obeCourseRequirementDtoList,
                                                      Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        Map<String, Object> resultMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(courseRequirementMatrixDtoList)) {
            ObeCourseWeightResult obeCourseWeightResult = trBasicInfoService.findCourseWeightResultRmi(obeCourseOutlineId, false);
            List<CourseWeightDto> courseWeightDtoList = obeCourseWeightResult.getSubmitForm();
            //课程目标指标点集合map
            Map<Long, CourseWeightDto> cultureProgramRequirementIdMap = courseWeightDtoList.stream().collect(Collectors.toMap(CourseWeightDto::getObeCultureProgramRequirementId, Function.identity(), (dto1, dto2) -> dto1));
            Map<Long, List<ObeSubRequirementsDto>> matrixSubRequirementIdMap = new HashMap<>();//课程目标指标点集合-多个目标共用一个指标点
            Map<Long, CourseTargetWebDto> courseTargetWebDtoMap = new HashMap<>();//课程目标集合

            ObeCourseOutline obeCourseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
            Objects.requireNonNull(obeCourseOutline, "未找到课程大纲信息");
            BasicCourse basicCourse = basicCourseService.getById(obeCourseOutline.getCourseId());
            Objects.requireNonNull(basicCourse, "未找到课程信息");
            for (CourseRequirementMatrixDto c : courseRequirementMatrixDtoList) {
                if (c.getCourseId().longValue() == basicCourse.getId().longValue()) {
                    List<TRBasicInfo> trBasicInfoList = trBasicInfoService.queryBasicInfoAll(obeCourseOutline.getCultureProgramId(), obeCourseOutline.getCourseId(), null, examId);
                    TRBasicInfo trBasicInfo = null;
                    if (!CollectionUtils.isEmpty(trBasicInfoList)) {
                        List<TRBasicInfo> trBasicTempInfoList = !CollectionUtils.isEmpty(trBasicInfoList) ? trBasicInfoList.stream().filter(s -> Objects.nonNull(s.getTeachClassName()) && Objects.equals(s.getTeachClassName(), SystemConstant.ALL)).collect(Collectors.toList()) : null;
                        trBasicInfo = !CollectionUtils.isEmpty(trBasicTempInfoList) ? trBasicTempInfoList.get(0) : trBasicInfo;
                    }
                    if (Objects.nonNull(trBasicInfo)) {
                        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = Objects.nonNull(trBasicInfo.getCourseEvaluationResult()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class) : null;
                        BigDecimal courseMatrixDegree = new BigDecimal(0);
                        if (Objects.nonNull(reportCourseEvaluationResultDto)) {
                            List<CourseTargetWebDto> courseTargetWebDtoList = reportCourseEvaluationResultDto.getTargetList();
                            OptionalDouble courseDegree = courseTargetWebDtoList.stream().filter(x -> Objects.nonNull(x.getEvaluationValue()) && x.getEvaluationValue().doubleValue() > 0).mapToDouble(x -> x.getEvaluationValue().doubleValue()).min();
                            if (courseDegree.isPresent()) {
                                courseMatrixDegree = new BigDecimal(courseDegree.getAsDouble()).setScale(2, BigDecimal.ROUND_HALF_UP);
                            }
                            courseTargetWebDtoMap = courseTargetWebDtoList.stream().collect(Collectors.toMap(CourseTargetWebDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                        }
                        ObeCourseRequirementDto obeCourseRequirementDto = new ObeCourseRequirementDto(c, courseMatrixDegree);
                        obeCourseRequirementDtoList.add(obeCourseRequirementDto);
                        List<MatrixRequirementDto> matrixRequirementDtoList = c.getRequirements();
                        if (!CollectionUtils.isEmpty(matrixRequirementDtoList)) {
                            List<ObeRequirementDto> obeRequirementDtoList = new ArrayList<>();
                            obeCourseRequirementDto.setObeRequirements(obeRequirementDtoList);
                            for (MatrixRequirementDto matrixRequirementDto : matrixRequirementDtoList) {
                                ObeRequirementDto obeRequirementDto = new ObeRequirementDto(matrixRequirementDto);
                                obeRequirementDtoList.add(obeRequirementDto);
                                List<MatrixSubRequirementDto> matrixSubRequirementDtoList = matrixRequirementDto.getSubRequirements();
                                if (!CollectionUtils.isEmpty(matrixSubRequirementDtoList)) {
                                    List<ObeSubRequirementsDto> obeSubRequirementsDtoList = new ArrayList<>();
                                    obeRequirementDto.setObeSubRequirements(obeSubRequirementsDtoList);
                                    for (MatrixSubRequirementDto matrixSubRequirementDto : matrixSubRequirementDtoList) {
                                        ObeSubRequirementsDto obeSubRequirementsDto = new ObeSubRequirementsDto(matrixSubRequirementDto);
                                        obeSubRequirementsDtoList.add(obeSubRequirementsDto);
                                        if (Objects.nonNull(matrixSubRequirementDto.getScale())) {
                                            //多个课程目标关联同一个指标点
                                            if (matrixSubRequirementIdMap.containsKey(matrixSubRequirementDto.getSubRequirementId())) {
                                                List<ObeSubRequirementsDto> obeRequirementsDtoList = matrixSubRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                Map<Long, CourseTargetWebDto> map = obeRequirementsDtoList.stream().collect(Collectors.toMap(ObeSubRequirementsDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                                                CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                if (Objects.nonNull(courseWeightDto)) {
                                                    CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                                    if (!map.containsKey(courseTargetWebDto.getTargetId())) {
                                                        obeRequirementsDtoList.add(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto));
                                                    }
                                                }
                                                matrixSubRequirementIdMap.put(matrixSubRequirementDto.getSubRequirementId(), obeRequirementsDtoList);
                                            } else {
                                                CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                if (Objects.nonNull(courseWeightDto)) {
                                                    CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                                    matrixSubRequirementIdMap.put(matrixSubRequirementDto.getSubRequirementId(), new ArrayList<>(Arrays.asList(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto))));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        //计算各个课程的指标点达成值
                        if (!CollectionUtils.isEmpty(matrixSubRequirementIdMap)) {
                            for (Map.Entry<Long, List<ObeSubRequirementsDto>> entry : matrixSubRequirementIdMap.entrySet()) {
                                OptionalDouble evaluationValue = entry.getValue().stream().mapToDouble(x -> Objects.nonNull(x.getEvaluationValue()) ? x.getEvaluationValue().doubleValue() : 0.0d).average();
                                if (evaluationValue.isPresent()) {
                                    ObeSubRequirementsDto obeSubRequirementsDto = entry.getValue().get(0);
                                    obeSubRequirementsDto.setCourseMatrixDegree(new BigDecimal(evaluationValue.getAsDouble()).setScale(2, BigDecimal.ROUND_HALF_UP));
                                    BigDecimal bigDecimal = new BigDecimal(evaluationValue.getAsDouble()).multiply(obeSubRequirementsDto.getScale()).setScale(2, BigDecimal.ROUND_HALF_UP);
                                    entry.getValue().stream().peek(x -> x.setMatrixDegree(bigDecimal)).collect(Collectors.toList());
                                }
                            }
                        }
                        //计算各个课程的指标点达成值
                        List<ObeRequirementDto> obeRequirementDtoList = obeCourseRequirementDto.getObeRequirements();
                        if (!CollectionUtils.isEmpty(obeRequirementDtoList)) {
                            for (ObeRequirementDto o : obeRequirementDtoList) {
                                List<ObeSubRequirementsDto> obeSubRequirementsDtoList = o.getObeSubRequirements();
                                for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                                    if (matrixSubRequirementIdMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                                        obeSubRequirementsDto.setMatrixDegree(matrixSubRequirementIdMap.get(obeSubRequirementsDto.getSubRequirementId()).get(0).getMatrixDegree());
                                        obeSubRequirementsDto.setCourseMatrixDegree(matrixSubRequirementIdMap.get(obeSubRequirementsDto.getSubRequirementId()).get(0).getCourseMatrixDegree());
                                    }
                                    if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                                        List<ObeSubRequirementsDto> obeSubRequirementsDtoListTemp = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getSubRequirementId());
                                        obeSubRequirementsDtoListTemp.add(obeSubRequirementsDto);
                                        obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getSubRequirementId(), obeSubRequirementsDtoListTemp);
                                    } else {
                                        obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getSubRequirementId(), new ArrayList<>(Arrays.asList(obeSubRequirementsDto)));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        if (CollectionUtils.isEmpty(obeCourseRequirementDtoList)) {
            throw ExceptionResultEnum.ERROR.exception("报告生成失败，请先完成当前方案下课程目标达成度计算");
        }
        resultMap.put(SystemConstant.OBE_COURSE_REQUIREMENT_DTO_LIST, obeCourseRequirementDtoList);
        resultMap.put(SystemConstant.OBE_SUB_REQUIREMENT_DTO_MAP, obeSubRequirementsDtoMap);
        return resultMap;
    }

    /**
     * 换算课程毕业达成度报表
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    @Override
    public ObeCourseRequirementDto matrixCourseReport(List<ObeCourseRequirementDto> obeCourseRequirementDtoList, Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        //计算指标和毕业达成度
        ObeCourseRequirementDto obeCourseRequirementResultDto = new ObeCourseRequirementDto();
        if (!CollectionUtils.isEmpty(obeCourseRequirementDtoList)) {
            obeCourseRequirementDtoList = this.matrixCourseReportCommon(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
            obeCourseRequirementDtoList = obeCourseRequirementDtoList.stream().map(ObeCourseRequirementDto::new).collect(Collectors.toList());
            obeCourseRequirementResultDto = obeCourseRequirementDtoList.get(obeCourseRequirementDtoList.size() - 1);
            obeCourseRequirementResultDto.updateInfo();
        }
        return obeCourseRequirementResultDto;
    }

    /**
     * 换算课程毕业达成度报表共用
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    @Override
    public List<ObeCourseRequirementDto> matrixCourseReportCommon(List<ObeCourseRequirementDto> obeCourseRequirementDtoList, Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        for (ObeCourseRequirementDto o : obeCourseRequirementDtoList) {
            List<ObeRequirementDto> obeRequirementDtoList = o.getObeRequirements();
            for (ObeRequirementDto obeRequirementDto : obeRequirementDtoList) {
                List<ObeSubRequirementsDto> obeSubRequirementsDtoList = obeRequirementDto.getObeSubRequirements();
                for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                    if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                        List<Double> obeMatrixDegreeList = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getSubRequirementId()).stream().filter(x -> Objects.nonNull(x.getMatrixDegree())).map(x -> x.getMatrixDegree().doubleValue()).collect(Collectors.toList());
                        Optional<Double> degree = obeMatrixDegreeList.stream().reduce(Double::sum);
                        if (degree.isPresent() && (Objects.isNull(obeRequirementDto.getMatrixDegree()) || obeRequirementDto.getMatrixDegree().doubleValue() == 0)) {
                            obeRequirementDto.setMatrixDegree(new BigDecimal(degree.get()).setScale(2, BigDecimal.ROUND_HALF_UP));
                        } else if (degree.isPresent() && degree.get().doubleValue() > 0 && Objects.nonNull(obeRequirementDto.getMatrixDegree()) && degree.get().doubleValue() < obeRequirementDto.getMatrixDegree().doubleValue()) {
                            obeRequirementDto.setMatrixDegree(new BigDecimal(degree.get()).setScale(2, BigDecimal.ROUND_HALF_UP));
                        }
                    }
                }
                OptionalDouble degree = obeRequirementDtoList.stream().filter(x -> Objects.nonNull(x.getMatrixDegree()) && x.getMatrixDegree().doubleValue() > 0).mapToDouble(x -> x.getMatrixDegree().doubleValue()).min();
                if (degree.isPresent()) {
                    o.setMatrixDegree(new BigDecimal(degree.getAsDouble()).setScale(2, BigDecimal.ROUND_HALF_UP));
                }
            }
        }
        return obeCourseRequirementDtoList;
    }

    /**
     * 查找课程毕业达成度报表列表
     *
     * @param iPage
     * @param professionalId
     * @param cultureProgramName
     * @return
     */
    @Override
    public IPage<ObeCourseRequirementListResult> obeCourseRequirementReportList(IPage<Map> iPage, Long professionalId, String cultureProgramName) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.obeCourseRequirementReportList(iPage, sysUser.getSchoolId(), professionalId, cultureProgramName, dpr);
    }

    /**
     * 根据培养方案删除报表
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public boolean removeReport(Long cultureProgramId) {
        ObeCourseRequirementReport obeCourseRequirementReport = obeCourseRequirementReportService.findByCultureProgramId(cultureProgramId);
        return Objects.nonNull(obeCourseRequirementReport) ? obeCourseRequirementReportService.removeById(obeCourseRequirementReport.getId()) : false;
    }

    /**
     * 根据培养方案查询报表
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    public ObeCourseRequirementReport findByCultureProgramId(Long cultureProgramId) {
        return obeCourseRequirementReportService.getOne(new QueryWrapper<ObeCourseRequirementReport>().lambda().eq(ObeCourseRequirementReport::getCultureProgramId, cultureProgramId));
    }
}
