package com.qmth.teachcloud.obe.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.enums.SourceEnum;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.mark.bean.archivescore.MarkStudentScoreVo;
import com.qmth.teachcloud.mark.dto.mark.ScoreItem;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.obe.been.dto.FinalScoreDto;
import com.qmth.teachcloud.obe.been.dto.TCFinalScoreDto;
import com.qmth.teachcloud.obe.been.excel.PaperStructDto;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.been.result.ObeScoreResult;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.entity.TCFinalScore;
import com.qmth.teachcloud.obe.entity.TCPaperStruct;
import com.qmth.teachcloud.obe.entity.TCUsualScore;
import com.qmth.teachcloud.obe.mapper.TCFinalScoreMapper;
import com.qmth.teachcloud.obe.mapper.TCUsualScoreMapper;
import com.qmth.teachcloud.obe.service.TCFinalScoreService;
import com.qmth.teachcloud.obe.service.TCPaperStructService;
import com.qmth.teachcloud.obe.service.TCUsualScoreService;
import com.qmth.teachcloud.obe.service.TRBasicInfoService;
import io.github.burukeyou.dataframe.iframe.SDFrame;
import io.github.burukeyou.dataframe.iframe.window.Sorter;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 期末考试成绩表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCFinalScoreServiceImpl extends ServiceImpl<TCFinalScoreMapper, TCFinalScore> implements TCFinalScoreService {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    MarkStudentService markStudentService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    TCUsualScoreService tcUsualScoreService;

    @Resource
    private TCUsualScoreMapper tcUsualScoreMapper;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    BasicExamStudentService basicExamStudentService;

    /**
     * 导入期末成绩excel
     *
     * @param file
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> finalScoreExcelImport(MultipartFile file, Long examId, Long courseId, String paperNumber, Long cultureProgramId) {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            printCommonService.scoreImportExcelValid(file, examId, paperNumber, courseId);
            Double totalScore = 100d;

            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<String[]> arrayList = excelReader.getDataArrayList();

            if (!CollectionUtils.isEmpty(arrayList)) {
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                List<TCFinalScore> tcFinalScoreList = new ArrayList<>(arrayList.size());
                String[] columnNames = excelReader.getColumnNames();
                for (int i = 0; i < columnNames.length; i++) {
                    if (i == 0 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "学号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                    } else if (i == 1 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "姓名")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                    } else if (i == 2 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "成绩")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第三行为成绩");
                    }
                }

                BasicCourse basicCourse = basicCourseService.getById(courseId);
                if (Objects.isNull(basicCourse)) {
                    throw ExceptionResultEnum.ERROR.exception("课程不存在");
                }
                TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber, examId);
                Objects.requireNonNull(tcPaperStruct, "未找到试卷结构");

                List<PaperStructDimensionResult> paperStructDimensionResultList = null;
                if (Objects.nonNull(tcPaperStruct.getPaperStructDimension())) {
                    paperStructDimensionResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStructDimension(), new TypeToken<List<PaperStructDimensionResult>>() {
                    }.getType());
                } else {
                    List<PaperStructDto> paperStructDtoList = GsonUtil.fromJson(tcPaperStruct.getPaperStruct(), new TypeToken<List<PaperStructDto>>() {
                    }.getType());
                    paperStructDimensionResultList = new ArrayList<>(paperStructDtoList.size());
                    for (PaperStructDto paperStructDto : paperStructDtoList) {
                        paperStructDimensionResultList.add(new PaperStructDimensionResult(paperStructDto.getMainNumber(), paperStructDto.getSubNumber(), tcPaperStruct.getExamId(), tcPaperStruct.getPaperNumber(), new BigDecimal(paperStructDto.getScore())));
                    }
                }
                Map<String, PaperStructDimensionResult> paperStructResultMap = paperStructDimensionResultList.stream().collect(Collectors.toMap(k -> k.getMainNumber() + "-" + k.getSubNumber(), Function.identity(), (dto1, dto2) -> dto1));

                for (int i = 0; i < arrayList.size(); i++) {
                    String[] strs = arrayList.get(i);
                    if (Objects.nonNull(strs) && strs.length > 0) {
                        if (strs.length != columnNames.length) {
                            throw ExceptionResultEnum.ERROR.exception("excel列和数据不匹配");
                        }
                        boolean error = false;
                        List<TCFinalScoreDto> tcFinalScoreDtoList = new ArrayList<>();
//                        TCFinalScore tcFinalScore = new TCFinalScore(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, SourceEnum.EXCEL_IMPORT, sysUser.getId());
                        TCFinalScore tcFinalScore = new TCFinalScore(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, SourceEnum.SYNC, sysUser.getId(), examId);
                        BigDecimal score = new BigDecimal(0);
                        for (int j = 0; j < strs.length; j++) {
                            if (Objects.isNull(strs[j]) || Objects.equals(strs[j].trim(), "")) {
                                errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j] + "]为空;").add("\r\n");
                                error = true;
                            } else {
                                if (Objects.equals(columnNames[j].trim(), "学号")) {
                                    tcFinalScore.setStudentCode(strs[j]);
                                } else if (Objects.equals(columnNames[j].trim(), "姓名")) {
                                    tcFinalScore.setName(strs[j]);
                                } else if (Objects.equals(columnNames[j].trim(), "成绩")) {
                                    Double scoreTemp = SystemConstant.isDouble(strs[j]);
                                    if (Objects.nonNull(scoreTemp)) {
                                        tcFinalScore.setScore(scoreTemp);
                                    } else {
                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]请输入数字").add("\r\n");
                                        error = true;
                                    }
                                } else {
                                    if (Objects.nonNull(SystemConstant.isDouble(strs[j]))) {
                                        if (!paperStructResultMap.containsKey(columnNames[j].trim())) {
                                            errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]题号不正确").add("\r\n");
                                            error = true;
                                        } else {
                                            PaperStructDimensionResult paperStructDimensionResult = paperStructResultMap.get(columnNames[j].trim());
                                            BigDecimal paperScore = paperStructDimensionResult.getScore();
                                            BigDecimal importScore = new BigDecimal(strs[j]);
                                            if (importScore.compareTo(paperScore) == 1) {
                                                errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]导入分数大于题号" + columnNames[j] + "的总分").add("\r\n");
                                                error = true;
                                            } else {
                                                score = score.add(importScore);
                                                tcFinalScoreDtoList.add(new TCFinalScoreDto(strs[j], columnNames[j]));
                                            }
                                        }
                                    } else {
                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]请输入数字").add("\r\n");
                                        error = true;
                                    }
                                }
                            }
                        }
                        if (CollectionUtils.isNotEmpty(tcFinalScoreDtoList)) {
                            if (score.doubleValue() > totalScore.doubleValue()) {
                                errorData.add("excel第").add((i + 1) + "").add("行[总分大于试卷总分]").add("\r\n");
                                error = true;
                            } else {
                                score = score.setScale(2, BigDecimal.ROUND_HALF_UP);
                                tcFinalScore.setScore(Double.parseDouble(SystemConstant.df.format(score)));
                                tcFinalScore.setScoreDetail(JacksonUtil.parseJson(tcFinalScoreDtoList));
                            }
                        }
                        if (!error) {
                            tcFinalScoreList.add(tcFinalScore);
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(tcFinalScoreList)) {
                    successData.add("共导入").add(tcFinalScoreList.size() + "").add("条数据");
                    List<TCFinalScore> tcFinalScoreDbSourceList = tcFinalScoreService.queryFinalScore(cultureProgramId, courseId, paperNumber, examId);
                    if (!CollectionUtils.isEmpty(tcFinalScoreDbSourceList)) {
                        tcFinalScoreList.stream().peek(s -> s.updateExamAndPaperNumberInfo(tcFinalScoreDbSourceList.get(0))).collect(Collectors.toList());
                    }
                    if (CollectionUtils.isEmpty(tcFinalScoreDbSourceList) || !CollectionUtils.isEqualCollection(tcFinalScoreDbSourceList, tcFinalScoreList)) {
                        Map<String, TCFinalScore> tcUsualScoreDbMap = tcFinalScoreDbSourceList.stream().collect(Collectors.toMap(TCFinalScore::getStudentCode, Function.identity(), (dto1, dto2) -> dto1));
                        AtomicInteger size = new AtomicInteger();
                        tcFinalScoreList.stream().peek(s -> {
                            if (Objects.nonNull(s.getStudentCode()) && tcUsualScoreDbMap.containsKey(s.getStudentCode().trim())) {
                                size.incrementAndGet();
                            }
                        }).collect(Collectors.toList());
                        if (size.get() != tcFinalScoreList.size()) {
                            throw ExceptionResultEnum.ERROR.exception("请勿导入其他班级数据");
                        }
                        tcFinalScoreService.removeByIds(tcFinalScoreDbSourceList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                        tcFinalScoreService.saveBatch(tcFinalScoreList);
                        trBasicInfoService.clearReportData(cultureProgramId, courseId, paperNumber, examId, true);
                    }
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 期末成绩列表（分页）
     *
     * @param iPage
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    @Override
    public IPage<TCFinalScore> finalScoreList(IPage<Map> iPage, Long examId, String paperNumber, Long cultureProgramId, Long courseId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.finalScoreList(iPage, cultureProgramId, courseId, examId, paperNumber, dpr);
    }

    /**
     * 查找期末成绩分数图
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param teachClassName
     * @param userId
     * @return
     */
    @Override
    public FinalScoreDto finalScoreScoreOverView(Long examId, String paperNumber, Long cultureProgramId, Long courseId, String teachClassName, Long userId) {
        return this.baseMapper.finalScoreScoreOverView(cultureProgramId, courseId, examId, paperNumber, teachClassName, userId);
    }

    /**
     * 查找期末成绩分数区间
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param start
     * @param end
     * @param teachClassName
     * @param userId
     * @return
     */
    @Override
    public int getCountByScoreRange(Long examId, String paperNumber, Long cultureProgramId, Long courseId, Double start, Double end, String teachClassName, Long userId) {
        return this.baseMapper.getCountByScoreRange(cultureProgramId, courseId, start, end, examId, paperNumber, teachClassName, userId);
    }

    /**
     * 查找考生成绩
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param teachClassName
     * @param userId
     * @return
     */
    @Override
    public List<FinalScoreResult> examStudentOverview(Long examId, String paperNumber, Long cultureProgramId, Long courseId, String teachClassName, Long userId) {
        return this.baseMapper.examStudentOverview(cultureProgramId, courseId, examId, paperNumber, teachClassName, userId);
    }

    /**
     * 同步期末成绩
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> finalScoreSync(Long examId, Long courseId, String paperNumber, Long cultureProgramId) {
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            BasicCourse basicCourse = basicCourseService.getById(courseId);
            Objects.requireNonNull(basicCourse, "未找到基础课程信息");
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");
            List<MarkStudentScoreVo> markStudentScoreVoList = markStudentService.listMarkStudentScoreList(examId, paperNumber);
            //不为空则从markPaper里同步基础数据，否则从basicExamStudent同步
            List<TCFinalScore> tcFinalScoreList = null;
            List<TCUsualScore> tcUsualScoreList = null;
            List<PaperStructDimensionResult> paperStructDimensionResultList = null;
            if (!CollectionUtils.isEmpty(markStudentScoreVoList)) {
                tcFinalScoreList = new ArrayList<>(markStudentScoreVoList.size());
                tcUsualScoreList = new ArrayList<>(markStudentScoreVoList.size());
                paperStructDimensionResultList = new ArrayList<>(markStudentScoreVoList.size());
                for (int i = 0; i < markStudentScoreVoList.size(); i++) {
                    MarkStudentScoreVo markStudentScoreVo = markStudentScoreVoList.get(i);
                    if (!CollectionUtils.isEmpty(markStudentScoreVo.getTotalScoreList())) {
                        List<TCFinalScoreDto> tcFinalScoreDtoList = new ArrayList<>();
                        for (ScoreItem scoreItem : markStudentScoreVo.getTotalScoreList()) {
                            String name = scoreItem.getMainNumber() + "-" + scoreItem.getSubNumber();
                            tcFinalScoreDtoList.add(new TCFinalScoreDto(scoreItem.getScore().toString(), name));
                            if (i == 0) {
                                paperStructDimensionResultList.add(
                                        new PaperStructDimensionResult(scoreItem.getMainNumber(), scoreItem.getSubNumber(), examId, paperNumber,
                                                new BigDecimal(scoreItem.getTotalScore())));
                            }
                        }
                        tcFinalScoreList.add(
                                new TCFinalScore(markStudentScoreVo, JacksonUtil.parseJson(tcFinalScoreDtoList),
                                        SourceEnum.SYNC, sysUser.getId(), cultureProgramId, courseId));
                        tcUsualScoreList.add(
                                new TCUsualScore(markStudentScoreVo,
                                        SourceEnum.SYNC, sysUser.getId(), cultureProgramId, courseId));
                    }
                }
                if (!CollectionUtils.isEmpty(paperStructDimensionResultList)) {
                    TCPaperStruct tcPaperStructDb = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber, examId);
                    if (Objects.nonNull(tcPaperStructDb)) {
                        tcPaperStructService.removeById(tcPaperStructDb.getId());
                    }
                    paperStructDimensionResultList = SDFrame.read(paperStructDimensionResultList).sortDesc(Sorter.sortDescBy(PaperStructDimensionResult::getMainNumber).sortDesc(PaperStructDimensionResult::getMainNumber)).toLists();
                    TCPaperStruct tcPaperStruct = new TCPaperStruct(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, JacksonUtil.parseJson(paperStructDimensionResultList),
                            sysUser.getId(), examId, Objects.nonNull(tcPaperStructDb)
                            && Objects.nonNull(cultureProgramId) && Objects.nonNull(tcPaperStructDb.getCultureProgramId()) && cultureProgramId.longValue() == tcPaperStructDb.getCultureProgramId().longValue()
                            && Objects.nonNull(courseId) && Objects.nonNull(tcPaperStructDb.getCourseId()) && courseId.longValue() == tcPaperStructDb.getCourseId().longValue()
                            && Objects.nonNull(paperNumber) && Objects.nonNull(tcPaperStructDb.getPaperNumber()) && Objects.equals(paperNumber, tcPaperStructDb.getPaperNumber())
                            && Objects.nonNull(examId) && Objects.nonNull(tcPaperStructDb.getExamId()) && examId.longValue() == tcPaperStructDb.getExamId().longValue() ? tcPaperStructDb.getPaperStructDimension() : null);
                    tcPaperStructService.saveOrUpdate(tcPaperStruct);
                }
            } else {
                List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda().eq(BasicExamStudent::getExamId, examId).eq(BasicExamStudent::getCourseId, courseId).eq(BasicExamStudent::getPaperNumber, paperNumber));
                if (!CollectionUtils.isEmpty(basicExamStudentList)) {
                    tcFinalScoreList = new ArrayList<>(basicExamStudentList.size());
                    tcUsualScoreList = new ArrayList<>(basicExamStudentList.size());
                    for (BasicExamStudent basicExamStudent : basicExamStudentList) {
                        tcFinalScoreList.add(
                                new TCFinalScore(basicExamStudent, basicCourse,
                                        SourceEnum.SYNC, sysUser.getId(), cultureProgramId, courseId));
                        tcUsualScoreList.add(
                                new TCUsualScore(basicExamStudent, basicCourse,
                                        SourceEnum.SYNC, sysUser.getId(), cultureProgramId, courseId));
                    }
                }
            }
            if (!CollectionUtils.isEmpty(tcFinalScoreList) && !CollectionUtils.isEmpty(tcUsualScoreList)) {
                List<TCFinalScore> tcFinalScoreNotEqPaperNumberList = tcFinalScoreService.queryFinalScoreNotEqPaperNumber(cultureProgramId, courseId, paperNumber, examId, null);
                List<TCFinalScore> tcFinalScoreNotEqExamAndPaperNumberList = tcFinalScoreService.queryFinalScoreNotEqExamIdAndPaperNumber(cultureProgramId, courseId, paperNumber, examId, null);
                if (!CollectionUtils.isEmpty(tcFinalScoreNotEqPaperNumberList)) {
                    for (TCFinalScore t : tcFinalScoreNotEqPaperNumberList) {
                        tcUsualScoreService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        tcFinalScoreService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        tcPaperStructService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        trBasicInfoService.clearReportData(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId(), true);
                    }
                }
                if (!CollectionUtils.isEmpty(tcFinalScoreNotEqExamAndPaperNumberList)) {
                    for (TCFinalScore t : tcFinalScoreNotEqExamAndPaperNumberList) {
                        tcUsualScoreService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        tcFinalScoreService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        tcPaperStructService.remove(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId());
                        trBasicInfoService.clearReportData(t.getCultureProgramId(), t.getCourseId(), t.getPaperNumber(), t.getExamId(), true);
                    }
                }
                successData.add("共同步").add(tcFinalScoreList.size() + "").add("条数据");
                List<TCFinalScore> tcFinalScoreDbSourceList = tcFinalScoreService.queryFinalScoreAll(cultureProgramId,
                        courseId, paperNumber, examId);
                boolean clearReport = false;
                if (CollectionUtils.isEmpty(tcFinalScoreDbSourceList) || !CollectionUtils.isEqualCollection(
                        tcFinalScoreDbSourceList, tcFinalScoreList)) {
                    tcFinalScoreService.remove(cultureProgramId, courseId, paperNumber, examId);
                    tcFinalScoreService.saveBatch(tcFinalScoreList);
                    tcUsualScoreService.remove(cultureProgramId, courseId, paperNumber, examId);
                    tcUsualScoreService.saveBatch(tcUsualScoreList);
                    tcPaperStructService.remove(cultureProgramId, courseId, paperNumber, examId);
                    clearReport = true;
                }
                if (clearReport) {
                    trBasicInfoService.clearReportData(cultureProgramId, courseId, paperNumber, examId, true);
                }
                if (!CollectionUtils.isEmpty(paperStructDimensionResultList)) {
                    TCPaperStruct tcPaperStructDb = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber, examId);
                    if (Objects.nonNull(tcPaperStructDb)) {
                        tcPaperStructService.removeById(tcPaperStructDb.getId());
                    }
                    paperStructDimensionResultList = SDFrame.read(paperStructDimensionResultList).sortDesc(Sorter.sortDescBy(PaperStructDimensionResult::getMainNumber).sortDesc(PaperStructDimensionResult::getMainNumber)).toLists();
                    TCPaperStruct tcPaperStruct = new TCPaperStruct(cultureProgramId, courseId, basicCourse.getCode(), basicCourse.getName(), paperNumber, JacksonUtil.parseJson(paperStructDimensionResultList),
                            sysUser.getId(), examId, Objects.nonNull(tcPaperStructDb)
                            && Objects.nonNull(cultureProgramId) && Objects.nonNull(tcPaperStructDb.getCultureProgramId()) && cultureProgramId.longValue() == tcPaperStructDb.getCultureProgramId().longValue()
                            && Objects.nonNull(courseId) && Objects.nonNull(tcPaperStructDb.getCourseId()) && courseId.longValue() == tcPaperStructDb.getCourseId().longValue()
                            && Objects.nonNull(paperNumber) && Objects.nonNull(tcPaperStructDb.getPaperNumber()) && Objects.equals(paperNumber, tcPaperStructDb.getPaperNumber())
                            && Objects.nonNull(examId) && Objects.nonNull(tcPaperStructDb.getExamId()) && examId.longValue() == tcPaperStructDb.getExamId().longValue() ? tcPaperStructDb.getPaperStructDimension() : null);
                    tcPaperStructService.saveOrUpdate(tcPaperStruct);
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return messageMap;
    }

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return tcFinalScoreService.remove(this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId));
    }

    /**
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TCFinalScore> queryWrapper = this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId);
        queryWrapper.lambda().eq(TCFinalScore::getCreateId, sysUser.getId());
        return tcFinalScoreService.remove(queryWrapper);
    }

    /**
     * 共用查询
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    protected QueryWrapper<TCFinalScore> commonQueryWrapper(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        QueryWrapper<TCFinalScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId)
                .eq(TCFinalScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.eq(TCFinalScore::getPaperNumber, paperNumber).or().isNull(TCFinalScore::getPaperNumber));
        }
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().eq(TCFinalScore::getExamId, examId);
        }
        return queryWrapper;
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TCFinalScore> queryFinalScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.queryFinalScore(cultureProgramId, courseId, paperNumber, examId, dpr);
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TCFinalScore> queryFinalScoreAll(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return this.baseMapper.queryFinalScore(cultureProgramId, courseId, paperNumber, examId, null);
    }

    /**
     * 查询期末成绩同步状态
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public Boolean queryFinalScoreSync(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return !CollectionUtils.isEmpty(this.queryFinalScoreAll(cultureProgramId, courseId, paperNumber, examId)) ? true : false;
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    @Override
    public List<TCFinalScore> queryFinalScoreNotEqPaperNumber(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId) {
        QueryWrapper<TCFinalScore> queryWrapper = this.commonTcFinalScoreQuery(cultureProgramId, courseId, paperNumber, userId);
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().eq(TCFinalScore::getExamId, examId);
        }
        return tcFinalScoreService.list(queryWrapper);
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    @Override
    public List<TCFinalScore> queryFinalScoreNotEqExamIdAndPaperNumber(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId) {
        QueryWrapper<TCFinalScore> queryWrapper = this.commonTcFinalScoreQuery(cultureProgramId, courseId, paperNumber, userId);
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().and(w -> w.ne(TCFinalScore::getExamId, examId).or().isNull(TCFinalScore::getExamId));
        }
        return tcFinalScoreService.list(queryWrapper);
    }

    /**
     * 期末成绩公共查询
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param userId
     * @return
     */
    protected QueryWrapper<TCFinalScore> commonTcFinalScoreQuery(Long cultureProgramId, Long courseId, String paperNumber, Long userId) {
        QueryWrapper<TCFinalScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(" distinct culture_program_id as cultureProgramId,course_id as courseId,exam_id as examId,paper_number as paperNumber");
        queryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId)
                .eq(TCFinalScore::getCourseId, courseId);
        if (Objects.nonNull(userId)) {
            queryWrapper.lambda().eq(TCFinalScore::getCreateId, userId);
        }
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.ne(TCFinalScore::getPaperNumber, paperNumber).or().isNull(TCFinalScore::getPaperNumber));
        }
        return queryWrapper;
    }

    /**
     * 获取期末考试同步锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    @Override
    public void getFinalScoreSyncLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.SYNC + cultureProgramId + "_" + courseId + "_" + paperNumber + "_" + examId;
        Object o = redisUtil.get(lockKey);
        if (Objects.nonNull(o)) {
            throw ExceptionResultEnum.ERROR.exception("正在同步期末成绩数据，请稍候再试！");
        }
    }

    /**
     * 获取期末考试导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    @Override
    public void getFinalScoreImportLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.IMPORT + cultureProgramId + "_" + courseId + "_" + paperNumber + "_" + examId;
        Object o = redisUtil.get(lockKey);
        if (Objects.nonNull(o)) {
            throw ExceptionResultEnum.ERROR.exception("正在导入期末成绩数据，请稍候再试！");
        }
    }

    /**
     * 成绩管理列表
     *
     * @param iPage
     * @param cultureProgramId 培养方案id
     * @param semesterId       学期id
     * @param courseId         课程id
     * @return
     */
    @Override
    public IPage<ObeScoreResult> scoreList(IPage<Map> iPage, Long cultureProgramId, Long semesterId, Long courseId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return tcUsualScoreMapper.scoreList(iPage, sysUser.getSchoolId(), cultureProgramId, semesterId, courseId, dpr);
    }

    /**
     * 获取成绩管理需要的锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    @Override
    public void getLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        tcFinalScoreService.getFinalScoreSyncLock(cultureProgramId, courseId, paperNumber, examId);
        tcFinalScoreService.getFinalScoreImportLock(cultureProgramId, courseId, paperNumber, examId);
        tcPaperStructService.getPaperStructSyncLock(cultureProgramId, courseId, paperNumber, examId);
        tcPaperStructService.getPaperStructImportLock(cultureProgramId, courseId, paperNumber, examId);
    }

    /**
     * 校验方案下课程是否已导入成绩
     *
     * @param cultureProgramId 培养方案ID
     * @param courseId         课程ID
     */
    @Override
    public void checkUsualOrFinalScoreExist(Long cultureProgramId, Long courseId) {
        QueryWrapper<TCUsualScore> tcUsualScoreQueryWrapper = new QueryWrapper<>();
        tcUsualScoreQueryWrapper.lambda().eq(TCUsualScore::getCultureProgramId, cultureProgramId)
                .eq(TCUsualScore::getCourseId, courseId);
        if (tcUsualScoreService.count(tcUsualScoreQueryWrapper) > 0) {
            throw ExceptionResultEnum.ERROR.exception("已导入平时成绩，无法删除");
        }
        QueryWrapper<TCFinalScore> tcFinalScoreQueryWrapper = new QueryWrapper<>();
        tcFinalScoreQueryWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId)
                .eq(TCFinalScore::getCourseId, courseId);
        if (tcFinalScoreService.count(tcFinalScoreQueryWrapper) > 0) {
            throw ExceptionResultEnum.ERROR.exception("已导入期末成绩，无法删除");
        }
    }

    /**
     * 更新成绩数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Boolean updateScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        UpdateWrapper<TCFinalScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(TCFinalScore::getCultureProgramId, cultureProgramId)
                .eq(TCFinalScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            updateWrapper.lambda().and(w -> w.eq(TCFinalScore::getPaperNumber, paperNumber).or().isNull(TCFinalScore::getPaperNumber));
        }
        if (Objects.nonNull(examId)) {
            updateWrapper.lambda().eq(TCFinalScore::getExamId, examId);
        }
        updateWrapper.lambda().set(TCFinalScore::getScore, null).set(TCFinalScore::getScoreDetail, null)
                .set(TCFinalScore::getUpdateId, sysUser.getId()).set(TCFinalScore::getUpdateTime, System.currentTimeMillis());
        return tcFinalScoreService.update(updateWrapper);
    }
}
