package com.qmth.teachcloud.obe.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.enums.SourceEnum;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.obe.been.dto.TCUsualScoreDto;
import com.qmth.teachcloud.obe.entity.TCUsualScore;
import com.qmth.teachcloud.obe.mapper.TCUsualScoreMapper;
import com.qmth.teachcloud.obe.service.TCUsualScoreService;
import com.qmth.teachcloud.obe.service.TRBasicInfoService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 平常作业成绩表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCUsualScoreServiceImpl extends ServiceImpl<TCUsualScoreMapper, TCUsualScore> implements TCUsualScoreService {
    private final static Logger log = LoggerFactory.getLogger(TCUsualScoreServiceImpl.class);

    @Resource
    TCUsualScoreService tcUsualScoreService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    /**
     * 平时成绩导入
     *
     * @param file             excel
     * @param cultureProgramId 培养方案id
     * @param courseId         课程id
     * @param examId
     * @param paperNumber
     * @param usualScoreList
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> usualScoreExcelImport(MultipartFile file, Long cultureProgramId, Long courseId, Long examId, String paperNumber, String usualScoreList) {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        Map<String, String> messageMap = new LinkedHashMap<>();

        BasicCourse basicCourse = basicCourseService.getById(courseId);
        Objects.requireNonNull(basicCourse, "未找到课程信息");

        try {
            Double totalScore = 100d;
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<String[]> arrayList = excelReader.getDataArrayList();
            if (!CollectionUtils.isEmpty(arrayList)) {
                log.info("arrayList:{}", JacksonUtil.parseJson(arrayList));
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                List<TCUsualScore> tcUsualScoreList = new ArrayList<>(arrayList.size());
                String[] columnNames = excelReader.getColumnNames();
                for (int i = 0; i < columnNames.length; i++) {
                    if (i == 0 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "学号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                    } else if (i == 1 && Objects.nonNull(columnNames[i]) && !Objects.equals(columnNames[i].trim(), "姓名")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                    }
                }

                Map<String, ExcelField> excelFieldMap = new HashMap<>();
                if (Objects.nonNull(usualScoreList)) {
                    List<ExcelField> excelFieldList = GsonUtil.fromJson(usualScoreList, new TypeToken<List<ExcelField>>() {
                    }.getType());
                    excelFieldMap = excelFieldList.stream().collect(Collectors.toMap(k -> k.getName(), Function.identity(), (dto1, dto2) -> dto1));
                }

                for (int i = 0; i < arrayList.size(); i++) {
                    String[] strs = arrayList.get(i);
                    if (Objects.nonNull(strs) && strs.length > 0) {
                        if (strs.length != columnNames.length) {
                            throw ExceptionResultEnum.ERROR.exception("excel列和数据不匹配");
                        }
                        boolean error = false;
                        List<TCUsualScoreDto> tcUsualScoreDtoList = new ArrayList<>();
                        TCUsualScore tcUsualScore = new TCUsualScore(cultureProgramId, basicCourse.getId(), basicCourse.getCode(), basicCourse.getName(), paperNumber, SourceEnum.SYNC, sysUser.getId(), examId);
                        for (int j = 0; j < strs.length; j++) {
                            if (Objects.isNull(strs[j]) || Objects.equals(strs[j].trim(), "")) {
                                errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j] + "]为空;").add("\r\n");
                                error = true;
                            } else {
                                if (Objects.equals(columnNames[j].trim(), "学号")) {
                                    tcUsualScore.setStudentCode(strs[j]);
                                } else if (Objects.equals(columnNames[j].trim(), "姓名")) {
                                    tcUsualScore.setName(strs[j]);
                                } else {
                                    Double score = SystemConstant.isDouble(strs[j]);
                                    if (Objects.nonNull(score)) {
                                        if (score.doubleValue() > totalScore.doubleValue()) {
                                            errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]总分大于试卷总分").add("\r\n");
                                            error = true;
                                        } else {
                                            if (excelFieldMap.containsKey(columnNames[j])) {
                                                ExcelField excelField = excelFieldMap.get(columnNames[j]);
                                                if (Objects.nonNull(excelField.getValue()) && !Objects.equals(excelField.getValue().trim(), "")) {
                                                    BigDecimal scoreFull = new BigDecimal(excelField.getValue());
                                                    BigDecimal scoreBd = new BigDecimal(score);
                                                    if (scoreBd.compareTo(scoreFull) == 1) {
                                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]总分大于评价方式满分").add("\r\n");
                                                        error = true;
                                                    } else {
                                                        scoreBd = scoreBd.divide(scoreFull, 2, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP);
                                                        strs[j] = scoreBd.toString();
                                                    }
                                                }
                                            }
                                            tcUsualScoreDtoList.add(new TCUsualScoreDto(strs[j], columnNames[j]));
                                        }
                                    } else {
                                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[j]).add("]请输入数字").add("\r\n");
                                        error = true;
                                    }
                                }
                            }
                        }
                        if (CollectionUtils.isNotEmpty(tcUsualScoreDtoList)) {
                            tcUsualScore.setScore(JacksonUtil.parseJson(tcUsualScoreDtoList));
                        }
                        if (!error) {
                            tcUsualScoreList.add(tcUsualScore);
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(tcUsualScoreList)) {
                    successData.add("共导入").add(tcUsualScoreList.size() + "").add("条数据");
                    List<TCUsualScore> tcUsualScoreDbSourceList = tcUsualScoreService.queryUsualScore(cultureProgramId, basicCourse.getId(), paperNumber, examId);
                    if (!CollectionUtils.isEmpty(tcUsualScoreDbSourceList)) {
                        tcUsualScoreList.stream().peek(s -> s.updateExamAndPaperNumberInfo(tcUsualScoreDbSourceList.get(0))).collect(Collectors.toList());
                    }
                    if (CollectionUtils.isEmpty(tcUsualScoreDbSourceList) || !CollectionUtils.isEqualCollection(tcUsualScoreDbSourceList, tcUsualScoreList)) {
                        Map<String, TCUsualScore> tcUsualScoreDbMap = tcUsualScoreDbSourceList.stream().collect(Collectors.toMap(TCUsualScore::getStudentCode, Function.identity(), (dto1, dto2) -> dto1));
                        AtomicInteger size = new AtomicInteger();
                        tcUsualScoreList.stream().peek(s -> {
                            if (Objects.nonNull(s.getStudentCode()) && tcUsualScoreDbMap.containsKey(s.getStudentCode().trim())) {
                                size.incrementAndGet();
                            }
                        }).collect(Collectors.toList());
                        if (size.get() != tcUsualScoreList.size()) {
                            throw ExceptionResultEnum.ERROR.exception("请勿导入其他班级数据");
                        }
                        tcUsualScoreService.removeByIds(tcUsualScoreDbSourceList.stream().map(BaseEntity::getId).collect(Collectors.toList()));
                        tcUsualScoreService.saveBatch(tcUsualScoreList);
                        trBasicInfoService.clearReportData(cultureProgramId, basicCourse.getId(), paperNumber, examId
                                , false);
                    }
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 平时成绩列表（分页）
     *
     * @param iPage
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public IPage<TCUsualScore> usualScoreList(IPage<Map> iPage, Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.usualScoreList(iPage, cultureProgramId, courseId, paperNumber, examId, dpr);
    }

    /**
     * 查询平时作业成绩
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TCUsualScore> queryUsualScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.queryUsualScore(cultureProgramId, courseId, paperNumber, examId, dpr);
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TCUsualScore> queryUsualScoreAll(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return this.baseMapper.queryUsualScore(cultureProgramId, courseId, paperNumber, examId, null);
    }

    /**
     * 获取平时成绩导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    @Override
    public void getUsualScoreImportLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        String lockKey = SystemConstant.REDIS_USUAL_SCORE_DATA_PREFIX + SystemConstant.IMPORT + cultureProgramId + "_" + courseId + "_" + paperNumber + "_" + examId;
        Object o = redisUtil.get(lockKey);
        if (Objects.nonNull(o)) {
            throw ExceptionResultEnum.ERROR.exception("正在导入平时成绩数据，请稍候再试！");
        }
    }

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return tcUsualScoreService.remove(this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId));
    }

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TCUsualScore> queryWrapper = this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId);
        queryWrapper.lambda().eq(TCUsualScore::getCreateId, sysUser.getId());
        return tcUsualScoreService.remove(queryWrapper);
    }

    /**
     * 共用查询
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    protected QueryWrapper<TCUsualScore> commonQueryWrapper(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        QueryWrapper<TCUsualScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TCUsualScore::getCultureProgramId, cultureProgramId)
                .eq(TCUsualScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.eq(TCUsualScore::getPaperNumber, paperNumber).or().isNull(TCUsualScore::getPaperNumber));
        }
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().eq(TCUsualScore::getExamId, examId);
        }
        return queryWrapper;
    }

    /**
     * 更新成绩数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Boolean updateScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        UpdateWrapper<TCUsualScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(TCUsualScore::getCultureProgramId, cultureProgramId)
                .eq(TCUsualScore::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            updateWrapper.lambda().and(w -> w.eq(TCUsualScore::getPaperNumber, paperNumber).or().isNull(TCUsualScore::getPaperNumber));
        }
        if (Objects.nonNull(examId)) {
            updateWrapper.lambda().eq(TCUsualScore::getExamId, examId);
        }
        updateWrapper.lambda().set(TCUsualScore::getScore, null).set(TCUsualScore::getUpdateId, sysUser.getId()).set(TCUsualScore::getUpdateTime, System.currentTimeMillis());
        return tcUsualScoreService.update(updateWrapper);
    }
}
