package com.qmth.teachcloud.obe.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aventrix.jnanoid.jnanoid.NanoIdUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.data.*;
import com.deepoove.poi.data.style.CellStyle;
import com.deepoove.poi.data.style.ParagraphStyle;
import com.deepoove.poi.data.style.RowStyle;
import com.google.common.collect.ImmutableMap;
import com.google.gson.reflect.TypeToken;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.util.GsonUtil;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.bean.archivescore.ScoreRangeVo;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.obe.been.dto.*;
import com.qmth.teachcloud.obe.been.obe.CourseSuggestDto;
import com.qmth.teachcloud.obe.been.report.*;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.been.result.ObeCourseWeightResult;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.been.result.report.word.common.CourseBasicBean;
import com.qmth.teachcloud.obe.been.result.report.word.common.CourseReportBean;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuCourseBasicBean;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuCourseReportBean;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuCourseSuggestDto;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuScoreOtherBean;
import com.qmth.teachcloud.obe.entity.*;
import com.qmth.teachcloud.obe.mapper.TRBasicInfoMapper;
import com.qmth.teachcloud.obe.service.*;
import org.apache.poi.xwpf.usermodel.ParagraphAlignment;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 报告基本情况表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TRBasicInfoServiceImpl extends ServiceImpl<TRBasicInfoMapper, TRBasicInfo> implements TRBasicInfoService {
    private final static Logger log = LoggerFactory.getLogger(TRBasicInfoServiceImpl.class);

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    BasicSemesterService basicSemesterService;

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    TRExamStudentService trExamStudentService;

    @Resource
    ObeCourseWeightService obeCourseWeightService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    ObeCourseOutlineService obeCourseOutlineService;

    @Resource
    ObeCultureProgramService obeCultureProgramService;

    @Resource
    BasicProfessionalService basicProfessionalService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    BasicExamStudentService basicExamStudentService;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    protected static final String tbBgColor = "F2F2F2";
    protected static final String tbPercentWidth = "100%";
    protected static final String[] scoreTitles = new String[]{"100-90", "89-80", "79-70", "69-60", "<60"};

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId cultureProgramId
     * @param courseId         courseId
     * @param paperNumber      paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TRBasicInfo> queryBasicInfo(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.queryBasicInfo(cultureProgramId, courseId, paperNumber, examId, dpr);
    }

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId cultureProgramId
     * @param courseId         courseId
     * @param paperNumber      paperNumber
     * @param examId
     * @return
     */
    @Override
    public List<TRBasicInfo> queryBasicInfoAll(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return this.baseMapper.queryBasicInfo(cultureProgramId, courseId, paperNumber, examId, null);
    }

    /**
     * 获取课程目标考核分布-分数图
     *
     * @param trBasicInfo
     * @param markPaper
     * @return
     */
    @Override
    public ReportScoreViewDto getScoreList(TRBasicInfo trBasicInfo, MarkPaper markPaper) {
        FinalScoreDto finalScoreDto = tcFinalScoreService.finalScoreScoreOverView(trBasicInfo.getExamId(), trBasicInfo.getPaperNumber(), trBasicInfo.getCultureProgramId(), trBasicInfo.getCourseId(), trBasicInfo.getTeachClassName(), Objects.nonNull(trBasicInfo.getCreateId()) && trBasicInfo.getCreateId().longValue() != -1 ? trBasicInfo.getCreateId() : null);
        Objects.requireNonNull(finalScoreDto, "未找到期末成绩分数信息");

        if (finalScoreDto.getStudentCount() == 0) {
            throw ExceptionResultEnum.ERROR.exception("成绩数据异常，未查询到学生，请检查");
        }

        ReportScoreViewDto reportScoreViewDto = null;
        TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(trBasicInfo.getCultureProgramId(), trBasicInfo.getCourseId(), trBasicInfo.getPaperNumber(), trBasicInfo.getExamId());
        if (Objects.nonNull(tcPaperStruct) && Objects.nonNull(tcPaperStruct.getPaperStruct()) && Objects.nonNull(tcPaperStruct.getTotalScore())) {
            BasicCourse basicCourse = basicCourseService.getById(trBasicInfo.getCourseId());
            markPaper = new MarkPaper(markPaper.getExamId(), markPaper.getCourseCode(), basicCourse.getName(), markPaper.getPaperNumber(), tcPaperStruct.getTotalScore(), tcPaperStruct.getPassScore());
        }
        fillScoreRange(finalScoreDto, trBasicInfo.getExamId(), trBasicInfo.getPaperNumber(), trBasicInfo.getCultureProgramId(), trBasicInfo.getCourseId(), markPaper, trBasicInfo.getTeachClassName(), trBasicInfo.getCreateId());
        List<ScoreRangeVo> scoreRangeVoList = finalScoreDto.getScoreRange();
        if (!CollectionUtils.isEmpty(scoreRangeVoList)) {
            Integer failCount = scoreRangeVoList.get(0).getStudentCount();//第一行数据默认为不及格
            BigDecimal failRate = new BigDecimal(scoreRangeVoList.get(0).getRate());

            scoreRangeVoList.remove(0);
            List<ReportScoreRangeViewDto> scoreRangeViewDtoList = GsonUtil.fromJson(JacksonUtil.parseJson(scoreRangeVoList), new TypeToken<List<ReportScoreRangeViewDto>>() {
            }.getType());
            reportScoreViewDto = new ReportScoreViewDto(failCount, failRate, scoreRangeViewDtoList);
        }
        return reportScoreViewDto;
    }

    /**
     * 获取计算参数
     *
     * @param trBasicInfo
     * @param paperStructDimensionResultList
     * @param obeCourseWeightResult
     * @return
     */
    @Override
    public Map<String, Object> getReportCalculateParams(TRBasicInfo trBasicInfo, List<PaperStructDimensionResult> paperStructDimensionResultList, ObeCourseWeightResult obeCourseWeightResult) {
        Map<String, Object> paramsMap = new HashMap<>();
        //查询考生数据
        List<FinalScoreResult> finalScoreResultList = tcFinalScoreService.examStudentOverview(trBasicInfo.getExamId(), trBasicInfo.getPaperNumber(), trBasicInfo.getCultureProgramId(), trBasicInfo.getCourseId(), trBasicInfo.getTeachClassName(), Objects.nonNull(trBasicInfo.getCreateId()) && trBasicInfo.getCreateId().longValue() != -1 ? trBasicInfo.getCreateId() : null);//考生成绩
        if (CollectionUtils.isEmpty(finalScoreResultList)) {
            throw ExceptionResultEnum.ERROR.exception("成绩数据异常，平时成绩与期末成绩里考生名单不一致，请检查");
        }

        Map<String, Map<Long, BigDecimal>> finalScoreExamStudentTargetMap = new LinkedHashMap<>();//汇总考生课程目标-课程目标卷面分
        Map<Long, Map<Long, DimensionDto>> targetDimensionMap = new LinkedHashMap<>();//课程目标-知识点-卷面分（word报表用）

        List<ReportExamStudentDto> examStudentList = null;//考生集合
        Map<String, FinalScoreResult> finalScoreResultMap = null;//考生成绩map
        Map<String, PaperStructDimensionResult> paperStructResultMap = null;//试卷蓝图map
        Map<Long, CourseTargetWordDto> targetWordMap = null;//word目标集合
        Map<Long, CourseTargetWebDto> targetWebMap = null;//web目标集合
        Map<String, List<ReportEvaluationDto>> usualScoreMap = null;//平时作业集合
        Long weightSettingSign = null, dimensionSign = null;//权重变化id,知识点变化id
        if (!CollectionUtils.isEmpty(finalScoreResultList)) {
            targetWordMap = new LinkedHashMap<>();
            targetWebMap = new LinkedHashMap<>();
            usualScoreMap = new LinkedHashMap<>();

            List<CourseWeightDto> courseWeightDtoList = obeCourseWeightResult.getSubmitForm();
            weightSettingSign = obeCourseWeightResult.getWeightSettingSign();
            dimensionSign = obeCourseWeightResult.getDimensionSign();

            for (CourseWeightDto courseWeightDto : courseWeightDtoList) {
                List<CourseWeightDetailDto> courseWeightDetailDtoList = courseWeightDto.getEvaluationList();
                CourseTargetWordDto courseTargetWordDto = new CourseTargetWordDto(courseWeightDto);
                targetWordMap.put(courseWeightDto.getCourseTargetId(), courseTargetWordDto);//word模版

                CourseTargetWebDto courseTargetWebDto = new CourseTargetWebDto(courseWeightDto);
                targetWebMap.put(courseWeightDto.getCourseTargetId(), courseTargetWebDto);//web模版

                if (!CollectionUtils.isEmpty(courseWeightDetailDtoList)) {
                    List<ReportEvaluationDto> reportEvaluationDtoList = new ArrayList<>(courseWeightDetailDtoList.size());
                    List<ReportEvaluationDto> usualScoreDetailList = new ArrayList<>(courseWeightDetailDtoList.size());
                    UsualScoreWordDto usualScoreWordDto = new UsualScoreWordDto();
                    usualScoreWordDto.setUsualScoreDetailList(usualScoreDetailList);
                    BigDecimal usualScoreTargetSumScore = new BigDecimal(0);//平常作业目标总分
                    for (CourseWeightDetailDto courseWeightDetailDto : courseWeightDetailDtoList) {
                        //期末考试
                        if (courseWeightDetailDto.getEnable() && Objects.equals(courseWeightDetailDto.getEvaluationName(), SystemConstant.FINAL_SCORE_STR)) {
                            FinalScoreWordDto finalScoreWordDto = new FinalScoreWordDto(courseWeightDto.getCourseTargetId(), courseWeightDetailDto);
                            courseTargetWordDto.setFinalScoreDto(finalScoreWordDto);
                            ReportEvaluationDto reportEvaluationDto = new ReportEvaluationDto(courseWeightDto.getCourseTargetId(), courseWeightDetailDto, new BigDecimal(0));
                            reportEvaluationDtoList.add(reportEvaluationDto);
                        }//非期末考试
                        else if (courseWeightDetailDto.getEnable() && !Objects.equals(courseWeightDetailDto.getEvaluationName(), SystemConstant.FINAL_SCORE_STR)) {
                            ReportEvaluationDto reportEvaluationDto = new ReportEvaluationDto(courseWeightDto.getCourseTargetId(), courseWeightDetailDto, new BigDecimal(0));
                            usualScoreTargetSumScore = usualScoreTargetSumScore.add(reportEvaluationDto.getTargetScore());
                            usualScoreDetailList.add(reportEvaluationDto);
                            reportEvaluationDtoList.add(reportEvaluationDto);
                            if (!usualScoreMap.containsKey(courseWeightDetailDto.getEvaluationName())) {
                                usualScoreMap.put(courseWeightDetailDto.getEvaluationName(), new ArrayList<>(Arrays.asList(reportEvaluationDto)));
                            } else {
                                List<ReportEvaluationDto> reportEvaluationDtos = usualScoreMap.get(courseWeightDetailDto.getEvaluationName());
                                reportEvaluationDtos.add(reportEvaluationDto);
                                usualScoreMap.put(courseWeightDetailDto.getEvaluationName(), reportEvaluationDtos);
                            }
                        }
                    }
                    usualScoreWordDto.setUsualScoreTargetSumScore(usualScoreTargetSumScore);
                    Collections.sort(reportEvaluationDtoList);
                    courseTargetWebDto.setEvaluationList(reportEvaluationDtoList);

                    courseTargetWordDto.setUsualScoreDto(usualScoreWordDto);
                }
            }

            //期末成绩明细map
            paperStructResultMap = paperStructDimensionResultList.stream().collect(Collectors.toMap(k -> k.getMainNumber() + "-" + k.getSubNumber(), Function.identity(), (dto1, dto2) -> dto1));
            finalScoreResultMap = finalScoreResultList.stream().collect(Collectors.toMap(FinalScoreResult::getStudentCode, Function.identity(), (dto1, dto2) -> dto1));

            examStudentList = new ArrayList<>(finalScoreResultList.size());
            for (int i = 0; i < finalScoreResultList.size(); i++) {
                FinalScoreResult finalScoreResult = finalScoreResultList.get(i);
                //解析期末成绩知识点
                List<TCFinalScoreDto> tcFinalScoreDtoList = GsonUtil.fromJson(finalScoreResult.getFinalScoreDetail(), new TypeToken<List<TCFinalScoreDto>>() {
                }.getType());
                for (TCFinalScoreDto t : tcFinalScoreDtoList) {
                    BigDecimal score = Objects.nonNull(t.getScore()) ? new BigDecimal(t.getScore()) : new BigDecimal(0);
                    PaperStructDimensionResult paperStructDimensionResult = paperStructResultMap.get(t.getName().toString());
                    if (Objects.nonNull(paperStructDimensionResult)) {
                        if (paperStructDimensionResult.getScore().doubleValue() < score.doubleValue()) {
                            throw ExceptionResultEnum.ERROR.exception("学生[" + finalScoreResult.getName() + "]准考证号[" + finalScoreResult.getStudentCode() + "]题号[" + t.getName() + "]该小题得分：" + score + "，该小题总分：" + paperStructDimensionResult.getScore() + "，该题学生得分大于该小题总分，请重新导入期末成绩");
                        }
                        List<CourseTargetWebDto> courseTargetDtoList = paperStructDimensionResult.getTargetList();
                        for (CourseTargetWebDto courseTargetDto : courseTargetDtoList) {
                            this.calculateFinalScoreByTargetExamStudent(finalScoreExamStudentTargetMap, finalScoreResult, courseTargetDto, score);
                            //知识点-期末考试明细分
                            List<DimensionDto> dimensionDtoList = courseTargetDto.getDimensionList();
                            for (DimensionDto dimensionDto : dimensionDtoList) {
                                if (i == 0) {//计算各知识点原始卷面分总和
                                    BigDecimal paperScore = paperStructDimensionResult.getScore();
                                    //目标-考生知识点去重
                                    if (!targetDimensionMap.containsKey(courseTargetDto.getTargetId())) {
                                        Map<Long, DimensionDto> dimensionMap = new LinkedHashMap<>();
                                        dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, paperScore));
                                        targetDimensionMap.put(courseTargetDto.getTargetId(), dimensionMap);
                                    } else {
                                        Map<Long, DimensionDto> dimensionMap = targetDimensionMap.get(courseTargetDto.getTargetId());
                                        if (!dimensionMap.containsKey(dimensionDto.getDimensionId())) {
                                            dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, paperScore));
                                        } else {
                                            DimensionDto dimensionDtoTemp = dimensionMap.get(dimensionDto.getDimensionId());
                                            dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, dimensionDtoTemp.getDimensionScore().add(paperScore)));
                                        }
                                        targetDimensionMap.put(courseTargetDto.getTargetId(), dimensionMap);
                                    }
                                }
                            }
                        }
                    }
                }

                Map<Long, CourseTargetWordDto> finalTargetWordMap = targetWordMap;
                Map<Long, CourseTargetWebDto> finalTargetWebMap = targetWebMap;
                targetDimensionMap.forEach((k, v) -> {
                    List<DimensionDto> dimensionDtoList = new ArrayList<>(v.values());
                    Collections.sort(dimensionDtoList);
                    Double dimensionScoreSum = dimensionDtoList.stream().mapToDouble(s -> s.getDimensionScore().doubleValue()).sum();
                    List<String> dimensionDtoNameList = dimensionDtoList.stream().map(s -> s.getDimensionName()).collect(Collectors.toList());
                    if (finalTargetWordMap.containsKey(k)) {
                        if (Objects.nonNull(finalTargetWordMap.get(k).getFinalScoreDto())) {
                            finalTargetWordMap.get(k).getFinalScoreDto().setDimensionList(dimensionDtoList);
                        }
                        finalTargetWordMap.get(k).setDimensionScoreSum(new BigDecimal(dimensionScoreSum));
                        finalTargetWordMap.get(k).setDimensionPoint(String.join(",", dimensionDtoNameList));
                    }
                    if (finalTargetWebMap.containsKey(k)) {
                        finalTargetWebMap.get(k).setDimensionScoreSum(new BigDecimal(dimensionScoreSum));
                        finalTargetWebMap.get(k).setDimensionPoint(String.join(",", dimensionDtoNameList));
                    }
                    Map<Long, BigDecimal> doubleMap = finalScoreExamStudentTargetMap.get(finalScoreResult.getStudentCode());
                    Map<Long, BigDecimal> mapTarget = finalScoreResult.getFinalScoreTarget();
                    mapTarget.put(k, doubleMap.get(k));
                });

                //解析平时作业分数图
                ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = this.calculateUsualScoreByExamStudent(finalScoreResult, usualScoreMap, targetWordMap);
                finalScoreResult.setUsualScoreDto(reportExamStudentUsualScoreDto);
                Map<Long, List<ReportExamStudentUsualScoreObjDto>> reportExamStudentUsualScoreDtoScoreMap = reportExamStudentUsualScoreDto.getScoreMap();
                if (!CollectionUtils.isEmpty(reportExamStudentUsualScoreDtoScoreMap)) {
                    Map<Long, BigDecimal> map = finalScoreExamStudentTargetMap.get(finalScoreResult.getStudentCode());
                    reportExamStudentUsualScoreDtoScoreMap.forEach((k, v) -> {
                        if (!map.containsKey(k)) {
                            map.put(k, new BigDecimal(0));
                        }
                    });
                    finalScoreExamStudentTargetMap.put(finalScoreResult.getStudentCode(), map);
                }
                examStudentList.add(new ReportExamStudentDto(finalScoreResult));
            }
        }

        paramsMap.put("finalScoreResultList", finalScoreResultList);
        paramsMap.put("usualScoreMap", usualScoreMap);
        paramsMap.put("finalScoreExamStudentTargetMap", finalScoreExamStudentTargetMap);
        paramsMap.put("examStudentList", examStudentList);
        paramsMap.put("finalScoreResultMap", finalScoreResultMap);
        paramsMap.put("targetWordMap", targetWordMap);
        paramsMap.put("targetWebMap", targetWebMap);
        paramsMap.put("weightSettingSign", weightSettingSign);
        paramsMap.put("dimensionSign", dimensionSign);
        return paramsMap;
    }

    /**
     * 计算课程考核成绩评价结果
     *
     * @param trBasicInfo
     * @return
     */
    @Override
    public ReportCourseEvaluationResultDto reportCalculateCourseEvaluationResult(TRBasicInfo trBasicInfo, Map<String, Object> paramsMap) {
        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = null;
        List<TRExamStudent> trExamStudentList = trExamStudentService.list(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
        if (!CollectionUtils.isEmpty(trExamStudentList)) {
            BigDecimal targetEvaluationSumValue = new BigDecimal(0);
            Map<Long, CourseTargetWebDto> targetWebMap = (Map<Long, CourseTargetWebDto>) paramsMap.get("targetWebMap");
            Map<Long, CourseTargetWordDto> targetWordMap = (Map<Long, CourseTargetWordDto>) paramsMap.get("targetWordMap");
            TRExamStudent targetAvg = trExamStudentList.get(trExamStudentList.size() - 2);//平均分
            if (Objects.nonNull(targetAvg) && Objects.equals(targetAvg.getName(), "平均分") && Objects.nonNull(targetAvg.getResultDetail())) {
                List<ReportExamStudentTargetDto> reportExamStudentTargetAvgDtoList = GsonUtil.fromJson(targetAvg.getResultDetail(), new TypeToken<List<ReportExamStudentTargetDto>>() {
                }.getType());
                List<TRExamStudent> trExamStudentNewList = new ArrayList<>();
                trExamStudentNewList.addAll(trExamStudentList);

                trExamStudentNewList.remove(trExamStudentNewList.size() - 1);
                trExamStudentNewList.remove(trExamStudentNewList.size() - 1);
                trExamStudentNewList.remove(trExamStudentNewList.size() - 1);
                Map<Long, BigDecimal> targetEvaluationValueMap = new HashMap<>();
                Map<Long, Integer> targetEvaluationValueSizeMap = new HashMap<>();
                for (TRExamStudent trExamStudent : trExamStudentNewList) {
                    List<ReportExamStudentTargetDto> targetList = GsonUtil.fromJson(trExamStudent.getResultDetail(), new TypeToken<List<ReportExamStudentTargetDto>>() {
                    }.getType());
                    for (ReportExamStudentTargetDto r : targetList) {
                        if (targetEvaluationValueMap.containsKey(r.getTargetId())) {
                            BigDecimal targetEvaluationValueTemp = targetEvaluationValueMap.get(r.getTargetId());
                            BigDecimal targetEvaluationValue = new BigDecimal(0);
                            targetEvaluationValue = targetEvaluationValue.add(r.getMatrixDegree()).add(targetEvaluationValueTemp);
                            targetEvaluationValueMap.put(r.getTargetId(), targetEvaluationValue);

                            Integer size = targetEvaluationValueSizeMap.get(r.getTargetId());
                            targetEvaluationValueSizeMap.put(r.getTargetId(), size + 1);
                        } else {
                            BigDecimal targetEvaluationValue = new BigDecimal(0);
                            targetEvaluationValue = targetEvaluationValue.add(r.getMatrixDegree());
                            targetEvaluationValueMap.put(r.getTargetId(), targetEvaluationValue);
                            targetEvaluationValueSizeMap.put(r.getTargetId(), 1);
                        }
                    }
                }
                for (ReportExamStudentTargetDto r : reportExamStudentTargetAvgDtoList) {
                    BigDecimal targetAvgSumScore = new BigDecimal(0);//平均分总和
                    BigDecimal targetSumScore = new BigDecimal(0);//目标分总和
                    if (targetWebMap.containsKey(r.getTargetId())) {
                        CourseTargetWebDto courseTargetWebDto = targetWebMap.get(r.getTargetId());
                        CourseTargetWordDto courseTargetWordDto = targetWordMap.get(r.getTargetId());

                        List<ReportEvaluationDto> reportEvaluationDtoList = courseTargetWebDto.getEvaluationList();
                        Map<Long, ReportEvaluationDto> reportEvaluationDtoMap = reportEvaluationDtoList.stream().collect(Collectors.toMap(ReportEvaluationDto::getEvaluationId, Function.identity(), (dto1, dto2) -> dto1));

                        ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = r.getFinalScore();
                        ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = r.getUsualScore();

                        //期末成绩换算
                        ReportEvaluationDto reportEvaluationDto = null;
                        if (Objects.nonNull(reportExamStudentFinalScoreDto)) {
                            reportEvaluationDto = reportEvaluationDtoMap.get(reportExamStudentFinalScoreDto.getEvaluationId());
                            reportEvaluationDto.setTargetAvgScore(reportExamStudentFinalScoreDto.getMatrixAvgScore());
                            targetAvgSumScore = targetAvgSumScore.add(reportEvaluationDto.getTargetAvgScore());
                            targetSumScore = targetSumScore.add(reportEvaluationDto.getTargetScore());
                        }

                        //平时作业换算
                        if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                            List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                            for (ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto : reportExamStudentUsualScoreObjDtoList) {
                                reportEvaluationDto = reportEvaluationDtoMap.get(reportExamStudentUsualScoreObjDto.getEvaluationId());
                                reportEvaluationDto.setTargetAvgScore(reportExamStudentUsualScoreObjDto.getMatrixAvgScore());
                                targetAvgSumScore = targetAvgSumScore.add(reportEvaluationDto.getTargetAvgScore());
                                targetSumScore = targetSumScore.add(reportEvaluationDto.getTargetScore());
                            }
                        }

                        //计算目标达成度
                        //目标下各平均分总和/科目目标分
                        BigDecimal targetEvaluationValue = new BigDecimal(0);
                        if (targetEvaluationValueMap.containsKey(r.getTargetId())) {
                            targetEvaluationValue = targetEvaluationValueMap.get(r.getTargetId()).divide(new BigDecimal(targetEvaluationValueSizeMap.get(r.getTargetId())), 4, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
                        }
                        courseTargetWebDto.setEvaluationValue(targetEvaluationValue);
                        if (targetEvaluationSumValue.compareTo(new BigDecimal(0)) == 0 || targetEvaluationSumValue.compareTo(targetEvaluationValue) == 1) {
                            targetEvaluationSumValue = targetEvaluationValue;
                        }
                        courseTargetWordDto.setEvaluationValue(targetEvaluationValue);
                    }
                }
                reportCourseEvaluationResultDto = new ReportCourseEvaluationResultDto(targetEvaluationSumValue, new ArrayList<>(targetWebMap.values()), (Long) paramsMap.get("weightSettingSign"));
            }
            paramsMap.put("targetWebMap", targetWebMap);
        }
        return reportCourseEvaluationResultDto;
    }

    /**
     * 获取报告
     *
     * @param markPaper
     * @param obeCourseWeightResult
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @return
     */
    @Override
    @Transactional
    public List<TRBasicInfo> getReportView(MarkPaper markPaper, ObeCourseWeightResult obeCourseWeightResult, Long cultureProgramId, Long courseId, Long examId, String paperNumber) {
        ObeCourseOutline obeCourseOutline = obeCourseOutlineService.findByCultureProgramIdAndCourseId(cultureProgramId, courseId);
        Objects.requireNonNull(obeCourseOutline, "未找到课程大纲信息");
        BasicSemester basicSemester = basicSemesterService.getById(obeCourseOutline.getSemesterId());
        Objects.requireNonNull(basicSemester, "未找到学期信息");

        List<FinalScoreResult> finalScoreResultList = tcFinalScoreService.examStudentOverview(Objects.nonNull(examId) ? examId : markPaper.getExamId(), Objects.nonNull(paperNumber) ? paperNumber : markPaper.getPaperNumber(), cultureProgramId, courseId, null, null);//考生成绩
        if (CollectionUtils.isEmpty(finalScoreResultList)) {
            throw ExceptionResultEnum.ERROR.exception("成绩数据异常，平时成绩与期末成绩里考生名单不一致，请检查");
        }

        Set<ScoreParamDto> scoreParamDtoSet = new LinkedHashSet<>();
        String nid = NanoIdUtils.randomNanoId();
        finalScoreResultList.stream().peek(s -> scoreParamDtoSet.add(new ScoreParamDto(s.getTeachClassName(), s.getTeachId()))).collect(Collectors.toSet());
        scoreParamDtoSet.add(new ScoreParamDto(SystemConstant.ALL + nid, -1L));

        List<TRBasicInfo> trBasicInfoList = new ArrayList<>();
        for (ScoreParamDto s : scoreParamDtoSet) {
            TRBasicInfo trBasicInfo = null;
            ReportCourseBasicInfoDto reportCourseBasicInfoDtoBasic = trBasicInfoService.setBasicInfo(trBasicInfo, obeCourseOutline, markPaper, obeCourseWeightResult);

            TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, markPaper.getPaperNumber(), Objects.nonNull(trBasicInfo) ? trBasicInfo.getExamId() : null);
            //课程目标考核分布-题目信息
            if (Objects.isNull(tcPaperStruct) || Objects.isNull(tcPaperStruct.getPaperStructDimension())) {
                throw ExceptionResultEnum.ERROR.exception("未找到试卷蓝图信息");
            }
            if (Objects.isNull(trBasicInfo)) {
                trBasicInfo = new TRBasicInfo(cultureProgramId, courseId, markPaper.getCourseCode(), markPaper.getCourseName(), markPaper.getPaperNumber(), basicSemester.getName(), s.getTeachId(), Objects.nonNull(trBasicInfo) ? trBasicInfo.getCourseSuggest() : null, Objects.nonNull(trBasicInfo) ? trBasicInfo.getCourseEnName() : null, tcPaperStruct.getExamId(), reportCourseBasicInfoDtoBasic);
            } else {
                trBasicInfo.setBasicInfo(tcPaperStruct.getExamId(), markPaper.getCourseCode(), markPaper.getCourseName(), markPaper.getPaperNumber(), basicSemester.getName(), cultureProgramId, courseId, trBasicInfo.getCourseEnName(), trBasicInfo.getCourseSuggest(), reportCourseBasicInfoDtoBasic);
            }
            if (Objects.nonNull(nid) && !Objects.equals(s.getTeachClassName(), SystemConstant.ALL + nid)) {
                trBasicInfo.setTeachClassName(s.getTeachClassName());
            }
            //课程基本情况
            ReportCourseBasicInfoDto reportCourseBasicInfoDto = new ReportCourseBasicInfoDto(trBasicInfo);
            trBasicInfo.getReportResult().setCourseBasicInfo(reportCourseBasicInfoDto);

            List<PaperStructDimensionResult> paperStructDimensionResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStructDimension(), new TypeToken<List<PaperStructDimensionResult>>() {
            }.getType());

            //课程目标考核分布
            ReportCourseEvaluationSpreadDto reportCourseEvaluationSpreadDto = new ReportCourseEvaluationSpreadDto(paperStructDimensionResultList);
            if (Objects.nonNull(reportCourseEvaluationSpreadDto)) {
                reportCourseEvaluationSpreadDto.setScoreList(trBasicInfoService.getScoreList(trBasicInfo, markPaper));
                trBasicInfo.setCourseEvaluationSpread(JacksonUtil.parseJson(reportCourseEvaluationSpreadDto));
                trBasicInfo.getReportResult().setCourseEvaluationSpreadInfo(reportCourseEvaluationSpreadDto);
            }

            Map<String, Object> paramsMap = trBasicInfoService.getReportCalculateParams(trBasicInfo, paperStructDimensionResultList, obeCourseWeightResult);

            //课程目标达成评价明细结果-考生集合
            List<ReportExamStudentDto> examStudentNewList = trExamStudentService.getExamStudentReportDetail(trBasicInfo, paramsMap);
            ReportCourseEvaluationResultDetailDto reportCourseEvaluationResultDetailDto = new ReportCourseEvaluationResultDetailDto(examStudentNewList);
            if (Objects.nonNull(reportCourseEvaluationResultDetailDto)) {
                trBasicInfo.getReportResult().setCourseEvaluationResultDetailInfo(reportCourseEvaluationResultDetailDto);
            }

            //课程考核成绩评价结果
            ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = trBasicInfoService.reportCalculateCourseEvaluationResult(trBasicInfo, paramsMap);
            if (Objects.nonNull(reportCourseEvaluationResultDto)) {
                trBasicInfo.setCourseEvaluationResult(JacksonUtil.parseJson(reportCourseEvaluationResultDto));
                trBasicInfo.getReportResult().setCourseEvaluationResultInfo(reportCourseEvaluationResultDto);
            }

            //存储数据
            if (Objects.nonNull(paramsMap.get("targetWordMap"))) {
                Map<Long, CourseTargetWordDto> targetWordMap = (Map<Long, CourseTargetWordDto>) paramsMap.get("targetWordMap");
                List<CourseTargetWordDto> courseTargetWordDtoList = new ArrayList<>(targetWordMap.values());
                Collections.sort(courseTargetWordDtoList);

                Map<Long, CourseTargetWebDto> targetWebMap = (Map<Long, CourseTargetWebDto>) paramsMap.get("targetWebMap");
                List<CourseTargetWebDto> courseTargetWebDtoList = new ArrayList<>(targetWebMap.values());
                Collections.sort(courseTargetWebDtoList);

                JSONObject jsonObject = new JSONObject();
                jsonObject.put("targetWordMap", courseTargetWordDtoList);
                jsonObject.put("targetWebMap", courseTargetWebDtoList);
                trBasicInfo.setCourseEvaluationResultDetail(jsonObject.toJSONString());
            }
            reportCourseBasicInfoDto.setParticipantCount(examStudentNewList.size() - 3);
            trBasicInfo.setParticipantCount(examStudentNewList.size() - 3);
            trBasicInfo.getReportResult().setCommonInfo(new ReportCommonDto(tcPaperStruct.getExamId(), markPaper.getCourseCode(), markPaper.getCourseName(), tcPaperStruct.getPaperNumber()));
            if (Objects.nonNull(nid) && Objects.equals(s.getTeachClassName(), SystemConstant.ALL + nid)) {
                trBasicInfo.setTeachClassName(SystemConstant.ALL);
                trBasicInfo.setCreateId(s.getTeachId());
            } else {
                trBasicInfo.setTeachClassName(s.getTeachClassName());
            }
            trBasicInfo.getReportResult().updateInfo(trBasicInfo.getTeachClassName(), trBasicInfo.getId(), trBasicInfo.getCourseSuggest());
            Map<String, Object> map = this.queryScatterYMap(trBasicInfo);
            LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = (LinkedMultiValueMap<Long, Map<String, String>>) map.get("finalScatterYMap");
            trBasicInfo.getReportResult().setCourseTargetScatterMap(finalScatterYMap);
            trBasicInfoList.add(trBasicInfo);
        }
        //课程目标达成评价明细结果-课程目标达成评价值图
        return trBasicInfoList;
    }

    /**
     * 构建word动态表格1
     *
     * @param courseTargetWordDtoList
     * @return
     */
    @Override
    public TableRenderData buildWordTable1(List<CourseTargetWordDto> courseTargetWordDtoList) {
        //table1课程目标目标与毕业要求指标点的对应关系
        RowRenderData header1 = Rows.of("课程目标", "毕业要求指标点", "毕业要求").rowStyle(this.getHeadRowStyle()).create();
        Tables.TableBuilder tableBuilder1 = Tables.ofPercentWidth(tbPercentWidth, new int[]{20, 40, 40});
        tableBuilder1.addRow(header1);

        for (int i = 0; i < courseTargetWordDtoList.size(); i++) {
            CourseTargetWordDto courseTargetWordDto = courseTargetWordDtoList.get(i);
            RowRenderData row = Rows.of(courseTargetWordDto.getTargetName(), courseTargetWordDto.getObeCultureProgramRequirementName(), courseTargetWordDto.getParentRequirementName()).rowStyle(this.getRowStyle()).create();
            tableBuilder1.addRow(row);
        }
        return tableBuilder1.left().create();
    }

    /**
     * 构建word动态表格2
     *
     * @param courseTargetWordDtoList
     * @return
     */
    @Override
    public TableRenderData buildWordTable2(List<CourseTargetWordDto> courseTargetWordDtoList) {
        Set<String> header2_1_set = new LinkedHashSet<>();
        header2_1_set.add("课程目标");
        header2_1_set.add("考核项目及比例（%）");

        Set<String> header2_2_set = new LinkedHashSet<>();
        String finalSocreTitle = null;
        for (int i = 0; i < courseTargetWordDtoList.size(); i++) {
            CourseTargetWordDto courseTargetWordDto = courseTargetWordDtoList.get(i);
            UsualScoreWordDto usualScoreWordDto = courseTargetWordDto.getUsualScoreDto();
            if (Objects.nonNull(usualScoreWordDto) && !CollectionUtils.isEmpty(usualScoreWordDto.getUsualScoreDetailList())) {
                Set<String> headTemp = usualScoreWordDto.getUsualScoreDetailList().stream().map(s -> s.getEvaluation()).collect(Collectors.toSet());
                header2_2_set.addAll(headTemp);
            }
            FinalScoreWordDto finalScoreWordDto = courseTargetWordDto.getFinalScoreDto();
            if (Objects.nonNull(finalScoreWordDto)) {
                finalSocreTitle = finalScoreWordDto.getEvaluation();
            }
        }
        List<String> list_header = new ArrayList<>(header2_2_set);
//        Collections.sort(list_header, Collections.reverseOrder());
        Collections.sort(list_header);

        List<CellRenderData> cells_1 = new ArrayList<>(), cells_2 = new ArrayList<>();
        cells_1.add(Cells.of("课程目标").center().create());
        cells_1.add(Cells.of("考核项目及比例（%）").center().create());
        cells_2.add(Cells.of("课程目标").center().create());

        List<String> list = new ArrayList<>(header2_1_set);
        if (Objects.nonNull(finalSocreTitle)) {
            list.add(finalSocreTitle);
        }
        list.addAll(list_header);

        for (int i = 2; i < list.size(); i++) {
            cells_2.add(Cells.of(list.get(i)).center().create());
            if (i < list.size() - 1) {
                cells_1.add(Cells.of("").center().create());
            }
        }
        cells_1.add(Cells.of("权重（%）").center().create());
        cells_2.add(Cells.of("权重（%）").center().create());

        RowRenderData[] rowRenderData2 = new RowRenderData[courseTargetWordDtoList.size() + 3];
        RowRenderData header2_1 = new RowRenderData();
        header2_1.setCells(cells_1);
        header2_1.setRowStyle(this.getHeadRowStyle());

        RowRenderData header2_2 = new RowRenderData();
        header2_2.setCells(cells_2);
        header2_2.setRowStyle(this.getHeadRowStyle());

        list.remove(0);
        list.remove(0);

        RowRenderData totalRow = new RowRenderData();
        List<CellRenderData> totalCell = new ArrayList<>();
        totalCell.add(Cells.of("合计").create());
        Map<String, BigDecimal> totalMap = new HashMap<>();
        BigDecimal weight = new BigDecimal(0);
        for (int i = 0; i < courseTargetWordDtoList.size(); i++) {
            CourseTargetWordDto courseTargetWordDto = courseTargetWordDtoList.get(i);
            weight = weight.add(courseTargetWordDto.getTargetSumWeight());
            Map<String, BigDecimal> map = new LinkedHashMap<>();
            RowRenderData row = new RowRenderData();
            List<CellRenderData> cell = new ArrayList<>();
            cell.add(Cells.of(courseTargetWordDto.getTargetName()).create());
            UsualScoreWordDto usualScoreWordDto = courseTargetWordDto.getUsualScoreDto();
            if (Objects.nonNull(usualScoreWordDto) && !CollectionUtils.isEmpty(usualScoreWordDto.getUsualScoreDetailList())) {
                List<ReportEvaluationDto> reportEvaluationDtoList = usualScoreWordDto.getUsualScoreDetailList();
                for (ReportEvaluationDto r : reportEvaluationDtoList) {
                    map.put(r.getEvaluation(), r.getTargetWeight());
                    if (totalMap.containsKey(r.getEvaluation())) {
                        BigDecimal bigDecimal = totalMap.get(r.getEvaluation());
                        bigDecimal = bigDecimal.add(r.getTargetWeight());
                        totalMap.put(r.getEvaluation(), bigDecimal);
                    } else {
                        totalMap.put(r.getEvaluation(), r.getTargetWeight());
                    }
                }
            }
            FinalScoreWordDto finalScoreWordDto = courseTargetWordDto.getFinalScoreDto();
            if (Objects.nonNull(finalScoreWordDto)) {
                map.put(finalScoreWordDto.getEvaluation(), finalScoreWordDto.getFinalScoreSumWeight());
                if (totalMap.containsKey(finalScoreWordDto.getEvaluation())) {
                    BigDecimal bigDecimal = totalMap.get(finalScoreWordDto.getEvaluation());
                    bigDecimal = bigDecimal.add(finalScoreWordDto.getFinalScoreSumWeight());
                    totalMap.put(finalScoreWordDto.getEvaluation(), bigDecimal);
                } else {
                    totalMap.put(finalScoreWordDto.getEvaluation(), finalScoreWordDto.getFinalScoreSumWeight());
                }
            }
            for (String s : list) {
                if (map.containsKey(s)) {
                    cell.add(Cells.of(SystemConstant.dfS.format(map.get(s))).create());
                } else {
                    cell.add(Cells.of("").create());
                }
            }
            cell.add(Cells.of(SystemConstant.dfS.format(courseTargetWordDto.getTargetSumWeight())).create());
            row.setCells(cell);
            row.setRowStyle(this.getRowStyle());
            rowRenderData2[i + 2] = row;
        }
        rowRenderData2[0] = header2_1;
        rowRenderData2[1] = header2_2;

        for (String s : list) {
            if (totalMap.containsKey(s)) {
                totalCell.add(Cells.of(SystemConstant.dfS.format(totalMap.get(s))).create());
            } else {
                totalCell.add(Cells.of("").create());
            }
        }
        totalCell.add(Cells.of(SystemConstant.dfS.format(weight)).create());
        totalRow.setCells(totalCell);
        totalRow.setRowStyle(this.getRowStyle());
        rowRenderData2[rowRenderData2.length - 1] = totalRow;

        // 表格合并,根据坐标
        MergeCellRule mergeCellRule2 = MergeCellRule.builder().map(MergeCellRule.Grid.of(0, 0), MergeCellRule.Grid.of(1, 0)).
                map(MergeCellRule.Grid.of(0, 1), MergeCellRule.Grid.of(0, list.size())).
                map(MergeCellRule.Grid.of(0, totalCell.size() - 1), MergeCellRule.Grid.of(1, totalCell.size() - 1)).
                build();

        Tables.TableBuilder tableBuilder2 = Tables.ofPercentWidth(tbPercentWidth);
        for (int i = 0; i < rowRenderData2.length; i++) {
            tableBuilder2.addRow(rowRenderData2[i]);
        }
        return tableBuilder2.mergeRule(mergeCellRule2).left().create();
    }

    /**
     * 构建word动态表格5
     *
     * @param trBasicInfo
     * @param courseReportBean
     * @return
     */
    @Override
    public TableRenderData buildWordTable5(TRBasicInfo trBasicInfo, CourseReportBean courseReportBean) {
        TableRenderData tableRenderDataExamStudent = null;
        Map<String, Object> map = this.buildExamStudentScoreMap(trBasicInfo);
        if (!CollectionUtils.isEmpty(map)) {
            List<CourseTargetWebDto> courseTargetWebDtoList = (List<CourseTargetWebDto>) map.get("courseTargetWebDtoList");
            tableRenderDataExamStudent = (TableRenderData) map.get("tableRenderDataExamStudent");
            List<BigDecimal> scoreList = (List<BigDecimal>) map.get("scoreList");
            LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = (LinkedMultiValueMap<Long, Map<String, String>>) map.get("finalScatterYMap");
            Map<String, Object> mapScore = this.buildScoreOtherMap(scoreList);
            map.putAll(mapScore);
            courseReportBean.setCourseTargetTable6(this.buildWordTable6(map));
            courseReportBean.setCourseTargetColumnDiagram1(this.buildCourseTargetColumnDiagram1(map));
            courseReportBean.setCourseTargetColumnDiagram2(this.buildCourseTargetColumnDiagram2(map));
            this.buildCourseTargetColumnDiagram3(finalScatterYMap, courseReportBean.getCourseBasicBean().getCourseSuggestList(), courseTargetWebDtoList, scoreList.size());
        }
        return tableRenderDataExamStudent;
    }

    /**
     * 构建word动态表格6
     *
     * @param map
     * @return
     */
    @Override
    public TableRenderData buildWordTable6(Map<String, Object> map) {
        Integer[] scoreSizes = (Integer[]) map.get("scoreSizes");
        BigDecimal[] scorePercent = (BigDecimal[]) map.get("scorePercent");
        Double maxScore = new BigDecimal((Double) map.get("maxScore")).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        Double minScore = new BigDecimal((Double) map.get("minScore")).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        Double avgScore = new BigDecimal((Double) map.get("avgScore")).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();

        RowRenderData[] rowRenderData = new RowRenderData[3];
        List<CellRenderData> cells_1 = new ArrayList<>(), cells_2 = new ArrayList<>(), cells_3 = new ArrayList<>();
        cells_1.add(Cells.of("达成度区间").center().create());
        cells_2.add(Cells.of("人数").create());
        cells_3.add(Cells.of("百分比").create());
        for (int i = 0; i < scoreTitles.length; i++) {
            cells_1.add(Cells.of(scoreTitles[i]).center().create());
            cells_2.add(Cells.of(scoreSizes[i] + "").create());
            cells_3.add(Cells.of(SystemConstant.df.format(scorePercent[i]) + "%").create());
        }
        cells_1.add(Cells.of("平均分").center().create());
        cells_1.add(Cells.of("最高分").center().create());
        cells_1.add(Cells.of("最低分").center().create());

        cells_2.add(Cells.of(SystemConstant.df.format(avgScore)).create());
        cells_2.add(Cells.of(SystemConstant.df.format(maxScore)).create());
        cells_2.add(Cells.of(SystemConstant.df.format(minScore)).create());

        cells_3.add(Cells.of(SystemConstant.df.format(avgScore)).create());
        cells_3.add(Cells.of(SystemConstant.df.format(maxScore)).create());
        cells_3.add(Cells.of(SystemConstant.df.format(minScore)).create());

        RowRenderData row_1 = new RowRenderData();
        row_1.setCells(cells_1);
        row_1.setRowStyle(this.getHeadRowStyle());

        RowRenderData row_2 = new RowRenderData();
        row_2.setCells(cells_2);
        row_2.setRowStyle(this.getRowStyle());

        RowRenderData row_3 = new RowRenderData();
        row_3.setCells(cells_3);
        row_3.setRowStyle(this.getRowStyle());

        rowRenderData[0] = row_1;
        rowRenderData[1] = row_2;
        rowRenderData[2] = row_3;

        MergeCellRule mergeCellRule6 = MergeCellRule.builder().
                map(MergeCellRule.Grid.of(1, 6), MergeCellRule.Grid.of(2, 6)).
                map(MergeCellRule.Grid.of(1, 7), MergeCellRule.Grid.of(2, 7)).
                map(MergeCellRule.Grid.of(1, 8), MergeCellRule.Grid.of(2, 8)).
                build();

        Tables.TableBuilder tableBuilder6 = Tables.ofPercentWidth(tbPercentWidth);
        for (int i = 0; i < rowRenderData.length; i++) {
            tableBuilder6.addRow(rowRenderData[i]);
        }
        return tableBuilder6.mergeRule(mergeCellRule6).left().create();
    }

    /**
     * 构建word图形表格1
     *
     * @param map
     * @return
     */
    @Override
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram1(Map<String, Object> map) {
        Integer[] scoreSizes = (Integer[]) map.get("scoreSizes");
        ChartMultiSeriesRenderData chart = Charts
                .ofMultiSeries("达成度分布柱状图", scoreTitles)
                .addSeries("人数", scoreSizes)
                .create();
        return chart;
    }

    /**
     * 构建word图形表格2
     *
     * @param map
     * @return
     */
    @Override
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram2(Map<String, Object> map) {
        List<CourseTargetWebDto> courseTargetWebDtoList = (List<CourseTargetWebDto>) map.get("courseTargetWebDtoList");
        List<String> courseDegreeList = courseTargetWebDtoList.stream().map(s -> s.getTargetName()).collect(Collectors.toList());
        List<BigDecimal> courseDegreeValueList = courseTargetWebDtoList.stream().map(s -> s.getEvaluationValue()).collect(Collectors.toList());
        String[] courseDegreeTitle = new String[courseDegreeList.size()];
        BigDecimal[] courseDegreeValues = new BigDecimal[courseDegreeValueList.size()];
        courseDegreeList.toArray(courseDegreeTitle);
        courseDegreeValueList.toArray(courseDegreeValues);
        ChartMultiSeriesRenderData chart = Charts
                .ofMultiSeries("课程目标达成度情况", courseDegreeTitle)
                .addSeries("达成度", courseDegreeValues)
                .create();
        return chart;
    }

    /**
     * 构建word图形表格3（散点图）
     *
     * @param finalScatterYMap
     * @param courseSuggestDtoList
     * @param courseTargetWebDtoList
     * @param examStudentSize
     */
    @Override
    public void buildCourseTargetColumnDiagram3(LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap,
                                                List<CourseSuggestDto> courseSuggestDtoList,
                                                List<CourseTargetWebDto> courseTargetWebDtoList,
                                                int examStudentSize) {
        Map<Long, CourseTargetWebDto> courseTargetWebDtoMap = courseTargetWebDtoList.stream().collect(Collectors.toMap(CourseTargetWebDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
        for (CourseSuggestDto courseSuggestDto : courseSuggestDtoList) {
            CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseSuggestDto.getTargetId());
            List<Map<String, String>> list = finalScatterYMap.get(courseSuggestDto.getTargetId());
            if (CollectionUtils.isEmpty(list)) {
                list = new ArrayList<>();
                for (int i = 0; i < examStudentSize; i++) {
                    list.add(ImmutableMap.of((i + 1) + "", "0"));
                }
            }
            if (Objects.nonNull(courseTargetWebDto)) {
                courseSuggestDto.setCourseTargetColumnDiagram3(this.buildScatterMap(list, courseTargetWebDto));
            }
        }
    }

    /**
     * 删除报表数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        return trBasicInfoService.remove(this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId));
    }

    /**
     * 删除报表数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param userId
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId, Long userId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TRBasicInfo> queryWrapper = this.commonQueryWrapper(cultureProgramId, courseId, paperNumber, examId);
        queryWrapper.lambda().eq(TRBasicInfo::getCreateId, sysUser.getId());
        return trBasicInfoService.remove(queryWrapper);
    }

    /**
     * 共用查询
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    protected QueryWrapper<TRBasicInfo> commonQueryWrapper(Long cultureProgramId, Long courseId, String paperNumber, Long examId) {
        QueryWrapper<TRBasicInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TRBasicInfo::getCultureProgramId, cultureProgramId)
                .eq(TRBasicInfo::getCourseId, courseId);
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().and(w -> w.eq(TRBasicInfo::getPaperNumber, paperNumber).or().isNull(TRBasicInfo::getPaperNumber));
        }
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().eq(TRBasicInfo::getExamId, examId);
        }
        return queryWrapper;
    }

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @param vaildWeight
     * @return
     */
    @Override
    public ObeCourseWeightResult findCourseWeightResultRmi(Long obeCourseOutlineId, boolean vaildWeight) {
        ObeCourseWeightResult obeCourseWeightResult = obeCourseWeightService.findCourseWeight(obeCourseOutlineId);
        Objects.requireNonNull(obeCourseWeightResult, "未设置课程目标，报告生成失败，请在课程大纲管理菜单设置课程目标");
//        Objects.requireNonNull(obeCourseWeightResult.getWeightSettingSign(), "课程权重变化id为空");
//        Objects.requireNonNull(obeCourseWeightResult.getDimensionSign(), "知识点变化id为空");
        List<CourseWeightDto> courseWeightDtoList = obeCourseWeightResult.getSubmitForm();
        if (CollectionUtils.isEmpty(courseWeightDtoList)) {
            throw ExceptionResultEnum.ERROR.exception("未设置课程目标，报告生成失败，请在课程大纲管理菜单设置课程目标");
        }
        if (vaildWeight) {
            for (CourseWeightDto c : obeCourseWeightResult.getSubmitForm()) {
                Objects.requireNonNull(c.getTotalWeight(), "课程目标权重未设置，报告生成失败，请在课程考核设置菜单设置课程的考核方式及权重");
            }
        }

        ObeCourseOutline obeCourseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
        TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(obeCourseOutline.getCultureProgramId(), obeCourseOutline.getCourseId(), null, null);
        if (Objects.nonNull(tcPaperStruct) && Objects.nonNull(tcPaperStruct.getPaperStructDimension())) {
            List<PaperStructDimensionResult> paperStructDimensionResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStructDimension(), new TypeToken<List<PaperStructDimensionResult>>() {
            }.getType());
            for (CourseWeightDto c : obeCourseWeightResult.getSubmitForm()) {
                Double score = paperStructDimensionResultList.stream().filter(s -> Objects.equals(s.getCourseTargetName(), c.getCourseTargetName()))
                        .mapToDouble(s -> s.getScore().doubleValue()).sum();
                Objects.requireNonNull(c.getTotalWeight(), "课程目标权重未设置，报告生成失败，请在课程考核设置菜单设置课程的考核方式及权重");
                List<CourseWeightDetailDto> courseWeightDetailDtoList = c.getEvaluationList();
                for (CourseWeightDetailDto courseWeightDetailDto : courseWeightDetailDtoList) {
                    if (Objects.equals(courseWeightDetailDto.getEvaluationName(), SystemConstant.FINAL_SCORE_STR)) {
                        courseWeightDetailDto.setTargetScore(new BigDecimal(score));
                        break;
                    }
                }
            }
        }
        return obeCourseWeightResult;
    }

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @return
     */
    @Override
    public List<CourseWeightDto> findCourseWeightDtoRmi(Long obeCourseOutlineId) {
        ObeCourseWeightResult obeCourseWeightResult = obeCourseWeightService.findCourseWeight(obeCourseOutlineId);
        if (Objects.nonNull(obeCourseWeightResult) && !CollectionUtils.isEmpty(obeCourseWeightResult.getSubmitForm())) {
            return obeCourseWeightResult.getSubmitForm();
        }
        return null;
    }

    /**
     * 清除报表
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @param clearDimension
     */
    @Override
    @Transactional
    public void clearReportData(Long cultureProgramId, Long courseId, String paperNumber, Long examId, boolean clearDimension) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        trExamStudentService.remove(cultureProgramId, courseId, paperNumber, examId);
        trBasicInfoService.remove(cultureProgramId, courseId, paperNumber, examId);
        if (clearDimension) {
            TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber, examId);
            if (Objects.nonNull(tcPaperStruct)) {
//                tcPaperStruct.setPaperStructDimension(null);
                tcPaperStruct.updateInfo(sysUser.getId());
                tcPaperStructService.updateById(tcPaperStruct);
            }
        }
    }

    /**
     * 报表基础信息
     *
     * @param trBasicInfo
     * @param obeCourseOutline
     * @param markPaper
     * @param obeCourseWeightResult
     * @return
     */
    @Override
    public ReportCourseBasicInfoDto setBasicInfo(TRBasicInfo trBasicInfo, ObeCourseOutline obeCourseOutline,
                                                 MarkPaper markPaper, ObeCourseWeightResult obeCourseWeightResult) {
        String college = null;
        if (Objects.isNull(trBasicInfo) || Objects.isNull(trBasicInfo.getCollege())) {
            BasicCourse basicCourse = basicCourseService.getById(obeCourseOutline.getCourseId());
            Objects.requireNonNull(basicCourse, "未找到课程信息");
            if (Objects.nonNull(basicCourse.getTeachingRoomId())) {
                SysOrg sysOrg = sysOrgService.getById(basicCourse.getTeachingRoomId());
                if (Objects.nonNull(sysOrg)) {
                    college = sysOrg.getName();
                }
            }
        }

        String teachingObject = Objects.nonNull(trBasicInfo) ? trBasicInfo.getTeachingObject() : null;
        if (Objects.nonNull(markPaper.getPaperNumber())) {
            ExamStudentDto examStudentDto = basicExamStudentService.queryExamStudent(markPaper.getExamId(), Objects.nonNull(trBasicInfo) ? trBasicInfo.getCourseId() : obeCourseOutline.getCourseId(), markPaper.getPaperNumber());
            teachingObject = Objects.nonNull(examStudentDto) ? examStudentDto.getClazzNames() : null;
        }

        ObeCultureProgram obeCultureProgram = obeCultureProgramService.getById(obeCourseOutline.getCultureProgramId());
        Objects.requireNonNull(obeCultureProgram, "未找到培养方案信息");
        BasicProfessional basicProfessional = basicProfessionalService.getById(obeCultureProgram.getProfessionalId());
        Objects.requireNonNull(basicProfessional, "未找到专业信息");
        Double expectValue = obeCourseWeightResult.getSubmitForm().get(0).getExpectValue();
        return new ReportCourseBasicInfoDto(college, teachingObject, Objects.nonNull(expectValue) ? new BigDecimal(expectValue) : null, basicProfessional.getName());
    }

    /**
     * 获取行样式
     *
     * @return
     */
    protected RowStyle getRowStyle() {
        RowStyle rowStyle = new RowStyle();
        ParagraphStyle paragraphStyle = new ParagraphStyle();
        paragraphStyle.setAlign(ParagraphAlignment.LEFT);
        CellStyle cellStyle = new CellStyle();
        cellStyle.setDefaultParagraphStyle(paragraphStyle);
        cellStyle.setVertAlign(XWPFTableCell.XWPFVertAlign.CENTER);
        rowStyle.setDefaultCellStyle(cellStyle);
        return rowStyle;
    }

    /**
     * 获取行样式
     *
     * @return
     */
    protected RowStyle getHeadRowStyle() {
        RowStyle rowStyle = new RowStyle();
        rowStyle.setRepeated(true);
        ParagraphStyle paragraphStyle = new ParagraphStyle();
        paragraphStyle.setAlign(ParagraphAlignment.CENTER);
        CellStyle cellStyle = new CellStyle();
        cellStyle.setDefaultParagraphStyle(paragraphStyle);
        cellStyle.setBackgroundColor(tbBgColor);
        rowStyle.setDefaultCellStyle(cellStyle);
        return rowStyle;
    }

    /**
     * 计算分数范围
     *
     * @param finalScoreDto
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param markPaper
     * @param teachClassName
     * @param userId
     */
    protected void fillScoreRange(FinalScoreDto finalScoreDto, Long examId, String paperNumber, Long cultureProgramId, Long courseId, MarkPaper markPaper, String teachClassName, Long userId) {
        //分数区间
        Double passScore = markPaper.getPassScore();//及格分
        Objects.requireNonNull(passScore, "及格分为空");

        Double totalScore = markPaper.getTotalScore();
        Objects.requireNonNull(totalScore, "试卷满分为空");

        List<ScoreRangeVo> scoreRange = new ArrayList<>();
        finalScoreDto.setScoreRange(scoreRange);

        //开始、结束分数区间
        Double startInterval = 0.5d, endInterval = 9.5d, increaseStartScore = passScore - startInterval,
                increaseEndScore = increaseStartScore + startInterval + endInterval;
        scoreRange.add(getScoreRangeVo(finalScoreDto.getStudentCount(), examId, paperNumber, cultureProgramId, courseId, 0.0d, increaseStartScore, teachClassName, userId));
        increaseStartScore = increaseStartScore + startInterval;
        increaseEndScore = increaseStartScore + endInterval;
        if (totalScore.doubleValue() - increaseEndScore.doubleValue() == startInterval || increaseEndScore.doubleValue() >= totalScore.doubleValue()) {
            increaseEndScore = totalScore;
            scoreRange.add(getScoreRangeVo(finalScoreDto.getStudentCount(), examId, paperNumber, cultureProgramId, courseId, increaseStartScore, increaseEndScore, teachClassName, userId));
        } else {
            while (increaseEndScore.doubleValue() < totalScore.doubleValue()) {
                scoreRange.add(getScoreRangeVo(finalScoreDto.getStudentCount(), examId, paperNumber, cultureProgramId, courseId, increaseStartScore, increaseEndScore, teachClassName, userId));
                increaseStartScore = increaseEndScore + startInterval;
                increaseEndScore = increaseStartScore + endInterval;
                if (totalScore.doubleValue() - increaseEndScore.doubleValue() == startInterval || increaseEndScore.doubleValue() >= totalScore.doubleValue()) {
                    increaseEndScore = totalScore;
                    scoreRange.add(getScoreRangeVo(finalScoreDto.getStudentCount(), examId, paperNumber, cultureProgramId, courseId, increaseStartScore, increaseEndScore, teachClassName, userId));
                }
            }
        }
    }

//    /**
//     * 计算考生期末成绩知识点纬度
//     *
//     * @param finalScoreExamStudentDimensionMap
//     * @param finalScoreResult
//     * @param courseTargetDto
//     * @param dimensionDto
//     * @param score
//     */
//    protected void calculateFinalScoreByDimensionExamStudent(Map<String, Map<Long, Map<Long, DimensionDto>>> finalScoreExamStudentDimensionMap,
//                                                             FinalScoreResult finalScoreResult,
//                                                             CourseTargetWebDto courseTargetDto,
//                                                             DimensionDto dimensionDto,
//                                                             Double score) {
//        if (!finalScoreExamStudentDimensionMap.containsKey(finalScoreResult.getStudentCode())) {
//            Map<Long, Map<Long, DimensionDto>> map = new LinkedHashMap<>();
//            Map<Long, DimensionDto> dimensionMap = new LinkedHashMap<>();
//            dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, score));
//            map.put(courseTargetDto.getTargetId(), dimensionMap);
//            finalScoreExamStudentDimensionMap.put(finalScoreResult.getStudentCode(), map);
//        } else {
//            Map<Long, Map<Long, DimensionDto>> map = finalScoreExamStudentDimensionMap.get(finalScoreResult.getStudentCode());
//            if (!map.containsKey(courseTargetDto.getTargetId())) {
//                Map<Long, DimensionDto> dimensionMap = new LinkedHashMap<>();
//                dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, score));
//                map.put(courseTargetDto.getTargetId(), dimensionMap);
//            } else {
//                Map<Long, DimensionDto> dimensionMap = map.get(courseTargetDto.getTargetId());
//                if (!dimensionMap.containsKey(dimensionDto.getDimensionId())) {
//                    dimensionMap.put(dimensionDto.getDimensionId(), new DimensionDto(courseTargetDto.getTargetId(), dimensionDto, score));
//                } else {
//                    DimensionDto dimensionDtoTemp = dimensionMap.get(dimensionDto.getDimensionId());
//                    dimensionDtoTemp.setDimensionScore(dimensionDtoTemp.getDimensionScore().doubleValue() + score.doubleValue());
//                }
//                map.put(courseTargetDto.getTargetId(), dimensionMap);
//            }
//            finalScoreExamStudentDimensionMap.put(finalScoreResult.getStudentCode(), map);
//        }
//    }

    /**
     * 计算考生期末成绩课程目标卷面分纬度
     *
     * @param finalScoreExamStudentTargetMap
     * @param finalScoreResult
     * @param courseTargetDto
     * @param score
     */
    protected void calculateFinalScoreByTargetExamStudent(Map<String, Map<Long, BigDecimal>> finalScoreExamStudentTargetMap,
                                                          FinalScoreResult finalScoreResult,
                                                          CourseTargetWebDto courseTargetDto,
                                                          BigDecimal score) {
        if (!finalScoreExamStudentTargetMap.containsKey(finalScoreResult.getStudentCode())) {
            Map<Long, BigDecimal> map = new LinkedHashMap<>();
            map.put(courseTargetDto.getTargetId(), score);
            finalScoreExamStudentTargetMap.put(finalScoreResult.getStudentCode(), map);
        } else {
            Map<Long, BigDecimal> map = finalScoreExamStudentTargetMap.get(finalScoreResult.getStudentCode());
            if (!map.containsKey(courseTargetDto.getTargetId())) {
                map.put(courseTargetDto.getTargetId(), score);
            } else {
                BigDecimal scoreTemp = map.get(courseTargetDto.getTargetId());
                map.put(courseTargetDto.getTargetId(), scoreTemp.add(score));
            }
            finalScoreExamStudentTargetMap.put(finalScoreResult.getStudentCode(), map);
        }
    }

    /**
     * 计算考生平时成绩
     *
     * @param finalScoreResult
     * @param usualScoreMap
     * @param targetWordMap
     * @return
     */
    protected ReportExamStudentUsualScoreDto calculateUsualScoreByExamStudent(FinalScoreResult finalScoreResult,
                                                                              Map<String, List<ReportEvaluationDto>> usualScoreMap,
                                                                              Map<Long, CourseTargetWordDto> targetWordMap) {
        List<TCUsualScoreDto> tcUsualScoreDtoList = GsonUtil.fromJson(finalScoreResult.getUsualScore(), new TypeToken<List<TCUsualScoreDto>>() {
        }.getType());
        Map<Long, List<ReportExamStudentUsualScoreObjDto>> scoreListMap = new LinkedHashMap<>();
        ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = new ReportExamStudentUsualScoreDto(scoreListMap);
        for (TCUsualScoreDto t : tcUsualScoreDtoList) {
            if (usualScoreMap.containsKey(t.getName())) {
                List<ReportEvaluationDto> reportEvaluationDtoList = usualScoreMap.get(t.getName());
                if (!CollectionUtils.isEmpty(reportEvaluationDtoList)) {
                    if (reportEvaluationDtoList.size() == 1) {
                        ReportEvaluationDto reportEvaluationDto = reportEvaluationDtoList.get(0);
                        List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = !scoreListMap.containsKey(reportEvaluationDto.getTargetId()) ? new ArrayList<>() : scoreListMap.get(reportEvaluationDto.getTargetId());
                        //计算平时作业转换
                        BigDecimal score = Objects.nonNull(t.getScore()) ? new BigDecimal(t.getScore()) : new BigDecimal(0);
                        CourseTargetWordDto courseTargetWordDto = targetWordMap.get(reportEvaluationDto.getTargetId());
                        UsualScoreWordDto usualScoreWordDto = courseTargetWordDto.getUsualScoreDto();
                        //卷面分/目标卷面总分*权重
                        BigDecimal matrixScore = score.divide(reportEvaluationDto.getTargetScore(), 4, BigDecimal.ROUND_HALF_UP).multiply(reportEvaluationDto.getTargetWeight()).setScale(2, BigDecimal.ROUND_HALF_UP);
                        reportExamStudentUsualScoreObjDtoList.add(new ReportExamStudentUsualScoreObjDto(reportEvaluationDto.getTargetId(), reportEvaluationDto, score, matrixScore, usualScoreWordDto.getUsualScoreTargetSumScore()));
                        scoreListMap.put(reportEvaluationDto.getTargetId(), reportExamStudentUsualScoreObjDtoList);
                    } else {
                        BigDecimal score = Objects.nonNull(t.getScore()) ? new BigDecimal(t.getScore()) : new BigDecimal(0);
                        Double targetWeightSum = reportEvaluationDtoList.stream().mapToDouble(s -> s.getTargetWeight().doubleValue()).sum();
                        for (ReportEvaluationDto reportEvaluationDto : reportEvaluationDtoList) {
                            List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = !scoreListMap.containsKey(reportEvaluationDto.getTargetId()) ? new ArrayList<>() : scoreListMap.get(reportEvaluationDto.getTargetId());
                            CourseTargetWordDto courseTargetWordDto = targetWordMap.get(reportEvaluationDto.getTargetId());
                            UsualScoreWordDto usualScoreWordDto = courseTargetWordDto.getUsualScoreDto();
                            BigDecimal matrixPageScore = new BigDecimal(0), matrixTargetScore = new BigDecimal(0), matrixScore = new BigDecimal(0);
                            if (targetWeightSum.doubleValue() > 0) {
                                matrixPageScore = score.multiply(reportEvaluationDto.getTargetWeight()).divide(new BigDecimal(targetWeightSum), 4, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
                                matrixTargetScore = SystemConstant.PERCENT.multiply(reportEvaluationDto.getTargetWeight()).divide(new BigDecimal(targetWeightSum), 4, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
                                matrixScore = matrixPageScore.divide(matrixTargetScore, 4, BigDecimal.ROUND_HALF_UP).multiply(reportEvaluationDto.getTargetWeight()).setScale(2, BigDecimal.ROUND_HALF_UP);
                            }
                            reportExamStudentUsualScoreObjDtoList.add(new ReportExamStudentUsualScoreObjDto(reportEvaluationDto.getTargetId(), reportEvaluationDto, score, matrixScore, usualScoreWordDto.getUsualScoreTargetSumScore(), matrixPageScore, matrixTargetScore));
                            scoreListMap.put(reportEvaluationDto.getTargetId(), reportExamStudentUsualScoreObjDtoList);
                        }
                    }
                }
            }
        }
        scoreListMap.forEach((k, v) -> {
            Collections.sort(v);
        });
        return reportExamStudentUsualScoreDto;
    }

    /**
     * 计算分数比率
     *
     * @param toltal
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param start
     * @param end
     * @param teachClassName
     * @param userId
     * @return
     */
    protected ScoreRangeVo getScoreRangeVo(int toltal, Long examId, String paperNumber, Long cultureProgramId, Long courseId, Double start, Double end, String teachClassName, Long userId) {
        int count = tcFinalScoreService.getCountByScoreRange(examId, paperNumber, cultureProgramId, courseId, start, end, teachClassName, Objects.nonNull(userId) && userId.longValue() != -1 ? userId : null);
        Double rate = null;
        if (toltal != 0) {
            rate = new BigDecimal(count).divide(new BigDecimal(toltal), 4, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        }
        return new ScoreRangeVo(count, start, end, rate);
    }

    /**
     * 获取散点图值
     *
     * @param scatterYMap
     * @param studentCode
     * @param reportExamStudentTargetDto
     * @return
     */
    @Override
    public Map<Long, ConcurrentMap<String, String>> getScatterValue(Map<Long, ConcurrentMap<String, String>> scatterYMap, String studentCode, ReportExamStudentTargetDto reportExamStudentTargetDto) {
        ConcurrentMap<String, String> map = null;
        if (!scatterYMap.containsKey(reportExamStudentTargetDto.getTargetId())) {
            map = new ConcurrentHashMap<>();
            map.put(studentCode, reportExamStudentTargetDto.getMatrixDegree().toString());
        } else {
            map = scatterYMap.get(reportExamStudentTargetDto.getTargetId());
            if (!map.containsKey(reportExamStudentTargetDto.getMatrixDegree().toString())) {
                map.put(studentCode, reportExamStudentTargetDto.getMatrixDegree().toString());
            }
        }
        scatterYMap.put(reportExamStudentTargetDto.getTargetId(), map);
        return scatterYMap;
    }

    /**
     * 获取最终散点图值
     *
     * @param scatterYMap
     * @return
     */
    @Override
    public LinkedMultiValueMap<Long, Map<String, String>> getFinalScatterValue(Map<Long, ConcurrentMap<String, String>> scatterYMap) {
        LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = new LinkedMultiValueMap<>();
        for (Map.Entry<Long, ConcurrentMap<String, String>> entry : scatterYMap.entrySet()) {
            ConcurrentMap<String, String> concurrentMap = entry.getValue();
            for (Map.Entry<String, String> entry1 : concurrentMap.entrySet()) {
                Map<String, String> mapNew = new HashMap<>();
                mapNew.put(entry1.getKey(), entry1.getValue().toString());
                finalScatterYMap.add(entry.getKey(), mapNew);
            }
        }
        return finalScatterYMap;
    }

    /**
     * 构建公共课程目标达成度模板
     *
     * @param trBasicInfo
     * @return
     */
    @Override
    public CourseReportBean buildCommonCourseDegreeReport(TRBasicInfo trBasicInfo) {
        //评价样本的基本信息
        JSONObject jsonObject = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResultDetail());
        List<CourseTargetWordDto> courseTargetWordDtoList = JSONArray.parseArray(jsonObject.get("targetWordMap").toString(), CourseTargetWordDto.class);

        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class);

        StringJoiner courseTarget = new StringJoiner("");
        courseTargetWordDtoList.stream().peek(s -> {
            courseTarget.add("      ").add(s.getTargetName()).add("——").add(s.getGraduationRequirementPoint()).add("\r\n");
        }).collect(Collectors.toList());

        CourseBasicBean courseBasicBean = new CourseBasicBean(trBasicInfo, courseTarget.toString(), reportCourseEvaluationResultDto.getTargetList().size());

        CourseReportBean courseReportBean = new CourseReportBean(trBasicInfo.getOpenTime() + "《" + trBasicInfo.getCourseName() + "》", courseBasicBean);
        //table1-课程目标目标与毕业要求指标点的对应关系
        courseReportBean.setCourseTargetTable1(trBasicInfoService.buildWordTable1(courseTargetWordDtoList));

        //table2-课程目标达成评价依据
        courseReportBean.setCourseTargetTable2(trBasicInfoService.buildWordTable2(courseTargetWordDtoList));

        //examstudent-课程目标达成评价依据-考生
        courseReportBean.setExamStudentTable1(trBasicInfoService.buildWordTable5(trBasicInfo, courseReportBean));
        return courseReportBean;
    }

    /**
     * 构建西安工业大学课程目标达成度模板
     *
     * @param trBasicInfo
     * @return
     */
    @Override
    public XagyuCourseReportBean buildXagyuCourseDegreeReport(TRBasicInfo trBasicInfo) {
        //评价样本的基本信息
        JSONObject jsonObject = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResultDetail());
        List<CourseTargetWordDto> courseTargetWordDtoList = JSONArray.parseArray(jsonObject.get("targetWordMap").toString(), CourseTargetWordDto.class);
        XagyuCourseBasicBean xagyucourseBasicBean = new XagyuCourseBasicBean(trBasicInfo);
        XagyuCourseReportBean xagyuCourseReportBean = new XagyuCourseReportBean(trBasicInfo.getOpenTime() + "《" + trBasicInfo.getCourseName() + "》", xagyucourseBasicBean);
        //table1-课程目标目标与毕业要求指标点的对应关系
        xagyuCourseReportBean.setXagyuCourseTargetTable1(trBasicInfoService.buildWordTable1(courseTargetWordDtoList));
        //examstudent-课程目标达成评价依据-考生
        xagyuCourseReportBean.setXagyuExamStudentTable1(trBasicInfoService.buildXagyuWordTable1(trBasicInfo, xagyuCourseReportBean));
        return xagyuCourseReportBean;
    }

    /**
     * 构建西安工业大学课程目标达成度-各课程目标考核样本数据详情
     *
     * @param trBasicInfo
     * @param xagyuCourseReportBean
     * @return
     */
    @Override
    public TableRenderData buildXagyuWordTable1(TRBasicInfo trBasicInfo, XagyuCourseReportBean xagyuCourseReportBean) {
        TableRenderData tableRenderDataExamStudent = null;
        Map<String, Object> map = this.buildExamStudentScoreMap(trBasicInfo);
        if (!CollectionUtils.isEmpty(map)) {
            List<CourseTargetWebDto> courseTargetWebDtoList = (List<CourseTargetWebDto>) map.get("courseTargetWebDtoList");
            tableRenderDataExamStudent = (TableRenderData) map.get("tableRenderDataExamStudent");
            List<BigDecimal> scoreList = (List<BigDecimal>) map.get("scoreList");
            LinkedMultiValueMap<String, BigDecimal> scoreListMap = (LinkedMultiValueMap<String, BigDecimal>) map.get("scoreListMap");
            LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = (LinkedMultiValueMap<Long, Map<String, String>>) map.get("finalScatterYMap");
            Map<String, Object> mapScore = this.buildXagyuScoreOtherMap(scoreListMap);
            xagyuCourseReportBean.setXagyuCourseTargetTable2(this.buildXagyuWordTable2(mapScore));
            xagyuCourseReportBean.setXagyuCourseTargetColumnDiagram1(this.buildCourseTargetColumnDiagram2(map));
            this.buildXagyuCourseTargetColumnDiagram1(finalScatterYMap, xagyuCourseReportBean.getXagyuCourseBasicBean().getCourseSuggestList(), courseTargetWebDtoList, scoreList.size());
        }
        return tableRenderDataExamStudent;
    }

    /**
     * 构建西安工业大学课程目标达成度（散点图）
     *
     * @param finalScatterYMap
     * @param courseSuggestDtoList
     * @param courseTargetWebDtoList
     * @param examStudentSize
     */
    @Override
    public void buildXagyuCourseTargetColumnDiagram1(LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap, List<XagyuCourseSuggestDto> courseSuggestDtoList, List<CourseTargetWebDto> courseTargetWebDtoList, int examStudentSize) {
        Map<Long, CourseTargetWebDto> courseTargetWebDtoMap = courseTargetWebDtoList.stream().collect(Collectors.toMap(CourseTargetWebDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
        for (XagyuCourseSuggestDto courseSuggestDto : courseSuggestDtoList) {
            CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseSuggestDto.getTargetId());
            List<Map<String, String>> list = finalScatterYMap.get(courseSuggestDto.getTargetId());
            if (CollectionUtils.isEmpty(list)) {
                list = new ArrayList<>();
                for (int i = 0; i < examStudentSize; i++) {
                    list.add(ImmutableMap.of((i + 1) + "", "0"));
                }
            }
            if (Objects.nonNull(courseTargetWebDto)) {
                courseSuggestDto.setXagyuCourseTargetColumnDiagram2(this.buildScatterMap(list, courseTargetWebDto));
            }
        }
    }

    /**
     * 构建考生成绩
     *
     * @param trBasicInfo
     * @return
     */
    @Override
    public Map<String, Object> buildExamStudentScoreMap(TRBasicInfo trBasicInfo) {
        Map<String, Object> map = new HashMap<>();
        TableRenderData tableRenderDataExamStudent = null;
        List<TRExamStudent> trExamStudentList = trExamStudentService.list(new QueryWrapper<TRExamStudent>().lambda()
                .eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()).and(w -> w.ne(TRExamStudent::getStudentCode, "目标分")));
        if (!CollectionUtils.isEmpty(trExamStudentList)) {
            List<CellRenderData> examStudent_cells_1 = new ArrayList<>(), examStudent_cells_2 = new ArrayList<>();
            examStudent_cells_1.add(Cells.of("序号").center().create());
            examStudent_cells_1.add(Cells.of("学号").center().create());
            examStudent_cells_1.add(Cells.of("姓名").center().create());
            examStudent_cells_1.add(Cells.of("行政班级").center().create());
            examStudent_cells_2.addAll(examStudent_cells_1);

            Map<String, Integer> targetUsualScoreSizeMap = new LinkedHashMap<>();
            Map<Long, BigDecimal> targetCourseDegreeMap = new LinkedHashMap<>();
            TRExamStudent trExamStudentTemp = trExamStudentList.get(0);

            RowRenderData[] rowRenderDataExamStudent = new RowRenderData[trExamStudentList.size() + 3];
            RowRenderData examStudent_header_1 = new RowRenderData();
            examStudent_header_1.setCells(examStudent_cells_1);
            examStudent_header_1.setRowStyle(this.getHeadRowStyle());

            RowRenderData examStudent_header_2 = new RowRenderData();
            examStudent_header_2.setCells(examStudent_cells_2);
            examStudent_header_2.setRowStyle(this.getHeadRowStyle());

            rowRenderDataExamStudent[0] = examStudent_header_1;
            rowRenderDataExamStudent[1] = examStudent_header_2;

            ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class);
            List<CourseTargetWebDto> courseTargetWebDtoList = reportCourseEvaluationResultDto.getTargetList();
            for (CourseTargetWebDto c : courseTargetWebDtoList) {
                if (!CollectionUtils.isEmpty(c.getEvaluationList())) {
                    targetUsualScoreSizeMap.put(c.getTargetName(), c.getEvaluationList().size() - 1);
                } else {
                    targetUsualScoreSizeMap.put(c.getTargetName(), 0);
                }
                targetCourseDegreeMap.put(c.getTargetId(), c.getEvaluationValue());
            }

            Double courseDegree = 0.0d;
            if (!CollectionUtils.isEmpty(targetCourseDegreeMap)) {
                courseDegree = targetCourseDegreeMap.values().stream().filter(s -> Objects.nonNull(s)).collect(Collectors.toList()).stream().mapToDouble(b -> b.doubleValue()).min().orElse(0);
            }
            //首行
            if (Objects.nonNull(trExamStudentTemp.getResultDetail())) {
                List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSONArray.parseArray(trExamStudentTemp.getResultDetail(), ReportExamStudentTargetDto.class);
                for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                    int finalScoreSize = Objects.nonNull(reportExamStudentTargetDto.getFinalScore()) ? 1 : 0;
                    int size = finalScoreSize + (Objects.nonNull(reportExamStudentTargetDto.getUsualScore()) && !CollectionUtils.isEmpty(reportExamStudentTargetDto.getUsualScore().getScoreList()) ? reportExamStudentTargetDto.getUsualScore().getScoreList().size() : 0);
                    examStudent_cells_1.add(Cells.of(reportExamStudentTargetDto.getTargetName()).center().create());
                    ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = reportExamStudentTargetDto.getUsualScore();
                    //平常作业
                    if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                        List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                        for (ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto : reportExamStudentUsualScoreObjDtoList) {
                            examStudent_cells_1.add(Cells.of("").center().create());
                            examStudent_cells_2.add(Cells.of(reportExamStudentUsualScoreObjDto.getEvaluation()).center().create());
                        }
                    }
                    //期末考试
                    ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = reportExamStudentTargetDto.getFinalScore();
                    if (Objects.nonNull(reportExamStudentFinalScoreDto) && Objects.equals(reportExamStudentFinalScoreDto.getEvaluation(), SystemConstant.FINAL_SCORE_STR)) {
                        examStudent_cells_2.add(Cells.of(SystemConstant.FINAL_SCORE_STR).center().create());
                    } else {
                        int usualScoreSize = reportExamStudentTargetDto.getUsualScore().getScoreList().size();
                        while ((size - usualScoreSize) >= 0) {
                            examStudent_cells_1.remove(examStudent_cells_1.size() - 1);
                            size = size - 1;
                        }
                    }
                }
                examStudent_cells_1.add(Cells.of("综合成绩").center().create());
                examStudent_cells_2.add(Cells.of("综合成绩").center().create());
                examStudent_cells_1.add(Cells.of("达成系数").center().create());
                examStudent_cells_2.add(Cells.of("达成系数").center().create());
            }

            LinkedMultiValueMap<String, BigDecimal> scoreListMap = new LinkedMultiValueMap<>();
            List<BigDecimal> scoreList = new ArrayList<>();
            List<TRExamStudent> sortTrExamStudentList = new ArrayList<>();
            sortTrExamStudentList.addAll(trExamStudentList.subList(trExamStudentList.size() - 2, trExamStudentList.size()));

            Collections.reverse(sortTrExamStudentList);
            trExamStudentList.remove(trExamStudentList.size() - 1);
            trExamStudentList.remove(trExamStudentList.size() - 1);
            trExamStudentList.addAll(sortTrExamStudentList);

            List<TRExamStudent> trExamStudentNewList = new ArrayList<>();
            trExamStudentNewList.addAll(trExamStudentList);
            trExamStudentNewList.remove(trExamStudentNewList.size() - 1);
            trExamStudentNewList.remove(trExamStudentNewList.size() - 1);

            BigDecimal targetAvgScore = new BigDecimal(trExamStudentNewList.stream().mapToDouble(s -> s.getScore().doubleValue()).sum()).divide(new BigDecimal(trExamStudentNewList.size()), 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            Map<Long, ConcurrentMap<String, String>> scatterYMap = new LinkedHashMap<>();
            for (int i = 0; i < trExamStudentList.size(); i++) {
                TRExamStudent trExamStudent = trExamStudentList.get(i);
                RowRenderData examStudent_row = new RowRenderData();
                List<CellRenderData> examStudent_cells = new ArrayList<>();
                switch (trExamStudent.getName()) {
                    case "平均分":
                        examStudent_cells.add(Cells.of("平均分").create());
                        examStudent_cells.add(Cells.of("平均分").create());
                        examStudent_cells.add(Cells.of("平均分").create());
                        break;
                    case "各课程目标平均分":
                        examStudent_cells.add(Cells.of("课程目标达成度").create());
                        examStudent_cells.add(Cells.of("课程目标达成度").create());
                        examStudent_cells.add(Cells.of("课程目标达成度").create());
                        break;
                    default:
                        examStudent_cells.add(Cells.of(String.valueOf(i + 1)).create());
                        examStudent_cells.add(Cells.of(trExamStudent.getStudentCode()).create());
                        examStudent_cells.add(Cells.of(trExamStudent.getName()).create());
                        break;
                }
                examStudent_cells.add(Cells.of(trExamStudent.getAdministrativeClass()).create());

                if (Objects.nonNull(trExamStudent.getResultDetail())) {
                    List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSONArray.parseArray(trExamStudent.getResultDetail(), ReportExamStudentTargetDto.class);
                    for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                        if (Objects.nonNull(reportExamStudentTargetDto)) {
                            BigDecimal matrixDegree = reportExamStudentTargetDto.getMatrixDegree();
                            if (!Objects.equals(trExamStudent.getName(), "平均分")
                                    && !Objects.equals(trExamStudent.getName(), "各课程目标平均分")
                                    && Objects.nonNull(matrixDegree)) {
                                trBasicInfoService.getScatterValue(scatterYMap, trExamStudent.getStudentCode(), reportExamStudentTargetDto);
                            }
                            BigDecimal scoreSum = new BigDecimal(0);
                            ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = reportExamStudentTargetDto.getUsualScore();
                            //平常作业
                            if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                                List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                                for (ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto : reportExamStudentUsualScoreObjDtoList) {
                                    switch (trExamStudent.getName()) {
                                        case "平均分":
                                            examStudent_cells.add(Cells.of(SystemConstant.df.format(reportExamStudentUsualScoreObjDto.getMatrixAvgScore())).create());
                                            break;
                                        case "各课程目标平均分":
                                            examStudent_cells.add(Cells.of(targetCourseDegreeMap.get(reportExamStudentTargetDto.getTargetId()) + "").create());
                                            break;
                                        default:
                                            examStudent_cells.add(Cells.of(SystemConstant.df.format(reportExamStudentUsualScoreObjDto.getScore())).create());
                                            scoreSum = scoreSum.add(reportExamStudentUsualScoreObjDto.getMatrixScore());
                                            break;
                                    }
                                }
                            }
                            //期末考试
                            ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = reportExamStudentTargetDto.getFinalScore();
                            if (Objects.nonNull(reportExamStudentFinalScoreDto) && Objects.equals(reportExamStudentFinalScoreDto.getEvaluation(), SystemConstant.FINAL_SCORE_STR)) {
//                                courseReportBean.getCourseBasicBean().setFinalScoreWeight(reportExamStudentFinalScoreDto.getTargetWeight());
                                switch (trExamStudent.getName()) {
                                    case "平均分":
                                        examStudent_cells.add(Cells.of(SystemConstant.df.format(reportExamStudentFinalScoreDto.getMatrixAvgScore())).create());
                                        break;
                                    case "各课程目标平均分":
                                        examStudent_cells.add(Cells.of(targetCourseDegreeMap.get(reportExamStudentTargetDto.getTargetId()) + "").create());
                                        break;
                                    default:
                                        examStudent_cells.add(Cells.of(SystemConstant.df.format(reportExamStudentFinalScoreDto.getTargetScoreSum())).create());
                                        scoreSum = scoreSum.add(reportExamStudentFinalScoreDto.getTargetMatrixSumScore());
                                        break;
                                }
                            }
                            if (!Objects.equals(trExamStudent.getName(), "平均分")
                                    && !Objects.equals(trExamStudent.getName(), "各课程目标平均分")) {
                                scoreListMap.add(reportExamStudentTargetDto.getTargetName(), scoreSum);
                            }
                        }
                    }
                }
                switch (trExamStudent.getName()) {
                    case "平均分":
                        examStudent_cells.add(Cells.of(SystemConstant.df.format(targetAvgScore.setScale(2, BigDecimal.ROUND_HALF_UP))).create());
                        break;
                    case "各课程目标平均分":
                        examStudent_cells.add(Cells.of("").create());
                        break;
                    default:
                        scoreList.add(trExamStudent.getScore());
                        examStudent_cells.add(Cells.of(SystemConstant.df.format(trExamStudent.getScore())).create());
                        break;
                }
                String key = examStudent_cells.get(examStudent_cells.size() - 1).getParagraphs().get(0).getContents().get(0).toString();
                if (!Objects.equals(key.trim(), "") && !Objects.equals(key.trim(), "null")) {
                    examStudent_cells.add(Cells.of(SystemConstant.df.format(new BigDecimal(key).divide(SystemConstant.PERCENT, 2, BigDecimal.ROUND_HALF_UP))).create());
                } else {
                    examStudent_cells.add(Cells.of("").create());
                }
                examStudent_row.setCells(examStudent_cells);
                examStudent_row.setRowStyle(this.getRowStyle());
                rowRenderDataExamStudent[i + 2] = examStudent_row;
            }
            LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = trBasicInfoService.getFinalScatterValue(scatterYMap);
            List<CellRenderData> cellRenderData_last_cell = new ArrayList<>();
            RowRenderData examStudent_last_row = new RowRenderData();
            cellRenderData_last_cell.add(Cells.of("课程达成度").create());
            cellRenderData_last_cell.add(Cells.of("课程达成度").create());
            cellRenderData_last_cell.add(Cells.of("课程达成度").create());
            cellRenderData_last_cell.add(Cells.of("课程达成度").create());

            int cellSize = rowRenderDataExamStudent[rowRenderDataExamStudent.length - 2].getCells().size();
            for (int i = 4; i < cellSize; i++) {
                cellRenderData_last_cell.add(Cells.of(courseDegree + "").center().create());
            }
            examStudent_last_row.setCells(cellRenderData_last_cell);
            examStudent_last_row.setRowStyle(this.getRowStyle());
            rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1] = examStudent_last_row;

            //表格合并,根据坐标
            MergeCellRule mergeCellRuleExamStudent = MergeCellRule.builder().build();
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 0), MergeCellRule.Grid.of(1, 0)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 1), MergeCellRule.Grid.of(1, 1)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 2), MergeCellRule.Grid.of(1, 2)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 3), MergeCellRule.Grid.of(1, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 3, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 3, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, 3)));

            //动态扩展列
            List<CellRenderData> cellRenderDataList = examStudent_header_1.getCells();
            for (int i = 0; i < cellRenderDataList.size(); i++) {
                CellRenderData cellRenderData = cellRenderDataList.get(i);
                String key = cellRenderData.getParagraphs().get(0).getContents().get(0).toString();
                if (targetUsualScoreSizeMap.containsKey(key)) {
                    Integer cellNum = targetUsualScoreSizeMap.get(key);
                    if (cellNum.intValue() > 0) {
                        mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, i), MergeCellRule.Grid.of(0, i + cellNum)));
                        mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, i), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, i + cellNum)));
                    }
                }
            }
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, 4), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, cellSize - 1)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1].getCells().size() - 2), MergeCellRule.Grid.of(1, rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1].getCells().size() - 2)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1].getCells().size() - 1), MergeCellRule.Grid.of(1, rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1].getCells().size() - 1)));

            Tables.TableBuilder tableBuilderExamStudent = Tables.ofPercentWidth(tbPercentWidth);
            for (int i = 0; i < rowRenderDataExamStudent.length; i++) {
                tableBuilderExamStudent.addRow(rowRenderDataExamStudent[i]);
            }
            tableRenderDataExamStudent = tableBuilderExamStudent.mergeRule(mergeCellRuleExamStudent).left().create();
            map.put("tableRenderDataExamStudent", tableRenderDataExamStudent);
            map.put("scoreList", scoreList);
            map.put("scoreListMap", scoreListMap);
            map.put("courseTargetWebDtoList", courseTargetWebDtoList);
            map.put("finalScatterYMap", finalScatterYMap);
        }
        return map;
    }

    /**
     * 构建分数图
     *
     * @param scoreList
     * @return
     */
    @Override
    public Map<String, Object> buildScoreOtherMap(List<BigDecimal> scoreList) {
        Map<String, Object> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(scoreList)) {
            Integer[] scoreSizes = new Integer[]{0, 0, 0, 0, 0};
            BigDecimal[] scorePercent = new BigDecimal[]{new BigDecimal(0), new BigDecimal(0), new BigDecimal(0), new BigDecimal(0), new BigDecimal(0)};
            Double maxScore = scoreList.stream().mapToDouble(s -> s.doubleValue()).max().orElse(0.0d);
            Double minScore = scoreList.stream().mapToDouble(s -> s.doubleValue()).min().orElse(0.0d);
            Double avgScore = scoreList.stream().mapToDouble(s -> s.doubleValue()).average().orElse(0.0d);
            for (BigDecimal d : scoreList) {
                if (d.doubleValue() <= 100 && d.doubleValue() >= 90d) {
                    scoreSizes[0] = scoreSizes[0] + 1;
                } else if (d.doubleValue() <= 89.99d && d.doubleValue() >= 80d) {
                    scoreSizes[1] = scoreSizes[1] + 1;
                } else if (d.doubleValue() <= 79.99d && d.doubleValue() >= 70d) {
                    scoreSizes[2] = scoreSizes[2] + 1;
                } else if (d.doubleValue() <= 69.99d && d.doubleValue() >= 60d) {
                    scoreSizes[3] = scoreSizes[3] + 1;
                } else {
                    scoreSizes[4] = scoreSizes[4] + 1;
                }
            }
            for (int i = 0; i < scoreSizes.length; i++) {
                scorePercent[i] = new BigDecimal(scoreSizes[i]).divide(new BigDecimal(scoreList.size()), 4, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP);
            }
            map.put("scoreSizes", scoreSizes);
            map.put("scorePercent", scorePercent);
            map.put("maxScore", maxScore);
            map.put("minScore", minScore);
            map.put("avgScore", avgScore);
        }
        return map;
    }

    /**
     * 构建西安工业大学课程目标达成度-分数图
     *
     * @param scoreListMap
     * @return
     */
    @Override
    public Map<String, Object> buildXagyuScoreOtherMap(LinkedMultiValueMap<String, BigDecimal> scoreListMap) {
        Map<String, Object> map = new LinkedHashMap<>();
        if (!CollectionUtils.isEmpty(scoreListMap)) {
            for (Map.Entry<String, List<BigDecimal>> entry : scoreListMap.entrySet()) {
                Integer[] scoreSizes = new Integer[]{0, 0, 0, 0, 0};
                BigDecimal[] scorePercent = new BigDecimal[]{new BigDecimal(0), new BigDecimal(0), new BigDecimal(0), new BigDecimal(0), new BigDecimal(0)};
                List<BigDecimal> scoreList = entry.getValue();
                for (BigDecimal d : scoreList) {
                    if (d.doubleValue() <= 100 && d.doubleValue() >= 90d) {
                        scoreSizes[0] = scoreSizes[0] + 1;
                    } else if (d.doubleValue() <= 89.99d && d.doubleValue() >= 80d) {
                        scoreSizes[1] = scoreSizes[1] + 1;
                    } else if (d.doubleValue() <= 79.99d && d.doubleValue() >= 70d) {
                        scoreSizes[2] = scoreSizes[2] + 1;
                    } else if (d.doubleValue() <= 69.99d && d.doubleValue() >= 60d) {
                        scoreSizes[3] = scoreSizes[3] + 1;
                    } else {
                        scoreSizes[4] = scoreSizes[4] + 1;
                    }
                }
                for (int i = 0; i < scoreSizes.length; i++) {
                    scorePercent[i] = new BigDecimal(scoreSizes[i]).divide(new BigDecimal(scoreList.size()), 4, BigDecimal.ROUND_HALF_UP).multiply(SystemConstant.PERCENT).setScale(2, BigDecimal.ROUND_HALF_UP);
                }
                map.put(entry.getKey(), new XagyuScoreOtherBean(entry.getKey(), scoreSizes, scorePercent));
            }
        }
        return map;
    }

    /**
     * 构建西安工业大学课程目标达成度-课程目标的评价标准
     *
     * @param map
     * @return
     */
    @Override
    public TableRenderData buildXagyuWordTable2(Map<String, Object> map) {
        RowRenderData[] rowRenderData = new RowRenderData[map.size() + 2];
        List<CellRenderData> cells_1 = new ArrayList<>(), cells_2 = new ArrayList<>();
        cells_1.add(Cells.of("课程目标").center().create());
        cells_1.add(Cells.of("评价标准").center().create());
        cells_1.add(Cells.of("").center().create());
        cells_1.add(Cells.of("").center().create());
        cells_1.add(Cells.of("").center().create());
        cells_1.add(Cells.of("").center().create());

        cells_2.add(Cells.of("").create());
        cells_2.add(Cells.of("优秀\n(100>x≥90)").create());
        cells_2.add(Cells.of("良好\n(90>x≥80)").create());
        cells_2.add(Cells.of("中\n(80>x≥70)").create());
        cells_2.add(Cells.of("及格\n(70>x≥60)").create());
        cells_2.add(Cells.of("不及格\n(x<60)").create());

        RowRenderData row_1 = new RowRenderData();
        row_1.setCells(cells_1);
        row_1.setRowStyle(this.getHeadRowStyle());

        RowRenderData row_2 = new RowRenderData();
        row_2.setCells(cells_2);
        row_2.setRowStyle(this.getHeadRowStyle());

        rowRenderData[0] = row_1;
        rowRenderData[1] = row_2;

        int index = 2;
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            if (Objects.nonNull(entry.getValue())) {
                RowRenderData score_row = new RowRenderData();
                List<CellRenderData> score_cells = new ArrayList<>();
                score_cells.add(Cells.of(entry.getKey()).create());
                XagyuScoreOtherBean xagyuScoreOtherBean = (XagyuScoreOtherBean) entry.getValue();
                Integer[] scoreSizes = xagyuScoreOtherBean.getScoreSizes();
                BigDecimal[] scorePercent = xagyuScoreOtherBean.getScorePercent();
                for (int i = 0; i < scoreSizes.length; i++) {
                    score_cells.add(Cells.of(scoreSizes[i] + "(" + SystemConstant.df.format(scorePercent[i]) + "%)").create());
                }
                score_row.setCells(score_cells);
                score_row.setRowStyle(this.getRowStyle());
                rowRenderData[index] = score_row;
                index++;
            }
        }

        MergeCellRule mergeCellRule2 = MergeCellRule.builder().
                map(MergeCellRule.Grid.of(0, 0), MergeCellRule.Grid.of(1, 0)).
                map(MergeCellRule.Grid.of(0, 1), MergeCellRule.Grid.of(0, cells_2.size() - 1)).
                build();

        Tables.TableBuilder tableBuilder2 = Tables.ofPercentWidth(tbPercentWidth);
        for (int i = 0; i < rowRenderData.length; i++) {
            tableBuilder2.addRow(rowRenderData[i]);
        }
        return tableBuilder2.mergeRule(mergeCellRule2).left().create();
    }

    /**
     * 查询散点图值
     *
     * @param trBasicInfo
     * @return
     */
    @Override
    public Map<String, Object> queryScatterYMap(TRBasicInfo trBasicInfo) {
        List<TRExamStudent> trExamStudentList = trExamStudentService.list(
                new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
        LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap = null;
        List<ReportExamStudentDto> examStudentList = null;
        if (!CollectionUtils.isEmpty(trExamStudentList)) {
            Map<Long, ConcurrentMap<String, String>> scatterYMap = new LinkedHashMap<>();
            examStudentList = new ArrayList<>(trExamStudentList.size());
            for (TRExamStudent trExamStudent : trExamStudentList) {
                if (!Objects.equals(trExamStudent.getName(), "各课程目标平均分")
                        && !Objects.equals(trExamStudent.getName(), "平均分")
                        && !Objects.equals(trExamStudent.getName(), "目标分")) {
                    List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSONArray.parseArray(trExamStudent.getResultDetail(), ReportExamStudentTargetDto.class);
                    for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                        if (Objects.nonNull(reportExamStudentTargetDto) && Objects.nonNull(reportExamStudentTargetDto.getMatrixDegree())) {
                            trBasicInfoService.getScatterValue(scatterYMap, trExamStudent.getStudentCode(), reportExamStudentTargetDto);
                        }
                    }
                }
                examStudentList.add(new ReportExamStudentDto(trExamStudent));
            }
            if (scatterYMap.size() > 0) {
                finalScatterYMap = trBasicInfoService.getFinalScatterValue(scatterYMap);
                if (Objects.nonNull(trBasicInfo.getCourseSuggest())) {
                    List<CourseSuggestDto> courseSuggestDtoList = JSONArray.parseArray(trBasicInfo.getCourseSuggest(), CourseSuggestDto.class);
                    List<CourseSuggestDto> CourseSuggestDtoFilterList = courseSuggestDtoList.stream().filter(s -> !scatterYMap.containsKey(s.getTargetId())).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(CourseSuggestDtoFilterList)) {
                        List<List<Map<String, String>>> collectionList = new ArrayList<>(finalScatterYMap.values());
                        List<Map<String, String>> list = collectionList.get(0);
                        List<Map<String, String>> listNew = new LinkedList<>();
                        for (Map<String, String> map : list) {
                            Map<String, String> mapTemp = new LinkedHashMap<>();
                            map.forEach((k, v) -> {
                                mapTemp.put(k, "0.0");
                            });
                            listNew.add(mapTemp);
                        }
                        for (CourseSuggestDto c : CourseSuggestDtoFilterList) {
                            finalScatterYMap.put(c.getTargetId(), listNew);
                        }
                    }
                }
            }
        }
        return ImmutableMap.of("examStudentList", examStudentList, "finalScatterYMap", finalScatterYMap);
    }

    /**
     * 构建散点图
     *
     * @param list
     * @param courseTargetWebDto
     * @return
     */
    protected ChartMultiSeriesRenderData buildScatterMap(List<Map<String, String>> list, CourseTargetWebDto courseTargetWebDto) {
        List<String> valueXList = new ArrayList<>();
        List<Number> titleYList = new ArrayList<>();
        int index = 1;
        for (Map<String, String> m : list) {
            for (Map.Entry<String, String> entry : m.entrySet()) {
                valueXList.add(index + "");
                titleYList.add(Double.parseDouble(entry.getValue()));
                ++index;
            }
        }
        String evaluationValue = Objects.nonNull(courseTargetWebDto.getEvaluationValue()) ? SystemConstant.df.format(courseTargetWebDto.getEvaluationValue()) : "0";
        ChartMultiSeriesRenderData scatterMap = Charts.ofMultiSeries(courseTargetWebDto.getTargetName(), valueXList.toArray(new String[valueXList.size()]))
                .addSeries("Y值", titleYList.toArray(new Number[titleYList.size()]))
                .setxAsixTitle("学生代号")
                .setyAsixTitle("达成值" + evaluationValue)
                .create();
        return scatterMap;
    }
}
