package com.qmth.teachcloud.task.job;

import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.threadPool.DynamicMyThreadPool;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.task.job.service.JobService;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;

/**
 * pdf自动生成任务
 */
public class CreatePdfTaskJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(CreatePdfTaskJob.class);

    @Resource
    JobService jobService;

    @Resource
    LockService lockService;

    @Resource
    DynamicMyThreadPool dynamicMyThreadPool;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) {
        if (lockService.trylock(LockType.CREATE_PDF, LockType.CREATE_PDF.name())) {
            try {
                jobService.createPdfTask();
                jobExecutionContext.setResult(dynamicMyThreadPool.toString());
            } catch (Exception e) {
                log.error(SystemConstant.LOG_ERROR, e);
            } finally {
                lockService.unlock(LockType.CREATE_PDF, LockType.CREATE_PDF.name());
            }
        }
    }
}
